@@ -1,7 +1,7 @@
 /* asn1.c
  * Routines for ASN.1 BER dissection
  *
- * $Id: asn1.c,v 1.16 2002/08/02 23:35:45 jmayer Exp $
+ * $Id: asn1.c,v 1.18 2003/04/28 00:31:26 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -29,7 +29,7 @@
 
 /*
  * MODULE INFORMATION
- * ------------------ 
+ * ------------------
  *     FILE     NAME:       g_asn1.c
  *     SYSTEM   NAME:       ASN1 Basic Encoding
  *     ORIGINAL AUTHOR(S):  Dirk Wisse
@@ -48,7 +48,7 @@
  *              asn1_int_decode (asn1, end_of_int, &integer);
  *              asn1_eoc_decode (asn1, end_of_seq);
  *              asn1_close (asn1, &offset);
- *              
+ *
  *              For indefinite encoding end_of_seq and &end_of_seq in the
  *              example above should be replaced by NULL.
  *              For indefinite decoding nothing has to be changed.
@@ -113,7 +113,7 @@ asn1_open(ASN1_SCK *asn1, tvbuff_t *tvb, int offset)
  * RETURNS:     void
  */
 
-void 
+void
 asn1_close(ASN1_SCK *asn1, int *offset)
 {
     *offset = asn1->offset;
@@ -332,7 +332,7 @@ asn1_eoc_decode (ASN1_SCK *asn1, int eoc)
 {
     int    ret;
     guchar ch;
-    
+
     if (eoc == -1) {
         ret = asn1_octet_decode (asn1, &ch);
         if (ret != ASN1_ERR_NOERROR)
@@ -592,35 +592,37 @@ done:
  *                  )
  * DESCRIPTION: Decodes Bit String.
  *              Parameters:
- *              asn1:   pointer to ASN1 socket.
- *              eoc: offset of end of encoding, or -1 if indefinite.
- *              bits:   pointer to begin of Bit String.
- *              size:   Size of Bit String in characters.
- *              len:    Length of Bit String in characters.
- *              unused: Number of unused bits in last character.
+ *              asn1:    pointer to ASN1 socket.
+ *              enc_len: length of value.
+ *              bits:    pointer to variable we set to point to strring
+ *              len:     Size of Bit String in characters.
+ *              unused:  Number of unused bits in last character.
  * RETURNS:     ASN1_ERR value (ASN1_ERR_NOERROR on success)
  */
 int
-asn1_bits_decode ( ASN1_SCK *asn1, int eoc, guchar **bits,
+asn1_bits_decode ( ASN1_SCK *asn1, int enc_len, guchar **bits,
 		     guint *len, guchar *unused)
-
 {
     int ret;
+    int eoc;
+    guchar *ptr;
 
+    eoc = asn1->offset + enc_len;
     *bits = NULL;
     ret = asn1_octet_decode (asn1, unused);
     if (ret != ASN1_ERR_NOERROR)
         return ret;
     *len = 0;
-    *bits = g_malloc(eoc - asn1->offset);
+    ptr = *bits = g_malloc(enc_len);
     while (asn1->offset < eoc) {
-        ret = asn1_octet_decode (asn1, (guchar *)bits++);
+        ret = asn1_octet_decode (asn1, (guchar *)ptr++);
         if (ret != ASN1_ERR_NOERROR) {
             g_free(*bits);
             *bits = NULL;
 	    return ret;
           }
     }
+    *len = ptr - *bits;
     return ASN1_ERR_NOERROR;
 }
 
@@ -829,7 +831,7 @@ asn1_oid_value_decode ( ASN1_SCK *asn1, int enc_len, subid_t **oid, guint *len)
     size = enc_len + 1;
     *oid = g_malloc(size * sizeof(gulong));
     optr = *oid;
- 
+
     ret = asn1_subid_decode (asn1, &subid);
     if (ret != ASN1_ERR_NOERROR) {
 	g_free(*oid);
