@@ -1,20 +1,20 @@
 /*
- * $Id: dfvm.h,v 1.6 2002/05/09 23:50:30 gram Exp $
+ * $Id: dfvm.h,v 1.8 2002/10/16 16:32:59 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
  * Copyright 2001 Gerald Combs
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
@@ -32,7 +32,7 @@
 typedef enum {
 	EMPTY,
 	FVALUE,
-	FIELD_ID,
+	HFINFO,
 	INSN_NUMBER,
 	REGISTER,
 	INTEGER,
@@ -43,9 +43,10 @@ typedef struct {
 	dfvm_value_type_t	type;
 
 	union {
-		fvalue_t	*fvalue;
-		guint32		numeric;
-		drange		*drange;
+		fvalue_t		*fvalue;
+		guint32			numeric;
+		drange			*drange;
+		header_field_info	*hfinfo;
 	} value;
 
 } dfvm_value_t;
@@ -67,7 +68,7 @@ typedef enum {
 	ANY_LT,
 	ANY_LE,
 	MK_RANGE
-	
+
 } dfvm_opcode_t;
 
 typedef struct {
