@@ -1,20 +1,20 @@
 /*
- * $Id: gencode.c,v 1.6 2002/03/02 20:48:11 guy Exp $
+ * $Id: gencode.c,v 1.9 2002/11/01 17:37:25 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
  * Copyright 2001 Gerald Combs
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
@@ -45,45 +45,57 @@ dfw_append_insn(dfwork_t *dfw, dfvm_insn_t *insn)
 
 /* returns register number */
 static int
-dfw_append_read_tree(dfwork_t *dfw, int field_id)
+dfw_append_read_tree(dfwork_t *dfw, header_field_info *hfinfo)
 {
 	dfvm_insn_t	*insn;
 	dfvm_value_t	*val1, *val2;
 	int		reg = -1;
+	gboolean	added_new_hfinfo = FALSE;
+
+	/* Rewind to find the first field of this name. */
+	while (hfinfo->same_name_prev) {
+		hfinfo = hfinfo->same_name_prev;
+	}
 
 	/* Keep track of which registers
-	 * were used for which field_id's so that we
+	 * were used for which hfinfo's so that we
 	 * can re-use registers. */
 	reg = GPOINTER_TO_UINT(
-			g_hash_table_lookup(dfw->loaded_fields,
-			GINT_TO_POINTER(field_id)));
+			g_hash_table_lookup(dfw->loaded_fields, hfinfo));
 	if (reg) {
 		/* Reg's are stored in has as reg+1, so
-		 * that the non-existence of a field_id in
+		 * that the non-existence of a hfinfo in
 		 * the hash, or 0, can be differentiated from
-		 * a field_id being loaded into register #0. */
+		 * a hfinfo being loaded into register #0. */
 		reg--;
 	}
 	else {
 		reg = dfw->next_register++;
 		g_hash_table_insert(dfw->loaded_fields,
-			GUINT_TO_POINTER(field_id),
-			GUINT_TO_POINTER(reg + 1));
+			hfinfo, GUINT_TO_POINTER(reg + 1));
 
-        /* Record the FIELD_ID in hash of interesting fields. */
-        g_hash_table_insert(dfw->interesting_fields,
-            GINT_TO_POINTER(field_id), GUINT_TO_POINTER(TRUE));
+		added_new_hfinfo = TRUE;
 	}
 
 	insn = dfvm_insn_new(READ_TREE);
-	val1 = dfvm_value_new(FIELD_ID);
-	val1->value.numeric = field_id;
+	val1 = dfvm_value_new(HFINFO);
+	val1->value.hfinfo = hfinfo;
 	val2 = dfvm_value_new(REGISTER);
 	val2->value.numeric = reg;
 
 	insn->arg1 = val1;
 	insn->arg2 = val2;
 	dfw_append_insn(dfw, insn);
+	
+	if (added_new_hfinfo) {
+		while (hfinfo) {
+			/* Record the FIELD_ID in hash of interesting fields. */
+			g_hash_table_insert(dfw->interesting_fields,
+			    GINT_TO_POINTER(hfinfo->id),
+			    GUINT_TO_POINTER(TRUE));
+			hfinfo = hfinfo->same_name_next;
+		}
+	}
 
 	return reg;
 }
@@ -119,7 +131,7 @@ dfw_append_mk_range(dfwork_t *dfw, stnode_t *node)
 	dfvm_value_t		*val;
 
 	hfinfo = sttype_range_hfinfo(node);
-	hf_reg = dfw_append_read_tree(dfw, hfinfo->id);
+	hf_reg = dfw_append_read_tree(dfw, hfinfo);
 
 	insn = dfvm_insn_new(MK_RANGE);
 
@@ -159,7 +171,7 @@ gen_relation(dfwork_t *dfw, dfvm_opcode_t op, stnode_t *st_arg1, stnode_t *st_ar
 
 	if (type1 == STTYPE_FIELD) {
 		hfinfo = stnode_data(st_arg1);
-		reg1 = dfw_append_read_tree(dfw, hfinfo->id);
+		reg1 = dfw_append_read_tree(dfw, hfinfo);
 
 		insn = dfvm_insn_new(IF_FALSE_GOTO);
 		jmp1 = dfvm_value_new(INSN_NUMBER);
@@ -178,7 +190,7 @@ gen_relation(dfwork_t *dfw, dfvm_opcode_t op, stnode_t *st_arg1, stnode_t *st_ar
 
 	if (type2 == STTYPE_FIELD) {
 		hfinfo = stnode_data(st_arg2);
-		reg2 = dfw_append_read_tree(dfw, hfinfo->id);
+		reg2 = dfw_append_read_tree(dfw, hfinfo);
 
 		insn = dfvm_insn_new(IF_FALSE_GOTO);
 		jmp2 = dfvm_value_new(INSN_NUMBER);
@@ -229,16 +241,25 @@ gen_test(dfwork_t *dfw, stnode_t *st_node)
 			break;
 
 		case TEST_OP_EXISTS:
-			val1 = dfvm_value_new(FIELD_ID);
+			val1 = dfvm_value_new(HFINFO);
 			hfinfo = stnode_data(st_arg1);
-			val1->value.numeric = hfinfo->id;
+
+			/* Rewind to find the first field of this name. */
+			while (hfinfo->same_name_prev) {
+				hfinfo = hfinfo->same_name_prev;
+			}
+			val1->value.hfinfo = hfinfo;
 			insn = dfvm_insn_new(CHECK_EXISTS);
 			insn->arg1 = val1;
 			dfw_append_insn(dfw, insn);
 
-            /* Record the FIELD_ID in hash of interesting fields. */
-            g_hash_table_insert(dfw->interesting_fields,
-                GINT_TO_POINTER(hfinfo->id), GUINT_TO_POINTER(TRUE));
+			/* Record the FIELD_ID in hash of interesting fields. */
+			while (hfinfo) {
+				g_hash_table_insert(dfw->interesting_fields,
+					GINT_TO_POINTER(hfinfo->id),
+					GUINT_TO_POINTER(TRUE));
+				hfinfo = hfinfo->same_name_next;
+			}
 
 			break;
 
@@ -331,7 +352,7 @@ typedef struct {
     int i;
     int *fields;
 } hash_key_iterator;
- 
+
 static void
 get_hash_key(gpointer key, gpointer value _U_, gpointer user_data)
 {
@@ -356,7 +377,7 @@ dfw_interesting_fields(dfwork_t *dfw, int *caller_num_fields)
 
     hki.fields = g_new(int, num_fields);
     hki.i = 0;
-        
+
     g_hash_table_foreach(dfw->interesting_fields, get_hash_key, &hki);
     *caller_num_fields = num_fields;
     return hki.fields;
