@@ -1,20 +1,20 @@
 /*
- * $Id: semcheck.c,v 1.11 2002/05/09 23:50:30 gram Exp $
+ * $Id: semcheck.c,v 1.15 2002/12/19 02:58:49 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
  * Copyright 2001 Gerald Combs
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
@@ -47,6 +47,7 @@ compatible_ftypes(ftenum_t a, ftenum_t b)
 	switch (a) {
 		case FT_NONE:
 		case FT_PROTOCOL:
+		case FT_FLOAT:
 		case FT_DOUBLE:
 		case FT_ABSOLUTE_TIME:
 		case FT_RELATIVE_TIME:
@@ -63,6 +64,7 @@ compatible_ftypes(ftenum_t a, ftenum_t b)
 			return (b == FT_ETHER || b == FT_BYTES || b == FT_UINT_BYTES);
 
 		case FT_BOOLEAN:
+		case FT_FRAMENUM:
 		case FT_UINT8:
 		case FT_UINT16:
 		case FT_UINT24:
@@ -73,6 +75,7 @@ compatible_ftypes(ftenum_t a, ftenum_t b)
 		case FT_INT32:
 			switch (b) {
 				case FT_BOOLEAN:
+				case FT_FRAMENUM:
 				case FT_UINT8:
 				case FT_UINT16:
 				case FT_UINT24:
@@ -85,7 +88,7 @@ compatible_ftypes(ftenum_t a, ftenum_t b)
 				default:
 					return FALSE;
 			}
-			
+
 		case FT_STRING:
 		case FT_STRINGZ:
 		case FT_UINT_STRING:
@@ -125,14 +128,15 @@ mk_uint32_fvalue(guint32 val)
 static fvalue_t*
 mk_fvalue_from_val_string(header_field_info *hfinfo, char *s)
 {
-	static true_false_string        default_tf = { "True", "False" };
-	true_false_string		*tf = &default_tf;
-	value_string			*vals;
+	static const true_false_string  default_tf = { "True", "False" };
+	const true_false_string		*tf = &default_tf;
+	const value_string		*vals;
 
 	/* Early return? */
 	switch(hfinfo->type) {
 		case FT_NONE:
 		case FT_PROTOCOL:
+		case FT_FLOAT:
 		case FT_DOUBLE:
 		case FT_ABSOLUTE_TIME:
 		case FT_RELATIVE_TIME:
@@ -150,6 +154,7 @@ mk_fvalue_from_val_string(header_field_info *hfinfo, char *s)
 			return FALSE;
 
 		case FT_BOOLEAN:
+		case FT_FRAMENUM:
 		case FT_UINT8:
 		case FT_UINT16:
 		case FT_UINT24:
@@ -159,7 +164,7 @@ mk_fvalue_from_val_string(header_field_info *hfinfo, char *s)
 		case FT_INT24:
 		case FT_INT32:
 			break;
-			
+
 		case FT_NUM_TYPES:
 			g_assert_not_reached();
 	}
@@ -220,6 +225,7 @@ is_bytes_type(enum ftenum type)
 
 		case FT_NONE:
 		case FT_PROTOCOL:
+		case FT_FLOAT:
 		case FT_DOUBLE:
 		case FT_ABSOLUTE_TIME:
 		case FT_RELATIVE_TIME:
@@ -229,6 +235,7 @@ is_bytes_type(enum ftenum type)
 		case FT_STRINGZ:
 		case FT_UINT_STRING:
 		case FT_BOOLEAN:
+		case FT_FRAMENUM:
 		case FT_UINT8:
 		case FT_UINT16:
 		case FT_UINT24:
@@ -240,7 +247,7 @@ is_bytes_type(enum ftenum type)
 		case FT_INT32:
 		case FT_INT64:
 			return FALSE;
-			
+
 		case FT_NUM_TYPES:
 			g_assert_not_reached();
 	}
@@ -345,7 +352,7 @@ check_relation_LHS_STRING(FtypeCanFunc can_func _U_, stnode_t *st_node,
 
 	type1 = stnode_type_id(st_arg1);
 	type2 = stnode_type_id(st_arg2);
-		
+
 	if (type2 == STTYPE_FIELD) {
 		hfinfo2 = stnode_data(st_arg2);
 		ftype2 = hfinfo2->type;
