@@ -1,5 +1,5 @@
 /*
- * $Id: ftype-double.c,v 1.7 2002/08/28 20:41:00 jmayer Exp $
+ * $Id: ftype-double.c,v 1.8 2003/02/08 04:22:37 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -124,6 +124,8 @@ ftype_register_double(void)
 		double_fvalue_new,
 		NULL,
 		val_from_string,
+		NULL,				/* val_to_string_repr */
+		NULL,				/* len_string_repr */
 
 		NULL,
 		NULL,
@@ -151,6 +153,8 @@ ftype_register_double(void)
 		double_fvalue_new,
 		NULL,
 		val_from_string,
+		NULL,				/* val_to_string_repr */
+		NULL,				/* len_string_repr */
 
 		NULL,
 		NULL,
