@@ -1,5 +1,5 @@
 /*
- * $Id: ftypes.c,v 1.7 2002/08/28 20:41:00 jmayer Exp $
+ * $Id: ftypes.c,v 1.8 2003/02/08 04:22:37 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -271,6 +271,20 @@ fvalue_length(fvalue_t *fv)
 		return fv->ftype->wire_size;
 }
 
+int
+fvalue_string_repr_len(fvalue_t *fv)
+{
+	g_assert(fv->ftype->len_string_repr);
+	return fv->ftype->len_string_repr(fv);
+}
+
+char *
+fvalue_to_string_repr(fvalue_t *fv)
+{
+	g_assert(fv->ftype->val_to_string_repr);
+	return fv->ftype->val_to_string_repr(fv);
+}
+
 typedef struct {
 	fvalue_t	*fv;
 	GByteArray	*bytes;
