@@ -1,7 +1,7 @@
 /* packet.c
  * Routines for packet disassembly
  *
- * $Id: packet.c,v 1.86 2003/01/22 06:26:36 guy Exp $
+ * $Id: packet.c,v 1.91 2003/04/23 10:20:27 sahlberg Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -249,34 +249,37 @@ void
 dissect_packet(epan_dissect_t *edt, union wtap_pseudo_header *pseudo_header,
 	       const guchar *pd, frame_data *fd, column_info *cinfo)
 {
-	int i;
-
-	if (cinfo != NULL) {
-		for (i = 0; i < cinfo->num_cols; i++) {
-			cinfo->col_buf[i][0] = '\0';
-			cinfo->col_data[i] = cinfo->col_buf[i];
-			cinfo->col_expr[i][0] = '\0';
-			cinfo->col_expr_val[i][0] = '\0';
-		}
-
-		col_set_writable(cinfo, TRUE);
-	}
+	if (cinfo != NULL)
+		col_init(cinfo);
 	edt->pi.current_proto = "<Missing Protocol Name>";
 	edt->pi.cinfo = cinfo;
 	edt->pi.fd = fd;
 	edt->pi.pseudo_header = pseudo_header;
 	edt->pi.data_src = NULL;
 	edt->pi.dl_src.type = AT_NONE;
+	edt->pi.dl_src.len = 0;
+	edt->pi.dl_src.data = NULL;
 	edt->pi.dl_dst.type = AT_NONE;
+	edt->pi.dl_dst.len = 0;
+	edt->pi.dl_dst.data = NULL;
 	edt->pi.net_src.type = AT_NONE;
+	edt->pi.net_src.len = 0;
+	edt->pi.net_src.data = NULL;
 	edt->pi.net_dst.type = AT_NONE;
+	edt->pi.net_dst.len = 0;
+	edt->pi.net_dst.data = NULL;
 	edt->pi.src.type = AT_NONE;
+	edt->pi.src.len = 0;
+	edt->pi.src.data = NULL;
 	edt->pi.dst.type = AT_NONE;
+	edt->pi.dst.len = 0;
+	edt->pi.dst.data = NULL;
 	edt->pi.ethertype = 0;
 	edt->pi.ipproto  = 0;
 	edt->pi.ipxptype = 0;
 	edt->pi.ctype = CT_NONE;
 	edt->pi.circuit_id = 0;
+	edt->pi.noreassembly_reason = "";
 	edt->pi.fragmented = FALSE;
 	edt->pi.in_error_pkt = FALSE;
 	edt->pi.ptype = PT_NONE;
@@ -284,6 +287,7 @@ dissect_packet(epan_dissect_t *edt, union wtap_pseudo_header *pseudo_header,
 	edt->pi.destport = 0;
 	edt->pi.match_port = 0;
 	edt->pi.can_desegment = 0;
+	edt->pi.want_pdu_tracking = 0;
 	edt->pi.p2p_dir = P2P_DIR_UNKNOWN;
 	edt->pi.private_data = NULL;
         edt->pi.oxid = 0;
@@ -484,6 +488,7 @@ call_dissector_work(dissector_handle_t handle, tvbuff_t *tvb,
 		pinfo->net_dst = save_net_dst;
 		pinfo->src = save_src;
 		pinfo->dst = save_dst;
+		pinfo->want_pdu_tracking = 0;
 	} else {
 		/*
 		 * Just call the subdissector.
@@ -492,6 +497,7 @@ call_dissector_work(dissector_handle_t handle, tvbuff_t *tvb,
 	}
 	pinfo->current_proto = saved_proto;
 	pinfo->can_desegment = saved_can_desegment;
+	pinfo->want_pdu_tracking -= !!(pinfo->want_pdu_tracking);
 	return ret;
 }
 
@@ -1070,6 +1076,7 @@ dissector_try_heuristic(heur_dissector_list_t sub_dissectors,
 	}
 	pinfo->current_proto = saved_proto;
 	pinfo->can_desegment=saved_can_desegment;
+	pinfo->want_pdu_tracking -= !!(pinfo->want_pdu_tracking);
 	return status;
 }
 
