@@ -1,7 +1,7 @@
 /* plugins.c
  * plugin routines
  *
- * $Id: plugins.c,v 1.62 2002/12/08 22:22:03 guy Exp $
+ * $Id: plugins.c,v 1.70 2003/05/01 21:10:42 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -65,6 +65,8 @@
 #include "packet-giop.h"
 #include "packet-tpkt.h"
 #include "packet-tcp.h"
+#include "tap.h"
+#include "asn1.h"
 #include "plugins/plugin_table.h"
 static plugin_address_table_t	patable;
 #endif
@@ -487,7 +489,50 @@ init_plugins(const char *plugin_dir)
 	patable.p_fragment_delete		= fragment_delete;
 	patable.p_show_fragment_tree		= show_fragment_tree;
 	patable.p_show_fragment_seq_tree	= show_fragment_seq_tree;
+	
+	patable.p_register_tap			= register_tap;
+	patable.p_tap_queue_packet		= tap_queue_packet;
 
+	patable.p_asn1_open			= asn1_open;
+	patable.p_asn1_close			= asn1_close;
+	patable.p_asn1_octet_decode		= asn1_octet_decode;
+	patable.p_asn1_tag_decode		= asn1_tag_decode;
+	patable.p_asn1_id_decode		= asn1_id_decode;
+	patable.p_asn1_length_decode		= asn1_length_decode;
+	patable.p_asn1_header_decode		= asn1_header_decode;
+	patable.p_asn1_eoc			= asn1_eoc;
+	patable.p_asn1_eoc_decode		= asn1_eoc_decode;
+	patable.p_asn1_null_decode		= asn1_null_decode;
+	patable.p_asn1_bool_decode		= asn1_bool_decode;
+	patable.p_asn1_int32_value_decode	= asn1_int32_value_decode;
+	patable.p_asn1_int32_decode		= asn1_int32_decode;
+	patable.p_asn1_uint32_value_decode	= asn1_uint32_value_decode;
+	patable.p_asn1_uint32_decode		= asn1_uint32_decode;
+	patable.p_asn1_bits_decode		= asn1_bits_decode;
+	patable.p_asn1_string_value_decode	= asn1_string_value_decode;
+	patable.p_asn1_string_decode		= asn1_string_decode;
+	patable.p_asn1_octet_string_decode	= asn1_octet_string_decode;
+	patable.p_asn1_subid_decode		= asn1_subid_decode;
+	patable.p_asn1_oid_value_decode		= asn1_oid_value_decode;
+	patable.p_asn1_oid_decode		= asn1_oid_decode;
+	patable.p_asn1_sequence_decode		= asn1_sequence_decode;
+	patable.p_asn1_err_to_str		= asn1_err_to_str;
+	
+	patable.p_proto_item_set_end		= proto_item_set_end;
+	patable.p_proto_tree_add_none_format	= proto_tree_add_none_format;
+
+	patable.p_except_init			= except_init;
+	patable.p_except_deinit			= except_deinit;
+	patable.p_except_rethrow		= except_rethrow;
+	patable.p_except_throwd			= except_throwd;
+	patable.p_except_throwf			= except_throwf;
+	patable.p_except_unhandled_catcher	= except_unhandled_catcher;
+	patable.p_except_take_data	       	= except_take_data;
+	patable.p_except_set_allocator		= except_set_allocator;
+	patable.p_except_alloc			= except_alloc;
+	patable.p_except_free			= except_free;
+	patable.p_except_pop			= except_pop;
+	patable.p_except_setup_try		= except_setup_try;
 #endif
 
 #ifdef WIN32
