@@ -1,7 +1,7 @@
 /* proto.c
  * Routines for protocol tree
  *
- * $Id: proto.c,v 1.80 2003/01/21 19:20:49 gerald Exp $
+ * $Id: proto.c,v 1.83 2003/04/29 21:27:14 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -1945,6 +1945,25 @@ proto_item_set_len(proto_item *pi, gint length)
 	fi->length = length;
 }
 
+/*
+ * Sets the length of the item based on its start and on the specified
+ * offset, which is the offset past the end of the item; as the start
+ * in the item is relative to the beginning of the data source tvbuff,
+ * we need to pass in a tvbuff - the end offset is relative to the beginning
+ * of that tvbuff.
+ */
+void
+proto_item_set_end(proto_item *pi, tvbuff_t *tvb, gint end)
+{
+	field_info *fi;
+
+	if (pi == NULL)
+		return;
+	fi = PITEM_FINFO(pi);
+	end += tvb_raw_offset(tvb);
+	fi->length = end - fi->start;
+}
+
 int
 proto_item_get_len(proto_item *pi)
 {
@@ -2808,7 +2827,6 @@ hfinfo_uint_vals_format(header_field_info *hfinfo)
 
 	switch(hfinfo->display) {
 		case BASE_DEC:
-		case BASE_BIN: /* I'm lazy */
 			format = "%s: %s (%u)";
 			break;
 		case BASE_OCT: /* I'm lazy */
@@ -2854,7 +2872,6 @@ hfinfo_uint_format(header_field_info *hfinfo)
 	} else {
 		switch(hfinfo->display) {
 			case BASE_DEC:
-			case BASE_BIN: /* I'm lazy */
 				format = "%s: %u";
 				break;
 			case BASE_OCT: /* I'm lazy */
@@ -2894,7 +2911,6 @@ hfinfo_int_vals_format(header_field_info *hfinfo)
 
 	switch(hfinfo->display) {
 		case BASE_DEC:
-		case BASE_BIN: /* I'm lazy */
 			format = "%s: %s (%d)";
 			break;
 		case BASE_OCT: /* I'm lazy */
@@ -2934,7 +2950,6 @@ hfinfo_int_format(header_field_info *hfinfo)
 	/* Pick the proper format string */
 	switch(hfinfo->display) {
 		case BASE_DEC:
-		case BASE_BIN: /* I'm lazy */
 			format = "%s: %d";
 			break;
 		case BASE_OCT: /* I'm lazy */
@@ -3254,7 +3269,6 @@ hfinfo_numeric_format(header_field_info *hfinfo)
 		switch(hfinfo->display) {
 			case BASE_DEC:
 			case BASE_OCT: /* I'm lazy */
-			case BASE_BIN: /* I'm lazy */
 				switch(hfinfo->type) {
 					case FT_UINT8:
 					case FT_UINT16:
@@ -3510,18 +3524,6 @@ proto_alloc_dfilter_string(field_info *finfo, guint8 *pd)
 					fvalue_get_floating(finfo->value));
 			break;
 
-		case FT_ETHER:
-			/*
-			 * 4 bytes for " == ".
-			 * 17 bytes for "NN:NN:NN:NN:NN:NN".
-			 * 1 byte for the trailing '\0'.
-			 */
-			dfilter_len = abbrev_len + 4 + 17 + 1;
-			buf = g_malloc0(dfilter_len);
-			snprintf(buf, dfilter_len, "%s == %s",
-					hfinfo->abbrev,
-					ether_to_str(fvalue_get(finfo->value)));
-			break;
 
 		case FT_ABSOLUTE_TIME:
 			/*
@@ -3566,24 +3568,21 @@ proto_alloc_dfilter_string(field_info *finfo, guint8 *pd)
 				 hfinfo->abbrev, value_str);
 			break;
 
+		case FT_ETHER:
 		case FT_BYTES:
 		case FT_UINT_BYTES:
 			/*
 			 * 4 bytes for " == ".
-			 * 3 bytes for each byte of the byte string, as
-			 * "NN:", minus 1 byte as there's no trailing ":".
 			 * 1 byte for the trailing '\0'.
 			 *
-			 * XXX - what are the other 2 bytes for?
 			 */
-			dfilter_len = fvalue_length(finfo->value)*3 - 1;
-			dfilter_len += abbrev_len + 4 + 2 + 1;
+			dfilter_len = fvalue_string_repr_len(finfo->value);
+			dfilter_len += abbrev_len + 4 + 1;
 			buf = g_malloc0(dfilter_len);
-			snprintf(buf, dfilter_len, "%s == %s",
-				 hfinfo->abbrev,
-				 /* XXX - bytes_to_str_punct() will truncate long strings with '...' */
-				 bytes_to_str_punct(fvalue_get(finfo->value),
-					 fvalue_length(finfo->value),':'));
+			snprintf(buf, dfilter_len, "%s == ", hfinfo->abbrev);
+            stringified = fvalue_to_string_repr(finfo->value);
+            strcat(buf, stringified);
+            g_free(stringified);
 			break;
 
 		default:
