@@ -9,7 +9,7 @@
  * 		the data of a backing tvbuff, or can be a composite of
  * 		other tvbuffs.
  *
- * $Id: tvbuff.c,v 1.40 2002/08/28 20:40:45 jmayer Exp $
+ * $Id: tvbuff.c,v 1.43 2003/04/30 02:35:23 gerald Exp $
  *
  * Copyright (c) 2000 by Gilbert Ramirez <gram@alumni.rice.edu>
  *
@@ -1617,6 +1617,41 @@ tvb_memeql(tvbuff_t *tvb, gint offset, const guint8 *str, gint size)
 	}
 }
 
+/* Convert a string from Unicode to ASCII.  At the moment we fake it by
+ * assuming all characters are ASCII  )-:  The caller must free the
+ * result returned.  The len parameter is the number of guint16's to
+ * convert from Unicode. */
+char *
+tvb_fake_unicode(tvbuff_t *tvb, int offset, int len, gboolean little_endian)
+{
+	char *buffer;
+	int i;
+	guint16 character;
+
+	/* Make sure we have enough data before allocating the buffer,
+	   so we don't blow up if the length is huge.
+	   We do so by attempting to fetch the last character; it'll
+	   throw an exception if it's past the end.
+
+           The byte order we use to fetch it is irrelevant here. */
+	tvb_get_letohs(tvb, offset + 2*(len - 1));
+
+	/* We know we won't throw an exception, so we don't have to worry
+	   about leaking this buffer. */
+	buffer = g_malloc(len + 1);
+
+	for (i = 0; i < len; i++) {
+		character = little_endian ? tvb_get_letohs(tvb, offset)
+					  : tvb_get_ntohs(tvb, offset);
+		buffer[i] = character & 0xff;
+		offset += 2;
+	}
+
+	buffer[len] = 0;
+
+	return buffer;
+}
+
 /*
  * Format the data in the tvb from offset for length ...
  */
@@ -1639,23 +1674,23 @@ tvb_format_text(tvbuff_t *tvb, gint offset, gint size)
 }
 
 /* Looks for a stringz (NUL-terminated string) in tvbuff and copies
- * no more than maxlength number of bytes, including terminating NUL, to buffer.
+ * no more than bufsize number of bytes, including terminating NUL, to buffer.
  * Returns length of string (not including terminating NUL), or -1 if the string was
  * truncated in the buffer due to not having reached the terminating NUL.
  * In this way, it acts like snprintf().
  *
  * When processing a packet where the remaining number of bytes is less
- * than maxlength, an exception is not thrown if the end of the packet
+ * than bufsize, an exception is not thrown if the end of the packet
  * is reached before the NUL is found. If no NUL is found before reaching
  * the end of the short packet, -1 is still returned, and the string
- * is truncated with a NUL, albeit not at buffer[maxlength], but
+ * is truncated with a NUL, albeit not at buffer[bufsize - 1], but
  * at the correct spot, terminating the string.
  *
  * *bytes_copied will contain the number of bytes actually copied,
  * including the terminating-NUL.
  */
 static gint
-_tvb_get_nstringz(tvbuff_t *tvb, gint offset, guint maxlength, guint8* buffer,
+_tvb_get_nstringz(tvbuff_t *tvb, gint offset, guint bufsize, guint8* buffer,
 		gint *bytes_copied)
 {
 	gint	stringlen;
@@ -1665,8 +1700,12 @@ _tvb_get_nstringz(tvbuff_t *tvb, gint offset, guint maxlength, guint8* buffer,
 
 	check_offset_length(tvb, offset, 0, &abs_offset, &junk_length);
 
-	if (maxlength == 0) {
+	if (bufsize == 0) {
+		*bytes_copied = 0;
+		return -1;
+	} else if (bufsize == 1) {
 		buffer[0] = 0;
+		*bytes_copied = 1;
 		return 0;
 	}
 
@@ -1680,19 +1719,25 @@ _tvb_get_nstringz(tvbuff_t *tvb, gint offset, guint maxlength, guint8* buffer,
 	}
 
 	/* This should not happen because check_offset_length() would
-	* have already thrown an exception if 'offset' were out-of-bounds.
-	*/
+	 * have already thrown an exception if 'offset' were out-of-bounds.
+	 */
 	g_assert(len != -1);
 
-	if ((guint)len < maxlength) {
+	/*
+	 * If we've been passed a negative number, bufsize will
+	 * be huge.
+	 */
+	g_assert(bufsize <= G_MAXINT);
+
+	if ((guint)len < bufsize) {
 		limit = len;
 		decreased_max = TRUE;
 	}
 	else {
-		limit = maxlength;
+		limit = bufsize;
 	}
 
-	stringlen = tvb_strnlen(tvb, abs_offset, limit);
+	stringlen = tvb_strnlen(tvb, abs_offset, limit - 1);
 	/* If NUL wasn't found, copy the data and return -1 */
 	if (stringlen == -1) {
 		tvb_memcpy(tvb, buffer, abs_offset, limit);
@@ -1715,24 +1760,24 @@ _tvb_get_nstringz(tvbuff_t *tvb, gint offset, guint maxlength, guint8* buffer,
 }
 
 /* Looks for a stringz (NUL-terminated string) in tvbuff and copies
- * no more than maxlength number of bytes, including terminating NUL, to buffer.
+ * no more than bufsize number of bytes, including terminating NUL, to buffer.
  * Returns length of string (not including terminating NUL), or -1 if the string was
  * truncated in the buffer due to not having reached the terminating NUL.
  * In this way, it acts like snprintf().
  *
  * When processing a packet where the remaining number of bytes is less
- * than maxlength, an exception is not thrown if the end of the packet
+ * than bufsize, an exception is not thrown if the end of the packet
  * is reached before the NUL is found. If no NUL is found before reaching
  * the end of the short packet, -1 is still returned, and the string
- * is truncated with a NUL, albeit not at buffer[maxlength], but
+ * is truncated with a NUL, albeit not at buffer[bufsize - 1], but
  * at the correct spot, terminating the string.
  */
 gint
-tvb_get_nstringz(tvbuff_t *tvb, gint offset, guint maxlength, guint8* buffer)
+tvb_get_nstringz(tvbuff_t *tvb, gint offset, guint bufsize, guint8* buffer)
 {
 	gint bytes_copied;
 
-	return _tvb_get_nstringz(tvb, offset, maxlength, buffer, &bytes_copied);
+	return _tvb_get_nstringz(tvb, offset, bufsize, buffer, &bytes_copied);
 }
 
 /* Like tvb_get_nstringz(), but never returns -1. The string is guaranteed to
@@ -1740,14 +1785,14 @@ tvb_get_nstringz(tvbuff_t *tvb, gint offset, guint maxlength, guint8* buffer)
  * a NUL is placed at the end of buffer to terminate it.
  */
 gint
-tvb_get_nstringz0(tvbuff_t *tvb, gint offset, guint maxlength, guint8* buffer)
+tvb_get_nstringz0(tvbuff_t *tvb, gint offset, guint bufsize, guint8* buffer)
 {
 	gint	len, bytes_copied;
 
-	len = _tvb_get_nstringz(tvb, offset, maxlength, buffer, &bytes_copied);
+	len = _tvb_get_nstringz(tvb, offset, bufsize, buffer, &bytes_copied);
 
 	if (len == -1) {
-		buffer[maxlength] = 0;
+		buffer[bufsize - 1] = 0;
 		return bytes_copied - 1;
 	}
 	else {
