@@ -9,7 +9,7 @@
  * 		the data of a backing tvbuff, or can be a composite of
  * 		other tvbuffs.
  *
- * $Id: tvbuff.h,v 1.29 2002/08/28 20:40:45 jmayer Exp $
+ * $Id: tvbuff.h,v 1.31 2003/04/30 02:35:23 gerald Exp $
  *
  * Copyright (c) 2000 by Gilbert Ramirez <gram@alumni.rice.edu>
  *
@@ -306,32 +306,39 @@ extern guint tvb_strsize(tvbuff_t *tvb, gint offset);
  * Returns -1 if 'maxlength' reached before finding EOS. */
 extern gint tvb_strnlen(tvbuff_t*, gint offset, guint maxlength);
 
+/* Convert a string from Unicode to ASCII.  At the moment we fake it by
+ * assuming all characters are ASCII  )-:  The caller must free the
+ * result returned.  The len parameter is the number of guint16's to
+ * convert from Unicode. */
+extern char *tvb_fake_unicode(tvbuff_t *tvb, int offset, int len,
+                              gboolean little_endian);
+
 /*
  * Format the data in the tvb from offset for size ...
  */
 extern guint8 * tvb_format_text(tvbuff_t *tvb, gint offset, gint size);
 
 /* Looks for a stringz (NUL-terminated string) in tvbuff and copies
- * no more than maxlength number of bytes, including terminating NUL, to buffer.
+ * no more than bufsize number of bytes, including terminating NUL, to buffer.
  * Returns length of string (not including terminating NUL), or -1 if the string was
  * truncated in the buffer due to not having reached the terminating NUL.
  * In this way, it acts like snprintf().
  *
  * When processing a packet where the remaining number of bytes is less
- * than maxlength, an exception is not thrown if the end of the packet
+ * than bufsize, an exception is not thrown if the end of the packet
  * is reached before the NUL is found. If no NUL is found before reaching
  * the end of the short packet, -1 is still returned, and the string
- * is truncated with a NUL, albeit not at buffer[maxlength], but
+ * is truncated with a NUL, albeit not at buffer[bufsize - 1], but
  * at the correct spot, terminating the string.
  */
-extern gint tvb_get_nstringz(tvbuff_t *tvb, gint offset, guint maxlength,
+extern gint tvb_get_nstringz(tvbuff_t *tvb, gint offset, guint bufsize,
     guint8* buffer);
 
 /* Like tvb_get_nstringz(), but never returns -1. The string is guaranteed to
  * have a terminating NUL. If the string was truncated when copied into buffer,
  * a NUL is placed at the end of buffer to terminate it.
  */
-extern gint tvb_get_nstringz0(tvbuff_t *tvb, gint offset, guint maxlength,
+extern gint tvb_get_nstringz0(tvbuff_t *tvb, gint offset, guint bufsize,
     guint8* buffer);
 
 /*
