@@ -1,7 +1,7 @@
 /* file.h
  * Definitions for file structures and routines
  *
- * $Id: file.h,v 1.98 2002/09/06 22:45:40 sahlberg Exp $
+ * $Id: file.h,v 1.99 2003/03/02 22:07:21 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -57,12 +57,7 @@ int print_packets(capture_file *cf, print_args_t *print_args);
 void change_time_formats(capture_file *);
 gboolean find_packet(capture_file *cf, dfilter_t *sfcode);
 
-typedef enum {
-  FOUND_FRAME,		/* found the frame */
-  NO_SUCH_FRAME,	/* no frame with that number */
-  FRAME_NOT_DISPLAYED	/* frame with that number isn't displayed */
-} goto_result_t;
-goto_result_t goto_frame(capture_file *cf, guint fnumber);
+gboolean goto_frame(capture_file *cf, guint fnumber);
 
 void select_packet(capture_file *, int);
 void unselect_packet(capture_file *);
