@@ -1,23 +1,23 @@
 /* dlg_utils.c
  * Utilities to use when constructing dialogs
  *
- * $Id: dlg_utils.c,v 1.7 2002/03/05 11:55:58 guy Exp $
+ * $Id: dlg_utils.c,v 1.10 2002/11/10 11:00:29 oabad Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
  * Copyright 1998 Gerald Combs
  *
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
@@ -32,6 +32,7 @@
 
 #include "gtkglobals.h"
 #include "ui_util.h"
+#include "compat_macros.h"
 
 static void
 dlg_activate (GtkWidget *widget, gpointer ok_button);
@@ -45,11 +46,14 @@ dlg_window_new(const gchar *title)
 {
 	GtkWidget *win;
 
+#if GTK_MAJOR_VERSION < 2
 	win = gtk_window_new(GTK_WINDOW_DIALOG);
+#else
+	win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+#endif
 	gtk_window_set_transient_for(GTK_WINDOW(win), GTK_WINDOW(top_level));
 	gtk_window_set_title(GTK_WINDOW(win), title);
-	gtk_signal_connect (GTK_OBJECT (win), "realize",
-		GTK_SIGNAL_FUNC (window_icon_realize_cb), NULL);
+	SIGNAL_CONNECT(win, "realize", window_icon_realize_cb, NULL);
 	return win;
 }
 
@@ -65,8 +69,7 @@ dlg_window_new(const gchar *title)
 void
 dlg_set_activate(GtkWidget *widget, GtkWidget *ok_button)
 {
-  gtk_signal_connect(GTK_OBJECT(widget), "activate",
-    GTK_SIGNAL_FUNC(dlg_activate), ok_button);
+  SIGNAL_CONNECT(widget, "activate", dlg_activate, ok_button);
 }
 
 static void
@@ -89,8 +92,7 @@ dlg_activate (GtkWidget *widget _U_, gpointer ok_button)
 void
 dlg_set_cancel(GtkWidget *widget, GtkWidget *cancel_button)
 {
-  gtk_signal_connect(GTK_OBJECT(widget), "key_press_event",
-    GTK_SIGNAL_FUNC(dlg_key_press), cancel_button);
+  SIGNAL_CONNECT(widget, "key_press_event", dlg_key_press, cancel_button);
 }
 
 static gint
@@ -107,6 +109,7 @@ dlg_key_press (GtkWidget *widget, GdkEventKey *event, gpointer cancel_button)
   return FALSE;
 }
 
+#if GTK_MAJOR_VERSION < 2
 /* Sigh.  GTK+ appears not to acknowledge that it should be possible
    to attach mnemonics to anything other than menu items; provide
    routines to create radio and check buttons with labels that
@@ -160,8 +163,9 @@ dlg_check_button_new_with_label_with_mnemonic(const gchar *label,
 			GtkAccelGroup *accel_group)
 {
   GtkWidget *check_button;
-	         
+
   check_button = gtk_check_button_new_with_label (label);
   dlg_fix_button_label(check_button, accel_group);
   return check_button;
 }
+#endif
