@@ -1,22 +1,22 @@
 /* file_dlg.c
  * Dialog boxes for handling files
  *
- * $Id: file_dlg.c,v 1.49 2002/03/05 11:55:58 guy Exp $
+ * $Id: file_dlg.c,v 1.55 2003/01/15 05:20:18 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
@@ -26,17 +26,7 @@
 #include "config.h"
 #endif
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
-#ifdef HAVE_DIRECT_H
-#include <direct.h>
-#endif
-
-#include <string.h>
-
-#include <glib.h>
+#include <gtk/gtk.h>
 
 #include <epan/filesystem.h>
 
@@ -51,6 +41,7 @@
 #include "file_dlg.h"
 #include "dlg_utils.h"
 #include "main.h"
+#include "compat_macros.h"
 
 static void file_open_ok_cb(GtkWidget *w, GtkFileSelection *fs);
 static void file_open_destroy_cb(GtkWidget *win, gpointer user_data);
@@ -76,7 +67,9 @@ file_open_cmd_cb(GtkWidget *w, gpointer data _U_)
 {
   GtkWidget	*main_vb, *filter_hbox, *filter_bt, *filter_te,
   		*m_resolv_cb, *n_resolv_cb, *t_resolv_cb;
+#if GTK_MAJOR_VERSION < 2
   GtkAccelGroup *accel_group;
+#endif
   /* No Apply button, and "OK" just sets our text widget, it doesn't
      activate it (i.e., it doesn't cause us to try to open the file). */
   static construct_args_t args = {
@@ -92,14 +85,15 @@ file_open_cmd_cb(GtkWidget *w, gpointer data _U_)
   }
 
   file_open_w = gtk_file_selection_new ("Ethereal: Open Capture File");
-  gtk_signal_connect(GTK_OBJECT(file_open_w), "destroy",
-	GTK_SIGNAL_FUNC(file_open_destroy_cb), NULL);
+  SIGNAL_CONNECT(file_open_w, "destroy", file_open_destroy_cb, NULL);
 
+#if GTK_MAJOR_VERSION < 2
   /* Accelerator group for the accelerators (or, as they're called in
      Windows and, I think, in Motif, "mnemonics"; Alt+<key> is a mnemonic,
      Ctrl+<key> is an accelerator). */
   accel_group = gtk_accel_group_new();
   gtk_window_add_accel_group(GTK_WINDOW(file_open_w), accel_group);
+#endif
 
   /* If we've opened a file, start out by showing the files in the directory
      in which that file resided. */
@@ -119,58 +113,73 @@ file_open_cmd_cb(GtkWidget *w, gpointer data _U_)
   gtk_widget_show(filter_hbox);
 
   filter_bt = gtk_button_new_with_label("Filter:");
-  gtk_signal_connect(GTK_OBJECT(filter_bt), "clicked",
-    GTK_SIGNAL_FUNC(display_filter_construct_cb), &args);
+  SIGNAL_CONNECT(filter_bt, "clicked", display_filter_construct_cb, &args);
+  SIGNAL_CONNECT(filter_bt, "destroy", filter_button_destroy_cb, NULL);
   gtk_box_pack_start(GTK_BOX(filter_hbox), filter_bt, FALSE, TRUE, 0);
   gtk_widget_show(filter_bt);
-  
+
   filter_te = gtk_entry_new();
-  gtk_object_set_data(GTK_OBJECT(filter_bt), E_FILT_TE_PTR_KEY, filter_te);
+  OBJECT_SET_DATA(filter_bt, E_FILT_TE_PTR_KEY, filter_te);
   gtk_box_pack_start(GTK_BOX(filter_hbox), filter_te, TRUE, TRUE, 3);
   gtk_widget_show(filter_te);
 
-  gtk_object_set_data(GTK_OBJECT(GTK_FILE_SELECTION(file_open_w)->ok_button),
-    E_RFILTER_TE_KEY, filter_te);
+  OBJECT_SET_DATA(GTK_FILE_SELECTION(file_open_w)->ok_button,
+                  E_RFILTER_TE_KEY, filter_te);
 
+#if GTK_MAJOR_VERSION < 2
   m_resolv_cb = dlg_check_button_new_with_label_with_mnemonic(
 		  "Enable _MAC name resolution", accel_group);
+#else
+  m_resolv_cb = gtk_check_button_new_with_mnemonic(
+		  "Enable _MAC name resolution");
+#endif
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(m_resolv_cb),
 	g_resolv_flags & RESOLV_MAC);
   gtk_box_pack_start(GTK_BOX(main_vb), m_resolv_cb, FALSE, FALSE, 0);
   gtk_widget_show(m_resolv_cb);
-  gtk_object_set_data(GTK_OBJECT(GTK_FILE_SELECTION(file_open_w)->ok_button),
-		  E_FILE_M_RESOLVE_KEY, m_resolv_cb);
+  OBJECT_SET_DATA(GTK_FILE_SELECTION(file_open_w)->ok_button,
+                  E_FILE_M_RESOLVE_KEY, m_resolv_cb);
 
+#if GTK_MAJOR_VERSION < 2
   n_resolv_cb = dlg_check_button_new_with_label_with_mnemonic(
 		  "Enable _network name resolution", accel_group);
+#else
+  n_resolv_cb = gtk_check_button_new_with_mnemonic(
+		  "Enable _network name resolution");
+#endif
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(n_resolv_cb),
 	g_resolv_flags & RESOLV_NETWORK);
   gtk_box_pack_start(GTK_BOX(main_vb), n_resolv_cb, FALSE, FALSE, 0);
   gtk_widget_show(n_resolv_cb);
-  gtk_object_set_data(GTK_OBJECT(GTK_FILE_SELECTION(file_open_w)->ok_button),
+  OBJECT_SET_DATA(GTK_FILE_SELECTION(file_open_w)->ok_button,
 		  E_FILE_N_RESOLVE_KEY, n_resolv_cb);
 
+#if GTK_MAJOR_VERSION < 2
   t_resolv_cb = dlg_check_button_new_with_label_with_mnemonic(
 		  "Enable _transport name resolution", accel_group);
+#else
+  t_resolv_cb = gtk_check_button_new_with_mnemonic(
+		  "Enable _transport name resolution");
+#endif
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(t_resolv_cb),
 	g_resolv_flags & RESOLV_TRANSPORT);
   gtk_box_pack_start(GTK_BOX(main_vb), t_resolv_cb, FALSE, FALSE, 0);
   gtk_widget_show(t_resolv_cb);
-  gtk_object_set_data(GTK_OBJECT(GTK_FILE_SELECTION(file_open_w)->ok_button),
+  OBJECT_SET_DATA(GTK_FILE_SELECTION(file_open_w)->ok_button,
 		  E_FILE_T_RESOLVE_KEY, t_resolv_cb);
-  
+
   /* Connect the ok_button to file_open_ok_cb function and pass along a
      pointer to the file selection box widget */
-  gtk_signal_connect(GTK_OBJECT (GTK_FILE_SELECTION(file_open_w)->ok_button),
-    "clicked", (GtkSignalFunc) file_open_ok_cb, file_open_w);
+  SIGNAL_CONNECT(GTK_FILE_SELECTION(file_open_w)->ok_button, "clicked",
+                 file_open_ok_cb, file_open_w);
 
-  gtk_object_set_data(GTK_OBJECT(GTK_FILE_SELECTION(file_open_w)->ok_button),
-      E_DFILTER_TE_KEY, gtk_object_get_data(GTK_OBJECT(w), E_DFILTER_TE_KEY));
+  OBJECT_SET_DATA(GTK_FILE_SELECTION(file_open_w)->ok_button,
+                  E_DFILTER_TE_KEY, OBJECT_GET_DATA(w, E_DFILTER_TE_KEY));
 
   /* Connect the cancel_button to destroy the widget */
-  gtk_signal_connect_object(GTK_OBJECT (GTK_FILE_SELECTION
-    (file_open_w)->cancel_button), "clicked", (GtkSignalFunc)
-    gtk_widget_destroy, GTK_OBJECT (file_open_w));
+  SIGNAL_CONNECT_OBJECT(GTK_FILE_SELECTION(file_open_w)->cancel_button,
+                        "clicked", (GtkSignalFunc)gtk_widget_destroy,
+                        file_open_w);
 
   /* Catch the "key_press_event" signal in the window, so that we can catch
      the ESC key being pressed and act as if the "Cancel" button had
@@ -188,9 +197,10 @@ file_open_ok_cb(GtkWidget *w, GtkFileSelection *fs) {
   int        err;
 
   cf_name = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION (fs)));
-  filter_te = gtk_object_get_data(GTK_OBJECT(w), E_RFILTER_TE_KEY);
-  rfilter = gtk_entry_get_text(GTK_ENTRY(filter_te));
+  filter_te = OBJECT_GET_DATA(w, E_RFILTER_TE_KEY);
+  rfilter = (gchar *)gtk_entry_get_text(GTK_ENTRY(filter_te));
   if (!dfilter_compile(rfilter, &rfcode)) {
+    g_free(cf_name);
     simple_dialog(ESD_TYPE_CRIT, NULL, dfilter_error_msg);
     return;
   }
@@ -201,6 +211,7 @@ file_open_ok_cb(GtkWidget *w, GtkFileSelection *fs) {
 	/* It's a directory - set the file selection box to display that
 	   directory, don't try to open the directory as a capture file. */
 	set_last_open_dir(cf_name);
+        g_free(cf_name);
 	gtk_file_selection_set_filename(GTK_FILE_SELECTION(fs), last_open_dir);
     	return;
   }
@@ -224,11 +235,11 @@ file_open_ok_cb(GtkWidget *w, GtkFileSelection *fs) {
 
   /* Set the global resolving variable */
   g_resolv_flags = 0;
-  m_resolv_cb = gtk_object_get_data(GTK_OBJECT(w), E_FILE_M_RESOLVE_KEY);
+  m_resolv_cb = OBJECT_GET_DATA(w, E_FILE_M_RESOLVE_KEY);
   g_resolv_flags |= GTK_TOGGLE_BUTTON (m_resolv_cb)->active ? RESOLV_MAC : RESOLV_NONE;
-  n_resolv_cb = gtk_object_get_data(GTK_OBJECT(w), E_FILE_N_RESOLVE_KEY);
+  n_resolv_cb = OBJECT_GET_DATA(w, E_FILE_N_RESOLVE_KEY);
   g_resolv_flags |= GTK_TOGGLE_BUTTON (n_resolv_cb)->active ? RESOLV_NETWORK : RESOLV_NONE;
-  t_resolv_cb = gtk_object_get_data(GTK_OBJECT(w), E_FILE_T_RESOLVE_KEY);
+  t_resolv_cb = OBJECT_GET_DATA(w, E_FILE_T_RESOLVE_KEY);
   g_resolv_flags |= GTK_TOGGLE_BUTTON (t_resolv_cb)->active ? RESOLV_TRANSPORT : RESOLV_NONE;
 
   /* We've crossed the Rubicon; get rid of the file selection box. */
@@ -252,7 +263,7 @@ file_open_ok_cb(GtkWidget *w, GtkFileSelection *fs) {
     g_free(cf_name);
     return;
   }
-    
+
   /* Save the name of the containing directory specified in the path name,
      if any; we can write over cf_name, which is a good thing, given that
      "get_dirname()" does write over its argument. */
@@ -263,19 +274,8 @@ file_open_ok_cb(GtkWidget *w, GtkFileSelection *fs) {
 }
 
 static void
-file_open_destroy_cb(GtkWidget *win, gpointer user_data _U_)
+file_open_destroy_cb(GtkWidget *win _U_, gpointer user_data _U_)
 {
-  GtkWidget *file_open_filter_w;
-
-  /* Is there a filter edit/selection dialog associated with this
-     Open Capture File dialog? */
-  file_open_filter_w = gtk_object_get_data(GTK_OBJECT(win), E_FILT_DIALOG_PTR_KEY);
-
-  if (file_open_filter_w != NULL) {
-    /* Yes.  Destroy it. */
-    gtk_widget_destroy(file_open_filter_w);
-  }
-
   /* Note that we no longer have a "Open Capture File" dialog box. */
   file_open_w = NULL;
 }
@@ -323,7 +323,7 @@ can_save_with_wiretap(int ft)
    "filtered" is TRUE if we're to save only the packets that passed
    the display filter (in which case we have to save it using Wiretap)
    and FALSE if we're to save the entire file (in which case, if we're
-   saving it in the type it has already, we can just copy it). 
+   saving it in the type it has already, we can just copy it).
 
    "marked" is TRUE if we have to save only the marked packets,
    the same remark as "filtered" applies.
@@ -357,8 +357,8 @@ set_file_type_list(GtkWidget *option_menu)
       /* Default to the same format as the file, if it's supported. */
       item_to_select = index;
     }
-    gtk_signal_connect(GTK_OBJECT(ft_menu_item), "activate",
-      GTK_SIGNAL_FUNC(select_file_type_cb), (gpointer)ft);
+    SIGNAL_CONNECT(ft_menu_item, "activate", select_file_type_cb,
+                   GINT_TO_POINTER(ft));
     gtk_menu_append(GTK_MENU(ft_menu), ft_menu_item);
     gtk_widget_show(ft_menu_item);
     index++;
@@ -371,7 +371,7 @@ set_file_type_list(GtkWidget *option_menu)
 static void
 select_file_type_cb(GtkWidget *w _U_, gpointer data)
 {
-  int new_filetype = (int)data;
+  int new_filetype = GPOINTER_TO_INT(data);
 
   if (filetype != new_filetype) {
     /* We can select only the filtered or marked packets to be saved if we can
@@ -435,8 +435,7 @@ file_save_as_cmd_cb(GtkWidget *w _U_, gpointer data _U_)
   filetype = cfile.cd_t;
 
   file_save_as_w = gtk_file_selection_new ("Ethereal: Save Capture File As");
-  gtk_signal_connect(GTK_OBJECT(file_save_as_w), "destroy",
-	GTK_SIGNAL_FUNC(file_save_as_destroy_cb), NULL);
+  SIGNAL_CONNECT(file_save_as_w, "destroy", file_save_as_destroy_cb, NULL);
 
   /* If we've opened a file, start out by showing the files in the directory
      in which that file resided. */
@@ -446,8 +445,7 @@ file_save_as_cmd_cb(GtkWidget *w _U_, gpointer data _U_)
   /* Connect the ok_button to file_save_as_ok_cb function and pass along a
      pointer to the file selection box widget */
   ok_bt = GTK_FILE_SELECTION (file_save_as_w)->ok_button;
-  gtk_signal_connect(GTK_OBJECT (ok_bt), "clicked",
-    (GtkSignalFunc) file_save_as_ok_cb, file_save_as_w);
+  SIGNAL_CONNECT(ok_bt, "clicked", file_save_as_ok_cb, file_save_as_w);
 
   /* Container for each row of widgets */
   main_vb = gtk_vbox_new(FALSE, 3);
@@ -455,7 +453,7 @@ file_save_as_cmd_cb(GtkWidget *w _U_, gpointer data _U_)
   gtk_box_pack_start(GTK_BOX(GTK_FILE_SELECTION(file_save_as_w)->action_area),
     main_vb, FALSE, FALSE, 0);
   gtk_widget_show(main_vb);
-  
+
   /*
    * XXX - should this be sensitive only if the current display filter
    * has rejected some packets, so that not all packets are currently
@@ -469,8 +467,7 @@ file_save_as_cmd_cb(GtkWidget *w _U_, gpointer data _U_)
   filter_cb = gtk_check_button_new_with_label("Save only packets currently being displayed");
   gtk_container_add(GTK_CONTAINER(main_vb), filter_cb);
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(filter_cb), FALSE);
-  gtk_signal_connect(GTK_OBJECT(filter_cb), "toggled",
-			GTK_SIGNAL_FUNC(toggle_filtered_cb), NULL);
+  SIGNAL_CONNECT(filter_cb, "toggled", toggle_filtered_cb, NULL);
   gtk_widget_set_sensitive(filter_cb, can_save_with_wiretap(filetype));
   gtk_widget_show(filter_cb);
 
@@ -487,15 +484,14 @@ file_save_as_cmd_cb(GtkWidget *w _U_, gpointer data _U_)
   gtk_container_add(GTK_CONTAINER(main_vb), mark_cb);
   marked = FALSE;
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(mark_cb), FALSE);
-  gtk_signal_connect(GTK_OBJECT(mark_cb), "toggled",
-		     GTK_SIGNAL_FUNC(toggle_marked_cb), NULL);
+  SIGNAL_CONNECT(mark_cb, "toggled", toggle_marked_cb, NULL);
   gtk_widget_show(mark_cb);
 
   /* File type row */
   ft_hb = gtk_hbox_new(FALSE, 3);
   gtk_container_add(GTK_CONTAINER(main_vb), ft_hb);
   gtk_widget_show(ft_hb);
-  
+
   ft_lb = gtk_label_new("File type:");
   gtk_box_pack_start(GTK_BOX(ft_hb), ft_lb, FALSE, FALSE, 0);
   gtk_widget_show(ft_lb);
@@ -517,9 +513,9 @@ file_save_as_cmd_cb(GtkWidget *w _U_, gpointer data _U_)
   file_set_save_marked_sensitive();
 
   /* Connect the cancel_button to destroy the widget */
-  gtk_signal_connect_object(GTK_OBJECT (GTK_FILE_SELECTION
-    (file_save_as_w)->cancel_button), "clicked", (GtkSignalFunc)
-    gtk_widget_destroy, GTK_OBJECT (file_save_as_w));
+  SIGNAL_CONNECT_OBJECT(GTK_FILE_SELECTION(file_save_as_w)->cancel_button,
+                        "clicked", (GtkSignalFunc)gtk_widget_destroy,
+                        file_save_as_w);
 
   /* Catch the "key_press_event" signal in the window, so that we can catch
      the ESC key being pressed and act as if the "Cancel" button had
@@ -564,17 +560,39 @@ file_set_save_marked_sensitive(void)
 static void
 file_save_as_ok_cb(GtkWidget *w _U_, GtkFileSelection *fs) {
   gchar	*cf_name;
+  gchar	*dirname;
 
   cf_name = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
-  gtk_widget_hide(GTK_WIDGET (fs));
-  gtk_widget_destroy(GTK_WIDGET (fs));
+
+  /* Perhaps the user specified a directory instead of a file.
+     Check whether they did. */
+  if (test_for_directory(cf_name) == EISDIR) {
+        /* It's a directory - set the file selection box to display that
+           directory, and leave the selection box displayed. */
+        set_last_open_dir(cf_name);
+        g_free(cf_name);
+        gtk_file_selection_set_filename(GTK_FILE_SELECTION(fs), last_open_dir);
+        return;
+  }
 
   /* Write out the packets (all, or only the ones that are currently
      displayed or marked) to the file with the specified name. */
-  save_cap_file(cf_name, &cfile, filtered, marked, filetype);
 
-  /* If "save_cap_file()" saved the file name we handed it, it saved
-     a copy, so we should free up our copy. */
+  if (! save_cap_file(cf_name, &cfile, filtered, marked, filetype)) {
+    /* The write failed; don't dismiss the open dialog box,
+       just leave it around so that the user can, after they
+       dismiss the alert box popped up for the error, try again. */
+    g_free(cf_name);
+    return;
+  }
+
+  /* The write succeeded; get rid of the file selection box. */
+  gtk_widget_hide(GTK_WIDGET (fs));
+  gtk_widget_destroy(GTK_WIDGET (fs));
+
+  /* Save the directory name for future file dialogs. */
+  dirname = get_dirname(cf_name);  /* Overwrites cf_name */
+  set_last_open_dir(dirname);
   g_free(cf_name);
 }
 
@@ -588,14 +606,11 @@ file_save_as_destroy_cb(GtkWidget *win _U_, gpointer user_data _U_)
 /* Reload a file using the current read and display filters */
 void
 file_reload_cmd_cb(GtkWidget *w, gpointer data _U_) {
-  /*GtkWidget *filter_te = gtk_object_get_data(GTK_OBJECT(w), E_DFILTER_TE_KEY);*/
-  GtkWidget *filter_te;
+  GtkWidget *filter_te = OBJECT_GET_DATA(w, E_DFILTER_TE_KEY);
   gchar *filename;
   gboolean is_tempfile;
   int err;
 
-  filter_te = gtk_object_get_data(GTK_OBJECT(w), E_DFILTER_TE_KEY);
-
   if (cfile.dfilter)
     g_free(cfile.dfilter);
   cfile.dfilter = g_strdup(gtk_entry_get_text(GTK_ENTRY(filter_te)));
