@@ -1,7 +1,7 @@
 /* io_stat.c
  * io_stat   2002 Ronnie Sahlberg
  *
- * $Id: io_stat.c,v 1.16 2003/01/15 05:58:50 guy Exp $
+ * $Id: io_stat.c,v 1.21 2003/04/23 08:20:05 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -39,11 +39,11 @@
 
 #include <gtk/gtk.h>
 #include "gtkglobals.h"
+#include "menu.h"
 #include "epan/epan_dissect.h"
 #include "epan/packet_info.h"
 #include "../tap.h"
 #include "../register.h"
-#include "io_stat.h"
 #include "simple_dialog.h"
 #include "../globals.h"
 #include "../color.h"
@@ -825,6 +825,7 @@ gtk_iostat_init(char *optarg _U_)
 		{0,	0x0000,	0x0000,	0xffff},
 		{0,	0xffff,	0x5000,	0xffff}
 	};
+	GString *error_string;
 
 	io=g_malloc(sizeof(io_stat_t));
 	io->needs_redraw=1;
@@ -872,7 +873,11 @@ gtk_iostat_init(char *optarg _U_)
 		io->graphs[i].filter_bt=NULL;
 	}
 
-	if(register_tap_listener("frame", &io->graphs[0], NULL, gtk_iostat_reset, gtk_iostat_packet, gtk_iostat_draw)){
+	error_string=register_tap_listener("frame", &io->graphs[0], NULL, gtk_iostat_reset, gtk_iostat_packet, gtk_iostat_draw);
+	if(error_string){
+		fprintf(stderr, "ethereal: Can't attach io_stat tap: %s\n",
+		    error_string->str);
+		g_string_free(error_string, TRUE);
 		g_free(io->graphs[0].counts);
 		io->graphs[0].counts=NULL;
 		io->graphs[0].display=0;
@@ -1055,7 +1060,7 @@ create_draw_area(io_stat_t *io, GtkWidget *box)
 	io->scrollbar_adjustment=(GtkAdjustment *)gtk_adjustment_new(0,0,0,0,0,0);
 	io->scrollbar=gtk_hscrollbar_new(io->scrollbar_adjustment);
 	gtk_widget_show(io->scrollbar);
-	gtk_box_pack_start(GTK_BOX(box), io->scrollbar, TRUE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(box), io->scrollbar, FALSE, FALSE, 0);
 	SIGNAL_CONNECT(io->scrollbar_adjustment, "value_changed", scrollbar_changed, io);
 }
 
@@ -1624,7 +1629,7 @@ init_io_stat_window(io_stat_t *io)
 	create_draw_area(io, vbox);
 
 	hbox=gtk_hbox_new(FALSE, 0);
-	gtk_container_add(GTK_CONTAINER(vbox), hbox);
+	gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
 	gtk_box_set_child_packing(GTK_BOX(vbox), hbox, FALSE, FALSE, 0, GTK_PACK_START);
 	gtk_widget_show(hbox);
 
@@ -1636,7 +1641,7 @@ init_io_stat_window(io_stat_t *io)
 }
 
 
-void 
+static void 
 gtk_iostat_cb(GtkWidget *w _U_, gpointer d _U_)
 {
 	gtk_iostat_init(NULL);
@@ -1650,6 +1655,11 @@ register_tap_listener_gtk_iostat(void)
 {
 	io_stat_widget_table = g_hash_table_new(io_stat_widget_hash,
 			io_stat_widget_equal);
-	register_ethereal_tap("io,stat", gtk_iostat_init, NULL, NULL);
+	register_ethereal_tap("io,stat", gtk_iostat_init);
 }
 
+void
+register_tap_menu_gtkiostat(void)
+{
+	register_tap_menu_item("IO/IO-Stat", gtk_iostat_cb);
+}
