@@ -1,6 +1,6 @@
 /* main.c
  *
- * $Id: main.c,v 1.280 2003/01/08 01:59:42 guy Exp $
+ * $Id: main.c,v 1.291 2003/04/23 20:51:57 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -56,26 +56,10 @@
 #include <pcap.h>
 #endif
 
-#ifdef HAVE_LIBZ
-#include <zlib.h>	/* to get the libz version number */
-#endif
-
 #ifdef NEED_SNPRINTF_H
 # include "snprintf.h"
 #endif
 
-#ifdef HAVE_SOME_SNMP
-
-#ifdef HAVE_NET_SNMP
-#include <net-snmp/version.h>
-#endif /* HAVE_NET_SNMP */
-
-#ifdef HAVE_UCD_SNMP
-#include <ucd-snmp/version.h>
-#endif /* HAVE_UCD_SNMP */
-
-#endif /* HAVE_SOME_SNMP */
-
 #ifdef NEED_STRERROR_H
 #include "strerror.h"
 #endif
@@ -127,10 +111,10 @@
 #include "register.h"
 #include "ringbuffer.h"
 #include "ui_util.h"
-#include "../ui_util.h"
 #include "image/clist_ascend.xpm"
 #include "image/clist_descend.xpm"
 #include "../tap.h"
+#include "../util.h"
 #include "compat_macros.h"
 
 #ifdef WIN32
@@ -154,7 +138,7 @@ guint	     m_font_height, m_font_width;
 PangoFontDescription *m_r_font, *m_b_font;
 #endif
 static guint    main_ctx, file_ctx, help_ctx;
-static GString *comp_info_str;
+static GString *comp_info_str, *runtime_info_str;
 gchar       *ethereal_path = NULL;
 gchar       *last_open_dir = NULL;
 gint   root_x = G_MAXINT, root_y = G_MAXINT, top_width, top_height;
@@ -187,14 +171,14 @@ void
 about_ethereal( GtkWidget *w _U_, gpointer data _U_ ) {
   simple_dialog(ESD_TYPE_INFO, NULL,
 		"Ethereal - Network Protocol Analyzer\n"
-		"Version " VERSION " (C) 1998-2002 Gerald Combs <gerald@ethereal.com>\n"
-                "Compiled %s\n\n"
+		"Version " VERSION " (C) 1998-2003 Gerald Combs <gerald@ethereal.com>\n"
+                "%s\n%s\n\n"
 
 		"Check the man page for complete documentation and\n"
 		"for the list of contributors.\n"
 
 		"\nSee http://www.ethereal.com/ for more information.",
-                 comp_info_str->str);
+                 comp_info_str->str, runtime_info_str->str);
 }
 
 #if GTK_MAJOR_VERSION < 2
@@ -230,9 +214,9 @@ goto_framenum_cb(GtkWidget *w _U_, gpointer data _U_)
 	hfinfo = finfo_selected->hfinfo;
 	g_assert(hfinfo);
 	if (hfinfo->type == FT_FRAMENUM) {
-		framenum = fvalue_get_integer(finfo_selected->value);
-		if (framenum != 0)
-			packet_list_set_selected_row(framenum - 1);
+	    framenum = fvalue_get_integer(finfo_selected->value);
+	    if (framenum != 0)
+		goto_frame(&cfile, framenum);
 	}
     }
 }
@@ -1028,6 +1012,8 @@ set_plist_font(PangoFontDescription *font)
 
 	gtk_widget_set_style(packet_list, style);
 #else
+        PangoLayout *layout;
+
         gtk_widget_modify_font(packet_list, font);
 #endif
 
@@ -1038,9 +1024,11 @@ set_plist_font(PangoFontDescription *font)
 		cfile.cinfo.col_width[i] = gdk_string_width(font,
 			get_column_longest_string(get_column_format(i)));
 #else
-		cfile.cinfo.col_width[i] =
-                    gdk_string_width(gdk_font_from_description(font),
-			get_column_longest_string(get_column_format(i)));
+                layout = gtk_widget_create_pango_layout(packet_list,
+		    get_column_longest_string(get_column_format(i)));
+                pango_layout_get_pixel_size(layout, &cfile.cinfo.col_width[i],
+                                            NULL);
+                g_object_unref(G_OBJECT(layout));
 #endif
 	}
 }
@@ -1186,8 +1174,8 @@ static void
 print_usage(gboolean print_ver) {
 
   if (print_ver) {
-    fprintf(stderr, "This is GNU " PACKAGE " " VERSION ", compiled %s\n",
-	  comp_info_str->str);
+    fprintf(stderr, "This is GNU " PACKAGE " " VERSION "\n%s\n%s\n",
+	  comp_info_str->str, runtime_info_str->str);
   }
 #ifdef HAVE_LIBPCAP
   fprintf(stderr, "\n%s [ -vh ] [ -klpQS ] [ -a <capture autostop condition> ] ...\n",
@@ -1216,7 +1204,8 @@ show_version(void)
   create_console();
 #endif
 
-  printf("%s %s, %s\n", PACKAGE, VERSION, comp_info_str->str);
+  printf("%s %s\n%s\n%s\n", PACKAGE, VERSION, comp_info_str->str,
+         runtime_info_str->str);
 }
 
 static int
@@ -1367,7 +1356,7 @@ typedef struct _ethereal_tap_list {
 static ethereal_tap_list *tap_list=NULL;
 
 void
-register_ethereal_tap(char *cmd, void (*func)(char *arg), char *dummy _U_, void (*dummy2)(void) _U_)
+register_ethereal_tap(char *cmd, void (*func)(char *arg))
 {
 	ethereal_tap_list *newtl;
 
@@ -1391,11 +1380,6 @@ main(int argc, char *argv[])
   int                  opt;
   extern char         *optarg;
   gboolean             arg_error = FALSE;
-#ifdef HAVE_LIBPCAP
-#ifdef HAVE_PCAP_VERSION
-  extern char          pcap_version[];
-#endif /* HAVE_PCAP_VERSION */
-#endif /* HAVE_LIBPCAP */
 
 #ifdef WIN32
   WSADATA 	       wsaData;
@@ -1478,6 +1462,9 @@ main(int argc, char *argv[])
      dissectors, and we must do it before we read the preferences, in
      case any dissectors register preferences. */
   epan_init(PLUGIN_DIR,register_all_protocols,register_all_protocol_handoffs);
+
+  /* Register all tap listeners; we do this before we parse the arguments,
+     as the "-z" argument can specify a registered tap. */
   register_all_tap_listeners();
 
   /* Now register the preferences for any non-dissector modules.
@@ -1517,7 +1504,7 @@ main(int argc, char *argv[])
       else if (strcmp(argv[2], "protocols") == 0)
         proto_registrar_dump_protocols();
       else {
-        fprintf(stderr, "tethereal: Invalid \"%s\" option for -G flag\n",
+        fprintf(stderr, "ethereal: Invalid \"%s\" option for -G flag\n",
                 argv[2]);
         exit(1);
       }
@@ -1594,9 +1581,8 @@ main(int argc, char *argv[])
 
   init_cap_file(&cfile);
 
-  /* Assemble the compile-time options */
-  comp_info_str = g_string_new("");
-
+  /* Assemble the compile-time version information string */
+  comp_info_str = g_string_new("Compiled ");
   g_string_append(comp_info_str, "with ");
   g_string_sprintfa(comp_info_str,
 #ifdef GTK_MAJOR_VERSION
@@ -1606,54 +1592,12 @@ main(int argc, char *argv[])
                     "GTK+ (version unknown)");
 #endif
 
-  g_string_append(comp_info_str, ", with ");
-  g_string_sprintfa(comp_info_str,
-#ifdef GLIB_MAJOR_VERSION
-                    "GLib %d.%d.%d", GLIB_MAJOR_VERSION, GLIB_MINOR_VERSION,
-                    GLIB_MICRO_VERSION);
-#else
-                    "GLib (version unknown)");
-#endif
+  g_string_append(comp_info_str, ", ");
+  get_compiled_version_info(comp_info_str);
 
-#ifdef HAVE_LIBPCAP
-  g_string_append(comp_info_str, ", with libpcap ");
-#ifdef HAVE_PCAP_VERSION
-  g_string_append(comp_info_str, pcap_version);
-#else /* HAVE_PCAP_VERSION */
-  g_string_append(comp_info_str, "(version unknown)");
-#endif /* HAVE_PCAP_VERSION */
-#else /* HAVE_LIBPCAP */
-  g_string_append(comp_info_str, ", without libpcap");
-#endif /* HAVE_LIBPCAP */
-
-#ifdef HAVE_LIBZ
-  g_string_append(comp_info_str, ", with libz ");
-#ifdef ZLIB_VERSION
-  g_string_append(comp_info_str, ZLIB_VERSION);
-#else /* ZLIB_VERSION */
-  g_string_append(comp_info_str, "(version unknown)");
-#endif /* ZLIB_VERSION */
-#else /* HAVE_LIBZ */
-  g_string_append(comp_info_str, ", without libz");
-#endif /* HAVE_LIBZ */
-
-/* Oh, this is pretty. */
-/* Oh, ha.  you think that was pretty.  Try this:! --Wes */
-#ifdef HAVE_SOME_SNMP
-
-#ifdef HAVE_UCD_SNMP
-  g_string_append(comp_info_str, ", with UCD-SNMP ");
-  g_string_append(comp_info_str, VersionInfo);
-#endif /* HAVE_UCD_SNMP */
-
-#ifdef HAVE_NET_SNMP
-  g_string_append(comp_info_str, ", with Net-SNMP ");
-  g_string_append(comp_info_str, netsnmp_get_version());
-#endif /* HAVE_NET_SNMP */
-
-#else /* no SNMP library */
-  g_string_append(comp_info_str, ", without UCD-SNMP or Net-SNMP");
-#endif /* HAVE_SOME_SNMP */
+  /* Assemble the run-time version information string */
+  runtime_info_str = g_string_new("Running ");
+  get_runtime_version_info(runtime_info_str);
 
   /* Now get our args */
   while ((opt = getopt(argc, argv, optstring)) != -1) {
@@ -2010,7 +1954,7 @@ main(int argc, char *argv[])
 #endif
 
   /* Build the column format array */
-  col_init(&cfile.cinfo, prefs->num_cols);
+  col_setup(&cfile.cinfo, prefs->num_cols);
   for (i = 0; i < cfile.cinfo.num_cols; i++) {
     cfile.cinfo.col_fmt[i] = get_column_format(i);
     cfile.cinfo.col_title[i] = g_strdup(get_column_title(i));
@@ -2022,6 +1966,7 @@ main(int argc, char *argv[])
       cfile.cinfo.col_buf[i] = (gchar *) g_malloc(sizeof(gchar) * COL_MAX_INFO_LEN);
     else
       cfile.cinfo.col_buf[i] = (gchar *) g_malloc(sizeof(gchar) * COL_MAX_LEN);
+    cfile.cinfo.col_fence[i] = 0;
     cfile.cinfo.col_expr[i] = (gchar *) g_malloc(sizeof(gchar) * COL_MAX_LEN);
     cfile.cinfo.col_expr_val[i] = (gchar *) g_malloc(sizeof(gchar) * COL_MAX_LEN);
   }
@@ -2130,10 +2075,12 @@ main(int argc, char *argv[])
      to a file that our parent will read? */
   if (!capture_child) {
 #endif
-    /* No.  Pop up the main window, and read in a capture file if
+    /* No.  Pop up the main window, register menus for taps (which we
+       must do after creating the main window, so that we can add
+       menu items to the main menu), and read in a capture file if
        we were told to. */
-
     create_main_window(pl_size, tv_size, bv_size, prefs);
+    register_all_tap_menus();
     set_menus_for_capture_file(FALSE);
 
     /* open tap windows after creating the main window to avoid GTK warnings */
