@@ -1,22 +1,22 @@
 /* main.h
  * Global defines, etc.
  *
- * $Id: main.h,v 1.26 2002/01/11 08:21:02 guy Exp $
+ * $Id: main.h,v 1.30 2002/12/19 02:58:53 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
@@ -58,33 +58,36 @@ typedef struct _selection_info {
   GtkWidget *text;
 } selection_info;
 
+#if GTK_MAJOR_VERSION < 2
 extern GtkStyle *item_style;
+#endif
 
 void about_ethereal( GtkWidget *, gpointer);
-void match_selected_cb_replace( GtkWidget *, gpointer);
-void match_selected_cb_and( GtkWidget *, gpointer);
-void match_selected_cb_or( GtkWidget *, gpointer);
-void match_selected_cb_not( GtkWidget *, gpointer);
-void match_selected_cb_and_not( GtkWidget *, gpointer);
-void match_selected_cb_or_not( GtkWidget *, gpointer);
-void prepare_selected_cb_replace( GtkWidget *, gpointer);
-void prepare_selected_cb_and( GtkWidget *, gpointer);
-void prepare_selected_cb_or( GtkWidget *, gpointer);
-void prepare_selected_cb_not( GtkWidget *, gpointer);
-void prepare_selected_cb_and_not( GtkWidget *, gpointer);
-void prepare_selected_cb_or_not( GtkWidget *, gpointer);
-void match_selected_cb_replace2( GtkWidget *, gpointer);
-void match_selected_cb_and2( GtkWidget *, gpointer);
-void match_selected_cb_or2( GtkWidget *, gpointer);
-void match_selected_cb_not2( GtkWidget *, gpointer);
-void match_selected_cb_and_not2( GtkWidget *, gpointer);
-void match_selected_cb_or_not2( GtkWidget *, gpointer);
-void prepare_selected_cb_replace2( GtkWidget *, gpointer);
-void prepare_selected_cb_and2( GtkWidget *, gpointer);
-void prepare_selected_cb_or2( GtkWidget *, gpointer);
-void prepare_selected_cb_not2( GtkWidget *, gpointer);
-void prepare_selected_cb_and_not2( GtkWidget *, gpointer);
-void prepare_selected_cb_or_not2( GtkWidget *, gpointer);
+void goto_framenum_cb(GtkWidget *, gpointer);
+void match_selected_cb_replace_ptree( GtkWidget *, gpointer);
+void match_selected_cb_and_ptree( GtkWidget *, gpointer);
+void match_selected_cb_or_ptree( GtkWidget *, gpointer);
+void match_selected_cb_not_ptree( GtkWidget *, gpointer);
+void match_selected_cb_and_ptree_not( GtkWidget *, gpointer);
+void match_selected_cb_or_ptree_not( GtkWidget *, gpointer);
+void prepare_selected_cb_replace_ptree( GtkWidget *, gpointer);
+void prepare_selected_cb_and_ptree( GtkWidget *, gpointer);
+void prepare_selected_cb_or_ptree( GtkWidget *, gpointer);
+void prepare_selected_cb_not_ptree( GtkWidget *, gpointer);
+void prepare_selected_cb_and_ptree_not( GtkWidget *, gpointer);
+void prepare_selected_cb_or_ptree_not( GtkWidget *, gpointer);
+void match_selected_cb_replace_plist( GtkWidget *, gpointer);
+void match_selected_cb_and_plist( GtkWidget *, gpointer);
+void match_selected_cb_or_plist( GtkWidget *, gpointer);
+void match_selected_cb_not_plist( GtkWidget *, gpointer);
+void match_selected_cb_and_plist_not( GtkWidget *, gpointer);
+void match_selected_cb_or_plist_not( GtkWidget *, gpointer);
+void prepare_selected_cb_replace_plist( GtkWidget *, gpointer);
+void prepare_selected_cb_and_plist( GtkWidget *, gpointer);
+void prepare_selected_cb_or_plist( GtkWidget *, gpointer);
+void prepare_selected_cb_not_plist( GtkWidget *, gpointer);
+void prepare_selected_cb_and_plist_not( GtkWidget *, gpointer);
+void prepare_selected_cb_or_plist_not( GtkWidget *, gpointer);
 void file_quit_cmd_cb(GtkWidget *, gpointer);
 void file_print_cmd_cb(GtkWidget *, gpointer);
 void file_print_packet_cmd_cb(GtkWidget *, gpointer);
@@ -98,7 +101,11 @@ void unmark_all_frames_cb(GtkWidget *w, gpointer);
 void update_marked_frames(void);
 
 char *boldify(const char *);
+#if GTK_MAJOR_VERSION < 2
 void set_fonts(GdkFont *regular, GdkFont *bold);
+#else
+void set_fonts(PangoFontDescription *regular, PangoFontDescription *bold);
+#endif
 void set_last_open_dir(char *dirname);
 
 #endif /* __MAIN_H__ */
