@@ -1,7 +1,7 @@
 /* menu.c
  * Menu routines
  *
- * $Id: menu.c,v 1.83 2003/01/22 00:40:36 sahlberg Exp $
+ * $Id: menu.c,v 1.92 2003/04/23 05:37:22 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -57,20 +57,16 @@
 #include <epan/plugins.h>
 #include "tcp_graph.h"
 #include <epan/epan_dissect.h>
-#include "io_stat.h"
-#include "rpc_stat.h"
-#include "rpc_progs.h"
-#include "dcerpc_stat.h"
-#include "smb_stat.h"
 #include "compat_macros.h"
 #include "gtkglobals.h"
+#include "../tap.h"
 
 GtkWidget *popup_menu_object;
 
 #define GTK_MENU_FUNC(a) ((GtkItemFactoryCallback)(a))
 
 static void menus_init(void);
-static void set_menu_sensitivity (gchar *, gint);
+static void set_menu_sensitivity (GtkItemFactory *, gchar *, gint);
 
 /* This is the GtkItemFactoryEntry structure used to generate new menus.
        Item 1: The menu path. The letter after the underscore indicates an
@@ -235,24 +231,6 @@ static GtkItemFactoryEntry menu_items[] =
     ITEM_FACTORY_ENTRY("/Tools/Protocol Hierarchy Statistics", NULL,
                        proto_hier_stats_cb, 0, NULL, NULL),
     ITEM_FACTORY_ENTRY("/Tools/Statistics", NULL, NULL, 0, "<Branch>", NULL),
-    ITEM_FACTORY_ENTRY("/Tools/Statistics/ONC-RPC", NULL, NULL, 0, "<Branch>",
-                       NULL),
-    ITEM_FACTORY_ENTRY("/Tools/Statistics/ONC-RPC/RTT", NULL, gtk_rpcstat_cb,
-                       0, NULL, NULL),
-    ITEM_FACTORY_ENTRY("/Tools/Statistics/ONC-RPC/Programs", NULL,
-                       gtk_rpcprogs_cb, 0, NULL, NULL),
-    ITEM_FACTORY_ENTRY("/Tools/Statistics/DCE-RPC", NULL, NULL, 0, "<Branch>",
-                       NULL),
-    ITEM_FACTORY_ENTRY("/Tools/Statistics/DCE-RPC/RTT", NULL, gtk_dcerpcstat_cb,
-                       0, NULL, NULL),
-    ITEM_FACTORY_ENTRY("/Tools/Statistics/IO", NULL, NULL, 0, "<Branch>",
-                       NULL),
-    ITEM_FACTORY_ENTRY("/Tools/Statistics/IO/IO-Stat", NULL, gtk_iostat_cb,
-                       0, NULL, NULL),
-    ITEM_FACTORY_ENTRY("/Tools/Statistics/SMB", NULL, NULL, 0, "<Branch>",
-                       NULL),
-    ITEM_FACTORY_ENTRY("/Tools/Statistics/SMB/RTT", NULL, gtk_smbstat_cb,
-                       0, NULL, NULL),
     ITEM_FACTORY_ENTRY("/_Help", NULL, NULL, 0, "<LastBranch>", NULL),
     ITEM_FACTORY_STOCK_ENTRY("/Help/_Help", NULL, help_cb, 0, GTK_STOCK_HELP),
     ITEM_FACTORY_ENTRY("/Help/<separator>", NULL, NULL, 0, "<Separator>", NULL),
@@ -363,7 +341,7 @@ static GtkItemFactoryEntry hexdump_menu_items[] =
 };
 
 static int initialize = TRUE;
-static GtkItemFactory *factory = NULL;
+static GtkItemFactory *main_menu_factory = NULL;
 static GtkItemFactory *packet_list_menu_factory = NULL;
 static GtkItemFactory *tree_view_menu_factory = NULL;
 static GtkItemFactory *hexdump_menu_factory = NULL;
@@ -383,7 +361,7 @@ get_main_menu(GtkWidget ** menubar, GtkAccelGroup ** table) {
   }
 
   if (menubar)
-    *menubar = factory->widget;
+    *menubar = main_menu_factory->widget;
 
   if (table)
     *table = grp;
@@ -415,66 +393,141 @@ menus_init(void) {
                     hexdump_menu_factory->widget);
     popup_menu_list = g_slist_append((GSList *)popup_menu_list, hexdump_menu_factory);
 
-    factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", grp);
-    gtk_item_factory_create_items_ac(factory, nmenu_items, menu_items, NULL,2);
+    main_menu_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", grp);
+    gtk_item_factory_create_items_ac(main_menu_factory, nmenu_items, menu_items, NULL,2);
     set_menus_for_unsaved_capture_file(FALSE);
     set_menus_for_capture_file(FALSE);
 #if 0
     /* Un-#if this when we actually implement Cut/Copy/Paste.
        Then make sure you enable them when they can be done. */
-    set_menu_sensitivity("/Edit/Cut", FALSE);
-    set_menu_sensitivity("/Edit/Copy", FALSE);
-    set_menu_sensitivity("/Edit/Paste", FALSE);
+    set_menu_sensitivity(main_menu_factory, "/Edit/Cut", FALSE);
+    set_menu_sensitivity(main_menu_factory, "/Edit/Copy", FALSE);
+    set_menu_sensitivity(main_menu_factory, "/Edit/Paste", FALSE);
 #endif
+
     set_menus_for_captured_packets(FALSE);
     set_menus_for_selected_packet(FALSE);
     set_menus_for_selected_tree_row(FALSE);
   }
 }
 
+/*
+ * Add a new menu item for a statistical tap.
+ * This must be called after we've created the main menu, so it can't
+ * be called from the routine that registers taps - we have to introduce
+ * another per-tap registration routine.
+ */
 void
-set_menu_sensitivity_meat(GtkItemFactory *ifactory, gchar *path, gint val) {
-	GtkWidget *menu = NULL;
+register_tap_menu_item(char *name, GtkItemFactoryCallback callback)
+{
+	static const char statspath[] = "/Tools/Statistics/";
+	char *p;
+	char *menupath;
+	size_t menupathlen;
+	GtkWidget *w;
+	GtkItemFactoryEntry *entry;
 
-	if((menu = gtk_item_factory_get_widget(ifactory, path)) != NULL) {
-		gtk_widget_set_sensitive(menu,val);
+	/*
+	 * The menu path must be relative.
+	 */
+	g_assert(*name != '/');
+
+	/*
+	 * Create any submenus required.
+	 */
+	p = name;
+	while ((p = strchr(p, '/')) != NULL) {
+		/*
+		 * OK, everything between "name" and "p" is
+		 * a menu relative subtree into which the menu item
+		 * will be placed.
+		 *
+		 * Construct the absolute path name of that subtree.
+		 */
+		menupathlen = sizeof statspath + (p - name);
+		menupath = g_malloc(menupathlen);
+		strcpy(menupath, statspath);
+		strncat(menupath, name, p - name);
+
+		/*
+		 * Does there exist an entry with that path in the main
+		 * menu item factory?
+		 */
+		w = gtk_item_factory_get_widget(main_menu_factory, menupath);
+		if (w == NULL) {
+			/*
+			 * No.  Create such an item as a subtree.
+			 */
+			entry = g_malloc0(sizeof (GtkItemFactoryEntry));
+			entry->path = menupath;
+			entry->item_type = "<Branch>";
+			gtk_item_factory_create_item(main_menu_factory, entry,
+			    NULL, 2);
+		}
+
+		/*
+		 * Skip over the '/' we found.
+		 */
+		p++;
 	}
+
+	/*
+	 * Construct the main menu path for the menu item.
+	 *
+	 * "sizeof statspath" includes the trailing '\0', so the sum
+	 * of that and the length of "name" is enough to hold a string
+	 * containing their concatenation.
+	 */
+	menupathlen = sizeof statspath + strlen(name);
+	menupath = g_malloc(menupathlen);
+	strcpy(menupath, statspath);
+	strcat(menupath, name);
+
+	/*
+	 * Construct an item factory entry for the item, and add it to
+	 * the main menu.
+	 */
+	entry = g_malloc0(sizeof (GtkItemFactoryEntry));
+	entry->path = menupath;
+	entry->callback = callback;
+	gtk_item_factory_create_item(main_menu_factory, entry, NULL, 2);
 }
 
-/* Enable/disable menu sensitivity                       */
-/* /menu/path            - old functionality             */
-/* <MenuName>/menu/path  - new functionality             */
-/* MenuName: <Main>, <PacketList>, <TreeView>, <HexDump> */
-/* XXX - is this really supposed to ignore all but the   */
-/* last component of the menu path?                      */
+
+/*
+ * Enable/disable menu sensitivity.
+ */
 static void
-set_menu_sensitivity (gchar *path, gint val) {
-  GSList *menu_list = popup_menu_list;
-  gchar *prefix;
-  gchar *shortpath;
+set_menu_sensitivity(GtkItemFactory *ifactory, gchar *path, gint val)
+{
+  GSList *menu_list;
+  GtkWidget *menu_item;
 
-  if ('<' == *path) {
-    /* New functionality => selective enable/disable per menu */
-    prefix=strchr(path, '/');
-    shortpath=strrchr(prefix, '/');
-
-    if (0 == strncmp(path, "<Main>", 6))
-      set_menu_sensitivity_meat(factory, prefix, val);
-    else if (0 == strncmp(path, "<PacketList>", 12))
-      set_menu_sensitivity_meat(packet_list_menu_factory, shortpath, val);
-    else if (0 == strncmp(path, "<TreeView>", 10))
-      set_menu_sensitivity_meat(tree_view_menu_factory, shortpath, val);
-    else if (0 == strncmp(path, "<HexDump>", 9))
-      set_menu_sensitivity_meat(hexdump_menu_factory, shortpath, val);
+  if (ifactory == NULL) {
+    /*
+     * Do it for all pop-up menus.
+     */
+    for (menu_list = popup_menu_list; menu_list != NULL;
+         menu_list = g_slist_next(menu_list))
+      set_menu_sensitivity(menu_list->data, path, val);
   } else {
-    /* Old functionality => enable/disable all menus with same shortpath */
-    shortpath = strrchr(path, '/');
-
-    set_menu_sensitivity_meat(factory, path, val);
-
-    while (menu_list != NULL) {
-  	  set_menu_sensitivity_meat(menu_list->data, shortpath, val);
-	  menu_list = g_slist_next(menu_list);
+    /*
+     * Do it for that particular menu.
+     */
+    if ((menu_item = gtk_item_factory_get_widget(ifactory, path)) != NULL) {
+      if (GTK_IS_MENU(menu_item)) {
+        /*
+         * "path" refers to a submenu; "gtk_item_factory_get_widget()"
+         * gets the menu, not the item that, when selected, pops up
+         * the submenu.
+         *
+         * We have to change the latter item's sensitivity, so that
+         * it shows up normally if sensitive and grayed-out if
+         * insensitive.
+         */
+        menu_item = gtk_menu_get_attach_widget(GTK_MENU(menu_item));
+      } 
+      gtk_widget_set_sensitive(menu_item, val);
     }
   }
 }
@@ -493,7 +546,7 @@ set_menu_object_data (gchar *path, gchar *key, gpointer data) {
   GSList *menu_list = popup_menu_list;
   gchar *shortpath = strrchr(path, '/');
 
-  set_menu_object_data_meat(factory, path, key, data);
+  set_menu_object_data_meat(main_menu_factory, path, key, data);
   while (menu_list != NULL) {
   	set_menu_object_data_meat(menu_list->data, shortpath, key, data);
 	menu_list = g_slist_next(menu_list);
@@ -569,10 +622,11 @@ popup_menu_handler(GtkWidget *widget, GdkEvent *event, gpointer data)
 void
 set_menus_for_capture_file(gboolean have_capture_file)
 {
-  set_menu_sensitivity("/File/Open...", have_capture_file);
-  set_menu_sensitivity("/File/Save As...", have_capture_file);
-  set_menu_sensitivity("/File/Close", have_capture_file);
-  set_menu_sensitivity("/File/Reload", have_capture_file);
+  set_menu_sensitivity(main_menu_factory, "/File/Open...", have_capture_file);
+  set_menu_sensitivity(main_menu_factory, "/File/Save As...",
+      have_capture_file);
+  set_menu_sensitivity(main_menu_factory, "/File/Close", have_capture_file);
+  set_menu_sensitivity(main_menu_factory, "/File/Reload", have_capture_file);
 }
 
 /* Enable or disable menu items based on whether you have an unsaved
@@ -580,7 +634,8 @@ set_menus_for_capture_file(gboolean have_capture_file)
 void
 set_menus_for_unsaved_capture_file(gboolean have_unsaved_capture_file)
 {
-  set_menu_sensitivity("/File/Save", have_unsaved_capture_file);
+  set_menu_sensitivity(main_menu_factory, "/File/Save",
+      have_unsaved_capture_file);
 }
 
 /* Enable or disable menu items based on whether there's a capture in
@@ -588,13 +643,16 @@ set_menus_for_unsaved_capture_file(gboolean have_unsaved_capture_file)
 void
 set_menus_for_capture_in_progress(gboolean capture_in_progress)
 {
-  set_menu_sensitivity("/File/Open...", !capture_in_progress);
-  set_menu_sensitivity("/Capture/Start...", !capture_in_progress);
+  set_menu_sensitivity(main_menu_factory, "/File/Open...",
+      !capture_in_progress);
+  set_menu_sensitivity(main_menu_factory, "/Capture/Start...",
+      !capture_in_progress);
   /*
    * XXX - this doesn't yet work in Win32.
    */
 #ifndef _WIN32
-  set_menu_sensitivity("/Capture/Stop", capture_in_progress);
+  set_menu_sensitivity(main_menu_factory, "/Capture/Stop",
+      capture_in_progress);
 #endif
 }
 
@@ -603,37 +661,74 @@ set_menus_for_capture_in_progress(gboolean capture_in_progress)
 void
 set_menus_for_captured_packets(gboolean have_captured_packets)
 {
-  set_menu_sensitivity("/File/Print...", have_captured_packets);
-  set_menu_sensitivity("/Edit/Find Frame...", have_captured_packets);
-  set_menu_sensitivity("/Edit/Find Next", have_captured_packets);
-  set_menu_sensitivity("/Edit/Find Previous", have_captured_packets);
-  set_menu_sensitivity("/Edit/Go To Frame...", have_captured_packets);
-  set_menu_sensitivity("/Display/Colorize Display...", have_captured_packets);
-  set_menu_sensitivity("/Tools/Summary", have_captured_packets);
-  set_menu_sensitivity("/Tools/Protocol Hierarchy Statistics", have_captured_packets);
-  set_menu_sensitivity("<PacketList>/Match", have_captured_packets);
-  set_menu_sensitivity("<PacketList>/Prepare", have_captured_packets);
+  set_menu_sensitivity(main_menu_factory, "/File/Print...",
+      have_captured_packets);
+  set_menu_sensitivity(packet_list_menu_factory, "/Print...",
+      have_captured_packets);
+  set_menu_sensitivity(main_menu_factory, "/Edit/Find Frame...",
+      have_captured_packets);
+  set_menu_sensitivity(main_menu_factory, "/Edit/Find Next",
+      have_captured_packets);
+  set_menu_sensitivity(main_menu_factory, "/Edit/Find Previous",
+      have_captured_packets);
+  set_menu_sensitivity(main_menu_factory, "/Edit/Go To Frame...",
+      have_captured_packets);
+  set_menu_sensitivity(main_menu_factory, "/Display/Colorize Display...",
+      have_captured_packets);
+  set_menu_sensitivity(packet_list_menu_factory, "/Colorize Display...",
+      have_captured_packets);
+  set_menu_sensitivity(main_menu_factory, "/Tools/Summary",
+      have_captured_packets);
+  set_menu_sensitivity(main_menu_factory,
+      "/Tools/Protocol Hierarchy Statistics", have_captured_packets);
+  set_menu_sensitivity(packet_list_menu_factory, "/Match",
+      have_captured_packets);
+  set_menu_sensitivity(packet_list_menu_factory, "/Prepare",
+      have_captured_packets);
+  set_menu_sensitivity(main_menu_factory, "/Tools/Statistics",
+      have_captured_packets);
 }
 
 /* Enable or disable menu items based on whether a packet is selected. */
 void
 set_menus_for_selected_packet(gboolean have_selected_packet)
 {
-  set_menu_sensitivity("/File/Print Packet", have_selected_packet);
-  set_menu_sensitivity("/Edit/Mark Frame", have_selected_packet);
-  set_menu_sensitivity("/Edit/Mark All Frames", have_selected_packet);
-  set_menu_sensitivity("/Edit/Unmark All Frames", have_selected_packet);
-  set_menu_sensitivity("/Display/Collapse All", have_selected_packet);
-  set_menu_sensitivity("/Display/Expand All", have_selected_packet);
-  set_menu_sensitivity("/Display/Show Packet In New Window", have_selected_packet);
-  set_menu_sensitivity("/Tools/Follow TCP Stream",
+  set_menu_sensitivity(main_menu_factory, "/File/Print Packet",
+      have_selected_packet);
+  set_menu_sensitivity(packet_list_menu_factory, "/Print Packet",
+      have_selected_packet);
+  set_menu_sensitivity(main_menu_factory, "/Edit/Mark Frame",
+      have_selected_packet);
+  set_menu_sensitivity(packet_list_menu_factory, "/Mark Frame",
+      have_selected_packet);
+  set_menu_sensitivity(main_menu_factory, "/Edit/Mark All Frames",
+      have_selected_packet);
+  set_menu_sensitivity(main_menu_factory, "/Edit/Unmark All Frames",
+      have_selected_packet);
+  set_menu_sensitivity(main_menu_factory, "/Display/Collapse All",
+      have_selected_packet);
+  set_menu_sensitivity(tree_view_menu_factory, "/Collapse All",
+      have_selected_packet);
+  set_menu_sensitivity(main_menu_factory, "/Display/Expand All",
+      have_selected_packet);
+  set_menu_sensitivity(tree_view_menu_factory, "/Expand All",
+      have_selected_packet);
+  set_menu_sensitivity(main_menu_factory, "/Display/Show Packet In New Window",
+      have_selected_packet);
+  set_menu_sensitivity(packet_list_menu_factory, "/Show Packet In New Window",
+      have_selected_packet);
+  set_menu_sensitivity(main_menu_factory, "/Tools/Follow TCP Stream",
       have_selected_packet ? (cfile.edt->pi.ipproto == 6) : FALSE);
-  set_menu_sensitivity("/Tools/Decode As...",
+  set_menu_sensitivity(NULL, "/Follow TCP Stream",
+      have_selected_packet ? (cfile.edt->pi.ipproto == 6) : FALSE);
+  set_menu_sensitivity(main_menu_factory, "/Tools/Decode As...",
       have_selected_packet && decode_as_ok());
-  set_menu_sensitivity("/Resolve Name",
+  set_menu_sensitivity(NULL, "/Decode As...",
+      have_selected_packet && decode_as_ok());
+  set_menu_sensitivity(tree_view_menu_factory, "/Resolve Name",
       have_selected_packet && g_resolv_flags == 0);
-  set_menu_sensitivity("/Tools/TCP Stream Analysis",
-            have_selected_packet ? (cfile.edt->pi.ipproto == 6) : FALSE);
+  set_menu_sensitivity(main_menu_factory, "/Tools/TCP Stream Analysis",
+      have_selected_packet ? (cfile.edt->pi.ipproto == 6) : FALSE);
 }
 
 /* Enable or disable menu items based on whether a tree row is selected
@@ -651,28 +746,35 @@ set_menus_for_selected_tree_row(gboolean have_selected_tree)
 	  properties = prefs_is_registered_protocol(proto_registrar_get_abbrev(hfinfo->parent));
 	}
 	if (hfinfo->type == FT_FRAMENUM) {
-	    set_menu_sensitivity("<Main>/Tools/Go To Corresponding Frame", TRUE);
-	    set_menu_sensitivity("<TreeView>/Go To Corresponding Frame", TRUE);
+	    set_menu_sensitivity(main_menu_factory,
+		"/Tools/Go To Corresponding Frame", TRUE);
+	    set_menu_sensitivity(tree_view_menu_factory,
+		"/Go To Corresponding Frame", TRUE);
 	} else {
-	    set_menu_sensitivity("<Main>/Tools/Go To Corresponding Frame", FALSE);
-	    set_menu_sensitivity("<TreeView>/Go To Corresponding Frame", FALSE);
+	    set_menu_sensitivity(main_menu_factory,
+		"/Tools/Go To Corresponding Frame", FALSE);
+	    set_menu_sensitivity(tree_view_menu_factory,
+		"/Go To Corresponding Frame", FALSE);
 	}
-	set_menu_sensitivity("<Main>/Display/Match",
+	set_menu_sensitivity(main_menu_factory, "/Display/Match",
 	  proto_can_match_selected(finfo_selected));
-	set_menu_sensitivity("<TreeView>/Match",
+	set_menu_sensitivity(tree_view_menu_factory, "/Match",
 	  proto_can_match_selected(finfo_selected));
-	set_menu_sensitivity("<Main>/Display/Prepare",
+	set_menu_sensitivity(main_menu_factory, "/Display/Prepare",
 	  proto_can_match_selected(finfo_selected));
-	set_menu_sensitivity("<TreeView>/Prepare",
+	set_menu_sensitivity(tree_view_menu_factory, "/Prepare",
 	  proto_can_match_selected(finfo_selected));
   } else {
-	set_menu_sensitivity("<Main>/Display/Match", FALSE);
-	set_menu_sensitivity("<TreeView>/Match", FALSE);
-	set_menu_sensitivity("<Main>/Display/Prepare", FALSE);
-	set_menu_sensitivity("<TreeView>/Prepare", FALSE);
-	set_menu_sensitivity("<Main>/Tools/Go To Corresponding Frame", FALSE);
-	set_menu_sensitivity("<TreeView>/Go To Corresponding Frame", FALSE);
+	set_menu_sensitivity(main_menu_factory, "/Display/Match", FALSE);
+	set_menu_sensitivity(tree_view_menu_factory, "/Match", FALSE);
+	set_menu_sensitivity(main_menu_factory, "/Display/Prepare", FALSE);
+	set_menu_sensitivity(tree_view_menu_factory, "/Prepare", FALSE);
+	set_menu_sensitivity(main_menu_factory,
+	    "/Tools/Go To Corresponding Frame", FALSE);
+	set_menu_sensitivity(tree_view_menu_factory,
+	    "/Go To Corresponding Frame", FALSE);
   }
 
-  set_menu_sensitivity("/Protocol Properties...", have_selected_tree && properties);
+  set_menu_sensitivity(tree_view_menu_factory, "/Protocol Properties...",
+      have_selected_tree && properties);
 }
