@@ -1,22 +1,22 @@
 /* prefs_dlg.c
  * Routines for handling preferences
  *
- * $Id: prefs_dlg.c,v 1.51 2002/08/02 22:34:54 jmayer Exp $
+ * $Id: prefs_dlg.c,v 1.59 2003/03/11 23:14:42 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
@@ -28,18 +28,7 @@
 
 #include <gtk/gtk.h>
 
-#include <stdlib.h>
 #include <string.h>
-#include <ctype.h>
-#include <errno.h>
-
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
-#ifdef HAVE_SYS_STAT_H
-#include <sys/stat.h>
-#endif
 
 #include <epan/filesystem.h>
 
@@ -58,6 +47,7 @@
 #include "ui_util.h"
 #include "dlg_utils.h"
 #include "simple_dialog.h"
+#include "compat_macros.h"
 
 #include "prefs-int.h"
 
@@ -73,7 +63,12 @@ static void     prefs_main_save_cb(GtkWidget *, gpointer);
 static void     prefs_main_cancel_cb(GtkWidget *, gpointer);
 static gboolean prefs_main_delete_cb(GtkWidget *, gpointer);
 static void     prefs_main_destroy_cb(GtkWidget *, gpointer);
-static void	prefs_tree_select_cb(GtkCTree *, GtkCTreeNode *, gint, gpointer);
+#if GTK_MAJOR_VERSION < 2
+static void	prefs_tree_select_cb(GtkCTree *, GtkCTreeNode *, gint,
+                                     gpointer);
+#else
+static void	prefs_tree_select_cb(GtkTreeSelection *, gpointer);
+#endif
 
 #define E_PRINT_PAGE_KEY   "printer_options_page"
 #define E_COLUMN_PAGE_KEY  "column_options_page"
@@ -83,10 +78,10 @@ static void	prefs_tree_select_cb(GtkCTree *, GtkCTreeNode *, gint, gpointer);
 #define E_NAMERES_PAGE_KEY "nameres_options_page"
 #define E_TOOLTIPS_KEY     "tooltips"
 
-#define FIRST_PROTO_PREFS_PAGE	6
+static int first_proto_prefs_page = -1;
 
-/* 
- * Keep a static pointer to the notebook to be able to choose the 
+/*
+ * Keep a static pointer to the notebook to be able to choose the
  * displayed page.
  */
 static GtkWidget *notebook;
@@ -107,11 +102,17 @@ static GtkWidget *prefs_w;
 static e_prefs saved_prefs;
 
 struct ct_struct {
+  GtkWidget    *main_vb;
   GtkWidget    *notebook;
-  GtkWidget    *ctree;
+  GtkWidget    *tree;
+#if GTK_MAJOR_VERSION < 2
   GtkCTreeNode *node;
+#else
+  GtkTreeIter  iter;
+#endif
   GtkTooltips  *tooltips;
   gint         page;
+  gboolean     is_protocol;
 };
 
 static void
@@ -205,54 +206,130 @@ static void
 module_prefs_show(module_t *module, gpointer user_data)
 {
   struct ct_struct *cts = user_data;
+  struct ct_struct child_cts;
   GtkWidget        *main_vb, *main_tb, *frame;
-  gchar            label_str[MAX_TREE_NODE_NAME_LEN], *label_ptr = label_str;
+  gchar            label_str[MAX_TREE_NODE_NAME_LEN];
+#if GTK_MAJOR_VERSION < 2
+  gchar            *label_ptr = label_str;
   GtkCTreeNode     *ct_node;
+#else
+  GtkTreeStore     *model;
+  GtkTreeIter      iter;
+#endif
 
-  /* Frame */
-  frame = gtk_frame_new(module->title);
-  gtk_widget_show(frame);
-
-  /* Main vertical box */
-  main_vb = gtk_vbox_new(FALSE, 5);
-  gtk_container_border_width(GTK_CONTAINER(main_vb), 5);
-  gtk_container_add(GTK_CONTAINER(frame), main_vb);
-
-  /* Main table */
-  main_tb = gtk_table_new(module->numprefs, 2, FALSE);
-  gtk_box_pack_start(GTK_BOX(main_vb), main_tb, FALSE, FALSE, 0);
-  gtk_table_set_row_spacings(GTK_TABLE(main_tb), 10);
-  gtk_table_set_col_spacings(GTK_TABLE(main_tb), 15);
-  gtk_object_set_data(GTK_OBJECT(main_tb), E_TOOLTIPS_KEY, cts->tooltips);
-
-  /* Add items for each of the preferences */
-  prefs_pref_foreach(module, pref_show, main_tb);
-
-  gtk_notebook_append_page(GTK_NOTEBOOK(cts->notebook), frame, NULL);
+  /*
+   * Add this module to the tree.
+   */
   strcpy(label_str, module->title);
-  ct_node = gtk_ctree_insert_node(GTK_CTREE(cts->ctree), cts->node, NULL, 
-  		&label_ptr, 5, NULL, NULL, NULL, NULL, TRUE, TRUE);
-  gtk_ctree_node_set_row_data(GTK_CTREE(cts->ctree), ct_node,
-  		GINT_TO_POINTER(cts->page));
-  cts->page++;
+#if GTK_MAJOR_VERSION < 2
+  ct_node = gtk_ctree_insert_node(GTK_CTREE(cts->tree), cts->node, NULL,
+  		&label_ptr, 5, NULL, NULL, NULL, NULL, !module->is_subtree,
+  		FALSE);
+#else
+  model = GTK_TREE_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(cts->tree)));
+  if (module->is_subtree)
+      gtk_tree_store_append(model, &iter, NULL);
+  else
+      gtk_tree_store_append(model, &iter, &cts->iter);
+#endif
 
-  /* Show 'em what we got */
-  gtk_widget_show_all(main_vb);
+  /*
+   * Is this a subtree?
+   */
+  if (module->is_subtree) {
+    /*
+     * Yes.
+     */
+
+    /* Note that there's no page attached to this item */
+#if GTK_MAJOR_VERSION < 2
+    gtk_ctree_node_set_row_data(GTK_CTREE(cts->tree), ct_node,
+  		GINT_TO_POINTER(-1));
+#else
+    gtk_tree_store_set(model, &iter, 0, label_str, 1, -1, -1);
+#endif
+
+    /*
+     * Walk the subtree and attach stuff to it.
+     */
+    child_cts = *cts;
+#if GTK_MAJOR_VERSION < 2
+    child_cts.node = ct_node;
+#else
+    child_cts.iter = iter;
+#endif
+    if (module == protocols_module)
+      child_cts.is_protocol = TRUE;
+    prefs_module_list_foreach(module->prefs, module_prefs_show, &child_cts);
+  } else {
+    /*
+     * No.
+     * Create a notebook page for it.
+     */
+
+    /* Frame */
+    frame = gtk_frame_new(module->title);
+    gtk_widget_show(frame);
+
+    /* Main vertical box */
+    main_vb = gtk_vbox_new(FALSE, 5);
+    gtk_container_border_width(GTK_CONTAINER(main_vb), 5);
+    gtk_container_add(GTK_CONTAINER(frame), main_vb);
+
+    /* Main table */
+    main_tb = gtk_table_new(module->numprefs, 2, FALSE);
+    gtk_box_pack_start(GTK_BOX(main_vb), main_tb, FALSE, FALSE, 0);
+    gtk_table_set_row_spacings(GTK_TABLE(main_tb), 10);
+    gtk_table_set_col_spacings(GTK_TABLE(main_tb), 15);
+    OBJECT_SET_DATA(main_tb, E_TOOLTIPS_KEY, cts->tooltips);
+
+    /* Add items for each of the preferences */
+    prefs_pref_foreach(module, pref_show, main_tb);
+
+    /* Add the page to the notebook */
+    gtk_notebook_append_page(GTK_NOTEBOOK(cts->notebook), frame, NULL);
+
+    /* Attach the page to the tree item */
+#if GTK_MAJOR_VERSION < 2
+    gtk_ctree_node_set_row_data(GTK_CTREE(cts->tree), ct_node,
+  		GINT_TO_POINTER(cts->page));
+#else
+    gtk_tree_store_set(model, &iter, 0, label_str, 1, cts->page, -1);
+#endif
+
+    /* If this is the first protocol page, remember its page number */
+    if (first_proto_prefs_page == -1)
+      first_proto_prefs_page = cts->page;
+    cts->page++;
+
+    /* Show 'em what we got */
+    gtk_widget_show_all(main_vb);
+  }
 }
 
 void
 prefs_cb(GtkWidget *w _U_, gpointer dummy _U_)
 {
-  GtkWidget        *main_vb, *top_hb, *bbox, *prefs_nb, *ct_sb, *frame,
-                   *ok_bt, *apply_bt, *save_bt, *cancel_bt;
-  GtkWidget        *print_pg, *column_pg, *stream_pg, *gui_pg;
+  GtkWidget         *top_hb, *bbox, *prefs_nb, *ct_sb, *frame,
+                    *ok_bt, *apply_bt, *save_bt, *cancel_bt;
+  GtkWidget         *print_pg, *column_pg, *stream_pg, *gui_pg;
 #ifdef HAVE_LIBPCAP
-  GtkWidget        *capture_pg;
+  GtkWidget         *capture_pg;
+#endif
+  GtkWidget         *nameres_pg;
+  gchar             label_str[MAX_TREE_NODE_NAME_LEN];
+  struct ct_struct  cts;
+#if GTK_MAJOR_VERSION < 2
+  gchar             *label_ptr = label_str;
+  GtkCTreeNode      *ct_node;
+#else
+  GtkTreeStore      *store;
+  GtkTreeSelection  *selection;
+  GtkCellRenderer   *renderer;
+  GtkTreeViewColumn *column;
+  gint              col_offset;
+  GtkTreeIter       iter;
 #endif
-  GtkWidget        *nameres_pg;
-  gchar            label_str[MAX_TREE_NODE_NAME_LEN], *label_ptr = label_str;
-  GtkCTreeNode     *ct_node;
-  struct ct_struct cts;
 
   if (prefs_w != NULL) {
     /* There's already a "Preferences" dialog box; reactivate it. */
@@ -265,11 +342,9 @@ prefs_cb(GtkWidget *w _U_, gpointer dummy _U_)
   copy_prefs(&saved_prefs, &prefs);
 
   prefs_w = dlg_window_new("Ethereal: Preferences");
-  gtk_signal_connect(GTK_OBJECT(prefs_w), "delete_event",
-    GTK_SIGNAL_FUNC(prefs_main_delete_cb), NULL);
-  gtk_signal_connect(GTK_OBJECT(prefs_w), "destroy",
-	GTK_SIGNAL_FUNC(prefs_main_destroy_cb), NULL);
-  
+  SIGNAL_CONNECT(prefs_w, "delete_event", prefs_main_delete_cb, NULL);
+  SIGNAL_CONNECT(prefs_w, "destroy", prefs_main_destroy_cb, NULL);
+
   /*
    * Unfortunately, we can't arrange that a GtkTable widget wrap an event box
    * around a table row, so the spacing between the preference item's label
@@ -279,14 +354,14 @@ prefs_cb(GtkWidget *w _U_, gpointer dummy _U_)
   cts.tooltips = gtk_tooltips_new();
 
   /* Container for each row of widgets */
-  main_vb = gtk_vbox_new(FALSE, 5);
-  gtk_container_border_width(GTK_CONTAINER(main_vb), 5);
-  gtk_container_add(GTK_CONTAINER(prefs_w), main_vb);
-  gtk_widget_show(main_vb);
-  
+  cts.main_vb = gtk_vbox_new(FALSE, 5);
+  gtk_container_border_width(GTK_CONTAINER(cts.main_vb), 5);
+  gtk_container_add(GTK_CONTAINER(prefs_w), cts.main_vb);
+  gtk_widget_show(cts.main_vb);
+
   /* Top row: Preferences tree and notebook */
   top_hb = gtk_hbox_new(FALSE, 10);
-  gtk_container_add(GTK_CONTAINER(main_vb), top_hb);
+  gtk_container_add(GTK_CONTAINER(cts.main_vb), top_hb);
   gtk_widget_show(top_hb);
 
   /* Place a Ctree on the left for preference categories */
@@ -296,62 +371,97 @@ prefs_cb(GtkWidget *w _U_, gpointer dummy _U_)
   gtk_container_add(GTK_CONTAINER(top_hb), ct_sb);
   gtk_widget_show(ct_sb);
 
-  cts.ctree = ctree_new(1, 0);
-  cts.page  = 0;
-  gtk_container_add(GTK_CONTAINER(ct_sb), cts.ctree);
+#if GTK_MAJOR_VERSION < 2
+  cts.tree = ctree_new(1, 0);
+  cts.node = NULL;
+#else
+  store = gtk_tree_store_new(2, G_TYPE_STRING, G_TYPE_INT);
+  cts.tree = tree_view_new(GTK_TREE_MODEL(store));
+  gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(cts.tree), FALSE);
+  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(cts.tree));
+  gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);
+  renderer = gtk_cell_renderer_text_new();
+  col_offset = gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(cts.tree),
+                                                           -1, "Name", renderer,
+                                                           "text", 0, NULL);
+  column = gtk_tree_view_get_column(GTK_TREE_VIEW(cts.tree),
+                                    col_offset - 1);
+  gtk_tree_view_column_set_sizing(GTK_TREE_VIEW_COLUMN(column),
+                                  GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+#endif
+  cts.page = 0;
+  gtk_container_add(GTK_CONTAINER(ct_sb), cts.tree);
+
+#if GTK_MAJOR_VERSION < 2
+  gtk_clist_set_column_auto_resize(GTK_CLIST(cts.tree), 0, TRUE);
+  SIGNAL_CONNECT(cts.tree, "tree-select-row", prefs_tree_select_cb, NULL);
+#else
+  SIGNAL_CONNECT(selection, "changed", prefs_tree_select_cb, NULL);
+#endif
+  gtk_widget_show(cts.tree);
 
-  gtk_clist_set_column_auto_resize(GTK_CLIST(cts.ctree), 0, TRUE);
-  gtk_signal_connect(GTK_OBJECT(cts.ctree), "tree-select-row",
-    GTK_SIGNAL_FUNC(prefs_tree_select_cb), NULL);
-  gtk_widget_show(cts.ctree);
-  
   /* A notebook widget sans tabs is used to flip between prefs */
   notebook = prefs_nb = gtk_notebook_new();
-  gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), FALSE);
-  gtk_notebook_set_show_border(GTK_NOTEBOOK(notebook), FALSE);
+  gtk_notebook_set_show_tabs(GTK_NOTEBOOK(prefs_nb), FALSE);
+  gtk_notebook_set_show_border(GTK_NOTEBOOK(prefs_nb), FALSE);
   gtk_container_add(GTK_CONTAINER(top_hb), prefs_nb);
   gtk_widget_show(prefs_nb);
-  
+
   /* Printing prefs */
   frame = gtk_frame_new("Printing");
   gtk_widget_show(GTK_WIDGET(frame));
   print_pg = printer_prefs_show();
   gtk_container_add(GTK_CONTAINER(frame), print_pg);
-  gtk_object_set_data(GTK_OBJECT(prefs_w), E_PRINT_PAGE_KEY, print_pg);
+  OBJECT_SET_DATA(prefs_w, E_PRINT_PAGE_KEY, print_pg);
   gtk_notebook_append_page (GTK_NOTEBOOK(prefs_nb), frame, NULL);
   strcpy(label_str, "Printing");
-  ct_node = gtk_ctree_insert_node(GTK_CTREE(cts.ctree), NULL, NULL, 
+#if GTK_MAJOR_VERSION < 2
+  ct_node = gtk_ctree_insert_node(GTK_CTREE(cts.tree), NULL, NULL,
   		&label_ptr, 5, NULL, NULL, NULL, NULL, TRUE, TRUE);
-  gtk_ctree_node_set_row_data(GTK_CTREE(cts.ctree), ct_node,
+  gtk_ctree_node_set_row_data(GTK_CTREE(cts.tree), ct_node,
   		GINT_TO_POINTER(cts.page));
+#else
+  gtk_tree_store_append(store, &iter, NULL);
+  gtk_tree_store_set(store, &iter, 0, label_str, 1, cts.page, -1);
+#endif
   cts.page++;
-    
+
   /* Column prefs */
   frame = gtk_frame_new("Columns");
   gtk_widget_show(GTK_WIDGET(frame));
   column_pg = column_prefs_show();
   gtk_container_add(GTK_CONTAINER(frame), column_pg);
-  gtk_object_set_data(GTK_OBJECT(prefs_w), E_COLUMN_PAGE_KEY, column_pg);
+  OBJECT_SET_DATA(prefs_w, E_COLUMN_PAGE_KEY, column_pg);
   gtk_notebook_append_page (GTK_NOTEBOOK(prefs_nb), frame, NULL);
   strcpy(label_str, "Columns");
-  ct_node = gtk_ctree_insert_node(GTK_CTREE(cts.ctree), NULL, NULL, 
+#if GTK_MAJOR_VERSION < 2
+  ct_node = gtk_ctree_insert_node(GTK_CTREE(cts.tree), NULL, NULL,
   		&label_ptr, 5, NULL, NULL, NULL, NULL, TRUE, TRUE);
-  gtk_ctree_node_set_row_data(GTK_CTREE(cts.ctree), ct_node,
+  gtk_ctree_node_set_row_data(GTK_CTREE(cts.tree), ct_node,
   		GINT_TO_POINTER(cts.page));
+#else
+  gtk_tree_store_append(store, &iter, NULL);
+  gtk_tree_store_set(store, &iter, 0, label_str, 1, cts.page, -1);
+#endif
   cts.page++;
-  
+
   /* TCP Streams prefs */
   frame = gtk_frame_new("TCP Streams");
   gtk_widget_show(GTK_WIDGET(frame));
   stream_pg = stream_prefs_show();
   gtk_container_add(GTK_CONTAINER(frame), stream_pg);
-  gtk_object_set_data(GTK_OBJECT(prefs_w), E_STREAM_PAGE_KEY, stream_pg);
+  OBJECT_SET_DATA(prefs_w, E_STREAM_PAGE_KEY, stream_pg);
   gtk_notebook_append_page (GTK_NOTEBOOK(prefs_nb), frame, NULL);
   strcpy(label_str, "TCP Streams");
-  ct_node = gtk_ctree_insert_node(GTK_CTREE(cts.ctree), NULL, NULL, 
+#if GTK_MAJOR_VERSION < 2
+  ct_node = gtk_ctree_insert_node(GTK_CTREE(cts.tree), NULL, NULL,
   		&label_ptr, 5, NULL, NULL, NULL, NULL, TRUE, TRUE);
-  gtk_ctree_node_set_row_data(GTK_CTREE(cts.ctree), ct_node,
+  gtk_ctree_node_set_row_data(GTK_CTREE(cts.tree), ct_node,
   		GINT_TO_POINTER(cts.page));
+#else
+  gtk_tree_store_append(store, &iter, NULL);
+  gtk_tree_store_set(store, &iter, 0, label_str, 1, cts.page, -1);
+#endif
   cts.page++;
 
   /* GUI prefs */
@@ -359,13 +469,18 @@ prefs_cb(GtkWidget *w _U_, gpointer dummy _U_)
   gtk_widget_show(GTK_WIDGET(frame));
   gui_pg = gui_prefs_show();
   gtk_container_add(GTK_CONTAINER(frame), gui_pg);
-  gtk_object_set_data(GTK_OBJECT(prefs_w), E_GUI_PAGE_KEY, gui_pg);
+  OBJECT_SET_DATA(prefs_w, E_GUI_PAGE_KEY, gui_pg);
   gtk_notebook_append_page (GTK_NOTEBOOK(prefs_nb), frame, NULL);
   strcpy(label_str, "User Interface");
-  ct_node = gtk_ctree_insert_node(GTK_CTREE(cts.ctree), NULL, NULL, 
+#if GTK_MAJOR_VERSION < 2
+  ct_node = gtk_ctree_insert_node(GTK_CTREE(cts.tree), NULL, NULL,
   		&label_ptr, 5, NULL, NULL, NULL, NULL, TRUE, TRUE);
-  gtk_ctree_node_set_row_data(GTK_CTREE(cts.ctree), ct_node,
+  gtk_ctree_node_set_row_data(GTK_CTREE(cts.tree), ct_node,
   		GINT_TO_POINTER(cts.page));
+#else
+  gtk_tree_store_append(store, &iter, NULL);
+  gtk_tree_store_set(store, &iter, 0, label_str, 1, cts.page, -1);
+#endif
   cts.page++;
 
 #ifdef HAVE_LIBPCAP
@@ -378,13 +493,18 @@ prefs_cb(GtkWidget *w _U_, gpointer dummy _U_)
   gtk_widget_show(GTK_WIDGET(frame));
   capture_pg = capture_prefs_show();
   gtk_container_add(GTK_CONTAINER(frame), capture_pg);
-  gtk_object_set_data(GTK_OBJECT(prefs_w), E_CAPTURE_PAGE_KEY, capture_pg);
+  OBJECT_SET_DATA(prefs_w, E_CAPTURE_PAGE_KEY, capture_pg);
   gtk_notebook_append_page (GTK_NOTEBOOK(prefs_nb), frame, NULL);
   strcpy(label_str, "Capture");
-  ct_node = gtk_ctree_insert_node(GTK_CTREE(cts.ctree), NULL, NULL, 
+#if GTK_MAJOR_VERSION < 2
+  ct_node = gtk_ctree_insert_node(GTK_CTREE(cts.tree), NULL, NULL,
   		&label_ptr, 5, NULL, NULL, NULL, NULL, TRUE, TRUE);
-  gtk_ctree_node_set_row_data(GTK_CTREE(cts.ctree), ct_node,
+  gtk_ctree_node_set_row_data(GTK_CTREE(cts.tree), ct_node,
   		GINT_TO_POINTER(cts.page));
+#else
+  gtk_tree_store_append(store, &iter, NULL);
+  gtk_tree_store_set(store, &iter, 0, label_str, 1, cts.page, -1);
+#endif
   cts.page++;
 #ifdef _WIN32
   }
@@ -396,59 +516,69 @@ prefs_cb(GtkWidget *w _U_, gpointer dummy _U_)
   gtk_widget_show(GTK_WIDGET(frame));
   nameres_pg = nameres_prefs_show();
   gtk_container_add(GTK_CONTAINER(frame), nameres_pg);
-  gtk_object_set_data(GTK_OBJECT(prefs_w), E_NAMERES_PAGE_KEY, nameres_pg);
+  OBJECT_SET_DATA(prefs_w, E_NAMERES_PAGE_KEY, nameres_pg);
   gtk_notebook_append_page (GTK_NOTEBOOK(prefs_nb), frame, NULL);
   strcpy(label_str, "Name resolution");
-  ct_node = gtk_ctree_insert_node(GTK_CTREE(cts.ctree), NULL, NULL, 
+#if GTK_MAJOR_VERSION < 2
+  ct_node = gtk_ctree_insert_node(GTK_CTREE(cts.tree), NULL, NULL,
   		&label_ptr, 5, NULL, NULL, NULL, NULL, TRUE, TRUE);
-  gtk_ctree_node_set_row_data(GTK_CTREE(cts.ctree), ct_node,
+  gtk_ctree_node_set_row_data(GTK_CTREE(cts.tree), ct_node,
   		GINT_TO_POINTER(cts.page));
+#else
+  gtk_tree_store_append(store, &iter, NULL);
+  gtk_tree_store_set(store, &iter, 0, label_str, 1, cts.page, -1);
+#endif
   cts.page++;
 
   /* Registered prefs */
   cts.notebook = prefs_nb;
-  strcpy(label_str, "Protocols");
-  cts.node = gtk_ctree_insert_node(GTK_CTREE(cts.ctree), NULL, NULL,
-  		&label_ptr, 5, NULL, NULL, NULL, NULL, FALSE, FALSE);
-  gtk_ctree_node_set_row_data(GTK_CTREE(cts.ctree), cts.node,
-  		GINT_TO_POINTER(-1));
-  gtk_ctree_node_set_selectable(GTK_CTREE(cts.ctree), cts.node, FALSE);
-
-  prefs_module_foreach(module_prefs_show, &cts);
-
+  cts.is_protocol = FALSE;
+  prefs_module_list_foreach(NULL, module_prefs_show, &cts);
 
   /* Button row: OK and cancel buttons */
   bbox = gtk_hbutton_box_new();
   gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), GTK_BUTTONBOX_END);
   gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 5);
-  gtk_container_add(GTK_CONTAINER(main_vb), bbox);
+  gtk_container_add(GTK_CONTAINER(cts.main_vb), bbox);
   gtk_widget_show(bbox);
-  
+
+#if GTK_MAJOR_VERSION < 2
   ok_bt = gtk_button_new_with_label ("OK");
-  gtk_signal_connect(GTK_OBJECT(ok_bt), "clicked",
-    GTK_SIGNAL_FUNC(prefs_main_ok_cb), GTK_OBJECT(prefs_w));
+#else
+  ok_bt = gtk_button_new_from_stock(GTK_STOCK_OK);
+#endif
+  SIGNAL_CONNECT(ok_bt, "clicked", prefs_main_ok_cb, prefs_w);
   GTK_WIDGET_SET_FLAGS(ok_bt, GTK_CAN_DEFAULT);
   gtk_box_pack_start (GTK_BOX (bbox), ok_bt, TRUE, TRUE, 0);
   gtk_widget_grab_default(ok_bt);
   gtk_widget_show(ok_bt);
 
+#if GTK_MAJOR_VERSION < 2
   apply_bt = gtk_button_new_with_label ("Apply");
-  gtk_signal_connect(GTK_OBJECT(apply_bt), "clicked",
-    GTK_SIGNAL_FUNC(prefs_main_apply_cb), GTK_OBJECT(prefs_w));
+#else
+  apply_bt = gtk_button_new_from_stock(GTK_STOCK_APPLY);
+#endif
+  SIGNAL_CONNECT(apply_bt, "clicked", prefs_main_apply_cb, prefs_w);
   GTK_WIDGET_SET_FLAGS(apply_bt, GTK_CAN_DEFAULT);
   gtk_box_pack_start(GTK_BOX (bbox), apply_bt, TRUE, TRUE, 0);
   gtk_widget_show(apply_bt);
 
+#if GTK_MAJOR_VERSION < 2
   save_bt = gtk_button_new_with_label ("Save");
-  gtk_signal_connect(GTK_OBJECT(save_bt), "clicked",
-    GTK_SIGNAL_FUNC(prefs_main_save_cb), GTK_OBJECT(prefs_w));
+#else
+  save_bt = gtk_button_new_from_stock(GTK_STOCK_SAVE);
+#endif
+  SIGNAL_CONNECT(save_bt, "clicked", prefs_main_save_cb, prefs_w);
   GTK_WIDGET_SET_FLAGS(save_bt, GTK_CAN_DEFAULT);
   gtk_box_pack_start (GTK_BOX (bbox), save_bt, TRUE, TRUE, 0);
   gtk_widget_show(save_bt);
-  
+
+#if GTK_MAJOR_VERSION < 2
   cancel_bt = gtk_button_new_with_label ("Cancel");
-  gtk_signal_connect(GTK_OBJECT(cancel_bt), "clicked",
-    GTK_SIGNAL_FUNC(prefs_main_cancel_cb), GTK_OBJECT(prefs_w));
+#else
+  cancel_bt = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
+#endif
+  SIGNAL_CONNECT(cancel_bt, "clicked", prefs_main_cancel_cb, prefs_w);
   GTK_WIDGET_SET_FLAGS(cancel_bt, GTK_CAN_DEFAULT);
   gtk_box_pack_start (GTK_BOX (bbox), cancel_bt, TRUE, TRUE, 0);
   gtk_widget_show(cancel_bt);
@@ -459,6 +589,10 @@ prefs_cb(GtkWidget *w _U_, gpointer dummy _U_)
   dlg_set_cancel(prefs_w, cancel_bt);
 
   gtk_widget_show(prefs_w);
+
+#if GTK_MAJOR_VERSION >= 2
+  g_object_unref(G_OBJECT(store));
+#endif
 }
 
 static void
@@ -488,7 +622,7 @@ create_preference_check_button(GtkWidget *main_tb, int table_position,
 	GtkTooltips *tooltips;
 	GtkWidget *check_box;
 
-	tooltips = gtk_object_get_data(GTK_OBJECT(main_tb), E_TOOLTIPS_KEY);
+	tooltips = OBJECT_GET_DATA(main_tb, E_TOOLTIPS_KEY);
 
 	set_option_label(main_tb, table_position, label_text, tooltip_text,
 	    tooltips);
@@ -515,7 +649,7 @@ create_preference_radio_buttons(GtkWidget *main_tb, int table_position,
 	const enum_val_t *enum_valp;
 	GtkWidget *event_box;
 
-	tooltips = gtk_object_get_data(GTK_OBJECT(main_tb), E_TOOLTIPS_KEY);
+	tooltips = OBJECT_GET_DATA(main_tb, E_TOOLTIPS_KEY);
 
 	set_option_label(main_tb, table_position, label_text, tooltip_text,
 	    tooltips);
@@ -602,7 +736,7 @@ create_preference_option_menu(GtkWidget *main_tb, int table_position,
 	const enum_val_t *enum_valp;
 	GtkWidget *event_box;
 
-	tooltips = gtk_object_get_data(GTK_OBJECT(main_tb), E_TOOLTIPS_KEY);
+	tooltips = OBJECT_GET_DATA(main_tb, E_TOOLTIPS_KEY);
 
 	set_option_label(main_tb, table_position, label_text, tooltip_text,
 	    tooltips);
@@ -668,7 +802,7 @@ create_preference_entry(GtkWidget *main_tb, int table_position,
 	GtkTooltips *tooltips;
 	GtkWidget *entry;
 
-	tooltips = gtk_object_get_data(GTK_OBJECT(main_tb), E_TOOLTIPS_KEY);
+	tooltips = OBJECT_GET_DATA(main_tb, E_TOOLTIPS_KEY);
 
 	set_option_label(main_tb, table_position, label_text, tooltip_text,
 	    tooltips);
@@ -737,10 +871,9 @@ pref_fetch(pref_t *pref, gpointer user_data)
 
   case PREF_STRING:
     str_val = gtk_entry_get_text(GTK_ENTRY(pref->control));
-    if (*pref->varp.string == NULL || strcmp(*pref->varp.string, str_val) != 0) {
+    if (strcmp(*pref->varp.string, str_val) != 0) {
       *pref_changed_p = TRUE;
-      if (*pref->varp.string != NULL)
-        g_free(*pref->varp.string);
+      g_free(*pref->varp.string);
       *pref->varp.string = g_strdup(str_val);
     }
     break;
@@ -811,39 +944,39 @@ prefs_main_ok_cb(GtkWidget *ok_bt _U_, gpointer parent_w)
   /* Fetch the preferences (i.e., make sure all the values set in all of
      the preferences panes have been copied to "prefs" and the registered
      preferences). */
-  printer_prefs_fetch(gtk_object_get_data(GTK_OBJECT(parent_w), E_PRINT_PAGE_KEY));
-  column_prefs_fetch(gtk_object_get_data(GTK_OBJECT(parent_w), E_COLUMN_PAGE_KEY));
-  stream_prefs_fetch(gtk_object_get_data(GTK_OBJECT(parent_w), E_STREAM_PAGE_KEY));
-  gui_prefs_fetch(gtk_object_get_data(GTK_OBJECT(parent_w), E_GUI_PAGE_KEY));
+  printer_prefs_fetch(OBJECT_GET_DATA(parent_w, E_PRINT_PAGE_KEY));
+  column_prefs_fetch(OBJECT_GET_DATA(parent_w, E_COLUMN_PAGE_KEY));
+  stream_prefs_fetch(OBJECT_GET_DATA(parent_w, E_STREAM_PAGE_KEY));
+  gui_prefs_fetch(OBJECT_GET_DATA(parent_w, E_GUI_PAGE_KEY));
 #ifdef HAVE_LIBPCAP
 #ifdef _WIN32
   /* Is WPcap loaded? */
   if (has_wpcap) {
 #endif /* _WIN32 */
-  capture_prefs_fetch(gtk_object_get_data(GTK_OBJECT(parent_w), E_CAPTURE_PAGE_KEY));
+  capture_prefs_fetch(OBJECT_GET_DATA(parent_w, E_CAPTURE_PAGE_KEY));
 #ifdef _WIN32
   }
 #endif /* _WIN32 */
 #endif /* HAVE_LIBPCAP */
-  nameres_prefs_fetch(gtk_object_get_data(GTK_OBJECT(parent_w), E_NAMERES_PAGE_KEY));
-  prefs_module_foreach(module_prefs_fetch, &must_redissect);
+  nameres_prefs_fetch(OBJECT_GET_DATA(parent_w, E_NAMERES_PAGE_KEY));
+  prefs_modules_foreach(module_prefs_fetch, &must_redissect);
 
   /* Now apply those preferences. */
-  printer_prefs_apply(gtk_object_get_data(GTK_OBJECT(parent_w), E_PRINT_PAGE_KEY));
-  column_prefs_apply(gtk_object_get_data(GTK_OBJECT(parent_w), E_COLUMN_PAGE_KEY));
-  stream_prefs_apply(gtk_object_get_data(GTK_OBJECT(parent_w), E_STREAM_PAGE_KEY));
-  gui_prefs_apply(gtk_object_get_data(GTK_OBJECT(parent_w), E_GUI_PAGE_KEY));
+  printer_prefs_apply(OBJECT_GET_DATA(parent_w, E_PRINT_PAGE_KEY));
+  column_prefs_apply(OBJECT_GET_DATA(parent_w, E_COLUMN_PAGE_KEY));
+  stream_prefs_apply(OBJECT_GET_DATA(parent_w, E_STREAM_PAGE_KEY));
+  gui_prefs_apply(OBJECT_GET_DATA(parent_w, E_GUI_PAGE_KEY));
 #ifdef HAVE_LIBPCAP
 #ifdef _WIN32
   /* Is WPcap loaded? */
   if (has_wpcap) {
 #endif /* _WIN32 */
-  capture_prefs_apply(gtk_object_get_data(GTK_OBJECT(parent_w), E_CAPTURE_PAGE_KEY));
+  capture_prefs_apply(OBJECT_GET_DATA(parent_w, E_CAPTURE_PAGE_KEY));
 #ifdef _WIN32
   }
 #endif /* _WIN32 */
 #endif /* HAVE_LIBPCAP */
-  nameres_prefs_apply(gtk_object_get_data(GTK_OBJECT(parent_w), E_NAMERES_PAGE_KEY));
+  nameres_prefs_apply(OBJECT_GET_DATA(parent_w, E_NAMERES_PAGE_KEY));
   prefs_apply_all();
 
   /* Now destroy the "Preferences" dialog. */
@@ -863,39 +996,39 @@ prefs_main_apply_cb(GtkWidget *apply_bt _U_, gpointer parent_w)
   /* Fetch the preferences (i.e., make sure all the values set in all of
      the preferences panes have been copied to "prefs" and the registered
      preferences). */
-  printer_prefs_fetch(gtk_object_get_data(GTK_OBJECT(parent_w), E_PRINT_PAGE_KEY));
-  column_prefs_fetch(gtk_object_get_data(GTK_OBJECT(parent_w), E_COLUMN_PAGE_KEY));
-  stream_prefs_fetch(gtk_object_get_data(GTK_OBJECT(parent_w), E_STREAM_PAGE_KEY));
-  gui_prefs_fetch(gtk_object_get_data(GTK_OBJECT(parent_w), E_GUI_PAGE_KEY));
+  printer_prefs_fetch(OBJECT_GET_DATA(parent_w, E_PRINT_PAGE_KEY));
+  column_prefs_fetch(OBJECT_GET_DATA(parent_w, E_COLUMN_PAGE_KEY));
+  stream_prefs_fetch(OBJECT_GET_DATA(parent_w, E_STREAM_PAGE_KEY));
+  gui_prefs_fetch(OBJECT_GET_DATA(parent_w, E_GUI_PAGE_KEY));
 #ifdef HAVE_LIBPCAP
 #ifdef _WIN32
   /* Is WPcap loaded? */
   if (has_wpcap) {
 #endif /* _WIN32 */
-  capture_prefs_fetch(gtk_object_get_data(GTK_OBJECT(parent_w), E_CAPTURE_PAGE_KEY));
+  capture_prefs_fetch(OBJECT_GET_DATA(parent_w, E_CAPTURE_PAGE_KEY));
 #ifdef _WIN32
   }
 #endif /* _WIN32 */
 #endif /* HAVE_LIBPCAP */
-  nameres_prefs_fetch(gtk_object_get_data(GTK_OBJECT(parent_w), E_NAMERES_PAGE_KEY));
-  prefs_module_foreach(module_prefs_fetch, &must_redissect);
+  nameres_prefs_fetch(OBJECT_GET_DATA(parent_w, E_NAMERES_PAGE_KEY));
+  prefs_modules_foreach(module_prefs_fetch, &must_redissect);
 
   /* Now apply those preferences. */
-  printer_prefs_apply(gtk_object_get_data(GTK_OBJECT(parent_w), E_PRINT_PAGE_KEY));
-  column_prefs_apply(gtk_object_get_data(GTK_OBJECT(parent_w), E_COLUMN_PAGE_KEY));
-  stream_prefs_apply(gtk_object_get_data(GTK_OBJECT(parent_w), E_STREAM_PAGE_KEY));
-  gui_prefs_apply(gtk_object_get_data(GTK_OBJECT(parent_w), E_GUI_PAGE_KEY));
+  printer_prefs_apply(OBJECT_GET_DATA(parent_w, E_PRINT_PAGE_KEY));
+  column_prefs_apply(OBJECT_GET_DATA(parent_w, E_COLUMN_PAGE_KEY));
+  stream_prefs_apply(OBJECT_GET_DATA(parent_w, E_STREAM_PAGE_KEY));
+  gui_prefs_apply(OBJECT_GET_DATA(parent_w, E_GUI_PAGE_KEY));
 #ifdef HAVE_LIBPCAP
 #ifdef _WIN32
   /* Is WPcap loaded? */
   if (has_wpcap) {
 #endif /* _WIN32 */
-  capture_prefs_apply(gtk_object_get_data(GTK_OBJECT(parent_w), E_CAPTURE_PAGE_KEY));
+  capture_prefs_apply(OBJECT_GET_DATA(parent_w, E_CAPTURE_PAGE_KEY));
 #ifdef _WIN32
   }
 #endif /* _WIN32 */
 #endif /* HAVE_LIBPCAP */
-  nameres_prefs_apply(gtk_object_get_data(GTK_OBJECT(parent_w), E_NAMERES_PAGE_KEY));
+  nameres_prefs_apply(OBJECT_GET_DATA(parent_w, E_NAMERES_PAGE_KEY));
   prefs_apply_all();
 
   if (must_redissect) {
@@ -915,22 +1048,22 @@ prefs_main_save_cb(GtkWidget *save_bt _U_, gpointer parent_w)
   /* Fetch the preferences (i.e., make sure all the values set in all of
      the preferences panes have been copied to "prefs" and the registered
      preferences). */
-  printer_prefs_fetch(gtk_object_get_data(GTK_OBJECT(parent_w), E_PRINT_PAGE_KEY));
-  column_prefs_fetch(gtk_object_get_data(GTK_OBJECT(parent_w), E_COLUMN_PAGE_KEY));
-  stream_prefs_fetch(gtk_object_get_data(GTK_OBJECT(parent_w), E_STREAM_PAGE_KEY));
-  gui_prefs_fetch(gtk_object_get_data(GTK_OBJECT(parent_w), E_GUI_PAGE_KEY));
+  printer_prefs_fetch(OBJECT_GET_DATA(parent_w, E_PRINT_PAGE_KEY));
+  column_prefs_fetch(OBJECT_GET_DATA(parent_w, E_COLUMN_PAGE_KEY));
+  stream_prefs_fetch(OBJECT_GET_DATA(parent_w, E_STREAM_PAGE_KEY));
+  gui_prefs_fetch(OBJECT_GET_DATA(parent_w, E_GUI_PAGE_KEY));
 #ifdef HAVE_LIBPCAP
 #ifdef _WIN32
   /* Is WPcap loaded? */
   if (has_wpcap) {
 #endif /* _WIN32 */
-  capture_prefs_fetch(gtk_object_get_data(GTK_OBJECT(parent_w), E_CAPTURE_PAGE_KEY));
+  capture_prefs_fetch(OBJECT_GET_DATA(parent_w, E_CAPTURE_PAGE_KEY));
 #ifdef _WIN32
   }
 #endif /* _WIN32 */
 #endif /* HAVE_LIBPCAP */
-  nameres_prefs_fetch(gtk_object_get_data(GTK_OBJECT(parent_w), E_NAMERES_PAGE_KEY));
-  prefs_module_foreach(module_prefs_fetch, &must_redissect);
+  nameres_prefs_fetch(OBJECT_GET_DATA(parent_w, E_NAMERES_PAGE_KEY));
+  prefs_modules_foreach(module_prefs_fetch, &must_redissect);
 
   /* Create the directory that holds personal configuration files, if
      necessary.  */
@@ -964,21 +1097,21 @@ prefs_main_save_cb(GtkWidget *save_bt _U_, gpointer parent_w)
 	   "Apply" after this, we know we have to redissect;
 
 	4) we did apply the protocol preferences, at least, in the past. */
-  printer_prefs_apply(gtk_object_get_data(GTK_OBJECT(parent_w), E_PRINT_PAGE_KEY));
-  column_prefs_apply(gtk_object_get_data(GTK_OBJECT(parent_w), E_COLUMN_PAGE_KEY));
-  stream_prefs_apply(gtk_object_get_data(GTK_OBJECT(parent_w), E_STREAM_PAGE_KEY));
-  gui_prefs_apply(gtk_object_get_data(GTK_OBJECT(parent_w), E_GUI_PAGE_KEY));
+  printer_prefs_apply(OBJECT_GET_DATA(parent_w, E_PRINT_PAGE_KEY));
+  column_prefs_apply(OBJECT_GET_DATA(parent_w, E_COLUMN_PAGE_KEY));
+  stream_prefs_apply(OBJECT_GET_DATA(parent_w, E_STREAM_PAGE_KEY));
+  gui_prefs_apply(OBJECT_GET_DATA(parent_w, E_GUI_PAGE_KEY));
 #ifdef HAVE_LIBPCAP
 #ifdef _WIN32
   /* Is WPcap loaded? */
   if (has_wpcap) {
 #endif /* _WIN32 */
-  capture_prefs_apply(gtk_object_get_data(GTK_OBJECT(parent_w), E_CAPTURE_PAGE_KEY));
+  capture_prefs_apply(OBJECT_GET_DATA(parent_w, E_CAPTURE_PAGE_KEY));
 #ifdef _WIN32
   }
 #endif /* _WIN32 */
 #endif /* HAVE_LIBPCAP */
-  nameres_prefs_apply(gtk_object_get_data(GTK_OBJECT(parent_w), E_NAMERES_PAGE_KEY));
+  nameres_prefs_apply(OBJECT_GET_DATA(parent_w, E_NAMERES_PAGE_KEY));
   prefs_apply_all();
 
   if (must_redissect) {
@@ -1017,13 +1150,9 @@ pref_revert(pref_t *pref, gpointer user_data)
     break;
 
   case PREF_STRING:
-    if (*pref->varp.string != pref->saved_val.string &&
-	(*pref->varp.string == NULL ||
-	 pref->saved_val.string == NULL ||
-	 strcmp(*pref->varp.string, pref->saved_val.string) != 0)) {
+    if (strcmp(*pref->varp.string, pref->saved_val.string) != 0) {
       *pref_changed_p = TRUE;
-      if (*pref->varp.string != NULL)
-        g_free(*pref->varp.string);
+      g_free(*pref->varp.string);
       *pref->varp.string = g_strdup(pref->saved_val.string);
     }
     break;
@@ -1063,14 +1192,14 @@ prefs_main_cancel_cb(GtkWidget *cancel_bt _U_, gpointer parent_w)
   copy_prefs(&prefs, &saved_prefs);
 
   /* Now revert the registered preferences. */
-  prefs_module_foreach(module_prefs_revert, &must_redissect);
+  prefs_modules_foreach(module_prefs_revert, &must_redissect);
 
   /* Now apply the reverted-to preferences. */
-  printer_prefs_apply(gtk_object_get_data(GTK_OBJECT(parent_w), E_PRINT_PAGE_KEY));
-  column_prefs_apply(gtk_object_get_data(GTK_OBJECT(parent_w), E_COLUMN_PAGE_KEY));
-  stream_prefs_apply(gtk_object_get_data(GTK_OBJECT(parent_w), E_STREAM_PAGE_KEY));
-  gui_prefs_apply(gtk_object_get_data(GTK_OBJECT(parent_w), E_GUI_PAGE_KEY));
-  nameres_prefs_apply(gtk_object_get_data(GTK_OBJECT(parent_w), E_NAMERES_PAGE_KEY));
+  printer_prefs_apply(OBJECT_GET_DATA(parent_w, E_PRINT_PAGE_KEY));
+  column_prefs_apply(OBJECT_GET_DATA(parent_w, E_COLUMN_PAGE_KEY));
+  stream_prefs_apply(OBJECT_GET_DATA(parent_w, E_STREAM_PAGE_KEY));
+  gui_prefs_apply(OBJECT_GET_DATA(parent_w, E_GUI_PAGE_KEY));
+  nameres_prefs_apply(OBJECT_GET_DATA(parent_w, E_NAMERES_PAGE_KEY));
   prefs_apply_all();
 
   gtk_widget_destroy(GTK_WIDGET(parent_w));
@@ -1096,35 +1225,36 @@ static void
 prefs_main_destroy_cb(GtkWidget *win _U_, gpointer user_data _U_)
 {
   /* Let the preference tabs clean up anything they've done. */
-  printer_prefs_destroy(gtk_object_get_data(GTK_OBJECT(prefs_w), E_PRINT_PAGE_KEY));
-  column_prefs_destroy(gtk_object_get_data(GTK_OBJECT(prefs_w), E_COLUMN_PAGE_KEY));
-  stream_prefs_destroy(gtk_object_get_data(GTK_OBJECT(prefs_w), E_STREAM_PAGE_KEY));
-  gui_prefs_destroy(gtk_object_get_data(GTK_OBJECT(prefs_w), E_GUI_PAGE_KEY));
+  printer_prefs_destroy(OBJECT_GET_DATA(prefs_w, E_PRINT_PAGE_KEY));
+  column_prefs_destroy(OBJECT_GET_DATA(prefs_w, E_COLUMN_PAGE_KEY));
+  stream_prefs_destroy(OBJECT_GET_DATA(prefs_w, E_STREAM_PAGE_KEY));
+  gui_prefs_destroy(OBJECT_GET_DATA(prefs_w, E_GUI_PAGE_KEY));
 #ifdef HAVE_LIBPCAP
 #ifdef _WIN32
   /* Is WPcap loaded? */
   if (has_wpcap) {
 #endif /* _WIN32 */
-  capture_prefs_destroy(gtk_object_get_data(GTK_OBJECT(prefs_w), E_CAPTURE_PAGE_KEY));
+  capture_prefs_destroy(OBJECT_GET_DATA(prefs_w, E_CAPTURE_PAGE_KEY));
 #ifdef _WIN32
   }
 #endif /* _WIN32 */
 #endif /* HAVE_LIBPCAP */
-  nameres_prefs_destroy(gtk_object_get_data(GTK_OBJECT(prefs_w), E_NAMERES_PAGE_KEY));
+  nameres_prefs_destroy(OBJECT_GET_DATA(prefs_w, E_NAMERES_PAGE_KEY));
 
   /* Free up the saved preferences (both for "prefs" and for registered
      preferences). */
   free_prefs(&saved_prefs);
-  prefs_module_foreach(module_prefs_clean, NULL);
+  prefs_modules_foreach(module_prefs_clean, NULL);
 
   /* Note that we no longer have a "Preferences" dialog box. */
   prefs_w = NULL;
+  first_proto_prefs_page = -1;
 }
 
 struct properties_data {
   GtkWidget *w;
   int page_num;
-  char *title;
+  const char *title;
 };
 
 /* XXX this way of searching the correct page number is really ugly ... */
@@ -1144,25 +1274,25 @@ module_search_properties(module_t *module, gpointer user_data)
 }
 
 void
-properties_cb(GtkWidget *w, gpointer dummy) 
+properties_cb(GtkWidget *w, gpointer dummy)
 {
-  gchar *title = NULL;
+  const gchar *title = NULL;
   struct properties_data p;
 
   if (finfo_selected) {
     header_field_info *hfinfo = finfo_selected->hfinfo;
     if (hfinfo->parent == -1) {
-      title = (gchar *)prefs_get_title_by_name(hfinfo->abbrev);
+      title = prefs_get_title_by_name(hfinfo->abbrev);
     } else {
-      title = (gchar *)
+      title =
 	prefs_get_title_by_name(proto_registrar_get_abbrev(hfinfo->parent));
     }
   } else {
     return;
   }
-  
+
   if (!title) return;
-  
+
   if (prefs_w != NULL) {
     reactivate_window(prefs_w);
   } else {
@@ -1170,20 +1300,41 @@ properties_cb(GtkWidget *w, gpointer dummy)
   }
 
   p.w = notebook;
-  p.page_num = FIRST_PROTO_PREFS_PAGE;
+  p.page_num = first_proto_prefs_page;
   p.title = title;
 
-  prefs_module_foreach(module_search_properties, &p);
-
+  prefs_module_list_foreach(protocols_module->prefs, module_search_properties,
+      &p);
 }
 
-/* Prefs tree selection callback.  The node data has been loaded with 
+/* Prefs tree selection callback.  The node data has been loaded with
    the proper notebook page to load. */
+#if GTK_MAJOR_VERSION < 2
 static void
-prefs_tree_select_cb(GtkCTree *ct, GtkCTreeNode *node, gint col _U_, gpointer dummy _U_)
+prefs_tree_select_cb(GtkCTree *ct, GtkCTreeNode *node, gint col _U_,
+                     gpointer dummy _U_)
+#else
+static void
+prefs_tree_select_cb(GtkTreeSelection *sel, gpointer dummy _U_)
+#endif
 {
-  gint page = GPOINTER_TO_INT(gtk_ctree_node_get_row_data(ct, node));
-  
-  if (page >= 0) 
+  gint page;
+#if GTK_MAJOR_VERSION >= 2
+  GtkTreeModel *model;
+  GtkTreeIter   iter;
+#endif
+
+#if GTK_MAJOR_VERSION < 2
+  page = GPOINTER_TO_INT(gtk_ctree_node_get_row_data(ct, node));
+
+  if (page >= 0)
     gtk_notebook_set_page(GTK_NOTEBOOK(notebook), page);
+#else
+  if (gtk_tree_selection_get_selected(sel, &model, &iter))
+  {
+    gtk_tree_model_get(model, &iter, 1, &page, -1);
+    if (page >= 0)
+      gtk_notebook_set_page(GTK_NOTEBOOK(notebook), page);
+  }
+#endif
 }
