@@ -2,7 +2,7 @@
  * Routines for AIM Instant Messenger (OSCAR) dissection
  * Copyright 2000, Ralf Hoelzer <ralf@well.com>
  *
- * $Id: packet-aim.c,v 1.21 2003/01/20 07:39:25 guy Exp $
+ * $Id: packet-aim.c,v 1.27 2003/04/30 02:35:18 gerald Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -45,6 +45,12 @@
 
 #define STRIP_TAGS 1
 
+typedef struct _aim_tlv {
+  guint16 valueid;
+  char *desc;
+  int datatype;
+} aim_tlv;
+
 /* channels */
 #define CHANNEL_NEW_CONN    0x01
 #define CHANNEL_SNAC_DATA   0x02
@@ -396,6 +402,25 @@ static const value_string aim_fnac_family_ssi[] = {
   { 0, NULL }
 };
 
+#define FAMILY_SSI_TYPE_BUDDY         0x0000
+#define FAMILY_SSI_TYPE_GROUP         0x0001
+#define FAMILY_SSI_TYPE_PERMIT        0x0002
+#define FAMILY_SSI_TYPE_DENY          0x0003
+#define FAMILY_SSI_TYPE_PDINFO        0x0004
+#define FAMILY_SSI_TYPE_PRESENCEPREFS 0x0005
+#define FAMILY_SSI_TYPE_ICONINFO      0x0014
+
+static const value_string aim_fnac_family_ssi_types[] = {
+  { FAMILY_SSI_TYPE_BUDDY, "Buddy" },
+  { FAMILY_SSI_TYPE_GROUP, "Group" },
+  { FAMILY_SSI_TYPE_PERMIT, "Permit" },
+  { FAMILY_SSI_TYPE_DENY, "Deny" },
+  { FAMILY_SSI_TYPE_PDINFO, "PDINFO" },
+  { FAMILY_SSI_TYPE_PRESENCEPREFS, "Presence Preferences" },
+  { FAMILY_SSI_TYPE_ICONINFO, "Icon Info" },
+  { 0, NULL }
+};
+
 static const value_string aim_fnac_family_icq[] = {
   { FAMILY_ICQ_ERROR, "Error" },
   { FAMILY_ICQ_LOGINREQUEST, "Login Request" },
@@ -405,6 +430,120 @@ static const value_string aim_fnac_family_icq[] = {
   { 0, NULL }
 };
 
+#define SIGNON_SCREENNAME     0x0001
+#define SIGNON_PASSWORD       0x0025
+#define SIGNON_CLIENTSTRING   0x0003
+#define SIGNON_CLIENTMAJOR    0x0017
+#define SIGNON_CLIENTMINOR    0x0018
+#define SIGNON_CLIENTPOINT    0x0019
+#define SIGNON_CLIENTBUILD    0x001a
+#define SIGNON_CLIENTCOUNTRY  0x000e
+#define SIGNON_CLIENTLANGUAGE 0x000f
+#define SIGNON_CLIENTUSESSI   0x004a
+
+static const aim_tlv aim_signon_signon_tlv[] = {
+  { SIGNON_SCREENNAME, "Screen Name", FT_STRING },
+  { SIGNON_PASSWORD, "Signon Challenge Response", FT_BYTES },
+  { SIGNON_CLIENTSTRING, "Login Request", FT_STRING },
+  { SIGNON_CLIENTMAJOR, "Client Major Version", FT_UINT16 },
+  { SIGNON_CLIENTMINOR, "Client Minor Version", FT_UINT16 },
+  { SIGNON_CLIENTPOINT, "Client Point", FT_UINT16 },
+  { SIGNON_CLIENTBUILD, "Client Build", FT_UINT16 },
+  { SIGNON_CLIENTCOUNTRY, "Client Country", FT_STRING },
+  { SIGNON_CLIENTLANGUAGE, "Client Language", FT_STRING },
+  { SIGNON_CLIENTUSESSI, "Use SSI", FT_UINT8 },
+  { 0, "Unknown", 0 }
+};
+
+#define SIGNON_LOGON_REPLY_SCREENNAME          0x0001
+#define SIGNON_LOGON_REPLY_ERRORURL            0x0004
+#define SIGNON_LOGON_REPLY_BOSADDR             0x0005
+#define SIGNON_LOGON_REPLY_AUTHCOOKIE          0x0006
+#define SIGNON_LOGON_REPLY_ERRORCODE           0x0008
+#define SIGNON_LOGON_REPLY_EMAILADDR           0x0011
+#define SIGNON_LOGON_REPLY_REGSTATUS           0x0013
+#define SIGNON_LOGON_REPLY_LATESTBETABUILD     0x0040
+#define SIGNON_LOGON_REPLY_LATESTBETAURL       0x0041
+#define SIGNON_LOGON_REPLY_LATESTBETAINFO      0x0042
+#define SIGNON_LOGON_REPLY_LATESTBETANAME      0x0043
+#define SIGNON_LOGON_REPLY_LATESTRELEASEBUILD  0x0044
+#define SIGNON_LOGON_REPLY_LATESTRELEASEURL    0x0045
+#define SIGNON_LOGON_REPLY_LATESTRELEASEINFO   0x0046
+#define SIGNON_LOGON_REPLY_LATESTRELEASENAME   0x0047
+
+static const aim_tlv aim_signon_logon_reply_tlv[] = {
+  { SIGNON_LOGON_REPLY_SCREENNAME, "Screen Name", FT_STRING },
+  { SIGNON_LOGON_REPLY_ERRORURL, "Error URL", FT_STRING },
+  { SIGNON_LOGON_REPLY_BOSADDR, "BOS Server Address", FT_STRING },
+  { SIGNON_LOGON_REPLY_AUTHCOOKIE, "Authorization Cookie", FT_BYTES },
+  { SIGNON_LOGON_REPLY_ERRORCODE, "Error Code", FT_UINT16 },
+  { SIGNON_LOGON_REPLY_EMAILADDR, "Account Email Address", FT_STRING },
+  { SIGNON_LOGON_REPLY_REGSTATUS, "Registration Status", FT_UINT16 },
+  { SIGNON_LOGON_REPLY_LATESTBETABUILD, "Latest Beta Build", FT_UINT32 },
+  { SIGNON_LOGON_REPLY_LATESTBETAURL, "Latest Beta URL", FT_STRING },
+  { SIGNON_LOGON_REPLY_LATESTBETAINFO, "Latest Beta Info", FT_STRING },
+  { SIGNON_LOGON_REPLY_LATESTBETANAME, "Latest Beta Name", FT_STRING },
+  { SIGNON_LOGON_REPLY_LATESTRELEASEBUILD, "Latest Release Build", FT_UINT32 },
+  { SIGNON_LOGON_REPLY_LATESTRELEASEURL, "Latest Release URL", FT_STRING },
+  { SIGNON_LOGON_REPLY_LATESTRELEASEINFO, "Latest Release Info", FT_STRING },
+  { SIGNON_LOGON_REPLY_LATESTRELEASENAME, "Latest Release Name", FT_STRING },
+  { 0, "Unknown", 0 }
+};
+
+#define FAMILY_BUDDYLIST_USERFLAGS      0x0001
+#define FAMILY_BUDDYLIST_MEMBERSINCE    0x0002
+#define FAMILY_BUDDYLIST_ONSINCE        0x0003
+#define FAMILY_BUDDYLIST_IDLETIME       0x0004
+#define FAMILY_BUDDYLIST_ICQSTATUS      0x0006
+#define FAMILY_BUDDYLIST_ICQIPADDR      0x000a
+#define FAMILY_BUDDYLIST_ICQSTUFF       0x000c
+#define FAMILY_BUDDYLIST_CAPINFO        0x000d
+#define FAMILY_BUDDYLIST_UNKNOWN        0x000e
+#define FAMILY_BUDDYLIST_SESSIONLEN     0x000f
+#define FAMILY_BUDDYLIST_ICQSESSIONLEN  0x0010
+
+static const aim_tlv aim_fnac_family_buddylist_oncoming_tlv[] = {
+  { FAMILY_BUDDYLIST_USERFLAGS, "User flags", FT_UINT16 },
+  { FAMILY_BUDDYLIST_MEMBERSINCE, "Member since date", FT_UINT32 },
+  { FAMILY_BUDDYLIST_ONSINCE, "Online since", FT_UINT32 },
+  { FAMILY_BUDDYLIST_IDLETIME, "Idle time (sec)", FT_UINT16 },
+  { FAMILY_BUDDYLIST_ICQSTATUS, "ICQ Online status", FT_UINT16 },
+  { FAMILY_BUDDYLIST_ICQIPADDR, "ICQ User IP Address", FT_UINT16 },
+  { FAMILY_BUDDYLIST_ICQSTUFF, "ICQ Info", FT_BYTES },
+  { FAMILY_BUDDYLIST_CAPINFO, "Capability Info", FT_BYTES },
+  { FAMILY_BUDDYLIST_UNKNOWN, "Unknown", FT_UINT16 },
+  { FAMILY_BUDDYLIST_SESSIONLEN, "Session Length (sec)", FT_UINT32 },
+  { FAMILY_BUDDYLIST_SESSIONLEN, "ICQ Session Length (sec)", FT_UINT32 },
+  { 0, "Unknown", 0 }
+};
+
+
+#define FAMILY_LOCATION_USERINFO_INFOENCODING  0x0001
+#define FAMILY_LOCATION_USERINFO_INFOMSG       0x0002
+#define FAMILY_LOCATION_USERINFO_AWAYENCODING  0x0003
+#define FAMILY_LOCATION_USERINFO_AWAYMSG       0x0004
+#define FAMILY_LOCATION_USERINFO_CAPS          0x0005
+
+static const aim_tlv aim_fnac_family_location_userinfo_tlv[] = {
+  { FAMILY_LOCATION_USERINFO_INFOENCODING, "Info Msg Encoding", FT_STRING },
+  { FAMILY_LOCATION_USERINFO_INFOMSG, "Info Message", FT_STRING },
+  { FAMILY_LOCATION_USERINFO_AWAYENCODING, "Away Msg Encoding", FT_STRING },
+  { FAMILY_LOCATION_USERINFO_AWAYMSG, "Away Message", FT_STRING },
+  { FAMILY_LOCATION_USERINFO_CAPS, "Capabilities", FT_BYTES },
+  { 0, "Unknown", 0 }
+};
+
+#define FAMILY_LOCATION_USERINFO_INFOTYPE_GENERALINFO  0x0001
+#define FAMILY_LOCATION_USERINFO_INFOTYPE_AWAYMSG      0x0003
+#define FAMILY_LOCATION_USERINFO_INFOTYPE_CAPS         0x0005
+
+static const value_string aim_snac_location_request_user_info_infotypes[] = {
+  { FAMILY_LOCATION_USERINFO_INFOTYPE_GENERALINFO, "Request General Info" },
+  { FAMILY_LOCATION_USERINFO_INFOTYPE_AWAYMSG, "Request Away Message" },
+  { FAMILY_LOCATION_USERINFO_INFOTYPE_CAPS, "Request Capabilities" },
+  { 0, NULL }
+};
+
 static int dissect_aim(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree);
 static guint get_aim_pdu_len(tvbuff_t *tvb, int offset);
 static void dissect_aim_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree);
@@ -433,7 +572,7 @@ static void dissect_aim_snac_location(tvbuff_t *tvb, packet_info *pinfo,
 				      int offset, proto_tree *tree, 
 				      guint16 subtype);
 static void dissect_aim_snac_location_request_user_information(tvbuff_t *tvb, int offset, proto_tree *tree);
-static void dissect_aim_snac_location_user_information(tvbuff_t *tvb, int offset, proto_tree *tree);
+static void dissect_aim_snac_location_user_information(tvbuff_t *tvb, packet_info *pinfo, int offset, proto_tree *tree);
 static void dissect_aim_snac_adverts(tvbuff_t *tvb, packet_info *pinfo, 
 				     int offset, proto_tree *tree, 
 				     guint16 subtype);
@@ -449,12 +588,17 @@ static void dissect_aim_snac_messaging(tvbuff_t *tvb, packet_info *pinfo,
 static void dissect_aim_snac_ssi(tvbuff_t *tvb, packet_info *pinfo, 
 				 int offset, proto_tree *tree, 
 				 guint16 subtype);
+static void dissect_aim_snac_ssi_list(tvbuff_t *tvb, packet_info *pinfo _U_, 
+				      int offset, proto_tree *tree, 
+				      guint16 subtype _U_);
 static void dissect_aim_flap_err(tvbuff_t *tvb, packet_info *pinfo, 
 				 int offset, proto_tree *tree);
 static void dissect_aim_close_conn(tvbuff_t *tvb, packet_info *pinfo, 
 				   int offset, proto_tree *tree);
 static void dissect_aim_unknown_channel(tvbuff_t *tvb, packet_info *pinfo, 
 					int offset, proto_tree *tree);
+static int dissect_aim_tlv(tvbuff_t *tvb, packet_info *pinfo, 
+			   int offset, proto_tree *tree, const aim_tlv *tlv);
 
 /* Initialize the protocol and registered fields */
 static int proto_aim = -1;
@@ -483,10 +627,20 @@ static int hf_aim_fnac_subtype_translate = -1;
 static int hf_aim_fnac_subtype_chatnav = -1;
 static int hf_aim_fnac_subtype_chat = -1;
 static int hf_aim_fnac_subtype_ssi = -1;
+static int hf_aim_fnac_subtype_ssi_version = -1;
+static int hf_aim_fnac_subtype_ssi_numitems = -1;
+static int hf_aim_fnac_subtype_ssi_buddyname_len = -1;
+static int hf_aim_fnac_subtype_ssi_buddyname = -1;
+static int hf_aim_fnac_subtype_ssi_gid = -1;
+static int hf_aim_fnac_subtype_ssi_bid = -1;
+static int hf_aim_fnac_subtype_ssi_type = -1;
+static int hf_aim_fnac_subtype_ssi_tlvlen = -1;
+static int hf_aim_fnac_subtype_ssi_data = -1;
 static int hf_aim_fnac_subtype_icq = -1;
 static int hf_aim_fnac_flags = -1;
 static int hf_aim_fnac_id = -1;
 static int hf_aim_infotype = -1;
+static int hf_aim_snac_location_request_user_info_infotype = -1;
 static int hf_aim_buddyname_len = -1;
 static int hf_aim_buddyname = -1;
 static int hf_aim_userinfo_warninglevel = -1;
@@ -495,6 +649,8 @@ static int hf_aim_userinfo_tlvcount = -1;
 /* Initialize the subtree pointers */
 static gint ett_aim          = -1;
 static gint ett_aim_fnac     = -1;
+static gint ett_aim_tlv      = -1;
+static gint ett_aim_ssi      = -1;
 
 /* desegmentation of AIM over TCP */
 static gboolean aim_desegment = TRUE;
@@ -604,7 +760,6 @@ static int get_buddyname( char *name, tvbuff_t *tvb, int len_offset, int name_of
   buddyname_length = tvb_get_guint8(tvb, len_offset);
 
   if(buddyname_length > MAX_BUDDYNAME_LENGTH ) buddyname_length = MAX_BUDDYNAME_LENGTH;
-  memset( name, '\0', sizeof(name));
   tvb_get_nstringz0(tvb, name_offset, buddyname_length, name);
 
   return buddyname_length;
@@ -746,7 +901,7 @@ static void dissect_aim_snac(tvbuff_t *tvb, packet_info *pinfo,
   if (check_col(pinfo->cinfo, COL_INFO))
     col_append_fstr(pinfo->cinfo, COL_INFO, ", Family: %s",
 		    val_to_str(family, aim_fnac_family_ids,
-			       "Unknown Family ID"));
+			       "Unknown Family ID=0x%04x"));
   
   switch(family)
     {
@@ -804,29 +959,9 @@ static void dissect_aim_snac_signon(tvbuff_t *tvb, packet_info *pinfo,
 static void dissect_aim_snac_signon_logon(tvbuff_t *tvb, packet_info *pinfo, 
 					  int offset, proto_tree *tree)
 {
-  guint8 buddyname_length = 0;
-  char buddyname[MAX_BUDDYNAME_LENGTH];
-
-  /* Info Type */
-  proto_tree_add_item(tree, hf_aim_infotype, tvb, offset, 2, FALSE);
-  offset += 2;
-
-  /* Unknown */
-  offset += 1;
-
-  /* Buddy Name */
-  buddyname_length = get_buddyname( buddyname, tvb, offset, offset + 1 );
-  
-  if (check_col(pinfo->cinfo, COL_INFO)) {
-    col_append_fstr(pinfo->cinfo, COL_INFO, ", Logon, Username: %s", 
-		    buddyname);
+  while (tvb_length_remaining(tvb, offset) > 0) {
+    offset = dissect_aim_tlv(tvb, pinfo, offset, tree, aim_signon_signon_tlv);
   }
-  
-  if(tree) {
-    proto_tree_add_text(tree, tvb, offset + 1, buddyname_length, 
-			"Screen Name: %s", buddyname);
-  }
-  offset += buddyname_length + 1;
 }
 
 static void dissect_aim_snac_signon_logon_reply(tvbuff_t *tvb, 
@@ -836,9 +971,10 @@ static void dissect_aim_snac_signon_logon_reply(tvbuff_t *tvb,
     if (check_col(pinfo->cinfo, COL_INFO)) 
       col_append_fstr(pinfo->cinfo, COL_INFO, ", Login information reply");
 
-    /* Show the undissected payload */
-    if (tvb_length_remaining(tvb, offset) > 0)
-      proto_tree_add_item(tree, hf_aim_data, tvb, offset, -1, FALSE);
+    while (tvb_length_remaining(tvb, offset) > 0) {
+      offset = dissect_aim_tlv(tvb, pinfo, offset, tree, 
+			       aim_signon_logon_reply_tlv);
+    }
 }
 
 static void dissect_aim_snac_signon_signon(tvbuff_t *tvb, packet_info *pinfo, 
@@ -1048,6 +1184,11 @@ static void dissect_aim_snac_buddylist(tvbuff_t *tvb, packet_info *pinfo,
 			  2, FALSE);
       offset += 2;
 
+      while (tvb_length_remaining(tvb, offset) > 0) {
+	offset = dissect_aim_tlv(tvb, pinfo, offset, tree, 
+				 aim_fnac_family_buddylist_oncoming_tlv);
+      }
+
       break;
       
     case FAMILY_BUDDYLIST_OFFGOING:
@@ -1113,7 +1254,7 @@ static void dissect_aim_snac_location(tvbuff_t *tvb, packet_info *pinfo,
     case FAMILY_LOCATION_USERINFO:
       if (check_col(pinfo->cinfo, COL_INFO)) 
 	col_add_fstr(pinfo->cinfo, COL_INFO, "User Information");
-      dissect_aim_snac_location_user_information(tvb, offset, tree);
+      dissect_aim_snac_location_user_information(tvb, pinfo, offset, tree);
       break;
     case FAMILY_LOCATION_WATCHERSUBREQ:
       if (check_col(pinfo->cinfo, COL_INFO)) 
@@ -1137,7 +1278,8 @@ static void dissect_aim_snac_location_request_user_information(tvbuff_t *tvb,
   guint8 buddyname_length = 0;
 
   /* Info Type */
-  proto_tree_add_item(tree, hf_aim_infotype, tvb, offset, 2, FALSE);
+  proto_tree_add_item(tree, hf_aim_snac_location_request_user_info_infotype, 
+		      tvb, offset, 2, FALSE);
   offset += 2;
 
   /* Buddy Name length */
@@ -1155,10 +1297,12 @@ static void dissect_aim_snac_location_request_user_information(tvbuff_t *tvb,
 }
 
 static void dissect_aim_snac_location_user_information(tvbuff_t *tvb, 
+						       packet_info *pinfo _U_, 
 						  int offset, proto_tree *tree)
 {
   guint8 buddyname_length = 0;
   guint16 tlv_count = 0;
+  guint16 i = 0;
 
   /* Buddy Name length */
   buddyname_length = tvb_get_guint8(tvb, offset);
@@ -1178,9 +1322,18 @@ static void dissect_aim_snac_location_user_information(tvbuff_t *tvb,
   proto_tree_add_item(tree, hf_aim_userinfo_tlvcount, tvb, offset, 2, FALSE);
   offset += 2;
 
-  /* Show the undissected payload */
-  if (tvb_length_remaining(tvb, offset) > 0)
-    proto_tree_add_item(tree, hf_aim_data, tvb, offset, -1, FALSE);
+  /* Dissect the TLV array containing general user status  */
+  while (i++ < tlv_count) {
+    offset = dissect_aim_tlv(tvb, pinfo, offset, tree, 
+			     aim_fnac_family_buddylist_oncoming_tlv);
+  }
+
+  /* Dissect the TLV array containing the away message (or whatever info was
+     specifically requested) */
+  while (tvb_length_remaining(tvb, offset) > 0) {
+    offset = dissect_aim_tlv(tvb, pinfo, offset, tree, 
+			     aim_fnac_family_location_userinfo_tlv);
+  }
 }
 
 static void dissect_aim_snac_adverts(tvbuff_t *tvb _U_, 
@@ -1343,9 +1496,84 @@ static void dissect_aim_snac_ssi(tvbuff_t *tvb, packet_info *pinfo _U_,
 				 int offset, proto_tree *tree, 
 				 guint16 subtype _U_)
 {
-  /* Show the undissected payload */
-  if (tvb_length_remaining(tvb, offset) > 0)
-    proto_tree_add_item(tree, hf_aim_data, tvb, offset, -1, FALSE);
+  switch(subtype)
+    {    
+    case FAMILY_SSI_LIST:
+      dissect_aim_snac_ssi_list(tvb, pinfo, offset, tree, subtype);
+      break;
+    default:
+      /* Show the undissected payload */
+      if (tvb_length_remaining(tvb, offset) > 0)
+	proto_tree_add_item(tree, hf_aim_data, tvb, offset, -1, FALSE);
+    }
+}
+
+static void dissect_aim_snac_ssi_list(tvbuff_t *tvb, packet_info *pinfo _U_, 
+				      int offset, proto_tree *tree, 
+				      guint16 subtype _U_)
+{
+  guint16 buddyname_length = 0;
+  guint16 tlv_len = 0;
+  proto_item *ti;
+  proto_tree *ssi_entry = NULL;
+
+  /* SSI Version */
+  proto_tree_add_item(tree, hf_aim_fnac_subtype_ssi_version, tvb, offset, 1,
+		      FALSE);
+  offset += 1;
+  
+  /* Number of items */
+  proto_tree_add_item(tree, hf_aim_fnac_subtype_ssi_numitems, tvb, offset, 2,
+		      FALSE);
+  offset += 2;
+  
+  while (tvb_length_remaining(tvb, offset) > 4) {
+    ti = proto_tree_add_text(tree, tvb, offset, 0, "SSI Entry");
+    ssi_entry = proto_item_add_subtree(ti, ett_aim_ssi);
+    
+    /* Buddy Name Length */
+    buddyname_length = tvb_get_ntohs(tvb, offset);
+    proto_tree_add_item(ssi_entry, hf_aim_fnac_subtype_ssi_buddyname_len, 
+			tvb, offset, 2, FALSE);
+    offset += 2;
+    
+    /* Buddy Name */
+    if (buddyname_length > 0) {
+      proto_tree_add_item(ssi_entry, hf_aim_fnac_subtype_ssi_buddyname, tvb, 
+			  offset, buddyname_length, FALSE);
+      offset += buddyname_length;
+    }
+    
+    /* Buddy group ID */
+    proto_tree_add_item(ssi_entry, hf_aim_fnac_subtype_ssi_gid, tvb, offset, 
+			2, FALSE);
+    offset += 2;
+    
+    /* Buddy ID */
+    proto_tree_add_item(ssi_entry, hf_aim_fnac_subtype_ssi_bid, tvb, offset, 
+			2, FALSE);
+    offset += 2;
+    
+    /* Buddy Type */
+    proto_tree_add_item(ssi_entry, hf_aim_fnac_subtype_ssi_type, tvb, offset,
+			2, FALSE);
+    offset += 2;
+    
+    /* Size of the following TLV in bytes (as opposed to the number of 
+       TLV objects in the chain) */
+    tlv_len = tvb_get_ntohs(tvb, offset);
+    proto_tree_add_item(ssi_entry, hf_aim_fnac_subtype_ssi_tlvlen, tvb, 
+			offset, 2, FALSE);
+    offset += 2;
+    
+    /* For now, we just dump the TLV contents as-is, since there is not a
+       TLV dissection utility that works based on total chain length */
+    if (tlv_len > 0) {
+      proto_tree_add_item(ssi_entry, hf_aim_data, tvb, offset, tlv_len, 
+			  FALSE);
+      offset += tlv_len;
+    }
+  }
 }
 
 static void dissect_aim_snac_fnac_subtype(tvbuff_t *tvb, int offset, 
@@ -1470,6 +1698,93 @@ static void dissect_aim_unknown_channel(tvbuff_t *tvb, packet_info *pinfo,
     proto_tree_add_item(tree, hf_aim_data, tvb, offset, -1, FALSE);
 }
 
+/* Dissect a TLV value */
+static int dissect_aim_tlv(tvbuff_t *tvb, packet_info *pinfo _U_, 
+			   int offset, proto_tree *tree, const aim_tlv *tlv)
+{
+  guint16 valueid;
+  guint16 length;
+  int i = 0;
+  const aim_tlv *tmp;
+  proto_item *ti1;
+  proto_tree *tlv_tree;
+  int orig_offset;
+  guint16 value16;
+  guint32 value32;
+
+  /* Record the starting offset so we can reuse it at the second pass */
+  orig_offset = offset;
+
+  /* Get the value ID */
+  valueid = tvb_get_ntohs(tvb, offset);
+  offset += 2;
+
+  /* Figure out which entry applies from the tlv list */
+  tmp = tlv;
+  while (tmp[i].valueid) {
+    if (tmp[i].valueid == valueid) {
+      /* We found a match */
+      break;
+    }
+    i++;
+  }
+
+  /* At this point, we are either pointing at the correct record, or 
+     we didn't find the record, and are pointing at the last item in the 
+     list */
+
+  length = tvb_get_ntohs(tvb, offset);
+  offset += 2;
+  offset += length;
+
+  if (tree) {
+    offset = orig_offset;
+    
+    /* Show the info in the top of the tree if it's one of the standard
+       data types */
+    if (tmp[i].datatype == FT_STRING) {
+      guint8 *buf;      
+      buf = g_malloc(length);
+      tvb_get_nstringz0(tvb, offset + 4, length, buf);
+      ti1 = proto_tree_add_text(tree, tvb, offset, length + 4, 
+				"%s: %s", tmp[i].desc, buf);
+      g_free(buf);
+    }
+    else if (tmp[i].datatype == FT_UINT16) {
+      value16 = tvb_get_ntohs(tvb, offset + 4);
+      ti1 = proto_tree_add_text(tree, tvb, offset, length + 4, 
+				"%s: %d", tmp[i].desc, value16);
+    }
+    else if (tmp[i].datatype == FT_UINT32) {
+      value32 = tvb_get_ntohl(tvb, offset + 4);
+      ti1 = proto_tree_add_text(tree, tvb, offset, length + 4, 
+				"%s: %d", tmp[i].desc, value32);
+    }
+    else {
+      ti1 = proto_tree_add_text(tree, tvb, offset, length + 4, 
+				"%s", tmp[i].desc);
+    }
+
+    tlv_tree = proto_item_add_subtree(ti1, ett_aim_tlv);
+
+    proto_tree_add_text(tlv_tree, tvb, offset, 2,
+			"Value ID: %s (0x%04x)", tmp[i].desc, valueid);
+    offset += 2;
+    
+    proto_tree_add_text(tlv_tree, tvb, offset, 2,
+			"Length: %d", length);
+    offset += 2;
+
+    ti1 = proto_tree_add_text(tlv_tree, tvb, offset, length,
+			      "Value");
+    offset += length;
+  }
+
+  /* Return the new length */
+  return offset;
+}
+
+
 /* Register the protocol with Ethereal */
 void
 proto_register_aim(void)
@@ -1552,6 +1867,33 @@ proto_register_aim(void)
     { &hf_aim_fnac_subtype_ssi,
       { "FNAC Subtype ID", "aim.fnac.subtype", FT_UINT16, BASE_HEX, VALS(aim_fnac_family_ssi), 0x0, "", HFILL }
     },
+    { &hf_aim_fnac_subtype_ssi_version,
+      { "SSI Version", "aim.fnac.ssi.version", FT_UINT8, BASE_HEX, NULL, 0x0, "", HFILL }
+    },
+    { &hf_aim_fnac_subtype_ssi_numitems,
+      { "SSI Object count", "aim.fnac.ssi.numitems", FT_UINT16, BASE_HEX, NULL, 0x0, "", HFILL }
+    },
+    { &hf_aim_fnac_subtype_ssi_buddyname_len,
+      { "SSI Buddy Name length", "aim.fnac.ssi.buddyname_len", FT_UINT16, BASE_HEX, NULL, 0x0, "", HFILL }
+    },
+    { &hf_aim_fnac_subtype_ssi_buddyname,
+      { "Buddy Name", "aim.fnac.ssi.buddyname", FT_STRING, BASE_NONE, NULL, 0x0, "", HFILL }
+    },
+    { &hf_aim_fnac_subtype_ssi_gid,
+      { "SSI Buddy Group ID", "aim.fnac.ssi.gid", FT_UINT16, BASE_HEX, NULL, 0x0, "", HFILL }
+    },
+    { &hf_aim_fnac_subtype_ssi_bid,
+      { "SSI Buddy ID", "aim.fnac.ssi.bid", FT_UINT16, BASE_HEX, NULL, 0x0, "", HFILL }
+    },
+    { &hf_aim_fnac_subtype_ssi_type,
+      { "SSI Buddy type", "aim.fnac.ssi.type", FT_UINT16, BASE_HEX, VALS(aim_fnac_family_ssi_types), 0x0, "", HFILL }
+    },
+    { &hf_aim_fnac_subtype_ssi_tlvlen,
+      { "SSI TLV Len", "aim.fnac.ssi.tlvlen", FT_UINT16, BASE_HEX, NULL, 0x0, "", HFILL }
+    },
+    { &hf_aim_fnac_subtype_ssi_data,
+      { "SSI Buddy Data", "aim.fnac.ssi.data", FT_UINT16, BASE_HEX, NULL, 0x0, "", HFILL }
+    },
     { &hf_aim_fnac_subtype_icq,
       { "FNAC Subtype ID", "aim.fnac.subtype", FT_UINT16, BASE_HEX, VALS(aim_fnac_family_icq), 0x0, "", HFILL }
     },
@@ -1564,6 +1906,11 @@ proto_register_aim(void)
     { &hf_aim_infotype,
       { "Infotype", "aim.infotype", FT_UINT16, BASE_HEX, NULL, 0x0, "", HFILL }
     },
+    { &hf_aim_snac_location_request_user_info_infotype,
+      { "Infotype", "aim.snac.location.request_user_info.infotype", FT_UINT16,
+	BASE_HEX, VALS(aim_snac_location_request_user_info_infotypes), 0x0,
+	"", HFILL }
+    },
     { &hf_aim_buddyname_len,
       { "Buddyname len", "aim.buddynamelen", FT_UINT8, BASE_DEC, NULL, 0x0, "", HFILL }
     },
@@ -1582,6 +1929,8 @@ proto_register_aim(void)
   static gint *ett[] = {
     &ett_aim,
     &ett_aim_fnac,
+    &ett_aim_tlv,
+    &ett_aim_ssi,
   };
   module_t *aim_module;
 
