@@ -1,8 +1,8 @@
 /* packet-clnp.c
  * Routines for ISO/OSI network and transport protocol packet disassembly
  *
- * $Id: packet-clnp.c,v 1.64 2003/01/20 05:42:30 guy Exp $
- * Laurent Deniel <deniel@worldnet.fr>
+ * $Id: packet-clnp.c,v 1.74 2003/04/22 22:03:45 deniel Exp $
+ * Laurent Deniel <laurent.deniel@free.fr>
  * Ralf Schneider <Ralf.Schneider@t-online.de>
  *
  * Ethereal - Network traffic analyzer
@@ -27,7 +27,6 @@
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
-
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
@@ -49,10 +48,6 @@ static gint ett_clnp_type      = -1;
 static gint ett_clnp_segments  = -1;
 static gint ett_clnp_segment   = -1;
 static gint ett_clnp_disc_pdu  = -1;
-static int  proto_cotp         = -1;
-static gint ett_cotp           = -1;
-static int  proto_cltp         = -1;
-static gint ett_cltp           = -1;
 
 static int hf_clnp_id          = -1;
 static int hf_clnp_length      = -1;
@@ -72,6 +67,19 @@ static int hf_clnp_segment_overlap_conflict = -1;
 static int hf_clnp_segment_multiple_tails = -1;
 static int hf_clnp_segment_too_long_segment = -1;
 static int hf_clnp_segment_error = -1;
+static int hf_clnp_reassembled_in = -1;
+
+static int  proto_cotp         = -1;
+static gint ett_cotp           = -1;
+
+static int hf_cotp_srcref = -1;
+static int hf_cotp_destref = -1;
+static int hf_cotp_type = -1;
+
+static int  proto_cltp         = -1;
+static gint ett_cltp           = -1;
+
+static int hf_cltp_type = -1;
 
 static const fragment_items clnp_frag_items = {
 	&ett_clnp_segment,
@@ -83,6 +91,7 @@ static const fragment_items clnp_frag_items = {
 	&hf_clnp_segment_multiple_tails,
 	&hf_clnp_segment_too_long_segment,
 	&hf_clnp_segment_error,
+	&hf_clnp_reassembled_in,
 	"segments"
 };
 
@@ -183,6 +192,25 @@ struct clnp_segment {
 #define CR_TPDU        		0xE	/* COTP */
 #define DT_TPDU        		0xF	/* COTP */
 
+static const value_string cotp_tpdu_type_abbrev_vals[] = {
+  { ED_TPDU,	"ED" },
+  { EA_TPDU,	"EA" },
+  { RJ_TPDU,	"RJ" },
+  { AK_TPDU,	"AK" },
+  { ER_TPDU,	"ER" },
+  { DR_TPDU,	"DR" },
+  { DC_TPDU,	"DC" },
+  { CC_TPDU,	"CC" },
+  { CR_TPDU,	"CR" },
+  { DT_TPDU,	"DT" },
+  { 0,		NULL }
+};
+
+static const value_string cltp_tpdu_type_abbrev_vals[] = {
+  { UD_TPDU,	"UD" },
+  { 0,		NULL }
+};
+
 /* field position */
 
 #define P_LI           		0
@@ -276,11 +304,16 @@ static const value_string tp_vpart_type_vals[] = {
 /* List of dissectors to call for COTP packets put atop the Inactive
    Subset of CLNP. */
 static heur_dissector_list_t cotp_is_heur_subdissector_list;
+/* List of dissectors to call for COTP packets put atop CLNP */
+static heur_dissector_list_t cotp_heur_subdissector_list;
+/* List of dissectors to call for CLNP packets */
+static heur_dissector_list_t clnp_heur_subdissector_list;
 
 /*
  * Reassembly of CLNP.
  */
 static GHashTable *clnp_segment_table = NULL;
+static GHashTable *clnp_reassembled_table = NULL;
 
 /* options */
 static guint tp_nsap_selector = NSEL_TP;
@@ -679,7 +712,9 @@ static int ositp_decode_DR(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
     return -1;
 
   dst_ref = tvb_get_ntohs(tvb, offset + P_DST_REF);
+
   src_ref = tvb_get_ntohs(tvb, offset + P_SRC_REF);
+
   reason  = tvb_get_guint8(tvb, offset + P_REASON_IN_DR);
 
   switch(reason) {
@@ -711,12 +746,10 @@ static int ositp_decode_DR(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
     cotp_tree = proto_item_add_subtree(ti, ett_cotp);
     proto_tree_add_text(cotp_tree, tvb, offset,      1,
 			"Length indicator: %u", li);
-    proto_tree_add_text(cotp_tree, tvb, offset +  1, 1,
-			"TPDU code: 0x%x (DR)", tpdu);
-    proto_tree_add_text(cotp_tree, tvb, offset +  2, 2,
-			"Destination reference: 0x%04x", dst_ref);
-    proto_tree_add_text(cotp_tree, tvb, offset +  4, 2,
-			"Source reference: 0x%04x", src_ref);
+    proto_tree_add_uint_format(cotp_tree, hf_cotp_type, tvb, offset +  1, 1, tpdu,
+			       "TPDU code: 0x%x (DR)", tpdu);
+    proto_tree_add_uint(cotp_tree, hf_cotp_destref, tvb, offset +  2, 2, dst_ref);
+    proto_tree_add_uint(cotp_tree, hf_cotp_srcref, tvb, offset +  4, 2, src_ref);
     proto_tree_add_text(cotp_tree, tvb, offset +  6, 1,
 			"Cause: %s", str);
   }
@@ -822,18 +855,15 @@ static int ositp_decode_DT(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
   offset += 1;
 
   if (tree) {
-    proto_tree_add_text(cotp_tree, tvb, offset, 1,
+    proto_tree_add_uint_format(cotp_tree, hf_cotp_type, tvb, offset, 1, tpdu, 
 			"TPDU code: 0x%x (DT)", tpdu);
-
   }
   offset += 1;
   li -= 1;
 
   if (is_class_234) {
-    if (tree) {
-      proto_tree_add_text(cotp_tree, tvb, offset, 2,
-			  "Destination reference: 0x%04x", dst_ref);
-    }
+    if (tree)
+      proto_tree_add_uint(cotp_tree, hf_cotp_destref, tvb, offset, 2, dst_ref);
     offset += 2;
     li -= 2;
   }
@@ -871,8 +901,14 @@ static int ositp_decode_DT(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
 	  /* Fill in other Dissectors using inactive subset here */
 	  call_dissector(data_handle,next_tvb, pinfo, tree);
 	}
-  } else
-	call_dissector(data_handle,next_tvb, pinfo, tree);
+  } else {
+        if (dissector_try_heuristic(cotp_heur_subdissector_list, next_tvb,
+				    pinfo, tree)) {
+		*subdissector_found = TRUE;
+	} else {
+		call_dissector(data_handle,next_tvb, pinfo, tree);
+	}
+  }
   offset += tvb_length_remaining(tvb, offset);
      /* we dissected all of the containing PDU */
 
@@ -945,16 +981,14 @@ static int ositp_decode_ED(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
   offset += 1;
 
   if (tree) {
-    proto_tree_add_text(cotp_tree, tvb, offset, 1,
+    proto_tree_add_uint_format(cotp_tree, hf_cotp_type, tvb, offset, 1, tpdu,
 			"TPDU code: 0x%x (ED)", tpdu);
   }
   offset += 1;
   li -= 1;
 
-  if (tree) {
-    proto_tree_add_text(cotp_tree, tvb, offset, 2,
-			"Destination reference: 0x%04x", dst_ref);
-  }
+  if (tree)
+    proto_tree_add_uint(cotp_tree, hf_cotp_destref, tvb, offset, 2, dst_ref);
   offset += 2;
   li -= 2;
 
@@ -1021,13 +1055,12 @@ static int ositp_decode_RJ(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
     cotp_tree = proto_item_add_subtree(ti, ett_cotp);
     proto_tree_add_text(cotp_tree, tvb, offset,      1,
 			"Length indicator: %u", li);
-    proto_tree_add_text(cotp_tree, tvb, offset +  1, 1,
+    proto_tree_add_uint_format(cotp_tree, hf_cotp_type, tvb, offset +  1, 1, tpdu,
 			"TPDU code: 0x%x (RJ)", tpdu);
     if (li == LI_NORMAL_RJ)
       proto_tree_add_text(cotp_tree, tvb, offset +  1, 1,
 			  "Credit: %u", cdt);
-    proto_tree_add_text(cotp_tree, tvb, offset +  2, 2,
-			"Destination reference: 0x%04x", dst_ref);
+    proto_tree_add_uint(cotp_tree, hf_cotp_destref, tvb, offset +  2, 2, dst_ref);
     if (li == LI_NORMAL_RJ)
       proto_tree_add_text(cotp_tree, tvb, offset +  4, 1,
 			  "Your TPDU number: 0x%02x", tpdu_nr);
@@ -1046,7 +1079,9 @@ static int ositp_decode_RJ(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
 } /* ositp_decode_RJ */
 
 static int ositp_decode_CC(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
-			 packet_info *pinfo, proto_tree *tree)
+			 packet_info *pinfo, proto_tree *tree,
+			 gboolean uses_inactive_subset,
+			 gboolean *subdissector_found)
 {
 
   /* CC & CR decoding in the same function */
@@ -1055,8 +1090,10 @@ static int ositp_decode_CC(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
   proto_item *ti;
   guint16 dst_ref, src_ref;
   guchar  class_option;
+  tvbuff_t *next_tvb;
 
   src_ref = tvb_get_ntohs(tvb, offset + P_SRC_REF);
+  
   class_option = (tvb_get_guint8(tvb, offset + P_CLASS_OPTION) >> 4 ) & 0x0F;
   if (class_option > 4)
     return -1;
@@ -1078,24 +1115,20 @@ static int ositp_decode_CC(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
   offset += 1;
 
   if (tree) {
-    proto_tree_add_text(cotp_tree, tvb, offset, 1,
+    proto_tree_add_uint_format(cotp_tree, hf_cotp_type, tvb, offset, 1, tpdu,
 			"TPDU code: 0x%x (%s)", tpdu,
 			(tpdu == CR_TPDU) ? "CR" : "CC");
   }
   offset += 1;
   li -= 1;
 
-  if (tree) {
-    proto_tree_add_text(cotp_tree, tvb, offset, 2,
-			"Destination reference: 0x%04x", dst_ref);
-  }
+  if (tree)
+    proto_tree_add_uint(cotp_tree, hf_cotp_destref, tvb, offset, 2, dst_ref);
   offset += 2;
   li -= 2;
 
-  if (tree) {
-    proto_tree_add_text(cotp_tree, tvb, offset, 2,
-			"Source reference: 0x%04x", src_ref);
-  }
+  if (tree)
+    proto_tree_add_uint(cotp_tree, hf_cotp_srcref, tvb, offset, 2, src_ref);
   offset += 2;
   li -= 2;
 
@@ -1110,8 +1143,17 @@ static int ositp_decode_CC(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
     ositp_decode_var_part(tvb, offset, li, class_option, cotp_tree);
   offset += li;
 
-  /* User data */
-  call_dissector(data_handle, tvb_new_subset(tvb, offset, -1, -1), pinfo, tree);
+  next_tvb = tvb_new_subset(tvb, offset, -1, -1);
+  if (!uses_inactive_subset){
+    if (dissector_try_heuristic(cotp_heur_subdissector_list, next_tvb,
+				pinfo, tree)) {
+      *subdissector_found = TRUE;
+    } else {
+      call_dissector(data_handle,next_tvb, pinfo, tree);
+    }
+  }
+  else
+    call_dissector(data_handle, next_tvb, pinfo, tree);
   offset += tvb_length_remaining(tvb, offset);
      /* we dissected all of the containing PDU */
 
@@ -1147,23 +1189,19 @@ static int ositp_decode_DC(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
   offset += 1;
 
   if (tree) {
-    proto_tree_add_text(cotp_tree, tvb, offset, 1,
+    proto_tree_add_uint_format(cotp_tree, hf_cotp_type, tvb, offset, 1, tpdu, 
 			"TPDU code: 0x%x (DC)", tpdu);
   }
   offset += 1;
   li -= 1;
 
-  if (tree) {
-    proto_tree_add_text(cotp_tree, tvb, offset, 2,
-			"Destination reference: 0x%04x", dst_ref);
-  }
+  if (tree)
+    proto_tree_add_uint(cotp_tree, hf_cotp_destref, tvb, offset, 2, dst_ref);
   offset += 2;
   li -= 2;
 
-  if (tree) {
-    proto_tree_add_text(cotp_tree, tvb, offset, 2,
-			"Source reference: 0x%04x", src_ref);
-  }
+  if (tree)
+    proto_tree_add_uint(cotp_tree, hf_cotp_srcref, tvb, offset, 2, src_ref);
   offset += 2;
   li -= 2;
 
@@ -1205,7 +1243,7 @@ static int ositp_decode_AK(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
     offset += 1;
 
     if (tree) {
-      proto_tree_add_text(cotp_tree, tvb, offset, 1,
+      proto_tree_add_uint_format(cotp_tree, hf_cotp_type, tvb, offset, 1, tpdu, 
 			  "TPDU code: 0x%x (AK)", tpdu);
       proto_tree_add_text(cotp_tree, tvb, offset, 1,
 			  "Credit: %u", cdt);
@@ -1213,10 +1251,8 @@ static int ositp_decode_AK(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
     offset += 1;
     li -= 1;
 
-    if (tree) {
-      proto_tree_add_text(cotp_tree, tvb, offset, 2,
-			  "Destination reference: 0x%04x", dst_ref);
-    }
+    if (tree)
+      proto_tree_add_uint(cotp_tree, hf_cotp_destref, tvb, offset, 2, dst_ref);
     offset += 2;
     li -= 2;
 
@@ -1250,16 +1286,14 @@ static int ositp_decode_AK(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
     offset += 1;
 
     if (tree) {
-      proto_tree_add_text(cotp_tree, tvb, offset, 1,
+      proto_tree_add_uint_format(cotp_tree, hf_cotp_type, tvb, offset, 1, tpdu, 
 			  "TPDU code: 0x%x (AK)", tpdu);
     }
     offset += 1;
     li -= 1;
 
-    if (tree) {
-      proto_tree_add_text(cotp_tree, tvb, offset, 2,
-			  "Destination reference: 0x%04x", dst_ref);
-    }
+    if (tree)
+      proto_tree_add_uint(cotp_tree, hf_cotp_destref, tvb, offset, 2, dst_ref);
     offset += 2;
     li -= 2;
 
@@ -1346,16 +1380,14 @@ static int ositp_decode_EA(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
   offset += 1;
 
   if (tree) {
-    proto_tree_add_text(cotp_tree, tvb, offset, 1,
+    proto_tree_add_uint_format(cotp_tree, hf_cotp_type, tvb, offset, 1, tpdu,
 			"TPDU code: 0x%x (EA)", tpdu);
   }
   offset += 1;
   li -= 1;
 
-  if (tree) {
-    proto_tree_add_text(cotp_tree, tvb, offset, 2,
-			"Destination reference: 0x%04x", dst_ref);
-  }
+  if (tree)
+    proto_tree_add_uint(cotp_tree, hf_cotp_destref, tvb, offset, 2, dst_ref);
   offset += 2;
   li -= 2;
 
@@ -1422,10 +1454,9 @@ static int ositp_decode_ER(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
     cotp_tree = proto_item_add_subtree(ti, ett_cotp);
     proto_tree_add_text(cotp_tree, tvb, offset,      1,
 			"Length indicator: %u", li);
-    proto_tree_add_text(cotp_tree, tvb, offset +  1, 1,
+    proto_tree_add_uint_format(cotp_tree, hf_cotp_type, tvb, offset +  1, 1, tpdu,
 			"TPDU code: 0x%x (ER)", tpdu);
-    proto_tree_add_text(cotp_tree, tvb, offset +  2, 2,
-			"Destination reference: 0x%04x", dst_ref);
+    proto_tree_add_uint(cotp_tree, hf_cotp_destref, tvb, offset +  2, 2, dst_ref);
     proto_tree_add_text(cotp_tree, tvb, offset +  4, 1,
 			"Reject cause: %s", str);
   }
@@ -1455,7 +1486,7 @@ static int ositp_decode_UD(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
   offset += 1;
 
   if (tree) {
-    proto_tree_add_text(cltp_tree, tvb, offset, 1,
+    proto_tree_add_uint_format(cltp_tree, hf_cltp_type, tvb, offset, 1, tpdu, 
 			"TPDU code: 0x%x (UD)", tpdu);
   }
   offset += 1;
@@ -1526,7 +1557,8 @@ static gboolean dissect_ositp_internal(tvbuff_t *tvb, packet_info *pinfo,
     switch (tpdu) {
       case CC_TPDU :
       case CR_TPDU :
-        new_offset = ositp_decode_CC(tvb, offset, li, tpdu, pinfo, tree);
+        new_offset = ositp_decode_CC(tvb, offset, li, tpdu, pinfo, tree,
+				     uses_inactive_subset, &subdissector_found);
         break;
       case DR_TPDU :
         new_offset = ositp_decode_DR(tvb, offset, li, tpdu, pinfo, tree);
@@ -1838,35 +1870,23 @@ static void dissect_clnp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
   offset = cnf_hdr_len;
 
-  /* If clnp_reassemble is on, and this is a segment, we have all the
+  /* If clnp_reassemble is on, this is a segment, we have all the
    * data in the segment, and the checksum is valid, then just add the
    * segment to the hashtable.
    */
   save_fragmented = pinfo->fragmented;
   if (clnp_reassemble && (cnf_type & CNF_SEG_OK) &&
 	((cnf_type & CNF_MORE_SEGS) || segment_offset != 0) &&
-	(tvb_reported_length(tvb) <= tvb_length(tvb)) &&
+	tvb_bytes_exist(tvb, offset, segment_length - cnf_hdr_len) &&
 	cksum_status != CKSUM_NOT_OK) {
-    fd_head = fragment_add(tvb, offset, pinfo, du_id, clnp_segment_table,
-			   segment_offset, segment_length - cnf_hdr_len,
+    fd_head = fragment_add_check(tvb, offset, pinfo, du_id, clnp_segment_table,
+			   clnp_reassembled_table, segment_offset,
+			   segment_length - cnf_hdr_len,
 			   cnf_type & CNF_MORE_SEGS);
 
     if (fd_head != NULL) {
-      /* OK, we have the complete reassembled payload.
-         Allocate a new tvbuff, referring to the reassembled payload. */
-      next_tvb = tvb_new_real_data(fd_head->data, fd_head->datalen,
-	fd_head->datalen);
-
-      /* Add the tvbuff to the list of tvbuffs to which the tvbuff we
-         were handed refers, so it'll get cleaned up when that tvbuff
-         is cleaned up. */
-      tvb_set_child_real_data_tvbuff(tvb, next_tvb);
-
-      /* Add the defragmented data to the data source list. */
-      add_new_data_source(pinfo, next_tvb, "Reassembled CLNP");
-
-      update_col_info = !show_fragment_tree(fd_head, &clnp_frag_items,
-        clnp_tree, pinfo, next_tvb);
+      next_tvb = process_reassembled_data(tvb, pinfo, "Reassembled CLNP",
+          fd_head, &clnp_frag_items, &update_col_info, clnp_tree);
     } else {
       /* We don't have the complete reassembled payload. */
       next_tvb = NULL;
@@ -1926,6 +1946,12 @@ static void dissect_clnp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
           return;	/* yes, it appears to be COTP or CLTP */
         }
       }
+      if (dissector_try_heuristic(clnp_heur_subdissector_list, next_tvb,
+				  pinfo, tree))	{
+          pinfo->fragmented = save_fragmented;
+          return;	/* yes, it appears to be COTP or CLTP */
+      }
+	
       break;
 
     case ER_NPDU:
@@ -1974,6 +2000,7 @@ static void
 clnp_reassemble_init(void)
 {
   fragment_table_init(&clnp_segment_table);
+  reassembled_table_init(&clnp_reassembled_table);
 }
 
 void proto_register_clnp(void)
@@ -2040,6 +2067,10 @@ void proto_register_clnp(void)
     { &hf_clnp_segments,
       { "CLNP Segments", "clnp.segments", FT_NONE, BASE_DEC, NULL, 0x0,
 	"CLNP Segments", HFILL }},
+
+    { &hf_clnp_reassembled_in,
+      { "Reassembled CLNP in frame", "clnp.reassembled_in", FT_FRAMENUM, BASE_NONE, NULL, 0x0,
+	"This CLNP packet is reassembled in this frame", HFILL }}
   };
   static gint *ett[] = {
     &ett_clnp,
@@ -2055,6 +2086,8 @@ void proto_register_clnp(void)
   proto_register_field_array(proto_clnp, hf, array_length(hf));
   proto_register_subtree_array(ett, array_length(ett));
   register_dissector("clnp", dissect_clnp, proto_clnp);
+  register_heur_dissector_list("clnp", &clnp_heur_subdissector_list);  
+  register_init_routine(clnp_reassemble_init);
 
   clnp_module = prefs_register_protocol(proto_clnp, NULL);
   prefs_register_uint_preference(clnp_module, "tp_nsap_selector",
@@ -2071,50 +2104,66 @@ void proto_register_clnp(void)
 	&clnp_reassemble);
 }
 
+void
+proto_reg_handoff_clnp(void)
+{
+  data_handle = find_dissector("data");
+
+  clnp_handle = create_dissector_handle(dissect_clnp, proto_clnp);
+  dissector_add("osinl", NLPID_ISO8473_CLNP, clnp_handle);
+  dissector_add("osinl", NLPID_NULL, clnp_handle); /* Inactive subset */
+  dissector_add("x.25.spi", NLPID_ISO8473_CLNP, clnp_handle);
+}
+
 void proto_register_cotp(void)
 {
-  /*        static hf_register_info hf[] = {
-                { &variable,
-                { "Name",           "cotp.abbreviation", TYPE, VALS_POINTER }},
-        };*/
-	static gint *ett[] = {
-		&ett_cotp,
-	};
+  static hf_register_info hf[] = {
+    { &hf_cotp_srcref,
+      { "Source reference", "cotp.srcref", FT_UINT16, BASE_HEX, NULL, 0x0,
+        "Source address reference", HFILL}},
+    { &hf_cotp_destref,
+      { "Destination reference", "cotp.destref", FT_UINT16, BASE_HEX, NULL, 0x0,
+        "Destination address reference", HFILL}}, 
+    { &hf_cotp_type,
+      { "COTP PDU Type", "cotp.type", FT_UINT8, BASE_HEX, VALS(cotp_tpdu_type_abbrev_vals), 0x0,
+        "COTP PDU Type", HFILL}},
+  };
+  static gint *ett[] = {
+	&ett_cotp,
+  };
 
-        proto_cotp = proto_register_protocol(PROTO_STRING_COTP, "COTP", "cotp");
- /*       proto_register_field_array(proto_cotp, hf, array_length(hf));*/
-	proto_register_subtree_array(ett, array_length(ett));
+  proto_cotp = proto_register_protocol(PROTO_STRING_COTP, "COTP", "cotp");
+  proto_register_field_array(proto_cotp, hf, array_length(hf));
+  proto_register_subtree_array(ett, array_length(ett));
 
-/* subdissector code */
-	register_heur_dissector_list("cotp_is", &cotp_is_heur_subdissector_list);
+  /* subdissector code in inactive subset */
+  register_heur_dissector_list("cotp_is", &cotp_is_heur_subdissector_list);
 
-	/* XXX - what about CLTP and proto_cltp? */
-	register_dissector("ositp", dissect_ositp, proto_cotp);
+  /* other COTP/ISO 8473 subdissectors */
+  register_heur_dissector_list("cotp", &cotp_heur_subdissector_list);
+
+  /* XXX - what about CLTP and proto_cltp? */
+  register_dissector("ositp", dissect_ositp, proto_cotp);
 }
 
 void proto_register_cltp(void)
 {
-  /*        static hf_register_info hf[] = {
-                { &variable,
-                { "Name",           "cltp.abbreviation", TYPE, VALS_POINTER }},
-        };*/
-	static gint *ett[] = {
-		&ett_cltp,
-	};
+  static hf_register_info hf[] = {
+    { &hf_cotp_srcref,
+      { "Source reference", "cotp.srcref", FT_UINT16, BASE_HEX, NULL, 0x0,
+        "Source address reference", HFILL}},
+    { &hf_cotp_destref,
+      { "Destination reference", "cotp.destref", FT_UINT16, BASE_HEX, NULL, 0x0,
+        "Destination address reference", HFILL}}, 
+    { &hf_cltp_type,
+      { "CLTP PDU Type", "cltp.type", FT_UINT8, BASE_HEX, VALS(cltp_tpdu_type_abbrev_vals), 0x0,
+        "CLTP PDU Type", HFILL}},
+  };
+  static gint *ett[] = {
+	&ett_cltp,
+  };
 
-        proto_cltp = proto_register_protocol(PROTO_STRING_CLTP, "CLTP", "cltp");
- /*       proto_register_field_array(proto_cotp, hf, array_length(hf));*/
-	proto_register_subtree_array(ett, array_length(ett));
-	register_init_routine(clnp_reassemble_init);
-}
-
-void
-proto_reg_handoff_clnp(void)
-{
-        data_handle = find_dissector("data");
-
-	clnp_handle = create_dissector_handle(dissect_clnp, proto_clnp);
-	dissector_add("osinl", NLPID_ISO8473_CLNP, clnp_handle);
-	dissector_add("osinl", NLPID_NULL, clnp_handle); /* Inactive subset */
-	dissector_add("x.25.spi", NLPID_ISO8473_CLNP, clnp_handle);
+  proto_cltp = proto_register_protocol(PROTO_STRING_CLTP, "CLTP", "cltp");
+  proto_register_field_array(proto_cltp, hf, array_length(hf));
+  proto_register_subtree_array(ett, array_length(ett));
 }
