@@ -1,8 +1,8 @@
 /* packet-dcerpc-reg.c
  * Routines for SMB \PIPE\winreg packet disassembly
- * Copyright 2001, 2002 Tim Potter <tpot@samba.org>
+ * Copyright 2001-2003 Tim Potter <tpot@samba.org>
  *
- * $Id: packet-dcerpc-reg.c,v 1.12 2002/08/29 19:05:40 guy Exp $
+ * $Id: packet-dcerpc-reg.c,v 1.16 2003/04/21 01:13:41 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -95,19 +95,12 @@ static int
 RegOpenHKLM_q(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	      proto_tree *tree, char *drep)
 {
-	dcerpc_info *di = (dcerpc_info *)pinfo->private_data;
-	dcerpc_call_value *dcv = (dcerpc_call_value *)di->call_data;
-
-	if (dcv->rep_frame != 0)
-		proto_tree_add_text(tree, tvb, offset, 0,
-				    "Reply in frame %u", dcv->rep_frame);
-
 	/* Parse packet */
 
 	offset = dissect_ndr_pointer(
 		tvb, offset, pinfo, tree, drep,
 		dissect_open_data,
-		NDR_POINTER_UNIQUE, "Unknown", -1, 0);
+		NDR_POINTER_UNIQUE, "Unknown", -1);
 
 	dcerpc_smb_check_long_frame(tvb, offset, pinfo, tree);
 
@@ -118,14 +111,8 @@ static int
 RegOpenHKLM_r(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	      proto_tree *tree, char *drep)
 {
-	dcerpc_info *di = (dcerpc_info *)pinfo->private_data;
-	dcerpc_call_value *dcv = (dcerpc_call_value *)di->call_data;
 	e_ctx_hnd policy_hnd;
 
-	if (dcv->req_frame != 0)
-		proto_tree_add_text(tree, tvb, offset, 0,
-				    "Request in frame %u", dcv->req_frame);
-
 	/* Parse packet */
 
 	offset = dissect_nt_policy_hnd(
@@ -150,19 +137,12 @@ static int
 RegOpenHKU_q(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	     proto_tree *tree, char *drep)
 {
-	dcerpc_info *di = (dcerpc_info *)pinfo->private_data;
-	dcerpc_call_value *dcv = (dcerpc_call_value *)di->call_data;
-
-	if (dcv->rep_frame != 0)
-		proto_tree_add_text(tree, tvb, offset, 0,
-				    "Reply in frame %u", dcv->rep_frame);
-
 	/* Parse packet */
 
 	offset = dissect_ndr_pointer(
 		tvb, offset, pinfo, tree, drep,
 		dissect_open_data,
-		NDR_POINTER_UNIQUE, "Unknown", -1, 0);
+		NDR_POINTER_UNIQUE, "Unknown", -1);
 
 	dcerpc_smb_check_long_frame(tvb, offset, pinfo, tree);
 
@@ -173,14 +153,8 @@ static int
 RegOpenHKU_r(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	     proto_tree *tree, char *drep)
 {
-	dcerpc_info *di = (dcerpc_info *)pinfo->private_data;
-	dcerpc_call_value *dcv = (dcerpc_call_value *)di->call_data;
 	e_ctx_hnd policy_hnd;
 
-	if (dcv->req_frame != 0)
-		proto_tree_add_text(tree, tvb, offset, 0,
-				    "Request in frame %u", dcv->req_frame);
-
 	/* Parse packet */
 
 	offset = dissect_nt_policy_hnd(
@@ -205,19 +179,12 @@ static int
 RegOpenHKCR_q(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	      proto_tree *tree, char *drep)
 {
-	dcerpc_info *di = (dcerpc_info *)pinfo->private_data;
-	dcerpc_call_value *dcv = (dcerpc_call_value *)di->call_data;
-
-	if (dcv->rep_frame != 0)
-		proto_tree_add_text(tree, tvb, offset, 0,
-				    "Reply in frame %u", dcv->rep_frame);
-
 	/* Parse packet */
 
 	offset = dissect_ndr_pointer(
 		tvb, offset, pinfo, tree, drep,
 		dissect_open_data,
-		NDR_POINTER_UNIQUE, "Unknown", -1, 0);
+		NDR_POINTER_UNIQUE, "Unknown", -1);
 
 	dcerpc_smb_check_long_frame(tvb, offset, pinfo, tree);
 
@@ -228,14 +195,8 @@ static int
 RegOpenHKCR_r(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	      proto_tree *tree, char *drep)
 {
-	dcerpc_info *di = (dcerpc_info *)pinfo->private_data;
-	dcerpc_call_value *dcv = (dcerpc_call_value *)di->call_data;
 	e_ctx_hnd policy_hnd;
 
-	if (dcv->req_frame != 0)
-		proto_tree_add_text(tree, tvb, offset, 0,
-				    "Request in frame %u", dcv->req_frame);
-
 	/* Parse packet */
 
 	offset = dissect_nt_policy_hnd(
@@ -260,13 +221,6 @@ static int
 RegClose_q(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	   proto_tree *tree, char *drep)
 {
-	dcerpc_info *di = (dcerpc_info *)pinfo->private_data;
-	dcerpc_call_value *dcv = (dcerpc_call_value *)di->call_data;
-
-	if (dcv->rep_frame != 0)
-		proto_tree_add_text(tree, tvb, offset, 0,
-				    "Reply in frame %u", dcv->rep_frame);
-
 	/* Parse packet */
 
 	offset = dissect_nt_policy_hnd(
@@ -282,13 +236,6 @@ static int
 RegClose_r(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	   proto_tree *tree, char *drep)
 {
-	dcerpc_info *di = (dcerpc_info *)pinfo->private_data;
-	dcerpc_call_value *dcv = (dcerpc_call_value *)di->call_data;
-
-	if (dcv->req_frame != 0)
-		proto_tree_add_text(tree, tvb, offset, 0,
-				    "Request in frame %u", dcv->req_frame);
-
 	/* Parse packet */
 
 	offset = dissect_nt_policy_hnd(
@@ -311,20 +258,13 @@ static int
 RegQueryKey_q(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	      proto_tree *tree, char *drep)
 {
-	dcerpc_info *di = (dcerpc_info *)pinfo->private_data;
-	dcerpc_call_value *dcv = (dcerpc_call_value *)di->call_data;
-
-	if (dcv->rep_frame != 0)
-		proto_tree_add_text(tree, tvb, offset, 0,
-				    "Reply in frame %u", dcv->rep_frame);
-
 	/* Parse packet */
 
 	offset = dissect_nt_policy_hnd(
 		tvb, offset, pinfo, tree, drep,
 		hf_hnd, NULL, FALSE, FALSE);
 
-	offset = dissect_ndr_nt_UNICODE_STRING(
+	offset = dissect_ndr_counted_string(
 		tvb, offset, pinfo, tree, drep, hf_querykey_class, 0);
 
 	dcerpc_smb_check_long_frame(tvb, offset, pinfo, tree);
@@ -336,16 +276,9 @@ static int
 RegQueryKey_r(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	      proto_tree *tree, char *drep)
 {
-	dcerpc_info *di = (dcerpc_info *)pinfo->private_data;
-	dcerpc_call_value *dcv = (dcerpc_call_value *)di->call_data;
-
-	if (dcv->req_frame != 0)
-		proto_tree_add_text(tree, tvb, offset, 0,
-				    "Request in frame %u", dcv->req_frame);
-
 	/* Parse packet */
 
-	offset = dissect_ndr_nt_UNICODE_STRING(
+	offset = dissect_ndr_counted_string(
 		tvb, offset, pinfo, tree, drep, hf_querykey_class, 0);
 
 	offset = dissect_ndr_uint32(
@@ -395,20 +328,13 @@ static int
 RegOpenEntry_q(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	       proto_tree *tree, char *drep)
 {
-	dcerpc_info *di = (dcerpc_info *)pinfo->private_data;
-	dcerpc_call_value *dcv = (dcerpc_call_value *)di->call_data;
-
-	if (dcv->rep_frame != 0)
-		proto_tree_add_text(tree, tvb, offset, 0,
-				    "Reply in frame %u", dcv->rep_frame);
-
 	/* Parse packet */
 
 	offset = dissect_nt_policy_hnd(
 		tvb, offset, pinfo, tree, drep,
 		hf_hnd, NULL, FALSE, FALSE);
 
-	offset = dissect_ndr_nt_UNICODE_STRING(
+	offset = dissect_ndr_counted_string(
 		tvb, offset, pinfo, tree, drep, hf_querykey_class, 0);
 
 	offset = dissect_ndr_uint32(
@@ -428,14 +354,8 @@ static int
 RegOpenEntry_r(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	       proto_tree *tree, char *drep)
 {
-	dcerpc_info *di = (dcerpc_info *)pinfo->private_data;
-	dcerpc_call_value *dcv = (dcerpc_call_value *)di->call_data;
 	e_ctx_hnd policy_hnd;
 
-	if (dcv->req_frame != 0)
-		proto_tree_add_text(tree, tvb, offset, 0,
-				    "Request in frame %u", dcv->req_frame);
-
 	/* Parse packet */
 
 	offset = dissect_nt_policy_hnd(
@@ -460,13 +380,6 @@ static int
 RegUnknown1A_q(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	       proto_tree *tree, char *drep)
 {
-	dcerpc_info *di = (dcerpc_info *)pinfo->private_data;
-	dcerpc_call_value *dcv = (dcerpc_call_value *)di->call_data;
-
-	if (dcv->rep_frame != 0)
-		proto_tree_add_text(tree, tvb, offset, 0,
-				    "Reply in frame %u", dcv->rep_frame);
-
 	/* Parse packet */
 
 	offset = dissect_nt_policy_hnd(
@@ -482,13 +395,6 @@ static int
 RegUnknown1A_r(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	       proto_tree *tree, char *drep)
 {
-	dcerpc_info *di = (dcerpc_info *)pinfo->private_data;
-	dcerpc_call_value *dcv = (dcerpc_call_value *)di->call_data;
-
-	if (dcv->req_frame != 0)
-		proto_tree_add_text(tree, tvb, offset, 0,
-				    "Request in frame %u", dcv->req_frame);
-
 	/* Parse packet */
 
 	offset = dissect_ndr_uint32(
@@ -511,13 +417,6 @@ static int
 RegEnumKey_q(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	     proto_tree *tree, char *drep)
 {
-	dcerpc_info *di = (dcerpc_info *)pinfo->private_data;
-	dcerpc_call_value *dcv = (dcerpc_call_value *)di->call_data;
-
-	if (dcv->rep_frame != 0)
-		proto_tree_add_text(tree, tvb, offset, 0,
-				    "Reply in frame %u", dcv->rep_frame);
-
 	/* Parse packet */
 
 	offset = dissect_nt_policy_hnd(
@@ -533,13 +432,6 @@ static int
 RegEnumKey_r(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	     proto_tree *tree, char *drep)
 {
-	dcerpc_info *di = (dcerpc_info *)pinfo->private_data;
-	dcerpc_call_value *dcv = (dcerpc_call_value *)di->call_data;
-
-	if (dcv->req_frame != 0)
-		proto_tree_add_text(tree, tvb, offset, 0,
-				    "Request in frame %u", dcv->req_frame);
-
 	/* Parse packet */
 
 	offset = dissect_ntstatus(
@@ -563,11 +455,6 @@ RegFoo_q(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	 proto_tree *tree, char *drep)
 {
 	dcerpc_info *di = (dcerpc_info *)pinfo->private_data;
-	dcerpc_call_value *dcv = (dcerpc_call_value *)di->call_data;
-
-	if (dcv->rep_frame != 0)
-		proto_tree_add_text(tree, tvb, offset, 0,
-				    "Reply in frame %u", dcv->rep_frame);
 
 	/* Parse packet */
 
@@ -581,11 +468,6 @@ RegFoo_r(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	 proto_tree *tree, char *drep)
 {
 	dcerpc_info *di = (dcerpc_info *)pinfo->private_data;
-	dcerpc_call_value *dcv = (dcerpc_call_value *)di->call_data;
-
-	if (dcv->req_frame != 0)
-		proto_tree_add_text(tree, tvb, offset, 0,
-				    "Request in frame %u", dcv->req_frame);
 
 	/* Parse packet */
 
