@@ -3,7 +3,7 @@
  *
  * Copyright 2001, Paul Ionescu	<paul@acorp.ro>
  *
- * $Id: packet-fr.c,v 1.32 2002/10/22 08:22:02 guy Exp $
+ * $Id: packet-fr.c,v 1.34 2003/04/29 17:56:47 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -45,7 +45,6 @@
 #include <string.h>
 #include <glib.h>
 #include <epan/packet.h>
-#include "packet-osi.h"
 #include "packet-llc.h"
 #include "packet-chdlc.h"
 #include "xdlc.h"
@@ -90,6 +89,8 @@ static gint hf_fr_chdlctype = -1;
 
 static dissector_handle_t data_handle;
 
+static dissector_table_t osinl_subdissector_table;
+
 static const true_false_string cmd_string = {
                 "Command",
                 "Response"
@@ -429,6 +430,9 @@ void proto_reg_handoff_fr(void)
 
   fr_handle = create_dissector_handle(dissect_fr, proto_fr);
   dissector_add("wtap_encap", WTAP_ENCAP_FRELAY, fr_handle);
+  dissector_add("wtap_encap", WTAP_ENCAP_FRELAY_WITH_PHDR, fr_handle);
   dissector_add("gre.proto", GRE_FR, fr_handle);
   data_handle = find_dissector("data");
+
+  osinl_subdissector_table = find_dissector_table("osinl");
 }
