@@ -1,7 +1,7 @@
 /* packet-icmpv6.c
  * Routines for ICMPv6 packet disassembly
  *
- * $Id: packet-icmpv6.c,v 1.69 2003/01/20 05:42:30 guy Exp $
+ * $Id: packet-icmpv6.c,v 1.72 2003/04/28 19:24:48 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -147,6 +147,8 @@ dissect_icmpv6opt(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tre
     struct nd_opt_hdr nd_opt_hdr, *opt;
     int len;
     char *typename;
+    static const guint8 nd_redirect_reserved[6] = {0, 0, 0, 0, 0, 0};
+    guint8 nd_redirect_res[6];
 
     if (!tree)
 	return;
@@ -216,7 +218,7 @@ again:
 	char *t;
 	int len, i, p;
 	len = (opt->nd_opt_len << 3) - sizeof(*opt);
-	t = (char *)malloc(len * 3);
+	t = g_malloc(len * 3);
 	memset(t, 0, len * 3);
 	p = offset + sizeof(*opt);
 	for (i = 0; i < len; i++) {
@@ -272,6 +274,13 @@ again:
 	break;
       }
     case ND_OPT_REDIRECTED_HEADER:
+	tvb_memcpy(tvb, (guint8 *)&nd_redirect_res, offset + 2, 6);
+	if (memcmp(nd_redirect_res, nd_redirect_reserved, 6) == 0)
+	   proto_tree_add_text(icmp6opt_tree, tvb,
+	    offset + 2, 6, "Reserved: 0 (correct)");
+	else
+	   proto_tree_add_text(icmp6opt_tree, tvb,
+	    offset +2, 6, "Reserved: MUST be 0 (incorrect!)");	
 	proto_tree_add_text(icmp6opt_tree, tvb,
 	    offset + 8, (opt->nd_opt_len << 3) - 8, "Redirected packet");
 	dissect_contained_icmpv6(tvb, offset + 8, pinfo, icmp6opt_tree);
@@ -1098,6 +1107,26 @@ dissect_icmpv6(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	    "Unknown");
 	len = sizeof(struct icmp6_nodeinfo);
 	break;
+    case ICMP6_MIP6_DHAAD_REQUEST:
+	typename = coltypename = "Dynamic Home Agent Address Discovery Request";
+	codename = "Should always be zero";
+	colcodename = NULL;
+	break;
+    case ICMP6_MIP6_DHAAD_REPLY:
+	typename = coltypename = "Dynamic Home Agent Address Discovery Reply";
+	codename = "Should always be zero";
+	colcodename = NULL;
+	break;
+    case ICMP6_MIP6_MPS:
+	typename = coltypename = "Mobile Prefix Solicitation";
+	codename = "Should always be zero";
+	colcodename = NULL;
+	break;
+    case ICMP6_MIP6_MPA:
+	typename = coltypename = "Mobile Prefix Advertisement";
+	codename = "Should always be zero";
+	colcodename = NULL;
+	break;
     }
 
     if (check_col(pinfo->cinfo, COL_INFO)) {
@@ -1368,6 +1397,53 @@ dissect_icmpv6(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		"Unknown"));
 	    dissect_nodeinfo(tvb, offset, pinfo, icmp6_tree);
 	    break;
+	case ICMP6_MIP6_DHAAD_REQUEST:
+	    proto_tree_add_text(icmp6_tree, tvb,
+		offset + 4, 2, "Identifier: %d (0x%02x)",
+		tvb_get_ntohs(tvb, offset + 4),
+		tvb_get_ntohs(tvb, offset + 4));
+	    proto_tree_add_text(icmp6_tree, tvb,
+		offset + 6, 2, "Reserved: %d",
+		tvb_get_ntohs(tvb, offset + 6));
+	    break;
+	case ICMP6_MIP6_DHAAD_REPLY:
+	    proto_tree_add_text(icmp6_tree, tvb,
+		offset + 4, 2, "Identifier: %d (0x%02x)",
+		tvb_get_ntohs(tvb, offset + 4),
+		tvb_get_ntohs(tvb, offset + 4));
+	    proto_tree_add_text(icmp6_tree, tvb,
+		offset + 6, 2, "Reserved: %d",
+		tvb_get_ntohs(tvb, offset + 6));
+	    /* TODO Show all Home Agent Addresses */
+	    break;
+	case ICMP6_MIP6_MPS:
+	    proto_tree_add_text(icmp6_tree, tvb,
+		offset + 4, 2, "Identifier: %d (0x%02x)",
+		tvb_get_ntohs(tvb, offset + 4),
+		tvb_get_ntohs(tvb, offset + 4));
+	    proto_tree_add_text(icmp6_tree, tvb,
+		offset + 6, 2, "Reserved: %d",
+		tvb_get_ntohs(tvb, offset + 6));
+	    break;
+	case ICMP6_MIP6_MPA:
+	    proto_tree_add_text(icmp6_tree, tvb,
+		offset + 4, 2, "Identifier: %d (0x%02x)",
+		tvb_get_ntohs(tvb, offset + 4),
+		tvb_get_ntohs(tvb, offset + 4));
+	    proto_tree_add_text(icmp6_tree, tvb,
+		offset + 6, 2,
+		decode_boolean_bitfield(tvb_get_guint8(tvb, offset + 6),
+		    0x8000, 16,
+		    "Managed Address Configuration",
+		    "No Managed Address Configuration"));
+	    proto_tree_add_text(icmp6_tree, tvb,
+		offset + 6, 2,
+		decode_boolean_bitfield(tvb_get_guint8(tvb, offset + 6),
+		    0x4000, 16,
+		    "Other Stateful Configuration",
+		    "No Other Stateful Configuration"));
+	    /* TODO Show all options */
+	    break;
 	default:
 	    next_tvb = tvb_new_subset(tvb, offset + sizeof(*dp), -1, -1);
 	    call_dissector(data_handle,next_tvb, pinfo, tree);
