@@ -3,7 +3,7 @@
  * Copyright 2000, Axis Communications AB
  * Inquiries/bugreports should be sent to Johan.Jorgensen@axis.com
  *
- * $Id: packet-ieee80211.c,v 1.81 2003/01/22 19:39:25 guy Exp $
+ * $Id: packet-ieee80211.c,v 1.86 2003/04/27 20:57:58 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -222,7 +222,7 @@ static char *wep_keystr[] = {NULL, NULL, NULL, NULL};
 #define TAG_TIM            0x05
 #define TAG_IBSS_PARAMETER 0x06
 #define TAG_CHALLENGE_TEXT 0x10
-#define TAG_ERP_INFO       0x2F
+#define TAG_ERP_INFO       0x2A
 #define TAG_EXT_SUPP_RATES 0x32
 
 /* ************************************************************************* */
@@ -328,6 +328,7 @@ static int hf_fragment_overlap_conflict = -1;
 static int hf_fragment_multiple_tails = -1;
 static int hf_fragment_too_long_fragment = -1;
 static int hf_fragment_error = -1;
+static int hf_reassembled_in = -1;
 
 
 static int proto_wlan_mgt = -1;
@@ -399,6 +400,7 @@ static const fragment_items frag_items = {
 	&hf_fragment_multiple_tails,
 	&hf_fragment_too_long_fragment,
 	&hf_fragment_error,
+	&hf_reassembled_in,
 	"fragments"
 };
 
@@ -876,12 +878,14 @@ add_tagged_field (proto_tree * tree, tvbuff_t * tvb, int offset)
       memset (out_buff, 0, SHORT_STR);
 
       snprintf (out_buff, SHORT_STR,
-		"ERP info: 0x%x (%sNon-ERP STAs, %suse protection)",
+		"ERP info: 0x%x (%sNon-ERP STAs, %suse protection, %s preambles)",
 		tag_data_ptr[0],
 		tag_data_ptr[0] & 0x01 ? "" : "no ",
-		tag_data_ptr[0] & 0x02 ? "" : "do not ");
+		tag_data_ptr[0] & 0x02 ? "" : "do not ",
+		tag_data_ptr[0] & 0x04 ? "short or long": "long");
       proto_tree_add_string (tree, tag_interpretation, tvb, offset + 2,
 			     tag_len, out_buff);
+
       break;
 
 
@@ -1759,7 +1763,7 @@ dissect_ieee80211_common (tvbuff_t * tvb, packet_info * pinfo,
 				     wlan_fragment_table,
 				     wlan_reassembled_table,
 				     frag_number,
-				     len,
+				     reported_len,
 				     more_frags);
     if (fd_head != NULL) {
       /*
@@ -1774,12 +1778,8 @@ dissect_ieee80211_common (tvbuff_t * tvb, packet_info * pinfo,
        * a non-fragmented frame.
        */
       if (fd_head->next != NULL) {
-        next_tvb = tvb_new_real_data(fd_head->data, fd_head->len, fd_head->len);
-        tvb_set_child_real_data_tvbuff(tvb, next_tvb);
-        add_new_data_source(pinfo, next_tvb, "Reassembled 802.11");
-
-        /* Show all fragments. */
-        show_fragment_seq_tree(fd_head, &frag_items, hdr_tree, pinfo, next_tvb);
+        next_tvb = process_reassembled_data(tvb, pinfo, "Reassembled 802.11",
+                fd_head, &frag_items, NULL, hdr_tree);
       } else {
       	/*
       	 * Not fragmented, really.
@@ -2070,6 +2070,16 @@ proto_register_ieee80211 (void)
      "associated stations"},
     {0x12, "Association denied due to requesting station not supporting all "
      "of the datarates in the BSSBasicServiceSet Parameter"},
+    {0x13, "Association denied due to requesting station not supporting "
+     "short preamble operation"},
+    {0x14, "Association denied due to requesting station not supporting "
+     "PBCC encoding"},
+    {0x15, "Association denied due to requesting station not supporting "
+     "channel agility"},
+    {0x19, "Association denied due to requesting station not supporting "
+     "short slot operation"},
+    {0x1A, "Association denied due to requesting station not supporting "
+     "DSSS-OFDM operation"},
     {0x00, NULL}
   };
 
@@ -2222,6 +2232,10 @@ proto_register_ieee80211 (void)
       {"802.11 Fragments", "wlan.fragments", FT_NONE, BASE_NONE, NULL, 0x0,
        "802.11 Fragments", HFILL }},
 
+    {&hf_reassembled_in,
+      {"Reassembled 802.11 in frame", "wlan.reassembled_in", FT_FRAMENUM, BASE_NONE, NULL, 0x0,
+       "This 802.11 packet is reassembled in this frame", HFILL }},
+
     {&hf_wep_iv,
      {"Initialization Vector", "wlan.wep.iv", FT_UINT24, BASE_HEX, NULL, 0,
       "Initialization Vector", HFILL }},
@@ -2628,8 +2642,6 @@ static int wep_decrypt(guint8 *buf, guint32 len, int key_override) {
   return 0;
 }
 
-/* XXX need to verify these malloc()s succeed */
-
 static void init_wepkeys(void) {
   char *tmp, *tmp2;
   guint8 *tmp3;
@@ -2655,8 +2667,8 @@ static void init_wepkeys(void) {
   if (wep_keylens != NULL)
     return;
 
-  wep_keys = malloc(num_wepkeys * sizeof(guint8*));
-  wep_keylens = malloc(num_wepkeys * sizeof(int));
+  wep_keys = g_malloc(num_wepkeys * sizeof(guint8*));
+  wep_keylens = g_malloc(num_wepkeys * sizeof(int));
 
   for (i = 0 ; i < num_wepkeys; i++) {
     wep_keys[i] = NULL;
@@ -2679,7 +2691,7 @@ static void init_wepkeys(void) {
 #endif
 #endif
 
-      wep_keys[i] = malloc(32 * sizeof(guint8));
+      wep_keys[i] = g_malloc(32 * sizeof(guint8));
       memset(wep_keys[i], 0, 32 * sizeof(guint8));
       tmp3 = wep_keys[i];
       while ((tmp != NULL) && (*tmp != 0)) {
