@@ -1,7 +1,7 @@
 /* packet-igmp.c   2001 Ronnie Sahlberg <See AUTHORS for email>
  * Routines for IGMP packet disassembly
  *
- * $Id: packet-igmp.c,v 1.21 2002/08/28 21:00:17 jmayer Exp $
+ * $Id: packet-igmp.c,v 1.22 2003/03/04 06:47:09 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -804,7 +804,7 @@ dissect_igmp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
 	switch (type) {
 
 	case IGMP_V1_HOST_MEMBERSHIP_QUERY:	/* 0x11 v1/v2/v3 */
-		if ( (pinfo->iplen-pinfo->iphdrlen*4)>=12 ) {
+		if ( (pinfo->iplen-pinfo->iphdrlen)>=12 ) {
 			/* version 3 */
 			offset = dissect_igmp_v3_query(tvb, pinfo, tree, type, offset);
 		} else {
@@ -867,7 +867,7 @@ dissect_igmp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
 		break;
 
 	case IGMP_TYPE_0x25:
-		if ( (pinfo->iplen-pinfo->iphdrlen*4)>=8 ) {
+		if ( (pinfo->iplen-pinfo->iphdrlen)>=8 ) {
 			/* if len of igmp packet>=8 we assume it is MSNIP */
 			offset = dissect_msnip(tvb, pinfo, parent_tree, offset);
 		} else {
