@@ -1,7 +1,7 @@
 /* packet-ip.c
  * Routines for IP and miscellaneous IP protocol packet disassembly
  *
- * $Id: packet-ip.c,v 1.182 2003/01/23 04:03:58 guy Exp $
+ * $Id: packet-ip.c,v 1.191 2003/04/29 17:24:35 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -97,6 +97,7 @@ static int hf_ip_fragment_overlap_conflict = -1;
 static int hf_ip_fragment_multiple_tails = -1;
 static int hf_ip_fragment_too_long_fragment = -1;
 static int hf_ip_fragment_error = -1;
+static int hf_ip_reassembled_in = -1;
 
 static gint ett_ip = -1;
 static gint ett_ip_dsfield = -1;
@@ -119,11 +120,11 @@ static const fragment_items ip_frag_items = {
 	&hf_ip_fragment_multiple_tails,
 	&hf_ip_fragment_too_long_fragment,
 	&hf_ip_fragment_error,
+	&hf_ip_reassembled_in,
 	"fragments"
 };
 
-/* Used by IPv6 as well, so not static */
-dissector_table_t ip_dissector_table;
+static dissector_table_t ip_dissector_table;
 
 static dissector_handle_t ip_handle;
 static dissector_handle_t data_handle;
@@ -321,11 +322,13 @@ static gint ett_icmp_mip_flags = -1;
  * defragmentation of IPv4
  */
 static GHashTable *ip_fragment_table = NULL;
+static GHashTable *ip_reassembled_table = NULL;
 
 static void
 ip_defragment_init(void)
 {
   fragment_table_init(&ip_fragment_table);
+  reassembled_table_init(&ip_reassembled_table);
 }
 
 void
@@ -818,7 +821,7 @@ dissect_ip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   guint16    flags;
   guint8     nxt;
   guint16    ipsum;
-  fragment_data *ipfd_head;
+  fragment_data *ipfd_head=NULL;
   tvbuff_t   *next_tvb;
   gboolean   update_col_info = TRUE;
   gboolean   save_fragmented;
@@ -831,6 +834,7 @@ dissect_ip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
      eip_current=0;
   }
   iph=&eip_arr[eip_current];
+  pinfo->private_data=iph;
 
   if (check_col(pinfo->cinfo, COL_PROTOCOL))
     col_set_str(pinfo->cinfo, COL_PROTOCOL, "IP");
@@ -918,7 +922,7 @@ dissect_ip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
   iph->ip_id  = tvb_get_ntohs(tvb, offset + 4);
   if (tree)
-    proto_tree_add_uint(ip_tree, hf_ip_id, tvb, offset + 4, 2, iph->ip_id);
+    proto_tree_add_uint_format(ip_tree, hf_ip_id, tvb, offset + 4, 2, iph->ip_id, "Identification: 0x%04x (%d)", iph->ip_id, iph->ip_id);
 
   iph->ip_off = tvb_get_ntohs(tvb, offset + 6);
   if (tree) {
@@ -1004,7 +1008,7 @@ dissect_ip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
   pinfo->iplen = iph->ip_len;
 
-  pinfo->iphdrlen = lo_nibble(iph->ip_v_hl);
+  pinfo->iphdrlen = hlen;
 
   SET_ADDRESS(&pinfo->net_src, AT_IPv4, 4, tvb_get_ptr(tvb, offset + IPH_SRC, 4));
   SET_ADDRESS(&pinfo->src, AT_IPv4, 4, tvb_get_ptr(tvb, offset + IPH_SRC, 4));
@@ -1021,30 +1025,18 @@ dissect_ip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
    */
   save_fragmented = pinfo->fragmented;
   if (ip_defragment && (iph->ip_off & (IP_MF|IP_OFFSET)) &&
-	tvb_reported_length(tvb) <= tvb_length(tvb) && ipsum == 0) {
-    ipfd_head = fragment_add(tvb, offset, pinfo, iph->ip_id,
+      tvb_bytes_exist(tvb, offset, pinfo->iplen - pinfo->iphdrlen) &&
+      ipsum == 0) {
+    ipfd_head = fragment_add_check(tvb, offset, pinfo, iph->ip_id,
 			     ip_fragment_table,
+			     ip_reassembled_table,
 			     (iph->ip_off & IP_OFFSET)*8,
-			     pinfo->iplen - (pinfo->iphdrlen*4),
+			     pinfo->iplen - pinfo->iphdrlen,
 			     iph->ip_off & IP_MF);
 
     if (ipfd_head != NULL) {
-      /* OK, we have the complete reassembled payload.
-         Allocate a new tvbuff, referring to the reassembled payload. */
-      next_tvb = tvb_new_real_data(ipfd_head->data, ipfd_head->datalen,
-	ipfd_head->datalen);
-
-      /* Add the tvbuff to the list of tvbuffs to which the tvbuff we
-         were handed refers, so it'll get cleaned up when that tvbuff
-         is cleaned up. */
-      tvb_set_child_real_data_tvbuff(tvb, next_tvb);
-
-      /* Add the defragmented data to the data source list. */
-      add_new_data_source(pinfo, next_tvb, "Reassembled IPv4");
-
-      /* show all fragments */
-      update_col_info = !show_fragment_tree(ipfd_head, &ip_frag_items,
-        ip_tree, pinfo, next_tvb);
+      next_tvb = process_reassembled_data(tvb, pinfo, "Reassembled IPv4",
+        ipfd_head, &ip_frag_items, &update_col_info, ip_tree);
     } else {
       /* We don't have the complete reassembled payload. */
       next_tvb = NULL;
@@ -1077,9 +1069,17 @@ dissect_ip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
   if (next_tvb == NULL) {
     /* Just show this as a fragment. */
-    if (check_col(pinfo->cinfo, COL_INFO))
+    if (check_col(pinfo->cinfo, COL_INFO)) {
       col_add_fstr(pinfo->cinfo, COL_INFO, "Fragmented IP protocol (proto=%s 0x%02x, off=%u)",
 	ipprotostr(iph->ip_p), iph->ip_p, (iph->ip_off & IP_OFFSET) * 8);
+    }
+    if( ipfd_head && ipfd_head->reassembled_in != pinfo->fd->num ){
+      if (check_col(pinfo->cinfo, COL_INFO)) {
+        col_append_fstr(pinfo->cinfo, COL_INFO, " [Reassembled in #%d]",
+          ipfd_head->reassembled_in);
+      }
+    }
+
     call_dissector(data_handle, tvb_new_subset(tvb, offset, -1, -1), pinfo,
                    tree);
     pinfo->fragmented = save_fragmented;
@@ -1694,6 +1694,10 @@ proto_register_ip(void)
 		{ &hf_ip_fragments,
 		{ "IP Fragments", "ip.fragments", FT_NONE, BASE_NONE, NULL, 0x0,
 			"IP Fragments", HFILL }},
+
+		{ &hf_ip_reassembled_in,
+		{ "Reassembled IP in frame", "ip.reassembled_in", FT_FRAMENUM, BASE_NONE, NULL, 0x0,
+			"This IP packet is reassembled in this frame", HFILL }}
 	};
 	static gint *ett[] = {
 		&ett_ip,
@@ -1727,7 +1731,7 @@ proto_register_ip(void)
 		"Reassemble fragmented IP datagrams",
 		"Whether fragmented IP datagrams should be reassembled",
 		&ip_defragment);
-	prefs_register_bool_preference(ip_module, "ip_summary_in_tree",
+	prefs_register_bool_preference(ip_module, "summary_in_tree",
 	    "Show IP summary in protocol tree",
 	    "Whether the IP summary line should be shown in the protocol tree",
 	    &ip_summary_in_tree);
