@@ -1,8 +1,12 @@
 /* packet-ipx.h
  * Routines for NetWare's IPX
  * Gilbert Ramirez <gram@alumni.rice.edu>
+ * NDPS support and SPX Defragmentation added by Greg Morris (gmorris@novell.com)
  *
- * $Id: packet-ipx.h,v 1.20 2002/10/15 04:25:42 guy Exp $
+ * Portions Copyright (c) by Gilbert Ramirez 2000-2002
+ * Portions Copyright (c) Novell, Inc. 2002-2003
+ *
+ * $Id: packet-ipx.h,v 1.27 2003/04/09 22:33:19 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -60,16 +64,6 @@ struct sap_query
 	guint16	server_type;	/* net order */
 };
 
-struct sap_server_ident
-{
-	guint16	server_type ;
-	char		server_name[48] ;
-	IPXNet		server_network ;
-	IPXNode		server_node ;
-	IPXPort		server_port ;
-	guint16	intermediate_network ;
-};
-
 #define IPX_RIP_REQUEST (0x1)
 #define IPX_RIP_RESPONSE (0x2)
 
@@ -137,7 +131,8 @@ struct ipx_rip_packet
 #define IPX_SOCKET_IPX_MESSAGE1		0x4003
 #define IPX_SOCKET_ADSM                 0x8522 /* www.tivoli.com */
 #define IPX_SOCKET_EIGRP                0x85be /* cisco ipx eigrp */
-#define IPX_SOCKET_WIDE_AREA_ROUTER	0x9001
+#define IPX_SOCKET_NLSP			0x9001 /* NetWare Link Services Protocol */
+#define IPX_SOCKET_IPXWAN               0x9004 /* IPX WAN (RFC 1362, NLSP spec) */
 #define IPX_SOCKET_SNMP_AGENT           0x900F /* RFC 1906 */
 #define IPX_SOCKET_SNMP_SINK            0x9010 /* RFC 1906 */
 #define IPX_SOCKET_PING_NOVELL          0x9086 /* In cisco this is set with: ipx ping-default novell */
@@ -150,6 +145,19 @@ struct ipx_rip_packet
 #define SPX_SOCKET_RMS                  0x90b6 /* NDPS Remote Management Service */
 #define SPX_SOCKET_NOTIFY_LISTENER      0x90b7 /* NDPS Notify Listener */
 
+extern const value_string ipx_socket_vals[];
+
+extern const value_string server_vals[];
+
 void capture_ipx(packet_counts *);
 
+/*
+ * Structure passed to SPX subdissectors, containing information from
+ * the SPX header that might be useful to the subdissector.
+ */
+typedef struct {
+	gboolean eom;			/* end-of-message flag in SPX header */
+	guint8	datastream_type;	/* datastream type from SPX header */
+} spx_info;
+
 #endif
