@@ -1,23 +1,23 @@
 /* packet-isis-snp.c
  * Routines for decoding isis complete & partial SNP and their payload
  *
- * $Id: packet-isis-snp.c,v 1.17 2002/08/02 23:35:52 jmayer Exp $
+ * $Id: packet-isis-snp.c,v 1.21 2003/03/31 07:44:09 guy Exp $
  * Stuart Stanley <stuarts@mxmail.net>
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
@@ -41,6 +41,7 @@
 static int hf_isis_csnp_pdu_length = -1;
 static gint ett_isis_csnp = -1;
 static gint ett_isis_csnp_lsp_entries = -1;
+static gint ett_isis_csnp_lsp_entry = -1;
 static gint ett_isis_csnp_authentication = -1;
 static gint ett_isis_csnp_clv_unknown = -1;
 
@@ -48,6 +49,7 @@ static gint ett_isis_csnp_clv_unknown = -1;
 static int hf_isis_psnp_pdu_length = -1;
 static gint ett_isis_psnp = -1;
 static gint ett_isis_psnp_lsp_entries = -1;
+static gint ett_isis_psnp_lsp_entry = -1;
 static gint ett_isis_psnp_authentication = -1;
 static gint ett_isis_psnp_clv_unknown = -1;
 
@@ -55,7 +57,9 @@ static void dissect_l1_snp_authentication_clv(tvbuff_t *tvb,
 	proto_tree *tree, int offset, int id_length, int length);
 static void dissect_l2_snp_authentication_clv(tvbuff_t *tvb,
 	proto_tree *tree, int offset, int id_length, int length);
-static void dissect_snp_lsp_entries(tvbuff_t *tvb,
+static void dissect_csnp_lsp_entries(tvbuff_t *tvb,
+	proto_tree *tree, int offset, int id_length, int length);
+static void dissect_psnp_lsp_entries(tvbuff_t *tvb,
 	proto_tree *tree, int offset, int id_length, int length);
 
 static const isis_clv_handle_t clv_l1_csnp_opts[] = {
@@ -63,7 +67,7 @@ static const isis_clv_handle_t clv_l1_csnp_opts[] = {
 		ISIS_CLV_L1_CSNP_LSP_ENTRIES,
 		"LSP entries",
 		&ett_isis_csnp_lsp_entries,
-		dissect_snp_lsp_entries
+		dissect_csnp_lsp_entries
 	},
 	{
 		ISIS_CLV_L1_CSNP_AUTHENTICATION_NS,
@@ -78,7 +82,7 @@ static const isis_clv_handle_t clv_l1_csnp_opts[] = {
 		dissect_l1_snp_authentication_clv
 	},
 	{
-		0, "", NULL, NULL 
+		0, "", NULL, NULL
 	}
 };
 
@@ -87,7 +91,7 @@ static const isis_clv_handle_t clv_l2_csnp_opts[] = {
 		ISIS_CLV_L2_CSNP_LSP_ENTRIES,
 		"LSP entries",
 		&ett_isis_csnp_lsp_entries,
-		dissect_snp_lsp_entries
+		dissect_csnp_lsp_entries
 	},
 	{
 		ISIS_CLV_L2_CSNP_AUTHENTICATION_NS,
@@ -102,7 +106,7 @@ static const isis_clv_handle_t clv_l2_csnp_opts[] = {
 		dissect_l2_snp_authentication_clv
 	},
 	{
-		0, "", NULL, NULL 
+		0, "", NULL, NULL
 	}
 };
 
@@ -111,7 +115,7 @@ static const isis_clv_handle_t clv_l1_psnp_opts[] = {
 		ISIS_CLV_L1_PSNP_LSP_ENTRIES,
 		"LSP entries",
 		&ett_isis_psnp_lsp_entries,
-		dissect_snp_lsp_entries
+		dissect_psnp_lsp_entries
 	},
 	{
 		ISIS_CLV_L1_PSNP_AUTHENTICATION_NS,
@@ -126,7 +130,7 @@ static const isis_clv_handle_t clv_l1_psnp_opts[] = {
 		dissect_l1_snp_authentication_clv
 	},
 	{
-		0, "", NULL, NULL 
+		0, "", NULL, NULL
 	}
 };
 
@@ -135,7 +139,7 @@ static const isis_clv_handle_t clv_l2_psnp_opts[] = {
 		ISIS_CLV_L2_PSNP_LSP_ENTRIES,
 		"LSP entries",
 		&ett_isis_psnp_lsp_entries,
-		dissect_snp_lsp_entries
+		dissect_psnp_lsp_entries
 	},
 	{
 		ISIS_CLV_L2_PSNP_AUTHENTICATION,
@@ -150,63 +154,117 @@ static const isis_clv_handle_t clv_l2_psnp_opts[] = {
 		dissect_l2_snp_authentication_clv
 	},
 	{
-		0, "", NULL, NULL 
+		0, "", NULL, NULL
 	}
 };
+
 /*
  * Name: dissect_snp_lsp_entries()
  *
  * Description:
  *	All the snp packets use a common payload format.  We have up
  *	to n entries (based on length), which are made of:
- *		2 : remaining life time
- *		8 : lsp id
- *		4 : sequence number
- *		2 : checksum
+ *		2         : remaining life time
+ *		id_length : lsp id
+ *		4         : sequence number
+ *		2         : checksum
  *
  * Input:
  *	tvbuff_t * : tvbuffer for packet data
  *	proto_tree * : protocol display tree to fill out.  May be NULL
  *	int : offset into packet data where we are.
- *	int : length of payload to decode.
  *	int : length of IDs in packet.
+ *	int : length of payload to decode.
  *
  * Output:
  *      void, but we will add to proto tree if !NULL.
  */
-static void 
-dissect_snp_lsp_entries(tvbuff_t *tvb, proto_tree *tree, int offset,
+static void
+dissect_csnp_lsp_entries(tvbuff_t *tvb, proto_tree *tree, int offset,
 	int id_length, int length)
 {
+        proto_tree *subtree,*ti;
+
 	while ( length > 0 ) {
 		if ( length < 2+id_length+2+4+2 ) {
 			isis_dissect_unknown(tvb, tree, offset,
-				"Short SNP header entry (%d vs %d)", length,
+				"Short CSNP header entry (%d vs %d)", length,
 				2+id_length+2+4+2 );
 			return;
 		}
-		
-		proto_tree_add_text(tree, tvb, offset, 2, "Remaining life      : %d",
+
+	        ti = proto_tree_add_text(tree, tvb, offset, 2+id_length+2+4+2,
+                                    "LSP-ID: %s, Sequence: 0x%08x, Lifetime: %5us, Checksum: 0x%04x",
+                                           print_system_id( tvb_get_ptr(tvb, offset+2, id_length+2), id_length+2 ),
+                                           tvb_get_ntohl(tvb, offset+2+id_length+2),
+                                           tvb_get_ntohs(tvb, offset),
+                                           tvb_get_ntohs(tvb, offset+2+id_length+2+4));
+
+                subtree = proto_item_add_subtree(ti,ett_isis_csnp_lsp_entry);
+
+		proto_tree_add_text(subtree, tvb, offset+2, 8,
+			"LSP-ID:             : %s",
+			print_system_id( tvb_get_ptr(tvb, offset+2, id_length+2), id_length+2 ));
+
+		proto_tree_add_text(subtree, tvb, offset+2+id_length+2, 4,
+			"LSP Sequence Number : 0x%08x",
+			tvb_get_ntohl(tvb, offset+2+id_length+2));
+
+		proto_tree_add_text(subtree, tvb, offset, 2,
+			"Remaining Lifetime  : %us",
 			tvb_get_ntohs(tvb, offset));
-		length -= 2;
-		offset += 2;
 
-		isis_lsp_decode_lsp_id(tvb, tree, offset,
-			 "LSP ID              ", id_length);
-		length -= id_length + 2;
-		offset += id_length + 2;
+		proto_tree_add_text(subtree, tvb, offset+2+id_length+2+4, 2,
+			"LSP checksum        : 0x%04x",
+			tvb_get_ntohs(tvb, offset+2+id_length+2+4));
 
-		proto_tree_add_text(tree, tvb, offset, 4, 
-			"LSP Sequence Number : 0x%04x",
-			tvb_get_ntohl(tvb, offset));
-		length -= 4;
-		offset += 4;
+		length -= 2+id_length+2+4+2;
+		offset += 2+id_length+2+4+2;
+	}
 
-		proto_tree_add_text(tree, tvb, offset, 2, 
-			"LSP checksum        : 0x%02x",
+}
+
+static void
+dissect_psnp_lsp_entries(tvbuff_t *tvb, proto_tree *tree, int offset,
+	int id_length, int length)
+{
+        proto_tree *subtree,*ti;
+
+	while ( length > 0 ) {
+		if ( length < 2+id_length+2+4+2 ) {
+			isis_dissect_unknown(tvb, tree, offset,
+				"Short PSNP header entry (%d vs %d)", length,
+				2+id_length+2+4+2 );
+			return;
+		}
+
+	        ti = proto_tree_add_text(tree, tvb, offset, 2+id_length+2+4+2,
+                                    "LSP-ID: %s, Sequence: 0x%08x, Lifetime: %5us, Checksum: 0x%04x",
+                                           print_system_id( tvb_get_ptr(tvb, offset+2, id_length+2), id_length+2 ),
+                                           tvb_get_ntohl(tvb, offset+2+id_length+2),
+                                           tvb_get_ntohs(tvb, offset),
+                                           tvb_get_ntohs(tvb, offset+2+id_length+2+4));
+
+                subtree = proto_item_add_subtree(ti,ett_isis_psnp_lsp_entry);
+
+		proto_tree_add_text(subtree, tvb, offset+2, 8,
+			"LSP-ID:             : %s",
+			print_system_id( tvb_get_ptr(tvb, offset+2, id_length+2), id_length+2 ));
+
+		proto_tree_add_text(subtree, tvb, offset+2+id_length+2, 4,
+			"LSP Sequence Number : 0x%08x",
+			tvb_get_ntohl(tvb, offset+2+id_length+2));
+
+		proto_tree_add_text(subtree, tvb, offset, 2,
+			"Remaining Lifetime  : %us",
 			tvb_get_ntohs(tvb, offset));
-		length -= 2;
-		offset += 2;
+
+		proto_tree_add_text(subtree, tvb, offset+2+id_length+2+4, 2,
+			"LSP checksum        : 0x%04x",
+			tvb_get_ntohs(tvb, offset+2+id_length+2+4));
+
+		length -= 2+id_length+2+4+2;
+		offset += 2+id_length+2+4+2;
 	}
 
 }
@@ -229,8 +287,8 @@ dissect_snp_lsp_entries(tvbuff_t *tvb, proto_tree *tree, int offset,
  * Output:
  *      void, but we will add to proto tree if !NULL.
  */
-void 
-isis_dissect_isis_csnp(tvbuff_t *tvb, proto_tree *tree, int offset,
+void
+isis_dissect_isis_csnp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset,
 	int type, int header_length, int id_length)
 {
 	proto_item	*ti;
@@ -252,21 +310,35 @@ isis_dissect_isis_csnp(tvbuff_t *tvb, proto_tree *tree, int offset,
 	offset += 2;
 
 	if (tree) {
-		proto_tree_add_text(csnp_tree, tvb, offset, id_length + 1, 
-			"Source id    : %s",
+		proto_tree_add_text(csnp_tree, tvb, offset, id_length + 1,
+			"Source-ID:    %s",
 				print_system_id( tvb_get_ptr(tvb, offset, id_length+1), id_length+1 ) );
 	}
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+		col_append_fstr(pinfo->cinfo, COL_INFO, ", Source-ID: %s",
+			print_system_id( tvb_get_ptr(tvb, offset, id_length+1), id_length+1 ) );
+	}
 	offset += id_length + 1;
 
 	if (tree) {
-		isis_lsp_decode_lsp_id(tvb, csnp_tree, offset,
-			"Start LSP id ", id_length );
+		proto_tree_add_text(csnp_tree, tvb, offset, id_length + 2,
+			"Start LSP-ID: %s",
+                                    print_system_id( tvb_get_ptr(tvb, offset, id_length+2), id_length+2 ) );                
+	}
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+		col_append_fstr(pinfo->cinfo, COL_INFO, ", Start LSP-ID: %s",
+			print_system_id( tvb_get_ptr(tvb, offset, id_length+2), id_length+2 ) );
 	}
 	offset += id_length + 2;
 
 	if (tree) {
-		isis_lsp_decode_lsp_id(tvb, csnp_tree, offset,
-			 "End   LSP id ", id_length );
+		proto_tree_add_text(csnp_tree, tvb, offset, id_length + 2,
+			"End LSP-ID: %s",
+                                    print_system_id( tvb_get_ptr(tvb, offset, id_length+2), id_length+2 ) );  
+	}
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+		col_append_fstr(pinfo->cinfo, COL_INFO, ", End LSP-ID: %s",
+			print_system_id( tvb_get_ptr(tvb, offset, id_length+2), id_length+2 ) );
 	}
 	offset += id_length + 2;
 
@@ -304,8 +376,8 @@ isis_dissect_isis_csnp(tvbuff_t *tvb, proto_tree *tree, int offset,
  * Output:
  *      void, but we will add to proto tree if !NULL.
  */
-void 
-isis_dissect_isis_psnp(tvbuff_t *tvb, proto_tree *tree, int offset,
+void
+isis_dissect_isis_psnp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset,
 	int type, int header_length, int id_length)
 {
 	proto_item	*ti;
@@ -328,9 +400,13 @@ isis_dissect_isis_psnp(tvbuff_t *tvb, proto_tree *tree, int offset,
 
 	if (tree) {
 		proto_tree_add_text(psnp_tree, tvb, offset, id_length + 1,
-			"Source id: %s",
+			"Source-ID: %s",
 			print_system_id( tvb_get_ptr(tvb, offset, id_length+1), id_length + 1 ) );
 	}
+	if (check_col(pinfo->cinfo, COL_INFO)) {
+		col_append_fstr(pinfo->cinfo, COL_INFO, ", Source-ID: %s",
+			print_system_id( tvb_get_ptr(tvb, offset, id_length+1), id_length+1 ) );
+	}
 	offset += id_length + 1;
 
 	len = pdu_length - header_length;
@@ -369,7 +445,7 @@ isis_dissect_isis_psnp(tvbuff_t *tvb, proto_tree *tree, int offset,
  * Output:
  *	void, will modify proto_tree if not null.
  */
-static void 
+static void
 dissect_l1_snp_authentication_clv(tvbuff_t *tvb, proto_tree *tree, int offset,
 	int id_length _U_, int length)
 {
@@ -394,7 +470,7 @@ dissect_l1_snp_authentication_clv(tvbuff_t *tvb, proto_tree *tree, int offset,
  * Output:
  *	void, will modify proto_tree if not null.
  */
-static void 
+static void
 dissect_l2_snp_authentication_clv(tvbuff_t *tvb, proto_tree *tree, int offset,
 	int id_length _U_, int length)
 {
@@ -405,25 +481,26 @@ dissect_l2_snp_authentication_clv(tvbuff_t *tvb, proto_tree *tree, int offset,
 /*
  * Name: isis_register_csnp()
  *
- * Description: 
+ * Description:
  *	Register our protocol sub-sets with protocol manager.
  *
- * Input: 
+ * Input:
  *	int : protocol index for the ISIS protocol
  *
  * Output:
  *	void
  */
-void 
+void
 isis_register_csnp(int proto_isis) {
 	static hf_register_info hf[] = {
 		{ &hf_isis_csnp_pdu_length,
-		{ "PDU length",		"isis.csnp.pdu_length", FT_UINT16, 
+		{ "PDU length",		"isis.csnp.pdu_length", FT_UINT16,
 		  BASE_DEC, NULL, 0x0, "", HFILL }},
 	};
 	static gint *ett[] = {
 		&ett_isis_csnp,
 		&ett_isis_csnp_lsp_entries,
+		&ett_isis_csnp_lsp_entry,
 		&ett_isis_csnp_authentication,
 		&ett_isis_csnp_clv_unknown,
 	};
@@ -436,25 +513,26 @@ isis_register_csnp(int proto_isis) {
 /*
  * Name: isis_register_psnp()
  *
- * Description: 
+ * Description:
  *	Register our protocol sub-sets with protocol manager.
  *
- * Input: 
+ * Input:
  *	int : protocol index for the ISIS protocol
  *
  * Output:
  *	void
  */
-void 
+void
 isis_register_psnp(int proto_isis) {
 	static hf_register_info hf[] = {
 		{ &hf_isis_psnp_pdu_length,
-		{ "PDU length",		"isis.psnp.pdu_length", FT_UINT16, 
+		{ "PDU length",		"isis.psnp.pdu_length", FT_UINT16,
 		  BASE_DEC, NULL, 0x0, "", HFILL }},
 	};
 	static gint *ett[] = {
 		&ett_isis_psnp,
 		&ett_isis_psnp_lsp_entries,
+		&ett_isis_psnp_lsp_entry,
 		&ett_isis_psnp_authentication,
 		&ett_isis_psnp_clv_unknown,
 	};
