@@ -2,7 +2,7 @@
  * Routines for IEEE 802.2 LLC layer
  * Gilbert Ramirez <gram@alumni.rice.edu>
  *
- * $Id: packet-llc.c,v 1.105 2002/12/10 02:49:31 guy Exp $
+ * $Id: packet-llc.c,v 1.109 2003/04/18 03:41:39 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -35,10 +35,12 @@
 #include "llcsaps.h"
 #include "bridged_pids.h"
 #include "ppptypes.h"
+#include "arcnet_pids.h"
 #include "packet-fc.h"
 #include "packet-ip.h"
 #include "packet-ipx.h"
 #include "packet-netbios.h"
+#include "packet-vines.h"
 #include <epan/sna-utils.h>
 
 #include "packet-llc.h"
@@ -64,6 +66,7 @@ static gint ett_llc_ctrl = -1;
 
 static dissector_table_t subdissector_table;
 static dissector_table_t cisco_subdissector_table;
+static dissector_table_t xid_subdissector_table;
 
 static dissector_handle_t bpdu_handle;
 static dissector_handle_t eth_handle;
@@ -131,9 +134,9 @@ const value_string sap_vals[] = {
 	{ SAP_NETWARE,        "NetWare" },
 	{ SAP_NETBIOS,        "NetBIOS" },
 	{ SAP_IBMNM,          "IBM Net Management" },
-	{ SAP_RPL1,           "Remote Program Load" },
+	{ SAP_HPEXT,          "HP Extended LLC" },
 	{ SAP_UB,             "Ungermann-Bass" },
-	{ SAP_RPL2,           "Remote Program Load" },
+	{ SAP_RPL,            "Remote Program Load" },
 	{ SAP_OSINL,          "ISO Network Layer" },
 	{ SAP_GLOBAL,         "Global LSAP" },
 	{ 0x00,               NULL }
@@ -239,6 +242,11 @@ capture_llc(const guchar *pd, int offset, int len, packet_counts *ld) {
 				capture_netbios(ld);
 				break;
 
+			case SAP_VINES1:
+			case SAP_VINES2:
+				capture_vines(ld);
+				break;
+
 			default:
 				ld->other++;
 				break;
@@ -321,16 +329,35 @@ dissect_llc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			);
 		}
 
-		next_tvb = tvb_new_subset(tvb, llc_header_len, -1, -1);
-		if (XDLC_IS_INFORMATION(control)) {
-			/* non-SNAP */
-			/* do lookup with the subdissector table */
-			if (!dissector_try_port(subdissector_table, dsap,
-			    next_tvb, pinfo, tree)) {
-				call_dissector(data_handle,next_tvb, pinfo, tree);
+		if (tvb_length_remaining(tvb, llc_header_len) > 0) {
+			next_tvb = tvb_new_subset(tvb, llc_header_len, -1, -1);
+			if (XDLC_IS_INFORMATION(control)) {
+				/*
+				 * Non-SNAP I or UI frame.
+				 * Try the regular LLC subdissector table
+				 * with the DSAP.
+				 */
+				if (!dissector_try_port(subdissector_table,
+				    dsap, next_tvb, pinfo, tree)) {
+					call_dissector(data_handle, next_tvb,
+					    pinfo, tree);
+				}
+			} else if ((control & (XDLC_U_MODIFIER_MASK|XDLC_U))
+			    == (XDLC_XID|XDLC_U)) {
+				/*
+				 * Non-SNAP XID frame.
+				 * Try the XID LLC subdissector table
+				 * with the DSAP.
+				 */
+				if (!dissector_try_port(xid_subdissector_table,
+				    dsap, next_tvb, pinfo, tree)) {
+					call_dissector(data_handle, next_tvb,
+					    pinfo, tree);
+				}
+			} else {
+				call_dissector(data_handle, next_tvb, pinfo,
+				    tree);
 			}
-		} else {
-			call_dissector(data_handle,next_tvb, pinfo, tree);
 		}
 	}
 }
@@ -532,6 +559,8 @@ proto_register_llc(void)
 	  "LLC SAP", FT_UINT8, BASE_HEX);
 	cisco_subdissector_table = register_dissector_table("llc.cisco_pid",
 	  "Cisco OUI PID", FT_UINT16, BASE_HEX);
+	xid_subdissector_table = register_dissector_table("llc.xid_dsap",
+	  "LLC XID SAP", FT_UINT8, BASE_HEX);
 
 	register_dissector("llc", dissect_llc, proto_llc);
 }
@@ -563,4 +592,10 @@ proto_reg_handoff_llc(void)
 	dissector_add("udp.port", UDP_PORT_LLC5, llc_handle);
 	/* IP-over-FC when we have the full FC frame */
 	dissector_add("fc.ftype", FC_FTYPE_IP, llc_handle);
+
+	/*
+	 * BACNET-over-ARCNET is really BACNET-over-802.2 LLC-over-ARCNET,
+	 * apparently.
+	 */
+	dissector_add("arcnet.protocol_id", ARCNET_PROTO_BACNET, llc_handle);
 }
