@@ -5,9 +5,9 @@
  * http://www.ietf.org/internet-drafts/draft-ietf-sigtran-m3ua-06.txt (expired)
  * http://www.ietf.org/rfc/rfc3332.txt
  *
- * Copyright 2000, 2001, 2002, 2003 Michael Tuexen <Michael.Tuexen [AT] siemens.com>
+ * Copyright 2000, 2001, 2002, 2003 Michael Tuexen <tuexen [AT] fh-muenster.de>
  *
- * $Id: packet-m3ua.c,v 1.24 2003/01/22 09:07:26 tuexen Exp $
+ * $Id: packet-m3ua.c,v 1.31 2003/04/22 13:47:38 tuexen Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -225,17 +225,6 @@ static const value_string message_class_type_acro_values[] = {
   { MESSAGE_CLASS_RKM_MESSAGE   * 256 + MESSAGE_TYPE_DEREG_RSP ,    "DEREG_RSP" },
   { 0,                           NULL } };
 
-
-
-
-
-#define PROTOCOL_DATA_OFFSET PARAMETER_VALUE_OFFSET
-
-
-
-#define TRAFFIC_MODE_TYPE_LENGTH 4
-#define TRAFFIC_MODE_TYPE_OFFSET PARAMETER_VALUE_OFFSET
-
 /* Initialize the protocol and registered fields */
 static int proto_m3ua = -1;
 static int hf_version = -1;
@@ -330,10 +319,8 @@ dissect_v5_common_header(tvbuff_t *common_header_tvb, packet_info *pinfo, proto_
   message_class  = tvb_get_guint8(common_header_tvb, MESSAGE_CLASS_OFFSET);
   message_type   = tvb_get_guint8(common_header_tvb, MESSAGE_TYPE_OFFSET);
 
-  if (check_col(pinfo->cinfo, COL_INFO)) {
-    col_append_str(pinfo->cinfo, COL_INFO, val_to_str(message_class * 256 + message_type, v5_message_class_type_acro_values, "reserved"));
-    col_append_str(pinfo->cinfo, COL_INFO, " ");
-  }
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_add_fstr(pinfo->cinfo, COL_INFO, "%s ", val_to_str(message_class * 256 + message_type, v5_message_class_type_acro_values, "reserved"));
 
   if (m3ua_tree) {
     /* add the components of the common header to the protocol tree */
@@ -355,10 +342,8 @@ dissect_common_header(tvbuff_t *common_header_tvb, packet_info *pinfo, proto_tre
   message_class  = tvb_get_guint8(common_header_tvb, MESSAGE_CLASS_OFFSET);
   message_type   = tvb_get_guint8(common_header_tvb, MESSAGE_TYPE_OFFSET);
 
-  if (check_col(pinfo->cinfo, COL_INFO)) {
-    col_append_str(pinfo->cinfo, COL_INFO, val_to_str(message_class * 256 + message_type, message_class_type_acro_values, "reserved"));
-    col_append_str(pinfo->cinfo, COL_INFO, " ");
-  }
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_add_fstr(pinfo->cinfo, COL_INFO,"%s ", val_to_str(message_class * 256 + message_type, message_class_type_acro_values, "reserved"));
 
   if (m3ua_tree) {
     /* add the components of the common header to the protocol tree */
@@ -381,6 +366,8 @@ dissect_network_appearance_parameter(tvbuff_t *parameter_tvb, proto_tree *parame
   proto_item_append_text(parameter_item, " (%u)", tvb_get_ntohl(parameter_tvb, NETWORK_APPEARANCE_OFFSET));
 }
 
+#define V5_PROTOCOL_DATA_OFFSET PARAMETER_VALUE_OFFSET
+
 static void
 dissect_v5_protocol_data_parameter(tvbuff_t *parameter_tvb, packet_info *pinfo, proto_tree *tree, proto_item *parameter_item)
 {
@@ -389,7 +376,7 @@ dissect_v5_protocol_data_parameter(tvbuff_t *parameter_tvb, packet_info *pinfo,
 
   length = tvb_get_ntohs(parameter_tvb, PARAMETER_LENGTH_OFFSET);
   protocol_data_length = length - PARAMETER_HEADER_LENGTH;
-  payload_tvb          = tvb_new_subset(parameter_tvb, PROTOCOL_DATA_OFFSET, protocol_data_length, protocol_data_length);
+  payload_tvb          = tvb_new_subset(parameter_tvb, V5_PROTOCOL_DATA_OFFSET, protocol_data_length, protocol_data_length);
   proto_item_append_text(parameter_item, " (SS7 message of %u byte%s)", protocol_data_length, plurality(protocol_data_length, "", "s"));
   proto_item_set_len(parameter_item, PARAMETER_HEADER_LENGTH);
   call_dissector(mtp3_handle, payload_tvb, pinfo, tree);
@@ -436,11 +423,10 @@ dissect_affected_destinations_parameter(tvbuff_t *parameter_tvb, proto_tree *par
 static void
 dissect_routing_context_parameter(tvbuff_t *parameter_tvb, proto_tree *parameter_tree, proto_item *parameter_item)
 {
-  guint16 number_of_contexts, context_number, parameter_length;
+  guint16 number_of_contexts, context_number;
   gint context_offset;
 
-  parameter_length   = tvb_get_ntohs(parameter_tvb, PARAMETER_LENGTH_OFFSET);
-  number_of_contexts = (parameter_length - PARAMETER_HEADER_LENGTH) >> 4;
+  number_of_contexts = (tvb_get_ntohs(parameter_tvb, PARAMETER_LENGTH_OFFSET) - PARAMETER_HEADER_LENGTH) >> 2;
   context_offset = PARAMETER_VALUE_OFFSET;
   for(context_number=1; context_number <= number_of_contexts; context_number++) {
     proto_tree_add_item(parameter_tree, hf_routing_context, parameter_tvb, context_offset, ROUTING_CONTEXT_LENGTH, NETWORK_BYTE_ORDER);
@@ -545,6 +531,9 @@ dissect_reason_parameter(tvbuff_t *parameter_tvb, proto_tree *parameter_tree, pr
   proto_item_append_text(parameter_item, " (%s)", val_to_str(tvb_get_ntohl(parameter_tvb, REASON_OFFSET), reason_values, "unknown"));
 }
 
+#define TRAFFIC_MODE_TYPE_LENGTH 4
+#define TRAFFIC_MODE_TYPE_OFFSET PARAMETER_VALUE_OFFSET
+
 static const value_string v5_traffic_mode_type_values[] = {
   { 1, "Over-ride"            },
   { 2, "Load-share"           },
@@ -760,36 +749,36 @@ dissect_asp_identifier_parameter(tvbuff_t *parameter_tvb, proto_tree *parameter_
   proto_item_append_text(parameter_item, " (%u)", tvb_get_ntohl(parameter_tvb, ASP_IDENTIFIER_OFFSET));
 }
 
+#define PROTOCOL_DATA_1_OFFSET PARAMETER_VALUE_OFFSET
+
 static void
 dissect_protocol_data_1_parameter(tvbuff_t *parameter_tvb, packet_info *pinfo, proto_tree *tree, proto_item *parameter_item)
 {
-  guint16 length, protocol_data_length;
+  guint16 protocol_data_length;
   tvbuff_t *payload_tvb;
 
-  length = tvb_get_ntohs(parameter_tvb, PARAMETER_LENGTH_OFFSET);
-  protocol_data_length = length - PARAMETER_HEADER_LENGTH;
-  payload_tvb          = tvb_new_subset(parameter_tvb, PROTOCOL_DATA_OFFSET, protocol_data_length, protocol_data_length);
+  protocol_data_length = tvb_get_ntohs(parameter_tvb, PARAMETER_LENGTH_OFFSET) - PARAMETER_HEADER_LENGTH;
+  payload_tvb          = tvb_new_subset(parameter_tvb, PROTOCOL_DATA_1_OFFSET, protocol_data_length, protocol_data_length);
   proto_item_append_text(parameter_item, " (SS7 message of %u byte%s)", protocol_data_length, plurality(protocol_data_length, "", "s"));
   proto_item_set_len(parameter_item, PARAMETER_HEADER_LENGTH);
   call_dissector(mtp3_handle, payload_tvb, pinfo, tree);
-
 }
 
 #define LI_OCTETT_LENGTH 1
 #define LI_OCTETT_OFFSET PARAMETER_VALUE_OFFSET
+#define PROTOCOL_DATA_2_OFFSET (PARAMETER_VALUE_OFFSET + LI_OCTETT_LENGTH)
 
 static void
 dissect_protocol_data_2_parameter(tvbuff_t *parameter_tvb, packet_info *pinfo, proto_tree *tree, proto_tree *parameter_tree, proto_item *parameter_item)
 {
   guint8 li;
-  guint16 length, protocol_data_length;
+  guint16 protocol_data_length;
   tvbuff_t *payload_tvb;
 
-  length               = tvb_get_ntohs(parameter_tvb, PARAMETER_LENGTH_OFFSET);
   li                   = tvb_get_guint8(parameter_tvb, LI_OCTETT_OFFSET);
-  protocol_data_length = length - PARAMETER_HEADER_LENGTH - LI_OCTETT_LENGTH;
-  payload_tvb          = tvb_new_subset(parameter_tvb, PROTOCOL_DATA_OFFSET + LI_OCTETT_LENGTH, protocol_data_length, protocol_data_length);
-  proto_tree_add_uint(parameter_tree, hf_li, parameter_tvb, LI_OCTETT_OFFSET, LI_OCTETT_LENGTH, li);
+  protocol_data_length = tvb_get_ntohs(parameter_tvb, PARAMETER_LENGTH_OFFSET) - PARAMETER_HEADER_LENGTH - LI_OCTETT_LENGTH;
+  payload_tvb          = tvb_new_subset(parameter_tvb, PROTOCOL_DATA_2_OFFSET, protocol_data_length, protocol_data_length);
+  proto_tree_add_item(parameter_tree, hf_li, parameter_tvb, LI_OCTETT_OFFSET, LI_OCTETT_LENGTH, NETWORK_BYTE_ORDER);
   proto_item_append_text(parameter_item, " (SS7 message of %u byte%s)", protocol_data_length, plurality(protocol_data_length, "", "s"));
   proto_item_set_len(parameter_item, PARAMETER_HEADER_LENGTH + LI_OCTETT_LENGTH);
   call_dissector(mtp3_handle, payload_tvb, pinfo, tree);
@@ -824,16 +813,16 @@ dissect_routing_key_parameter(tvbuff_t *parameter_tvb, packet_info *pinfo, proto
 }
 
 static const value_string registration_result_status_values[] = {
-  { 0,           "Successfully Registered" } ,
-  { 1,           "Error - Unknown" } ,
-  { 2,           "Error - Invalid DPC" } ,
-  { 3,           "Error - Invalid Network Appearance" } ,
-  { 4,           "Error - Invalid Routing Key" } ,
-  { 5,           "Error - Permission Denied" } ,
-  { 6,           "Error - Overlapping (Non-unique) Routing Key" } ,
-  { 7,           "Error - Routing Key not Provisioned" } ,
-  { 8,           "Error - Insufficient Resources" } ,
-  { 0,           NULL } };
+  { 0, "Successfully Registered" } ,
+  { 1, "Error - Unknown" } ,
+  { 2, "Error - Invalid DPC" } ,
+  { 3, "Error - Invalid Network Appearance" } ,
+  { 4, "Error - Invalid Routing Key" } ,
+  { 5, "Error - Permission Denied" } ,
+  { 6, "Error - Overlapping (Non-unique) Routing Key" } ,
+  { 7, "Error - Routing Key not Provisioned" } ,
+  { 8, "Error - Insufficient Resources" } ,
+  { 0, NULL } };
 
 #define REG_RES_IDENTIFIER_LENGTH 4
 #define REG_RES_STATUS_LENGTH     4
@@ -1028,35 +1017,34 @@ dissect_circuit_range_parameter(tvbuff_t *parameter_tvb, proto_tree *parameter_t
 #define DATA_OPC_OFFSET   PARAMETER_VALUE_OFFSET
 #define DATA_DPC_OFFSET   (DATA_OPC_OFFSET + DATA_OPC_LENGTH)
 #define DATA_SI_OFFSET    (DATA_DPC_OFFSET + DATA_DPC_LENGTH)
-#define DATA_NI_OFFSET    (DATA_SI_OFFSET + DATA_SI_LENGTH)
-#define DATA_MP_OFFSET    (DATA_NI_OFFSET + DATA_NI_LENGTH)
-#define DATA_SLS_OFFSET   (DATA_MP_OFFSET + DATA_MP_LENGTH)
+#define DATA_NI_OFFSET    (DATA_SI_OFFSET  + DATA_SI_LENGTH)
+#define DATA_MP_OFFSET    (DATA_NI_OFFSET  + DATA_NI_LENGTH)
+#define DATA_SLS_OFFSET   (DATA_MP_OFFSET  + DATA_MP_LENGTH)
 #define DATA_ULP_OFFSET   (DATA_SLS_OFFSET + DATA_SLS_LENGTH)
 
 static void
 dissect_protocol_data_parameter(tvbuff_t *parameter_tvb, packet_info *pinfo, proto_tree *tree, proto_tree *parameter_tree, proto_item *parameter_item)
 {
-  guint16 length, ulp_length;
-  guint8 si;
+  guint16 ulp_length;
   tvbuff_t *payload_tvb;
 
-  length = tvb_get_ntohs(parameter_tvb, PARAMETER_LENGTH_OFFSET);
-  ulp_length = length - PARAMETER_HEADER_LENGTH - DATA_HDR_LENGTH;
-  payload_tvb          = tvb_new_subset(parameter_tvb, DATA_ULP_OFFSET, ulp_length, ulp_length);
-  si  = tvb_get_guint8(parameter_tvb, DATA_SI_OFFSET);
-  proto_tree_add_item(parameter_tree, hf_protocol_data_opc, parameter_tvb, DATA_OPC_OFFSET, DATA_OPC_LENGTH, NETWORK_BYTE_ORDER);
-  proto_tree_add_item(parameter_tree, hf_protocol_data_dpc, parameter_tvb, DATA_DPC_OFFSET, DATA_DPC_LENGTH, NETWORK_BYTE_ORDER);
-  proto_tree_add_item(parameter_tree, hf_protocol_data_si,  parameter_tvb, DATA_SI_OFFSET,  DATA_SI_LENGTH,  NETWORK_BYTE_ORDER);
-  proto_tree_add_item(parameter_tree, hf_protocol_data_ni,  parameter_tvb, DATA_NI_OFFSET,  DATA_NI_LENGTH,  NETWORK_BYTE_ORDER);
-  proto_tree_add_item(parameter_tree, hf_protocol_data_mp,  parameter_tvb, DATA_MP_OFFSET,  DATA_MP_LENGTH,  NETWORK_BYTE_ORDER);
-  proto_tree_add_item(parameter_tree, hf_protocol_data_sls, parameter_tvb, DATA_SLS_OFFSET, DATA_SLS_LENGTH, NETWORK_BYTE_ORDER);
+  ulp_length  = tvb_get_ntohs(parameter_tvb, PARAMETER_LENGTH_OFFSET) - PARAMETER_HEADER_LENGTH - DATA_HDR_LENGTH;
 
-  proto_item_append_text(parameter_item, " (SS7 message of %u byte%s)", ulp_length, plurality(ulp_length, "", "s"));
-  proto_item_set_len(parameter_item, PARAMETER_HEADER_LENGTH + DATA_HDR_LENGTH);
+  if (parameter_tree) {
+    proto_tree_add_item(parameter_tree, hf_protocol_data_opc, parameter_tvb, DATA_OPC_OFFSET, DATA_OPC_LENGTH, NETWORK_BYTE_ORDER);
+    proto_tree_add_item(parameter_tree, hf_protocol_data_dpc, parameter_tvb, DATA_DPC_OFFSET, DATA_DPC_LENGTH, NETWORK_BYTE_ORDER);
+    proto_tree_add_item(parameter_tree, hf_protocol_data_si,  parameter_tvb, DATA_SI_OFFSET,  DATA_SI_LENGTH,  NETWORK_BYTE_ORDER);
+    proto_tree_add_item(parameter_tree, hf_protocol_data_ni,  parameter_tvb, DATA_NI_OFFSET,  DATA_NI_LENGTH,  NETWORK_BYTE_ORDER);
+    proto_tree_add_item(parameter_tree, hf_protocol_data_mp,  parameter_tvb, DATA_MP_OFFSET,  DATA_MP_LENGTH,  NETWORK_BYTE_ORDER);
+    proto_tree_add_item(parameter_tree, hf_protocol_data_sls, parameter_tvb, DATA_SLS_OFFSET, DATA_SLS_LENGTH, NETWORK_BYTE_ORDER);
 
-  if (!dissector_try_port(si_dissector_table, si, payload_tvb, pinfo, tree)) {
-    call_dissector(data_handle, payload_tvb, pinfo, tree);
+    proto_item_append_text(parameter_item, " (SS7 message of %u byte%s)", ulp_length, plurality(ulp_length, "", "s"));
+    proto_item_set_len(parameter_item, PARAMETER_HEADER_LENGTH + DATA_HDR_LENGTH);
   }
+  
+  payload_tvb = tvb_new_subset(parameter_tvb, DATA_ULP_OFFSET, ulp_length, ulp_length);
+  if (!dissector_try_port(si_dissector_table, tvb_get_guint8(parameter_tvb, DATA_SI_OFFSET), payload_tvb, pinfo, tree))
+    call_dissector(data_handle, payload_tvb, pinfo, tree);
 }
 
 #define CORR_ID_OFFSET PARAMETER_VALUE_OFFSET
@@ -1073,18 +1061,18 @@ dissect_correlation_identifier_parameter(tvbuff_t *parameter_tvb, proto_tree *pa
 #define REG_STATUS_OFFSET  PARAMETER_VALUE_OFFSET
 
 static const value_string registration_status_values[] = {
-  {  0,           "Successfully Registered" },
-  {  1,           "Error - Unknown" },
-  {  2,           "Error - Invalid DPC" },
-  {  3,           "Error - Invalid Network Appearance" },
-  {  4,           "Error - Invalid Routing Key" },
-  {  5,           "Error - Permission Denied" },
-  {  6,           "Error - Cannot Support Unique Routing" },
-  {  7,           "Error - Routing Key not Currently Provisioned" },
-  {  8,           "Error - Insufficient Resources" },
-  {  9,           "Error - Unsupported RK parameter Field" },
-  { 10,           "Error - Unsupported/Invalid Traffic Handling Mode" },
-  {  0,           NULL } };
+  {  0, "Successfully Registered" },
+  {  1, "Error - Unknown" },
+  {  2, "Error - Invalid DPC" },
+  {  3, "Error - Invalid Network Appearance" },
+  {  4, "Error - Invalid Routing Key" },
+  {  5, "Error - Permission Denied" },
+  {  6, "Error - Cannot Support Unique Routing" },
+  {  7, "Error - Routing Key not Currently Provisioned" },
+  {  8, "Error - Insufficient Resources" },
+  {  9, "Error - Unsupported RK parameter Field" },
+  { 10, "Error - Unsupported/Invalid Traffic Handling Mode" },
+  {  0, NULL } };
 
 static void
 dissect_registration_status_parameter(tvbuff_t *parameter_tvb, proto_tree *parameter_tree, proto_item *parameter_item)
@@ -1097,13 +1085,13 @@ dissect_registration_status_parameter(tvbuff_t *parameter_tvb, proto_tree *param
 #define DEREG_STATUS_OFFSET  PARAMETER_VALUE_OFFSET
 
 static const value_string deregistration_status_values[] = {
-  { 0,          "Successfully Deregistered" },
-  { 1,          "Error - Unknown" },
-  { 2,          "Error - Invalid Routing Context" },
-  { 3,          "Error - Permission Denied" },
-  { 4,          "Error - Not Registered" },
-  { 5,          "Error - ASP Currently Active for Routing Context" },
-  { 0,          NULL } };
+  { 0, "Successfully Deregistered" },
+  { 1, "Error - Unknown" },
+  { 2, "Error - Invalid Routing Context" },
+  { 3, "Error - Permission Denied" },
+  { 4, "Error - Not Registered" },
+  { 5, "Error - ASP Currently Active for Routing Context" },
+  { 0, NULL } };
 
 static void
 dissect_deregistration_status_parameter(tvbuff_t *parameter_tvb, proto_tree *parameter_tree, proto_item *parameter_item)
@@ -1116,10 +1104,9 @@ static void
 dissect_registration_results_parameter(tvbuff_t *parameter_tvb, packet_info *pinfo, proto_tree *tree, proto_tree *parameter_tree)
 {
   tvbuff_t *parameters_tvb;
-  guint16 length, parameters_length;
+  guint16 parameters_length;
 
-  length            = tvb_get_ntohs(parameter_tvb, PARAMETER_LENGTH_OFFSET);
-  parameters_length = length - PARAMETER_HEADER_LENGTH;
+  parameters_length = tvb_get_ntohs(parameter_tvb, PARAMETER_LENGTH_OFFSET) - PARAMETER_HEADER_LENGTH;
   parameters_tvb    = tvb_new_subset(parameter_tvb, PARAMETER_VALUE_OFFSET, parameters_length, parameters_length);
   dissect_parameters(parameters_tvb, pinfo, tree, parameter_tree);
 }
@@ -1128,10 +1115,9 @@ static void
 dissect_deregistration_results_parameter(tvbuff_t *parameter_tvb, packet_info *pinfo, proto_tree *tree, proto_tree *parameter_tree)
 {
   tvbuff_t *parameters_tvb;
-  guint16 length, parameters_length;
+  guint16 parameters_length;
 
-  length            = tvb_get_ntohs(parameter_tvb, PARAMETER_LENGTH_OFFSET);
-  parameters_length = length - PARAMETER_HEADER_LENGTH;
+  parameters_length = tvb_get_ntohs(parameter_tvb, PARAMETER_LENGTH_OFFSET) - PARAMETER_HEADER_LENGTH;
   parameters_tvb    = tvb_new_subset(parameter_tvb, PARAMETER_VALUE_OFFSET, parameters_length, parameters_length);
   dissect_parameters(parameters_tvb, pinfo, tree, parameter_tree);
 }
@@ -1189,13 +1175,17 @@ dissect_v5_parameter(tvbuff_t *parameter_tvb, packet_info *pinfo, proto_tree *tr
   length         = tvb_get_ntohs(parameter_tvb, PARAMETER_LENGTH_OFFSET);
   padding_length = tvb_length(parameter_tvb) - length;
 
+  if (!tree && tag != V5_PROTOCOL_DATA_PARAMETER_TAG)
+    return;	/* Nothing to do here */
+
   /* create proto_tree stuff */
   parameter_item   = proto_tree_add_text(m3ua_tree, parameter_tvb, PARAMETER_HEADER_OFFSET, tvb_length(parameter_tvb), val_to_str(tag, v5_parameter_tag_values, "Unknown parameter"));
   parameter_tree   = proto_item_add_subtree(parameter_item, ett_parameter);
 
-  /* add tag and length to the sua tree */
+  /* add tag and length to the parameter tree */
   proto_tree_add_item(parameter_tree, hf_v5_parameter_tag, parameter_tvb, PARAMETER_TAG_OFFSET,    PARAMETER_TAG_LENGTH,    NETWORK_BYTE_ORDER);
   proto_tree_add_item(parameter_tree, hf_parameter_length, parameter_tvb, PARAMETER_LENGTH_OFFSET, PARAMETER_LENGTH_LENGTH, NETWORK_BYTE_ORDER);
+
   switch(tag) {
   case V5_NETWORK_APPEARANCE_PARAMETER_TAG:
     dissect_network_appearance_parameter(parameter_tvb, parameter_tree, parameter_item);
@@ -1299,6 +1289,7 @@ static const value_string v6_parameter_tag_values[] = {
   { V6_REGISTRATION_RESULTS_PARAMETER_TAG,         "Registration results" },
   { V6_DEREGISTRATION_RESULTS_PARAMETER_TAG,       "De-registration results" },
   { 0,                           NULL } };
+  
 static void
 dissect_v6_parameter(tvbuff_t *parameter_tvb, packet_info *pinfo, proto_tree *tree, proto_tree *m3ua_tree)
 {
@@ -1311,12 +1302,15 @@ dissect_v6_parameter(tvbuff_t *parameter_tvb, packet_info *pinfo, proto_tree *tr
   length         = tvb_get_ntohs(parameter_tvb, PARAMETER_LENGTH_OFFSET);
   padding_length = tvb_length(parameter_tvb) - length;
 
+  if (!tree && tag != V6_PROTOCOL_DATA_1_PARAMETER_TAG && tag != V6_PROTOCOL_DATA_2_PARAMETER_TAG)
+    return;	/* Nothing to do here */
+
   /* create proto_tree stuff */
   parameter_item   = proto_tree_add_text(m3ua_tree, parameter_tvb, PARAMETER_HEADER_OFFSET, tvb_length(parameter_tvb), val_to_str(tag, v6_parameter_tag_values, "Unknown parameter"));
   parameter_tree   = proto_item_add_subtree(parameter_item, ett_parameter);
 
-  /* add tag and length to the sua tree */
-  proto_tree_add_item(parameter_tree, hf_v5_parameter_tag, parameter_tvb, PARAMETER_TAG_OFFSET,    PARAMETER_TAG_LENGTH,    NETWORK_BYTE_ORDER);
+  /* add tag and length to the parameter tree */
+  proto_tree_add_item(parameter_tree, hf_v6_parameter_tag, parameter_tvb, PARAMETER_TAG_OFFSET,    PARAMETER_TAG_LENGTH,    NETWORK_BYTE_ORDER);
   proto_tree_add_item(parameter_tree, hf_parameter_length, parameter_tvb, PARAMETER_LENGTH_OFFSET, PARAMETER_LENGTH_LENGTH, NETWORK_BYTE_ORDER);
 
   switch(tag) {
@@ -1474,12 +1468,16 @@ dissect_parameter(tvbuff_t *parameter_tvb, packet_info *pinfo, proto_tree *tree,
   length         = tvb_get_ntohs(parameter_tvb, PARAMETER_LENGTH_OFFSET);
   padding_length = tvb_length(parameter_tvb) - length;
 
+
+  if (!tree && tag != PROTOCOL_DATA_PARAMETER_TAG)
+    return;	/* Nothing to do here */
+
   /* create proto_tree stuff */
   parameter_item   = proto_tree_add_text(m3ua_tree, parameter_tvb, PARAMETER_HEADER_OFFSET, tvb_length(parameter_tvb), val_to_str(tag, parameter_tag_values, "Unknown parameter"));
   parameter_tree   = proto_item_add_subtree(parameter_item, ett_parameter);
 
-  /* add tag and length to the sua tree */
-  proto_tree_add_item(parameter_tree, hf_v5_parameter_tag, parameter_tvb, PARAMETER_TAG_OFFSET,    PARAMETER_TAG_LENGTH,    NETWORK_BYTE_ORDER);
+  /* add tag and length to the parameter tree */
+  proto_tree_add_item(parameter_tree, hf_parameter_tag,    parameter_tvb, PARAMETER_TAG_OFFSET,    PARAMETER_TAG_LENGTH,    NETWORK_BYTE_ORDER);
   proto_tree_add_item(parameter_tree, hf_parameter_length, parameter_tvb, PARAMETER_LENGTH_OFFSET, PARAMETER_LENGTH_LENGTH, NETWORK_BYTE_ORDER);
 
   switch(tag) {
@@ -1609,8 +1607,11 @@ dissect_message(tvbuff_t *message_tvb, packet_info *pinfo, proto_tree *tree, pro
   	dissect_v5_common_header(common_header_tvb, pinfo, m3ua_tree);
   else
     dissect_common_header(common_header_tvb, pinfo, m3ua_tree);
-  if (m3ua_tree)
-    dissect_parameters(parameters_tvb, pinfo, tree, m3ua_tree);
+
+  /*  Need to dissect (certain) parameters even when !tree, so subdissectors
+   *  (e.g., MTP3) are always called.
+   */
+  dissect_parameters(parameters_tvb, pinfo, tree, m3ua_tree);
 }
 
 static void
@@ -1632,6 +1633,7 @@ dissect_m3ua(tvbuff_t *message_tvb, packet_info *pinfo, proto_tree *tree)
   } else {
     m3ua_tree = NULL;
   };
+
   /* dissect the message */
   dissect_message(message_tvb, pinfo, tree, m3ua_tree);
 }
