@@ -2,23 +2,25 @@
  * Routines for NetWare Core Protocol
  * Gilbert Ramirez <gram@alumni.rice.edu>
  * Modified to allow NCP over TCP/IP decodes by James Coe <jammer@cin.net>
+ * Modified to decode server op-lock
+ * & NDS packets by Greg Morris <gmorris@novell.com>
  *
- * $Id: packet-ncp.c,v 1.65 2002/08/02 23:35:54 jmayer Exp $
+ * $Id: packet-ncp.c,v 1.72 2002/10/11 19:36:13 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
  * Copyright 2000 Gerald Combs
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
@@ -28,6 +30,14 @@
 # include "config.h"
 #endif
 
+#ifdef HAVE_SYS_TYPES_H
+# include <sys/types.h>
+#endif
+
+#ifdef HAVE_NETINET_IN_H
+# include <netinet/in.h>
+#endif
+
 #include <string.h>
 #include <glib.h>
 #include <epan/packet.h>
@@ -64,16 +74,23 @@ static int hf_ncp_data_bytes = -1;
 static int hf_ncp_missing_fraglist_count = -1;
 static int hf_ncp_missing_data_offset = -1;
 static int hf_ncp_missing_data_count = -1;
+static int hf_ncp_oplock_flag = -1;
+static int hf_ncp_oplock_handle = -1;
 static int hf_ncp_completion_code = -1;
 static int hf_ncp_connection_status = -1;
 static int hf_ncp_slot = -1;
 static int hf_ncp_control_code = -1;
+static int hf_ncp_fragment_handle = -1;
+static int hf_lip_echo = -1;
+/*static int hf_ping_version = -1;*/
 
 gint ett_ncp = -1;
+gint ett_nds = -1;
 static gint ett_ncp_system_flags = -1;
 
 /* desegmentation of NCP over TCP */
 static gboolean ncp_desegment = TRUE;
+/*static int ncp_nds_true = FALSE;*/
 
 static dissector_handle_t data_handle;
 
@@ -83,6 +100,7 @@ static dissector_handle_t data_handle;
 #define NCP_RQST_HDR_LENGTH	7
 #define NCP_RPLY_HDR_LENGTH	8
 
+
 /* Hash functions */
 gint  ncp_equal (gconstpointer v, gconstpointer v2);
 guint ncp_hash  (gconstpointer v);
@@ -123,6 +141,10 @@ static const value_string ncp_ip_signature[] = {
 	(c) 1996 by Steve Conner & Diane Conner
 	Published by Annabooks, San Diego, California
         ISBN: 0-929392-31-0
+        
+ And:
+    http:developer.novell.com
+    NCP documentation        
 
 */
 
@@ -144,8 +166,10 @@ static value_string ncp_type_vals[] = {
 	{ NCP_SERVICE_REPLY,	"Service reply" },
 	{ NCP_WATCHDOG,		"Watchdog" },
 	{ NCP_DEALLOCATE_SLOT,	"Destroy service connection" },
+    { NCP_BROADCAST_SLOT,   "Server Broadcast" },
 	{ NCP_BURST_MODE_XFER,	"Burst mode transfer" },
 	{ NCP_POSITIVE_ACK,	"Request being processed" },
+    { NCP_LIP_ECHO, "Large Internet Packet Echo" },
 	{ 0,			NULL }
 };
 
@@ -179,6 +203,8 @@ dissect_ncp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 	int				offset;
 	gint				length_remaining;
 	tvbuff_t       			*next_tvb;
+	guint32				testvar = 0;
+	guint8				subfunction;
 
 	if (check_col(pinfo->cinfo, COL_PROTOCOL))
 		col_set_str(pinfo->cinfo, COL_PROTOCOL, "NCP");
@@ -218,15 +244,16 @@ dissect_ncp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 	/* Record the offset where the NCP common header starts */
 	commhdr = hdr_offset;
 
-	header.type		= tvb_get_ntohs(tvb, commhdr);
+	header.type		    = tvb_get_ntohs(tvb, commhdr);
 	header.sequence		= tvb_get_guint8(tvb, commhdr+2);
 	header.conn_low		= tvb_get_guint8(tvb, commhdr+3);
 	header.conn_high	= tvb_get_guint8(tvb, commhdr+5);
 
 	if (check_col(pinfo->cinfo, COL_INFO)) {
-		col_add_fstr(pinfo->cinfo, COL_INFO,
+	    col_add_fstr(pinfo->cinfo, COL_INFO,
 		    "%s",
 		    val_to_str(header.type, ncp_type_vals, "Unknown type (0x%04x)"));
+
 	}
 
 	nw_connection = (header.conn_high << 16) + header.conn_low;
@@ -254,16 +281,16 @@ dissect_ncp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 	 */
 	switch (header.type) {
 
-	case NCP_ALLOCATE_SLOT:		/* Allocate Slot Request */
-	case NCP_SERVICE_REQUEST:	/* Server NCP Request */
-	case NCP_SERVICE_REPLY:		/* Server NCP Reply */
-	case NCP_WATCHDOG:		/* Watchdog Packet */
-	case NCP_DEALLOCATE_SLOT:	/* Deallocate Slot Request */
-	case NCP_POSITIVE_ACK:		/* Positive Acknowledgement */
-	default:
+	case NCP_BROADCAST_SLOT:    /* Server Broadcast */
 		proto_tree_add_uint(ncp_tree, hf_ncp_seq,	tvb, commhdr + 2, 1, header.sequence);
 		proto_tree_add_uint(ncp_tree, hf_ncp_connection,tvb, commhdr + 3, 3, nw_connection);
 		proto_tree_add_item(ncp_tree, hf_ncp_task,	tvb, commhdr + 4, 1, FALSE);
+		proto_tree_add_item(ncp_tree, hf_ncp_oplock_flag, tvb, commhdr + 9, 1, FALSE);
+		proto_tree_add_item(ncp_tree, hf_ncp_oplock_handle, tvb, commhdr + 10, 4, FALSE);
+		break;
+
+	case NCP_LIP_ECHO:    /* Lip Echo Packet */
+		proto_tree_add_item(ncp_tree, hf_lip_echo, tvb, commhdr, 2, FALSE);
 		break;
 
 	case NCP_BURST_MODE_XFER:	/* Packet Burst Packet */
@@ -360,6 +387,28 @@ dissect_ncp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 		proto_tree_add_item(ncp_tree, hf_ncp_missing_fraglist_count,
 		    tvb, commhdr + 34, 2, FALSE);
 		break;
+
+	case NCP_ALLOCATE_SLOT:		/* Allocate Slot Request */
+		length_remaining = tvb_length_remaining(tvb, commhdr + 4);
+		if (length_remaining > 4) { 
+			testvar = tvb_get_ntohl(tvb, commhdr+4);
+			if (testvar == 0x4c495020) {
+				proto_tree_add_item(ncp_tree, hf_lip_echo, tvb, commhdr+4, 13, FALSE);
+				break;
+			}
+		}
+		/* otherwise fall through */
+    
+	case NCP_POSITIVE_ACK:		/* Positive Acknowledgement */
+	case NCP_SERVICE_REQUEST:	/* Server NCP Request */
+	case NCP_SERVICE_REPLY:		/* Server NCP Reply */
+	case NCP_WATCHDOG:		/* Watchdog Packet */
+	case NCP_DEALLOCATE_SLOT:	/* Deallocate Slot Request */
+	default:
+		proto_tree_add_uint(ncp_tree, hf_ncp_seq,	tvb, commhdr + 2, 1, header.sequence);
+		proto_tree_add_uint(ncp_tree, hf_ncp_connection,tvb, commhdr + 3, 3, nw_connection);
+		proto_tree_add_item(ncp_tree, hf_ncp_task,	tvb, commhdr + 4, 1, FALSE);
+		break;
 	}
 
 	/*
@@ -368,11 +417,46 @@ dissect_ncp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 	switch (header.type) {
 
 	case NCP_ALLOCATE_SLOT:		/* Allocate Slot Request */
+		length_remaining = tvb_length_remaining(tvb, commhdr + 4);
+		if (length_remaining > 4) {
+			testvar = tvb_get_ntohl(tvb, commhdr+4);
+			if (testvar == 0x4c495020) {
+				proto_tree_add_text(ncp_tree, tvb, commhdr, -1,
+				    "Lip Echo Packet");
+				/*break;*/
+			}
+		}
+
 	case NCP_SERVICE_REQUEST:	/* Server NCP Request */
 	case NCP_DEALLOCATE_SLOT:	/* Deallocate Slot Request */
+	case NCP_BROADCAST_SLOT:	/* Server Broadcast Packet */
 		next_tvb = tvb_new_subset(tvb, hdr_offset, -1, -1);
-		dissect_ncp_request(next_tvb, pinfo, nw_connection,
-			header.sequence, header.type, ncp_tree);
+		if (tvb_get_guint8(tvb, commhdr+6) == 0x68) {
+			subfunction = tvb_get_guint8(tvb, commhdr+7);
+			switch (subfunction) {
+
+			case 0x02:	/* NDS Frag Packet to decode */
+				dissect_nds_request(next_tvb, pinfo,
+				    nw_connection, header.sequence,
+				    header.type, ncp_tree);
+				break;
+
+			case 0x01:	/* NDS Ping */
+				dissect_ping_req(next_tvb, pinfo,
+				    nw_connection, header.sequence,
+				    header.type, ncp_tree);
+				break;
+
+			default:
+				dissect_ncp_request(next_tvb, pinfo,
+				    nw_connection, header.sequence,
+				    header.type, ncp_tree);
+				break;
+			 }
+		} else {
+			dissect_ncp_request(next_tvb, pinfo, nw_connection,
+			    header.sequence, header.type, ncp_tree);
+		}
 		break;
 
 	case NCP_SERVICE_REPLY:		/* Server NCP Reply */
@@ -441,9 +525,14 @@ dissect_ncp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 		}
 		break;
 
+	case NCP_LIP_ECHO:		/* LIP Echo Packet */
+		proto_tree_add_text(ncp_tree, tvb, commhdr, -1,
+		    "Lip Echo Packet");
+		break;
+
 	default:
 		if (tree) {
-			proto_tree_add_text(ncp_tree, tvb, commhdr + 6, -1,
+		    proto_tree_add_text(ncp_tree, tvb, commhdr + 6, -1,
 			    "%s packets not supported yet",
 			    val_to_str(header.type, ncp_type_vals,
 				"Unknown type (0x%04x)"));
@@ -472,11 +561,12 @@ get_ncp_pdu_len(tvbuff_t *tvb, int offset)
   signature = tvb_get_ntohl(tvb, offset);
   if (signature != NCPIP_RQST && signature != NCPIP_RPLY)
     return tvb_length_remaining(tvb, offset);
-  
+
   /*
    * Get the length of the NCP-over-TCP packet.  Strip off the "has
    * signature" flag.
    */
+
   return tvb_get_ntohl(tvb, offset + 4) & 0x7fffffff;
 }
 
@@ -534,6 +624,14 @@ proto_register_ncp(void)
       { "Task Number",     	"ncp.task",
 	FT_UINT8, BASE_DEC, NULL, 0x0,
 	"", HFILL }},
+    { &hf_ncp_oplock_flag,
+      { "Oplock Flag",    "ncp.oplock_flag",
+	FT_UINT8, BASE_HEX, NULL, 0x0,
+	"", HFILL }},
+    { &hf_ncp_oplock_handle,
+      { "File Handle",    "ncp.oplock_handle",
+	FT_UINT16, BASE_HEX, NULL, 0x0,
+	"", HFILL }},
     { &hf_ncp_stream_type,
       { "Stream Type",     	"ncp.stream_type",
 	FT_UINT8, BASE_HEX, NULL, 0x0,
@@ -618,10 +716,20 @@ proto_register_ncp(void)
       { "Control Code",    "ncp.control_code",
 	FT_UINT8, BASE_DEC, NULL, 0x0,
 	"", HFILL }},
+    { &hf_ncp_fragment_handle,
+      { "Fragment Handle",    "ncp.fragger_hndl",
+    FT_UINT16, BASE_HEX, NULL, 0x0,
+    "", HFILL }},
+    { &hf_lip_echo,
+      { "Large Internet Packet Echo",    "ncp.lip_echo",
+    FT_STRING, BASE_NONE, NULL, 0x0,
+    "", HFILL }},
+  
   };
   static gint *ett[] = {
     &ett_ncp,
     &ett_ncp_system_flags,
+    &ett_nds,
   };
   module_t *ncp_module;
 
@@ -652,3 +760,5 @@ proto_reg_handoff_ncp(void)
 
   data_handle = find_dissector("data");
 }
+
+
