@@ -1,5 +1,5 @@
 /* packet-nfs.h (c) 1999 Uwe Girlich */
-/* $Id: packet-nfs.h,v 1.11 2002/05/21 21:13:42 guy Exp $ */
+/* $Id: packet-nfs.h,v 1.13 2003/04/01 04:38:05 guy Exp $ */
 
 #ifndef __PACKET_NFS_H__
 #define __PACKET_NFS_H__
@@ -103,6 +103,9 @@ int dissect_fhandle(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *t
 int dissect_nfs_fh3(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree,
     char *name);
 
+int dissect_nfs_post_op_attr(tvbuff_t *tvb, int offset, proto_tree *tree, 
+		char* name);
+
 void nfs_name_snoop_add_name(int xid, tvbuff_t *tvb, int name_offset, int name_len, int parent_offset, int parent_len, unsigned char *name);
 
 
@@ -110,7 +113,7 @@ extern gboolean nfs_fhandle_reqrep_matching;
 extern GHashTable *nfs_fhandle_frame_table;
 typedef struct nfs_fhandle_data {
 	int len;
-	unsigned char *fh;
+	const unsigned char *fh;
 	tvbuff_t *tvb;
 } nfs_fhandle_data_t;
 void dissect_fhandle_hidden(packet_info *pinfo, proto_tree *tree, nfs_fhandle_data_t *nfd);
