@@ -2,7 +2,7 @@
  * Routines for SMB Browser packet dissection
  * Copyright 1999, Richard Sharpe <rsharpe@ns.aus.com>
  *
- * $Id: packet-smb-browse.c,v 1.24 2002/08/28 21:00:31 jmayer Exp $
+ * $Id: packet-smb-browse.c,v 1.30 2003/04/30 02:35:19 gerald Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -41,6 +41,7 @@
 #include "alignment.h"
 
 #include "packet-smb-browse.h"
+#include "packet-dcerpc.h"
 
 static int proto_smb_browse = -1;
 static int hf_command = -1;
@@ -48,6 +49,10 @@ static int hf_update_count = -1;
 static int hf_periodicity = -1;
 static int hf_server_name = -1;
 static int hf_mb_server_name = -1;
+static int hf_mb_reset_command = -1;
+static int hf_mb_reset_demote = -1;
+static int hf_mb_reset_flush = -1;
+static int hf_mb_reset_stop = -1;
 static int hf_os_major = -1;
 static int hf_os_minor = -1;
 static int hf_server_type = -1;
@@ -106,7 +111,7 @@ static gint ett_browse_flags = -1;
 static gint ett_browse_election_criteria = -1;
 static gint ett_browse_election_os = -1;
 static gint ett_browse_election_desire = -1;
-
+static gint ett_browse_reset_cmd_flags = -1;
 
 #define SERVER_WORKSTATION		0
 #define SERVER_SERVER			1
@@ -161,6 +166,28 @@ static const value_string server_types[] = {
 	{0,	NULL}
 };
 
+static const value_string resetbrowserstate_command_names[] = {
+  { 0x01, "Stop being a master browser and become a backup browser"},
+  { 0x02, "Discard browse lists, stop being a master browser, and try again"},
+  { 0x04, "Stop being a master browser for ever"},
+  { 0, NULL}
+};
+
+static true_false_string tfs_demote_to_backup = {
+	"Demote an LMB to a Backup Browser",
+	"Do not demote an LMB to a Backup Browser"
+};
+
+static true_false_string tfs_flush_browse_list = {
+	"Flush the Browse List",
+	"Do not Flush the Browse List"
+};
+
+static true_false_string tfs_stop_being_lmb = {
+	"Stop Being a Local Master Browser",
+	"Do not Stop Being a Local Master Browser"
+};
+
 static const true_false_string tfs_workstation = {
 	"This is a Workstation",
 	"This is NOT a Workstation"
@@ -308,6 +335,7 @@ static const true_false_string tfs_desire_nt = {
 #define BROWSE_BECOME_BACKUP			11
 #define BROWSE_DOMAIN_ANNOUNCEMENT		12
 #define BROWSE_MASTER_ANNOUNCEMENT		13
+#define BROWSE_RESETBROWSERSTATE_ANNOUNCEMENT   14
 #define BROWSE_LOCAL_MASTER_ANNOUNCEMENT	15
 
 static const value_string commands[] = {
@@ -319,6 +347,7 @@ static const value_string commands[] = {
 	{BROWSE_BECOME_BACKUP,		"Become Backup Browser"},
 	{BROWSE_DOMAIN_ANNOUNCEMENT,	"Domain/Workgroup Announcement"},
 	{BROWSE_MASTER_ANNOUNCEMENT,	"Master Announcement"},
+	{BROWSE_RESETBROWSERSTATE_ANNOUNCEMENT, "Reset Browser State Announcement"},
 	{BROWSE_LOCAL_MASTER_ANNOUNCEMENT,"Local Master Announcement"},
 	{0,				NULL}
 };
@@ -434,16 +463,35 @@ dissect_election_criterion(tvbuff_t *tvb, proto_tree *parent_tree, int offset)
 /*
  * XXX - this causes non-browser packets to have browser fields.
  */
-void
-dissect_smb_server_type_flags(tvbuff_t *tvb, packet_info *pinfo,
-    proto_tree *parent_tree, int offset, gboolean infoflag)
+int
+dissect_smb_server_type_flags(tvbuff_t *tvb, int offset, packet_info *pinfo,
+			      proto_tree *parent_tree, char *drep, 
+			      gboolean infoflag)
 {
 	proto_tree *tree = NULL;
 	proto_item *item = NULL;
 	guint32 flags;
 	int i;
 
-	flags = tvb_get_letohl(tvb, offset);
+	if (drep != NULL) {
+		/*
+		 * Called from a DCE RPC protocol dissector, for a
+		 * protocol where a 32-bit NDR integer contains
+		 * an server type mask; extract the server type mask
+		 * with an NDR call.
+		 */
+		offset = dissect_ndr_uint32(
+			tvb, offset, pinfo, tree, drep, hf_server_type, &flags);
+	} else {
+		/*
+		 * Called from SMB browser or RAS, where the server type
+		 * mask is just a 4-byte little-endian quantity with no
+		 * special NDR alignment requirement; extract it with
+		 * "tvb_get_letohl()".
+		 */
+		flags = tvb_get_letohl(tvb, offset);
+		offset += 4;
+	}
 
 	if (parent_tree) {
 		item = proto_tree_add_uint(parent_tree, hf_server_type, tvb, offset, 4, flags);
@@ -512,6 +560,7 @@ dissect_smb_server_type_flags(tvbuff_t *tvb, packet_info *pinfo,
 	proto_tree_add_boolean(tree, hf_server_type_domainenum,
 		tvb, offset, 4, flags);
 
+	return offset;
 }
 
 
@@ -525,7 +574,7 @@ dissect_mailslot_browse(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tr
 	guint32 periodicity;
 	char host_name[17];
 	guint namelen;
-	guint8 server_count;
+	guint8 server_count, reset_cmd;
 	int i;
 	guint32 uptime;
 
@@ -563,7 +612,7 @@ dissect_mailslot_browse(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tr
 	switch (cmd) {
 	case BROWSE_DOMAIN_ANNOUNCEMENT:
 	case BROWSE_LOCAL_MASTER_ANNOUNCEMENT:
-	case BROWSE_HOST_ANNOUNCE:
+	case BROWSE_HOST_ANNOUNCE: {
 		/* update count */
 		proto_tree_add_item(tree, hf_update_count, tvb, offset, 1, TRUE);
 		offset += 1;
@@ -577,7 +626,7 @@ dissect_mailslot_browse(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tr
 		offset += 4;
 
 		/* server name */
-		tvb_get_nstringz0(tvb, offset, 16, host_name);
+		tvb_get_nstringz0(tvb, offset, sizeof(host_name), host_name);
 		if (check_col(pinfo->cinfo, COL_INFO)) {
 			col_append_fstr(pinfo->cinfo, COL_INFO, " %s", host_name);
 		}
@@ -599,8 +648,8 @@ dissect_mailslot_browse(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tr
 		offset += 1;
 
 		/* server type flags */
-		dissect_smb_server_type_flags(tvb, pinfo, tree, offset, TRUE);
-		offset += 4;
+		offset = dissect_smb_server_type_flags(
+			tvb, offset, pinfo, tree, NULL, TRUE);
 
 		if (cmd == BROWSE_DOMAIN_ANNOUNCEMENT) {
 			/*
@@ -637,8 +686,10 @@ dissect_mailslot_browse(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tr
 			tvb, offset, namelen, TRUE);
 		offset += namelen;
 		break;
+	}
+	case BROWSE_REQUEST_ANNOUNCE: {
+		char *computer_name;
 
-	case BROWSE_REQUEST_ANNOUNCE:
 		/* unused/unknown flags */
 		proto_tree_add_item(tree, hf_unused_flags,
 			tvb, offset, 1, TRUE);
@@ -648,8 +699,19 @@ dissect_mailslot_browse(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tr
 		namelen = tvb_strsize(tvb, offset);
 		proto_tree_add_item(tree, hf_response_computer_name,
 			tvb, offset, namelen, TRUE);
+
+		computer_name = g_malloc(namelen);
+		tvb_get_nstringz0(tvb, offset, namelen, computer_name);
+
+		if (check_col(pinfo->cinfo, COL_INFO))
+			col_append_fstr(
+				pinfo->cinfo, COL_INFO, " %s", computer_name);
+
+		g_free(computer_name);
+
 		offset += namelen;
 		break;
+	}
 
 	case BROWSE_ELECTION_REQUEST:
 		/* election version */
@@ -716,6 +778,26 @@ dissect_mailslot_browse(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tr
 		offset += namelen;
 		break;
 
+	case BROWSE_RESETBROWSERSTATE_ANNOUNCEMENT: {
+	        proto_tree *sub_tree;
+	        proto_item *reset_item;
+
+		/* the subcommand follows ... one of three values */
+
+	        reset_cmd = tvb_get_guint8(tvb, offset);
+		reset_item = proto_tree_add_uint(tree, hf_mb_reset_command, tvb, 
+						 offset, 1, reset_cmd);
+		sub_tree = proto_item_add_subtree(item, ett_browse_reset_cmd_flags);
+		proto_tree_add_boolean(sub_tree, hf_mb_reset_demote, tvb, 
+				       offset, 1, reset_cmd);
+		proto_tree_add_boolean(sub_tree, hf_mb_reset_flush, tvb, 
+				       offset, 1, reset_cmd);
+		proto_tree_add_boolean(sub_tree, hf_mb_reset_stop, tvb, 
+				       offset, 1, reset_cmd);
+		offset += 1;
+		break;
+	}
+
 	case BROWSE_BECOME_BACKUP:
 		/* name of browser to promote */
 		namelen = tvb_strsize(tvb, offset);
@@ -791,14 +873,14 @@ dissect_mailslot_lanman(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tr
 	case BROWSE_DOMAIN_ANNOUNCEMENT:
 	case BROWSE_LOCAL_MASTER_ANNOUNCEMENT:
 	case BROWSE_HOST_ANNOUNCE:
+
 		/* update count */
 		proto_tree_add_item(tree, hf_update_count, tvb, offset, 1, TRUE);
 		offset += 1;
 
 		/* server type flags */
-		dissect_smb_server_type_flags(tvb, pinfo, tree, offset,
-		    hf_server_type);
-		offset += 4;
+		offset = dissect_smb_server_type_flags(
+			tvb, offset, pinfo, tree, NULL, TRUE);
 
 		/* OS major version */
 		proto_tree_add_item(tree, hf_os_major, tvb, offset, 1, TRUE);
@@ -863,6 +945,19 @@ proto_register_smb_browse(void)
 			{ "Master Browser Server Name", "browser.mb_server", FT_STRING, BASE_NONE,
 			NULL, 0, "BROWSE Master Browser Server Name", HFILL }},
 
+		{ &hf_mb_reset_command,
+		  { "ResetBrowserState Command", "browser.reset_cmd", FT_UINT8,
+		    BASE_HEX, VALS(&resetbrowserstate_command_names), 0,
+		    "ResetBrowserState Command", HFILL }},
+		{ &hf_mb_reset_demote,
+		  { "Demote LMB", "browser.reset_cmd.demote", FT_BOOLEAN, 
+		    8, TFS(&tfs_demote_to_backup), 0x01, "Demote LMB", HFILL}}, 
+		{ &hf_mb_reset_flush,
+		  { "Flush Browse List", "browser.reset_cmd.flush", FT_BOOLEAN,
+		    8, TFS(&tfs_flush_browse_list), 0x02, "Flush Browse List", HFILL}},
+		{ &hf_mb_reset_stop,
+		  { "Stop Being LMB", "browser.reset_cmd.stop_lmb", FT_BOOLEAN,
+		    8, TFS(&tfs_stop_being_lmb), 0x04, "Stop Being LMB", HFILL}},
 		{ &hf_os_major,
 			{ "OS Major Version", "browser.os_major", FT_UINT8, BASE_DEC,
 			NULL, 0, "Operating System Major Version", HFILL }},
@@ -1078,7 +1173,8 @@ proto_register_smb_browse(void)
 		&ett_browse_flags,
 		&ett_browse_election_criteria,
 		&ett_browse_election_os,
-		&ett_browse_election_desire
+		&ett_browse_election_desire,
+		&ett_browse_reset_cmd_flags,
 	};
 
     	proto_smb_browse = proto_register_protocol("Microsoft Windows Browser Protocol",
