@@ -2,7 +2,7 @@
  * Routines for SMB packet dissection
  * Copyright 1999, Richard Sharpe <rsharpe@ns.aus.com>
  *
- * $Id: packet-smb-common.h,v 1.15 2002/08/28 21:00:31 jmayer Exp $
+ * $Id: packet-smb-common.h,v 1.21 2003/04/28 04:44:54 tpot Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -45,9 +45,9 @@
 
 int dissect_smb_unknown(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset);
 
-int display_unicode_string(tvbuff_t *tvb, proto_tree *tree, int offset, int hf_index);
+int display_unicode_string(tvbuff_t *tvb, proto_tree *tree, int offset, int hf_index, char **data);
 
-int display_ms_string(tvbuff_t *tvb, proto_tree *tree, int offset, int hf_index);
+int display_ms_string(tvbuff_t *tvb, proto_tree *tree, int offset, int hf_index, char **data);
 
 const gchar *get_unicode_or_ascii_string(tvbuff_t *tvb, int *offsetp,
     gboolean useunicode, int *len, gboolean nopad, gboolean exactlen,
@@ -55,10 +55,28 @@ const gchar *get_unicode_or_ascii_string(tvbuff_t *tvb, int *offsetp,
 
 int dissect_smb_64bit_time(tvbuff_t *tvb, proto_tree *tree, int offset, int hf_date);
 
-int dissect_nt_sid(tvbuff_t *tvb, int offset, proto_tree *parent_tree, char *name);
+int dissect_nt_sid(tvbuff_t *tvb, int offset, proto_tree *parent_tree, 
+		   char *name, char **sid_str);
+
+/* Stuff for dissecting NT access masks */
+
+typedef void (nt_access_mask_fn_t)(tvbuff_t *tvb, gint offset,
+				   proto_tree *tree, guint32 access);
+
+struct access_mask_info {
+	char *specific_rights_name;
+	nt_access_mask_fn_t *specific_rights_fn;
+};
 
 int
-dissect_nt_sec_desc(tvbuff_t *tvb, int offset, proto_tree *parent_tree, int len);
+dissect_nt_access_mask(tvbuff_t *tvb, gint offset, packet_info *pinfo,
+		       proto_tree *tree, char *drep, int hfindex,
+		       struct access_mask_info *ami);
+
+int
+dissect_nt_sec_desc(tvbuff_t *tvb, int offset, packet_info *pinfo,
+		    proto_tree *parent_tree, char *drep, int len, 
+		    struct access_mask_info *ami);
 
 extern const value_string share_type_vals[];
 
