@@ -2,7 +2,7 @@
  * Routines for SMB net logon packet dissection
  * Copyright 2000, Jeffrey C. Foster <jfoste@woodward.com>
  *
- * $Id: packet-smb-logon.c,v 1.28 2002/08/28 21:00:31 jmayer Exp $
+ * $Id: packet-smb-logon.c,v 1.32 2003/05/01 18:11:20 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -224,13 +224,13 @@ dissect_smb_logon_request(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tre
 	/*** 0x00 (LM1.0/LM2.0 LOGON Request) ***/
 
 	/* computer name */
-	offset = display_ms_string(tvb, tree, offset, hf_computer_name);
+	offset = display_ms_string(tvb, tree, offset, hf_computer_name, NULL);
 
 	/* user name */
-	offset = display_ms_string(tvb, tree, offset, hf_user_name);
+	offset = display_ms_string(tvb, tree, offset, hf_user_name, NULL);
 
 	/* mailslot name */
-	offset = display_ms_string(tvb, tree, offset, hf_mailslot_name);
+	offset = display_ms_string(tvb, tree, offset, hf_mailslot_name, NULL);
 
 	/*$$$$$ here add the Mailslot to the response list (if needed) */
 
@@ -256,10 +256,10 @@ dissect_smb_logon_LM10_resp(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *t
 	/*** 0x01 LanMan 1.0 Logon response ***/
 
 	/* user name */
-	offset = display_ms_string(tvb, tree, offset, hf_user_name);
+	offset = display_ms_string(tvb, tree, offset, hf_user_name, NULL);
 
 	/* script name */
-	offset = display_ms_string(tvb, tree, offset, hf_script_name);
+	offset = display_ms_string(tvb, tree, offset, hf_script_name, NULL);
 
 	return offset;
 }
@@ -274,10 +274,10 @@ dissect_smb_logon_2(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree, int
 	/*** 0x04  LM1.0 Query - Distributed Query Response ***/
 
 	/* computer name */
-	offset = display_ms_string(tvb, tree, offset, hf_computer_name);
+	offset = display_ms_string(tvb, tree, offset, hf_computer_name, NULL);
 
 	/* mailslot name */
-	offset = display_ms_string(tvb, tree, offset, hf_mailslot_name);
+	offset = display_ms_string(tvb, tree, offset, hf_mailslot_name, NULL);
 
 	/* NT version */
   	proto_tree_add_item(tree, hf_nt_version, tvb, offset, 2, TRUE);
@@ -297,7 +297,7 @@ dissect_smb_logon_LM20_resp(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *t
 	/*** 0x06 (LM2.0 LOGON Response)	***/
 
 	/* server name */
-	offset = display_ms_string(tvb, tree, offset, hf_server_name);
+	offset = display_ms_string(tvb, tree, offset, hf_server_name, NULL);
 
 	/* LM token */
 	offset = display_LM_token(tvb, offset, tree);
@@ -310,13 +310,21 @@ dissect_smb_logon_LM20_resp(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *t
 static int
 dissect_smb_pdc_query(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree, int offset)
 {
+	char *name = NULL;
+
 	/*** 0x07 Query for Primary PDC  ***/
 
 	/* computer name */
-	offset = display_ms_string(tvb, tree, offset, hf_computer_name);
+	offset = display_ms_string(tvb, tree, offset, hf_computer_name, &name);
+
+	if (name && check_col(pinfo->cinfo, COL_INFO)) {
+		col_append_fstr(pinfo->cinfo, COL_INFO, " from %s", name);
+		g_free(name);
+		name = NULL;
+	}
 
 	/* mailslot name */
-	offset = display_ms_string(tvb, tree, offset, hf_mailslot_name);
+	offset = display_ms_string(tvb, tree, offset, hf_mailslot_name, NULL);
 
 	if (tvb_reported_length_remaining(tvb, offset) > 2) {
 		/*
@@ -331,7 +339,7 @@ dissect_smb_pdc_query(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree, i
 		if (offset % 2) offset++;      /* word align ... */
 
 		/* Unicode computer name */
-		offset = display_unicode_string(tvb, tree, offset, hf_unicode_computer_name);
+		offset = display_unicode_string(tvb, tree, offset, hf_unicode_computer_name, NULL);
 
 		/* NT version */
 	  	proto_tree_add_item(tree, hf_nt_version, tvb, offset, 4, TRUE);
@@ -355,21 +363,34 @@ dissect_smb_pdc_startup(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree,
 	/*** 0x08  Announce startup of PDC ***/
 
 	/* pdc name */
-	offset = display_ms_string(tvb, tree, offset, hf_pdc_name);
+	offset = display_ms_string(tvb, tree, offset, hf_pdc_name, NULL);
 
 	/* A short Announce will not have the rest */
 
 	if (tvb_reported_length_remaining(tvb, offset) != 0) {
+	  char *name = NULL;
 
 	  if (offset % 2) offset++;      /* word align ... */
 
 	  /* pdc name */
-	  offset = display_unicode_string(tvb, tree, offset, hf_unicode_pdc_name);
+	  offset = display_unicode_string(tvb, tree, offset, hf_unicode_pdc_name, &name);
+
+	  if (name && check_col(pinfo->cinfo, COL_INFO)) {
+		  col_append_fstr(pinfo->cinfo, COL_INFO, ": host %s", name);
+		  g_free(name);
+		  name = NULL;
+	  }
 
 	  if (offset % 2) offset++;
 
 	  /* domain name */
-	  offset = display_unicode_string(tvb, tree, offset, hf_domain_name);
+	  offset = display_unicode_string(tvb, tree, offset, hf_domain_name, &name);
+
+	  if (name && check_col(pinfo->cinfo, COL_INFO)) {
+		  col_append_fstr(pinfo->cinfo, COL_INFO, ", domain %s", name);
+		  g_free(name);
+		  name = NULL;
+	  }
 
 	  /* NT version */
   	  proto_tree_add_item(tree, hf_nt_version, tvb, offset, 4, TRUE);
@@ -434,10 +455,10 @@ dissect_announce_change(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree,
 	offset += 4;
 
 	/* pdc name */
-	offset = display_ms_string(tvb, tree, offset, hf_pdc_name);
+	offset = display_ms_string(tvb, tree, offset, hf_pdc_name, NULL);
 
 	/* domain name */
-	offset = display_ms_string(tvb, tree, offset, hf_domain_name);
+	offset = display_ms_string(tvb, tree, offset, hf_domain_name, NULL);
 
 	if (offset % 2) offset++;      /* word align ... */
 
@@ -446,10 +467,10 @@ dissect_announce_change(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree,
 		 * XXX - older protocol versions don't have this stuff?
 		 */
 		/* pdc name */
-		offset = display_unicode_string(tvb, tree, offset, hf_unicode_pdc_name);
+		offset = display_unicode_string(tvb, tree, offset, hf_unicode_pdc_name, NULL);
 
 		/* domain name */
-		offset = display_unicode_string(tvb, tree, offset, hf_domain_name);
+		offset = display_unicode_string(tvb, tree, offset, hf_domain_name, NULL);
 
 		/* DB count */
 		info_count = tvb_get_letohl(tvb, offset);
@@ -489,7 +510,8 @@ dissect_announce_change(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree,
 			offset = ((offset + 3)/4)*4;
 
 			/* Domain SID */
-			offset = dissect_nt_sid(tvb, offset, tree, "Domain");
+			offset = dissect_nt_sid(
+				tvb, offset, tree, "Domain", NULL);
 		}
 
 		/* NT version */
@@ -520,13 +542,13 @@ dissect_smb_sam_logon_req(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tre
 	offset += 2;
 
 	/* computer name */
-	offset = display_unicode_string(tvb, tree, offset, hf_unicode_computer_name);
+	offset = display_unicode_string(tvb, tree, offset, hf_unicode_computer_name, NULL);
 
 	/* user name */
-	offset = display_unicode_string(tvb, tree, offset, hf_user_name);
+	offset = display_unicode_string(tvb, tree, offset, hf_user_name, NULL);
 
 	/* mailslot name */
-	offset = display_ms_string(tvb, tree, offset, hf_mailslot_name);
+	offset = display_ms_string(tvb, tree, offset, hf_mailslot_name, NULL);
 
 	/* account control */
 	offset = dissect_account_control(tvb, tree, offset);
@@ -542,7 +564,7 @@ dissect_smb_sam_logon_req(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tre
 		offset = ((offset + 3)/4)*4;
 
 		/* Domain SID */
-		offset = dissect_nt_sid(tvb, offset, tree, "Domain");
+		offset = dissect_nt_sid(tvb, offset, tree, "Domain", NULL);
 	}
 
 	/* NT version */
@@ -566,7 +588,7 @@ dissect_smb_no_user(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree, int
 	/* 0x0B (Announce no user on machine) */
 
 	/* computer name */
-	offset = display_ms_string(tvb, tree, offset, hf_computer_name);
+	offset = display_ms_string(tvb, tree, offset, hf_computer_name, NULL);
 
 	return offset;
 }
@@ -624,10 +646,10 @@ dissect_smb_acc_update(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree,
 	offset += 4;
 
 	/* computer name */
-	offset = display_ms_string(tvb, tree, offset, hf_computer_name);
+	offset = display_ms_string(tvb, tree, offset, hf_computer_name, NULL);
 
 	/* user name */
-	offset = display_ms_string(tvb, tree, offset, hf_user_name);
+	offset = display_ms_string(tvb, tree, offset, hf_user_name, NULL);
 
 	/* update type */
 	proto_tree_add_item(tree, hf_update_type, tvb, offset, 2, TRUE);
@@ -683,13 +705,13 @@ dissect_smb_sam_logon_resp(tvbuff_t *tvb, packet_info *pinfo _U_,
 	/* Netlogon command 0x13 - decode the SAM logon response from server */
 
 	/* server name */
-	offset = display_unicode_string(tvb, tree, offset, hf_server_name);
+	offset = display_unicode_string(tvb, tree, offset, hf_server_name, NULL);
 
 	/* user name */
-	offset = display_unicode_string(tvb, tree, offset, hf_user_name);
+	offset = display_unicode_string(tvb, tree, offset, hf_user_name, NULL);
 
 	/* domain name */
-	offset = display_unicode_string(tvb, tree, offset, hf_domain_name);
+	offset = display_unicode_string(tvb, tree, offset, hf_domain_name, NULL);
 
 	/* NT version */
 	proto_tree_add_item(tree, hf_nt_version, tvb, offset, 4, TRUE);
@@ -728,7 +750,10 @@ dissect_smb_sam_logon_resp(tvbuff_t *tvb, packet_info *pinfo _U_,
 #define LOGON_SAM_RESPONSE_DURING_LOGON		0x14
 #define LOGON_SAM_USER_UNKNOWN			0x15
 #define LOGON_SAM_INTERROGATE_RESPONSE		0x16
-#define LOGON_LAST_CMD				0x17
+#define LOGON_SAM_AD_USER_UNKNOWN		0x17
+#define LOGON_SAM_UNKNOWN_18			0x18
+#define LOGON_SAM_AD_LOGON_RESPONSE		0x19
+#define LOGON_LAST_CMD				0x20
 
 static const value_string commands[] = {
 	{LOGON_LM10_LOGON_REQUEST,	"LM1.0/LM2.0 LOGON Request"},
@@ -754,6 +779,9 @@ static const value_string commands[] = {
 	{LOGON_SAM_RESPONSE_DURING_LOGON,"SAM Response during LOGON pause"},
 	{LOGON_SAM_USER_UNKNOWN,	"SAM Response - user unknown"},
 	{LOGON_SAM_INTERROGATE_RESPONSE,"SAM Response to Interrogate Request"},
+	{LOGON_SAM_AD_USER_UNKNOWN,	"SAM Active Directory Response - user unknown"},
+	{LOGON_SAM_UNKNOWN_18,		"SAM unknown command 0x18"},
+	{LOGON_SAM_AD_LOGON_RESPONSE,	"Active Directory Response to SAM LOGON request"},
 	{0,	NULL}
 };
 
@@ -780,8 +808,11 @@ static int (*dissect_smb_logon_cmds[])(tvbuff_t *tvb, packet_info *pinfo, proto_
 	dissect_smb_sam_logon_req,  /* 0x12 (SAM LOGON request )	*/
 	dissect_smb_sam_logon_resp, /* 0x13 (SAM LOGON response)	*/
 	dissect_smb_unknown,        /* 0x14 (SAM Response during LOGON Pause) */
-	dissect_smb_unknown,        /* 0x15 (SAM Response User Unknown)      */
-	dissect_smb_unknown,        /* 0x16 (SAM Response to Interrogate)   */
+	dissect_smb_unknown,        /* 0x15 (SAM Response User Unknown)	*/
+	dissect_smb_unknown,        /* 0x16 (SAM Response to Interrogate)*/
+	dissect_smb_unknown,        /* 0x17 (SAM AD response User Unknown*/
+	dissect_smb_unknown,        /* 0x18 (Unknown command)		*/
+	dissect_smb_unknown         /* 0x19 (SAM LOGON AD response)	*/
 };
 
 
