@@ -10,7 +10,7 @@
  *
  * See RFCs 2570-2576 for SNMPv3
  *
- * $Id: packet-snmp.c,v 1.102 2002/11/11 17:34:22 guy Exp $
+ * $Id: packet-snmp.c,v 1.108 2003/04/27 20:57:58 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -52,6 +52,7 @@
 #include <epan/conversation.h>
 #include "etypes.h"
 #include "packet-ipx.h"
+#include "packet-hpext.h"
 
 #ifdef HAVE_SOME_SNMP
 
@@ -69,12 +70,15 @@
 # include <ucd-snmp/default_store.h>
 # include <ucd-snmp/read_config.h>
 # include <ucd-snmp/tools.h>
-# define netsnmp_ds_set_boolean ds_set_boolean
-# define netsnmp_ds_set_int ds_set_int
+#endif /* HAVE_NET_SNMP */
+
+#ifndef NETSNMP_DS_LIBRARY_ID
 # define NETSNMP_DS_LIBRARY_ID DS_LIBRARY_ID
 # define NETSNMP_DS_LIB_NO_TOKEN_WARNINGS DS_LIB_NO_TOKEN_WARNINGS
 # define NETSNMP_DS_LIB_PRINT_SUFFIX_ONLY DS_LIB_PRINT_SUFFIX_ONLY
-#endif /* HAVE_NET_SNMP */
+# define netsnmp_ds_set_boolean ds_set_boolean
+# define netsnmp_ds_set_int ds_set_int
+#endif
 
 #ifdef WIN32
 # include <epan/filesystem.h>
@@ -102,29 +106,41 @@
 #include "packet-snmp.h"
 #include "format-oid.h"
 
-/* Null string of type "guchar[]". */
-static const guchar nullstring[] = "";
-
 /* Take a pointer that may be null and return a pointer that's not null
-   by turning null pointers into pointers to the above null string. */
-#define	SAFE_STRING(s)	(((s) != NULL) ? (s) : nullstring)
+   by turning null pointers into pointers to the above null string,
+   and, if the argument pointer wasn't null, make sure we handle
+   non-printable characters in the string by escaping them. */
+#define	SAFE_STRING(s, l)	(((s) != NULL) ? format_text((s), (l)) : "")
 
 static int proto_snmp = -1;
-static int proto_smux = -1;
 
 static gint ett_snmp = -1;
-static gint ett_smux = -1;
 static gint ett_parameters = -1;
 static gint ett_parameters_qos = -1;
 static gint ett_global = -1;
 static gint ett_flags = -1;
 static gint ett_secur = -1;
 
+static int hf_snmp_version = -1;
+static int hf_snmp_community = -1;
+static int hf_snmp_pdutype = -1;
+static int hf_snmp_agent = -1;
+static int hf_snmp_enterprise = -1;
+static int hf_snmp_traptype = -1;
+static int hf_snmp_spectraptype = -1;
+static int hf_snmp_timestamp = -1;
 static int hf_snmpv3_flags = -1;
 static int hf_snmpv3_flags_auth = -1;
 static int hf_snmpv3_flags_crypt = -1;
 static int hf_snmpv3_flags_report = -1;
 
+static int proto_smux = -1;
+
+static gint ett_smux = -1;
+
+static int hf_smux_version = -1;
+static int hf_smux_pdutype = -1;
+
 static dissector_handle_t snmp_handle;
 static dissector_handle_t data_handle;
 
@@ -237,26 +253,63 @@ static const value_string smux_sout[] = {
 };
 
 /* Error status values */
+#ifndef SNMP_ERR_NOERROR
 #define SNMP_ERR_NOERROR		0
+#endif
+#ifndef SNMP_ERR_TOOBIG
 #define SNMP_ERR_TOOBIG			1
+#endif
+#ifndef SNMP_ERR_NOSUCHNAME
 #define SNMP_ERR_NOSUCHNAME		2
+#endif
+#ifndef SNMP_ERR_BADVALUE
 #define SNMP_ERR_BADVALUE		3
+#endif
+#ifndef SNMP_ERR_READONLY
 #define SNMP_ERR_READONLY		4
-#define SNMP_ERR_GENERROR		5
-
+#endif
+#ifndef SNMP_ERR_GENERR
+#define SNMP_ERR_GENERR			5
+#endif
+#ifndef SNMP_ERR_NOACCESS
 #define SNMP_ERR_NOACCESS		6
+#endif
+#ifndef SNMP_ERR_WRONGTYPE
 #define SNMP_ERR_WRONGTYPE		7
+#endif
+#ifndef SNMP_ERR_WRONGLENGTH
 #define SNMP_ERR_WRONGLENGTH		8
+#endif
+#ifndef SNMP_ERR_WRONGENCODING
 #define SNMP_ERR_WRONGENCODING		9
+#endif
+#ifndef SNMP_ERR_WRONGVALUE
 #define SNMP_ERR_WRONGVALUE		10
+#endif
+#ifndef SNMP_ERR_NOCREATION
 #define SNMP_ERR_NOCREATION		11
+#endif
+#ifndef SNMP_ERR_INCONSISTENTVALUE
 #define SNMP_ERR_INCONSISTENTVALUE	12
+#endif
+#ifndef SNMP_ERR_RESOURCEUNAVAILABLE
 #define SNMP_ERR_RESOURCEUNAVAILABLE	13
+#endif
+#ifndef SNMP_ERR_COMMITFAILED
 #define SNMP_ERR_COMMITFAILED		14
+#endif
+#ifndef SNMP_ERR_UNDOFAILED
 #define SNMP_ERR_UNDOFAILED		15
+#endif
+#ifndef SNMP_ERR_AUTHORIZATIONERROR
 #define SNMP_ERR_AUTHORIZATIONERROR	16
+#endif
+#ifndef SNMP_ERR_NOTWRITABLE
 #define SNMP_ERR_NOTWRITABLE		17
+#endif
+#ifndef SNMP_ERR_INCONSISTENTNAME
 #define SNMP_ERR_INCONSISTENTNAME	18
+#endif
 
 static const value_string error_statuses[] = {
 	{ SNMP_ERR_NOERROR,		"NO ERROR" },
@@ -264,7 +317,7 @@ static const value_string error_statuses[] = {
 	{ SNMP_ERR_NOSUCHNAME,		"NO SUCH NAME" },
 	{ SNMP_ERR_BADVALUE,		"BAD VALUE" },
 	{ SNMP_ERR_READONLY,		"READ ONLY" },
-	{ SNMP_ERR_GENERROR,		"GENERIC ERROR" },
+	{ SNMP_ERR_GENERR,		"GENERIC ERROR" },
 	{ SNMP_ERR_NOACCESS,		"NO ACCESS" },
 	{ SNMP_ERR_WRONGTYPE,		"WRONG TYPE" },
 	{ SNMP_ERR_WRONGLENGTH,		"WRONG LENGTH" },
@@ -283,13 +336,27 @@ static const value_string error_statuses[] = {
 
 /* General SNMP V1 Traps */
 
+#ifndef SNMP_TRAP_COLDSTART
 #define SNMP_TRAP_COLDSTART		0
+#endif
+#ifndef SNMP_TRAP_WARMSTART
 #define SNMP_TRAP_WARMSTART		1
+#endif
+#ifndef SNMP_TRAP_LINKDOWN
 #define SNMP_TRAP_LINKDOWN		2
+#endif
+#ifndef SNMP_TRAP_LINKUP
 #define SNMP_TRAP_LINKUP		3
+#endif
+#ifndef SNMP_TRAP_AUTHFAIL
 #define SNMP_TRAP_AUTHFAIL		4
+#endif
+#ifndef SNMP_TRAP_EGPNEIGHBORLOSS
 #define SNMP_TRAP_EGPNEIGHBORLOSS	5
+#endif
+#ifndef SNMP_TRAP_ENTERPRISESPECIFIC
 #define SNMP_TRAP_ENTERPRISESPECIFIC	6
+#endif
 
 static const value_string trap_types[] = {
 	{ SNMP_TRAP_COLDSTART,		"COLD START" },
@@ -473,10 +540,10 @@ format_oid(subid_t *oid, guint oid_length)
 	 * Get the decoded form of the OID, and add its length to the
 	 * length of the result string.
 	 *
-	 * XXX - check for "malloc" and "sprint_realloc_objid()" failure.
+	 * XXX - check for "sprint_realloc_objid()" failure.
 	 */
 	oid_string_len = 256;
-	oid_string = malloc(oid_string_len);
+	oid_string = g_malloc(oid_string_len);
 	*oid_string = '\0';
 	oid_out_len = 0;
 	sprint_realloc_objid(&oid_string, &oid_string_len, &oid_out_len, 1,
@@ -515,7 +582,7 @@ check_var_length(guint vb_length, guint required_length)
 		/* Enough room for the largest "Length is XXX,
 		   should be XXX" message - 10 digits for each
 		   XXX. */
-		buf = malloc(sizeof badlen_fmt + 10 + 10);
+		buf = g_malloc(sizeof badlen_fmt + 10 + 10);
 		sprintf(buf, badlen_fmt, vb_length, required_length);
 		return buf;
 	}
@@ -602,10 +669,10 @@ format_var(struct variable_list *variable, subid_t *variable_oid,
 	variable->val_len = val_len;
 
 	/*
-	 * XXX - check for "malloc" and "sprint_realloc_objid()" failure.
+	 * XXX - check for "sprint_realloc_objid()" failure.
 	 */
 	buf_len = 256;
-	buf = malloc(buf_len);
+	buf = g_malloc(buf_len);
 	*buf = '\0';
 	out_len = 0;
 	sprint_realloc_value(&buf, &buf_len, &out_len, 1,  variable_oid,
@@ -786,9 +853,8 @@ snmp_variable_decode(proto_tree *snmp_tree,
 			} else {
 				proto_tree_add_text(snmp_tree, asn1->tvb, offset,
 				    length,
-				    "Value: %s: %.*s", vb_type_name,
-				    (int)vb_length,
-				    SAFE_STRING(vb_octet_string));
+				    "Value: %s: %s", vb_type_name,
+				    SAFE_STRING(vb_octet_string, vb_length));
 			}
 #endif /* HAVE_SOME_SNMP */
 		}
@@ -884,6 +950,8 @@ dissect_common_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	subid_t *enterprise;
 	guint enterprise_length;
 
+	guint32 agent_ipaddr;
+
 	guint8 *agent_address;
 	guint agent_address_length;
 
@@ -912,8 +980,8 @@ dissect_common_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		col_add_str(pinfo->cinfo, COL_INFO, pdu_type_string);
 	length = asn1.offset - start;
 	if (tree) {
-		proto_tree_add_text(tree, tvb, offset, length,
-		    "PDU type: %s", pdu_type_string);
+		proto_tree_add_uint(tree, hf_snmp_pdutype, tvb, offset, length,
+		    pdu_type);
 	}
 	offset += length;
 
@@ -995,8 +1063,8 @@ dissect_common_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		}
 		if (tree) {
 			oid_string = format_oid(enterprise, enterprise_length);
-			proto_tree_add_text(tree, tvb, offset, length,
-			    "Enterprise: %s", oid_string);
+			proto_tree_add_string(tree, hf_snmp_enterprise, tvb,
+			    offset, length, oid_string);
 			g_free(oid_string);
 		}
 		g_free(enterprise);
@@ -1039,10 +1107,10 @@ dissect_common_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo,
 				    "Agent address: <length is %u, not 4>",
 				    agent_address_length);
 			} else {
-				proto_tree_add_text(tree, tvb, offset,
-				    length,
-				    "Agent address: %s",
-				    ip_to_str(agent_address));
+				memcpy((guint8 *)&agent_ipaddr, agent_address,
+				    agent_address_length);
+				proto_tree_add_ipv4(tree, hf_snmp_agent, tvb,
+				    offset, length, agent_ipaddr);
 			}
 		}
 		g_free(agent_address);
@@ -1056,9 +1124,8 @@ dissect_common_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo,
 			return;
 		}
 		if (tree) {
-			proto_tree_add_text(tree, tvb, offset, length,
-			    "Trap type: %s",
-			    val_to_str(trap_type, trap_types, "Unknown (%u)"));
+			proto_tree_add_uint(tree, hf_snmp_traptype, tvb,
+			    offset, length, trap_type);
 		}
 		offset += length;
 
@@ -1070,9 +1137,8 @@ dissect_common_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo,
 			return;
 		}
 		if (tree) {
-			proto_tree_add_text(tree, tvb, offset, length,
-			    "Specific trap type: %u (%#x)",
-			    specific_type, specific_type);
+			proto_tree_add_uint(tree, hf_snmp_spectraptype, tvb,
+			    offset, length, specific_type);
 		}
 		offset += length;
 
@@ -1100,8 +1166,8 @@ dissect_common_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		}
 		length = asn1.offset - start;
 		if (tree) {
-			proto_tree_add_text(tree, tvb, offset, length,
-			    "Timestamp: %u", timestamp);
+			proto_tree_add_uint(tree, hf_snmp_timestamp, tvb,
+			    offset, length, timestamp);
 		}
 		offset += length;
 		break;
@@ -1314,6 +1380,7 @@ dissect_snmp_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	guint32 enginetime;
 
 	guchar *msgflags;
+	guchar *commustr;
 	guchar *community;
 	guchar *secparm;
 	guchar *cengineid;
@@ -1374,9 +1441,8 @@ dissect_snmp_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		return;
 	}
 	if (snmp_tree) {
-		proto_tree_add_text(snmp_tree, tvb, offset, length,
-		    "Version: %s",
-		    val_to_str(version, versions, "Unknown version %#x"));
+		proto_tree_add_uint(snmp_tree, hf_snmp_version, tvb, offset,
+		    length, version);
 	}
 	offset += length;
 
@@ -1392,9 +1458,13 @@ dissect_snmp_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo,
 			return;
 		}
 		if (tree) {
-			proto_tree_add_text(snmp_tree, tvb, offset, length,
-			    "Community: %.*s", community_length,
-			    SAFE_STRING(community));
+			commustr = g_malloc(community_length+1);
+			memcpy(commustr, community, community_length);
+			commustr[community_length] = '\0';
+
+			proto_tree_add_string(snmp_tree, hf_snmp_community,
+			    tvb, offset, length, commustr);
+			g_free(commustr);
 		}
 		g_free(community);
 		offset += length;
@@ -1568,9 +1638,8 @@ dissect_snmp_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo,
 			}
 			if (secur_tree) {
 				proto_tree_add_text(secur_tree, tvb, offset,
-				    length, "User Name: %.*s",
-				    username_length,
-				    SAFE_STRING(username));
+				    length, "User Name: %s",
+				    SAFE_STRING(username, username_length));
 			}
 			g_free(username);
 			offset += length;
@@ -1668,8 +1737,8 @@ dissect_snmp_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		}
 		if (snmp_tree) {
 			proto_tree_add_text(snmp_tree, tvb, offset, length,
-			    "Context Name: %.*s", cname_length,
-			    SAFE_STRING(cname));
+			    "Context Name: %s",
+			    SAFE_STRING(cname, cname_length));
 		}
 		g_free(cname);
 		offset += length;
@@ -1762,8 +1831,8 @@ dissect_smux_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo,
 			col_add_str(pinfo->cinfo, COL_INFO, pdu_type_string);
 		length = asn1.offset - start;
 		if (tree) {
-			proto_tree_add_text(smux_tree, tvb, offset, length,
-			    "PDU type: %s", pdu_type_string);
+			proto_tree_add_uint(smux_tree, hf_smux_pdutype, tvb,
+			    offset, length, pdu_type);
 		}
 		offset += length;
 		ret = asn1_uint32_decode (&asn1, &version, &length);
@@ -1773,8 +1842,8 @@ dissect_smux_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo,
 			return;
 		}
 		if (tree) {
-			proto_tree_add_text(smux_tree, tvb, offset, length,
-			    "Version: %d", version);
+			proto_tree_add_uint(smux_tree, hf_smux_version, tvb,
+			    offset, length, version);
 		}
 		offset += length;
 
@@ -1802,8 +1871,8 @@ dissect_smux_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		}
 		if (tree) {
 			proto_tree_add_text(smux_tree, tvb, offset, length,
-			    "Application: %.*s", application_length,
-			     SAFE_STRING(application));
+			    "Application: %s",
+			     SAFE_STRING(application, application_length));
 		}
 		g_free(application);
 		offset += length;
@@ -1817,8 +1886,8 @@ dissect_smux_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		}
 		if (tree) {
 			proto_tree_add_text(smux_tree, tvb, offset, length,
-			    "Password: %.*s", password_length,
-			    SAFE_STRING(password));
+			    "Password: %s",
+			    SAFE_STRING(password, password_length));
 		}
 		g_free(password);
 		offset += length;
@@ -1831,8 +1900,8 @@ dissect_smux_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo,
 			col_add_str(pinfo->cinfo, COL_INFO, pdu_type_string);
 		length = asn1.offset - start;
 		if (tree) {
-			proto_tree_add_text(smux_tree, tvb, offset, length,
-			    "PDU type: %s", pdu_type_string);
+			proto_tree_add_uint(smux_tree, hf_smux_pdutype, tvb,
+			    offset, length, pdu_type);
 		}
 		offset += length;
 		ret = asn1_uint32_value_decode (&asn1, pdu_length, &cause);
@@ -1857,8 +1926,8 @@ dissect_smux_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo,
 			col_add_str(pinfo->cinfo, COL_INFO, pdu_type_string);
 		length = asn1.offset - start;
 		if (tree) {
-			proto_tree_add_text(smux_tree, tvb, offset, length,
-			    "PDU type: %s", pdu_type_string);
+			proto_tree_add_uint(smux_tree, hf_smux_pdutype, tvb,
+			    offset, length, pdu_type);
 		}
 		offset += length;
 		ret = asn1_oid_decode (&asn1, &regid, &regid_length, &length);
@@ -1910,8 +1979,8 @@ dissect_smux_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo,
 			col_add_str(pinfo->cinfo, COL_INFO, pdu_type_string);
 		length = asn1.offset - start;
 		if (tree) {
-			proto_tree_add_text(smux_tree, tvb, offset, length,
-			    "PDU type: %s", pdu_type_string);
+			proto_tree_add_uint(smux_tree, hf_smux_pdutype, tvb,
+			    offset, length, pdu_type);
 		}
 		offset += length;
 		ret = asn1_uint32_value_decode (&asn1, pdu_length, &priority);
@@ -1936,8 +2005,8 @@ dissect_smux_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo,
 			col_add_str(pinfo->cinfo, COL_INFO, pdu_type_string);
 		length = asn1.offset - start;
 		if (tree) {
-			proto_tree_add_text(smux_tree, tvb, offset, length,
-			    "PDU type: %s", pdu_type_string);
+			proto_tree_add_uint(smux_tree, hf_smux_pdutype, tvb,
+			    offset, length, pdu_type);
 		}
 		offset += length;
 		ret = asn1_uint32_value_decode (&asn1, pdu_length, &commit);
@@ -2014,6 +2083,30 @@ proto_register_snmp(void)
 #endif
 
 	static hf_register_info hf[] = {
+		{ &hf_snmp_version,
+		{ "Version", "snmp.version", FT_UINT8, BASE_DEC, VALS(versions),
+		    0x0, "", HFILL }},
+		{ &hf_snmp_community,
+		{ "Community", "snmp.community", FT_STRING, BASE_NONE, NULL,
+		    0x0, "", HFILL }},
+		{ &hf_snmp_pdutype,
+		{ "PDU type", "snmp.pdutype", FT_UINT8, BASE_DEC, VALS(pdu_types),
+		    0x0, "", HFILL }},
+		{ &hf_snmp_agent,
+		{ "Agent address", "snmp.agent", FT_IPv4, BASE_NONE, NULL,
+		    0x0, "", HFILL }},
+		{ &hf_snmp_enterprise,
+		{ "Enterprise", "snmp.enterprise", FT_STRING, BASE_NONE, NULL,
+		    0x0, "", HFILL }},
+		{ &hf_snmp_traptype,
+		{ "Trap type", "snmp.traptype", FT_UINT8, BASE_DEC, VALS(trap_types),
+		    0x0, "", HFILL }},
+		{ &hf_snmp_spectraptype,
+		{ "Specific trap type", "snmp.spectraptype", FT_UINT32, BASE_DEC, NULL,
+		    0x0, "", HFILL }},
+		{ &hf_snmp_timestamp,
+		{ "Timestamp", "snmp.timestamp", FT_UINT8, BASE_DEC, NULL,
+		    0x0, "", HFILL }},
 		{ &hf_snmpv3_flags,
 		{ "SNMPv3 Flags", "snmpv3.flags", FT_UINT8, BASE_HEX, NULL,
 		    0x0, "", HFILL }},
@@ -2026,10 +2119,9 @@ proto_register_snmp(void)
 		{ &hf_snmpv3_flags_report,
 		{ "Reportable", "snmpv3.flags.report", FT_BOOLEAN, 8,
 		    TFS(&flags_set_truth), TH_REPORT, "", HFILL }},
-        };
+	};
 	static gint *ett[] = {
 		&ett_snmp,
-		&ett_smux,
 		&ett_parameters,
 		&ett_parameters_qos,
 		&ett_global,
@@ -2069,11 +2161,9 @@ proto_register_snmp(void)
 	init_mib();
 	read_configs();
 #endif /* HAVE_SOME_SNMP */
-        proto_snmp = proto_register_protocol("Simple Network Management Protocol",
+	proto_snmp = proto_register_protocol("Simple Network Management Protocol",
 	    "SNMP", "snmp");
-        proto_smux = proto_register_protocol("SNMP Multiplex Protocol",
-	    "SMUX", "smux");
-        proto_register_field_array(proto_snmp, hf, array_length(hf));
+	proto_register_field_array(proto_snmp, hf, array_length(hf));
 	proto_register_subtree_array(ett, array_length(ett));
 	snmp_handle = create_dissector_handle(dissect_snmp, proto_snmp);
 }
@@ -2081,14 +2171,41 @@ proto_register_snmp(void)
 void
 proto_reg_handoff_snmp(void)
 {
-	dissector_handle_t smux_handle;
-
 	dissector_add("udp.port", UDP_PORT_SNMP, snmp_handle);
 	dissector_add("udp.port", UDP_PORT_SNMP_TRAP, snmp_handle);
-	smux_handle = create_dissector_handle(dissect_smux, proto_smux);
-	dissector_add("tcp.port", TCP_PORT_SMUX, smux_handle);
 	dissector_add("ethertype", ETHERTYPE_SNMP, snmp_handle);
 	dissector_add("ipx.socket", IPX_SOCKET_SNMP_AGENT, snmp_handle);
 	dissector_add("ipx.socket", IPX_SOCKET_SNMP_SINK, snmp_handle);
+	dissector_add("hpext.dxsap", HPEXT_SNMP, snmp_handle);
 	data_handle = find_dissector("data");
 }
+
+void
+proto_register_smux(void)
+{
+	static hf_register_info hf[] = {
+		{ &hf_smux_version,
+		{ "Version", "smux.version", FT_UINT8, BASE_DEC, NULL,
+		    0x0, "", HFILL }},
+		{ &hf_smux_pdutype,
+		{ "PDU type", "smux.pdutype", FT_UINT8, BASE_DEC, VALS(smux_types),
+		    0x0, "", HFILL }},
+	};
+	static gint *ett[] = {
+		&ett_smux,
+	};
+
+	proto_smux = proto_register_protocol("SNMP Multiplex Protocol",
+	    "SMUX", "smux");
+	proto_register_field_array(proto_smux, hf, array_length(hf));
+	proto_register_subtree_array(ett, array_length(ett));
+}
+
+void
+proto_reg_handoff_smux(void)
+{
+	dissector_handle_t smux_handle;
+
+	smux_handle = create_dissector_handle(dissect_smux, proto_smux);
+	dissector_add("tcp.port", TCP_PORT_SMUX, smux_handle);
+}
