@@ -1,7 +1,7 @@
 /* packet-tcp.c
  * Routines for TCP packet disassembly
  *
- * $Id: packet-tcp.c,v 1.170 2002/12/19 11:22:38 sahlberg Exp $
+ * $Id: packet-tcp.c,v 1.191 2003/04/23 10:20:29 sahlberg Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -93,6 +93,8 @@ static int hf_tcp_analysis_lost_packet = -1;
 static int hf_tcp_analysis_ack_lost_packet = -1;
 static int hf_tcp_analysis_keep_alive = -1;
 static int hf_tcp_analysis_duplicate_ack = -1;
+static int hf_tcp_analysis_duplicate_ack_num = -1;
+static int hf_tcp_analysis_duplicate_ack_frame = -1;
 static int hf_tcp_analysis_zero_window = -1;
 static int hf_tcp_analysis_zero_window_probe = -1;
 static int hf_tcp_analysis_zero_window_violation = -1;
@@ -103,6 +105,21 @@ static int hf_tcp_segment_overlap_conflict = -1;
 static int hf_tcp_segment_multiple_tails = -1;
 static int hf_tcp_segment_too_long_fragment = -1;
 static int hf_tcp_segment_error = -1;
+static int hf_tcp_option_mss = -1;
+static int hf_tcp_option_mss_val = -1;
+static int hf_tcp_option_wscale = -1;
+static int hf_tcp_option_wscale_val = -1;
+static int hf_tcp_option_sack_perm = -1;
+static int hf_tcp_option_sack = -1;
+static int hf_tcp_option_sack_sle = -1;
+static int hf_tcp_option_sack_sre = -1;
+static int hf_tcp_option_echo = -1;
+static int hf_tcp_option_echo_reply = -1;
+static int hf_tcp_option_time_stamp = -1;
+static int hf_tcp_option_cc = -1;
+static int hf_tcp_option_ccnew = -1;
+static int hf_tcp_option_ccecho = -1;
+static int hf_tcp_option_md5 = -1;
 
 static gint ett_tcp = -1;
 static gint ett_tcp_flags = -1;
@@ -127,6 +144,7 @@ static const fragment_items tcp_segment_items = {
 	&hf_tcp_segment_multiple_tails,
 	&hf_tcp_segment_too_long_fragment,
 	&hf_tcp_segment_error,
+	NULL,
 	"Segments"
 };
 
@@ -163,11 +181,11 @@ struct tcp_unacked {
 };
 
 /* Idea for gt: either x > y, or y is much bigger (assume wrap) */
-#define GT_SEQ(x, y) ((gint32)(y - x) < 0)
-#define LT_SEQ(x, y) ((gint32)(x - y) < 0)
-#define GE_SEQ(x, y) ((gint32)(y - x) <= 0)
-#define LE_SEQ(x, y) ((gint32)(x - y) <= 0)
-#define EQ_SEQ(x, y) (x == y)
+#define GT_SEQ(x, y) ((gint32)((y) - (x)) < 0)
+#define LT_SEQ(x, y) ((gint32)((x) - (y)) < 0)
+#define GE_SEQ(x, y) ((gint32)((y) - (x)) <= 0)
+#define LE_SEQ(x, y) ((gint32)((x) - (y)) <= 0)
+#define EQ_SEQ(x, y) ((x) == (y))
 
 static GMemChunk *tcp_acked_chunk = NULL;
 static int tcp_acked_count = 5000;	/* one for almost every other segment in the capture */
@@ -183,6 +201,8 @@ struct tcp_acked {
 	guint32 frame_acked;
 	nstime_t ts;
 	guint8 flags;
+	guint32 dupack_num;	/* dup ack number */
+	guint32 dupack_frame;	/* dup ack to frame # */
 };
 static GHashTable *tcp_analyze_acked_table = NULL;
 
@@ -214,8 +234,199 @@ struct tcp_analysis {
 	guint32 base_seq1;
 	struct tcp_unacked *ual2;	/* UnAcked List 2*/
 	guint32 base_seq2;
+
+	/* these two lists are used to track when PDUs may start
+	   inside a segment.
+	*/
+	struct tcp_next_pdu *pdu_seq1;
+	struct tcp_next_pdu *pdu_seq2;
 };
 
+
+static GMemChunk *tcp_next_pdu_chunk = NULL;
+static int tcp_next_pdu_count = 20;
+struct tcp_next_pdu {
+	struct tcp_next_pdu *next;
+	guint32 seq;
+};
+static GHashTable *tcp_pdu_tracking_table = NULL;
+
+
+static struct tcp_analysis *
+get_tcp_conversation_data(packet_info *pinfo)
+{
+	conversation_t *conv=NULL;
+	struct tcp_analysis *tcpd=NULL;
+
+	/* Have we seen this conversation before? */
+	if( (conv=find_conversation(&pinfo->src, &pinfo->dst, pinfo->ptype, pinfo->srcport, pinfo->destport, 0)) == NULL){
+		/* No this is a new conversation. */
+		conv=conversation_new(&pinfo->src, &pinfo->dst, pinfo->ptype, pinfo->srcport, pinfo->destport, 0);
+	}
+
+	/* check if we have any data for this conversation */
+	tcpd=conversation_get_proto_data(conv, proto_tcp);
+	if(!tcpd){
+		/* No no such data yet. Allocate and init it */
+		tcpd=g_mem_chunk_alloc(tcp_analysis_chunk);
+		tcpd->ual1=NULL;
+		tcpd->base_seq1=0;
+		tcpd->ual2=NULL;
+		tcpd->base_seq2=0;
+
+		tcpd->pdu_seq1=NULL;
+		tcpd->pdu_seq2=NULL;
+
+		conversation_add_proto_data(conv, proto_tcp, tcpd);
+	}
+
+	return tcpd;
+}
+
+/* This function is called from the tcp analysis code to provide
+   clues on how the seq and ack numbers are changed.
+   To prevent the next_pdu lists from growing uncontrollable in size we
+   use this function to do the following :
+   IF we see an ACK then we assume that the left edge of the window has changed
+      at least to this point and assuming it is rare with reordering and
+      trailing duplicate/retransmitted segments, we just assume that after
+      we have seen the ACK we will not see any more segments prior to the 
+      ACK value.
+      If we will not see any segments prior to the ACK value then we can just
+      delete all next_pdu entries that describe pdu's starting prior to the 
+      ACK.
+      If this heuristics is prooved to be too simplistic we can just enhance it
+      later.
+*/   
+/* XXX this function should be ehnanced to handle sequence number wrapping */
+/* XXX to handle retransmissions and reordered packets maybe we should only
+       discard entries that are more than (guesstimate) 50kb older than the
+       specified sequence number ?
+*/
+static void
+prune_next_pdu_list(struct tcp_next_pdu **tnp, guint32 seq)
+{
+	struct tcp_next_pdu *tmptnp;
+
+	if(*tnp == NULL){
+		return;
+	}
+
+	for(tmptnp=*tnp;tmptnp;tmptnp=tmptnp->next){
+		if(tmptnp->seq<=seq){
+			struct tcp_next_pdu *oldtnp;
+			oldtnp=tmptnp;
+
+			if(tmptnp==*tnp){
+				tmptnp=tmptnp->next;
+				*tnp=tmptnp;
+				g_mem_chunk_free(tcp_next_pdu_chunk, oldtnp);
+				if(!tmptnp){
+					return;
+				}
+				continue;
+			} else {
+				for(tmptnp=*tnp;tmptnp;tmptnp=tmptnp->next){
+					if(tmptnp->next==oldtnp){
+						tmptnp->next=oldtnp->next;
+						g_mem_chunk_free(tcp_next_pdu_chunk, oldtnp);
+						break;
+					}
+				}
+				if(!tmptnp){
+					return;
+				}
+			}
+		}
+	}
+}
+		
+
+/* if we know that a PDU starts inside this segment, return the adjusted 
+   offset to where that PDU starts or just return offset back
+   and let TCP try to find out what it can about this segment
+*/
+static int
+scan_for_next_pdu(packet_info *pinfo, int offset, guint32 seq, guint32 nxtseq)
+{
+	struct tcp_analysis *tcpd=NULL;
+	struct tcp_next_pdu *tnp=NULL;
+	int direction;
+
+	if(!pinfo->fd->flags.visited){
+		/* find(or create if needed) the conversation for this tcp session */
+		tcpd=get_tcp_conversation_data(pinfo);
+		/* check direction and get pdu start lists */
+		direction=CMP_ADDRESS(&pinfo->src, &pinfo->dst);
+		/* if the addresses are equal, match the ports instead */
+		if(direction==0) {
+			direction= (pinfo->srcport > pinfo->destport)*2-1;
+		}
+		if(direction>=0){
+			tnp=tcpd->pdu_seq1;
+		} else {
+			tnp=tcpd->pdu_seq2;
+		}
+
+		/* scan and see if we find any pdus starting inside this tvb */
+		for(;tnp;tnp=tnp->next){
+			/* XXX here we should also try to handle sequence number
+			   wrapping
+			*/
+			if(seq<tnp->seq && nxtseq>tnp->seq){
+				g_hash_table_insert(tcp_pdu_tracking_table, 
+					(void *)pinfo->fd->num, (void *)tnp->seq);
+				offset+=tnp->seq-seq;
+				break;
+			}
+		}
+	} else {
+		guint32 pduseq;
+
+		pduseq=(guint32)g_hash_table_lookup(tcp_pdu_tracking_table, (void *)pinfo->fd->num);
+		if(pduseq){
+			offset+=pduseq-seq;
+		}
+	}
+
+	return offset;
+}
+
+/* if we saw a PDU that extended beyond the end of the segment,
+   use this function to remember where the next pdu starts
+*/
+static void
+pdu_store_sequencenumber_of_next_pdu(packet_info *pinfo, guint32 nxtpdu)
+{
+	struct tcp_analysis *tcpd=NULL;
+	struct tcp_next_pdu *tnp=NULL;
+	int direction;
+
+	/* find(or create if needed) the conversation for this tcp session */
+	tcpd=get_tcp_conversation_data(pinfo);
+
+	tnp=g_mem_chunk_alloc(tcp_next_pdu_chunk);
+	tnp->seq=nxtpdu;
+
+	/* check direction and get pdu start list */
+	direction=CMP_ADDRESS(&pinfo->src, &pinfo->dst);
+	/* if the addresses are equal, match the ports instead */
+	if(direction==0) {
+		direction= (pinfo->srcport > pinfo->destport)*2-1;
+	}
+	if(direction>=0){
+		tnp->next=tcpd->pdu_seq1;
+		tcpd->pdu_seq1=tnp;
+	} else {
+		tnp->next=tcpd->pdu_seq2;
+		tcpd->pdu_seq2=tnp;
+	}
+	/*QQQ 
+	  Add check for ACKs and purge list of sequence numbers
+	  already acked.
+	*/
+}
+
 static void
 tcp_get_relative_seq_ack(guint32 frame, guint32 *seq, guint32 *ack)
 {
@@ -242,6 +453,8 @@ tcp_analyze_get_acked_struct(guint32 frame, gboolean createflag)
 		ta->ts.secs=0;
 		ta->ts.nsecs=0;
 		ta->flags=0;
+		ta->dupack_num=0;
+		ta->dupack_frame=0;
 		g_hash_table_insert(tcp_analyze_acked_table, (void *)frame, ta);
 	}
 	return ta;
@@ -250,32 +463,17 @@ tcp_analyze_get_acked_struct(guint32 frame, gboolean createflag)
 static void
 tcp_analyze_sequence_number(packet_info *pinfo, guint32 seq, guint32 ack, guint32 seglen, guint8 flags, guint16 window)
 {
-	conversation_t *conv=NULL;
 	struct tcp_analysis *tcpd=NULL;
 	int direction;
 	struct tcp_unacked *ual1=NULL;
 	struct tcp_unacked *ual2=NULL;
 	struct tcp_unacked *ual=NULL;
-	guint32 base_seq;
-	guint32 base_ack;
+	guint32 base_seq=0;
+	guint32 base_ack=0;
+	struct tcp_next_pdu **tnp=NULL;
 
-	/* Have we seen this conversation before? */
-	if( (conv=find_conversation(&pinfo->src, &pinfo->dst, pinfo->ptype, pinfo->srcport, pinfo->destport, 0)) == NULL){
-		/* No this is a new conversation. */
-		conv=conversation_new(&pinfo->src, &pinfo->dst, pinfo->ptype, pinfo->srcport, pinfo->destport, 0);
-	}
-
-	/* check if we have any data for this conversation */
-	tcpd=conversation_get_proto_data(conv, proto_tcp);
-	if(!tcpd){
-		/* No no such data yet. Allocate and init it */
-		tcpd=g_mem_chunk_alloc(tcp_analysis_chunk);
-		tcpd->ual1=NULL;
-		tcpd->base_seq1=0;
-		tcpd->ual2=NULL;
-		tcpd->base_seq2=0;
-		conversation_add_proto_data(conv, proto_tcp, tcpd);
-	}
+	/* find(or create if needed) the conversation for this tcp session */
+	tcpd=get_tcp_conversation_data(pinfo);
 
 	/* check direction and get ua lists */
 	direction=CMP_ADDRESS(&pinfo->src, &pinfo->dst);
@@ -286,20 +484,31 @@ tcp_analyze_sequence_number(packet_info *pinfo, guint32 seq, guint32 ack, guint3
 	if(direction>=0){
 		ual1=tcpd->ual1;
 		ual2=tcpd->ual2;
+		tnp=&tcpd->pdu_seq2;
 		base_seq=tcpd->base_seq1;
 		base_ack=tcpd->base_seq2;
 	} else {
 		ual1=tcpd->ual2;
 		ual2=tcpd->ual1;
+		tnp=&tcpd->pdu_seq1;
 		base_seq=tcpd->base_seq2;
 		base_ack=tcpd->base_seq1;
 	}
 
-	if(base_seq==0){
-		base_seq=seq;
+	if(tcp_relative_seq){
+		if(base_seq==0){
+			base_seq=seq;
+		}
+		if(base_ack==0){
+			base_ack=ack;
+		}
 	}
-	if(base_ack==0){
-		base_ack=ack;
+
+	/* To handle FIN, just add 1 to the length.
+	   else the ACK following the FIN-ACK will look like it was
+	   outside the window. */
+	if( flags&TH_FIN ){
+		seglen+=1;
 	}
 
 	/* handle the sequence numbers */
@@ -321,8 +530,10 @@ tcp_analyze_sequence_number(packet_info *pinfo, guint32 seq, guint32 ack, guint3
 		ual1->ts.secs=pinfo->fd->abs_secs;
 		ual1->ts.nsecs=pinfo->fd->abs_usecs*1000;
 		ual1->window=window;
-		base_seq=seq;
-		base_ack=ack;
+		if(tcp_relative_seq){
+			base_seq=seq;
+			base_ack=ack;
+		}
 		goto seq_finished;
 	}
 
@@ -339,18 +550,15 @@ tcp_analyze_sequence_number(packet_info *pinfo, guint32 seq, guint32 ack, guint3
 		ual1->ts.secs=pinfo->fd->abs_secs;
 		ual1->ts.nsecs=pinfo->fd->abs_usecs*1000;
 		ual1->window=window;
-		base_seq=seq;
+		if(tcp_relative_seq){
+			base_seq=seq;
+			base_ack=ack;
+		}
 		goto seq_finished;
 	}
 
 	/* if we get past here we know that ual1 points to a segment */
 
-	/* To handle FIN, just pretend they have a length of 1.
-	   else the ACK following the FIN-ACK will look like it was
-	   outside the window. */
-	if( (!seglen) && (flags&TH_FIN) ){
-		seglen=1;
-	}
 
 	/* if seq is beyond ual1->nextseq we have lost a segment */
 	if (GT_SEQ(seq, ual1->nextseq)) {
@@ -464,6 +672,7 @@ seq_finished:
 			ual=ual2->next;
 			g_mem_chunk_free(tcp_unacked_chunk, ual2);
 		}
+		prune_next_pdu_list(tnp, ack-base_ack);
 		goto ack_finished;
 	}
 
@@ -486,6 +695,7 @@ seq_finished:
 			ual=ual2->next;
 			g_mem_chunk_free(tcp_unacked_chunk, ual2);
 		}
+		prune_next_pdu_list(tnp, ack-base_ack);
 		goto ack_finished;
 	}
 
@@ -521,7 +731,7 @@ seq_finished:
 			tmpual=ual->next;
 			g_mem_chunk_free(tcp_unacked_chunk, ual);
 		}
-
+		prune_next_pdu_list(tnp, ack-base_ack);
 	}
 
 ack_finished:
@@ -577,6 +787,8 @@ ack_finished:
 				struct tcp_acked *ta;
 				ta=tcp_analyze_get_acked_struct(pinfo->fd->num, TRUE);
 				ta->flags|=TCP_A_DUPLICATE_ACK;
+				ta->dupack_num=ual->num_acks-1;
+				ta->dupack_frame=ual->ack_frame;
 			}
 		}		
 
@@ -706,11 +918,17 @@ tcp_print_sequence_number_analysis(packet_info *pinfo, tvbuff_t *tvb, proto_tree
 				col_prepend_fstr(pinfo->cinfo, COL_INFO, "[TCP Keep-Alive] ");
 			}
 		}
-		if( ta->flags&TCP_A_DUPLICATE_ACK ){
-			proto_tree_add_none_format(flags_tree, hf_tcp_analysis_duplicate_ack, tvb, 0, 0, "This is a TCP duplicate ack");
-			if(check_col(pinfo->cinfo, COL_INFO)){
-				col_prepend_fstr(pinfo->cinfo, COL_INFO, "[TCP Duplicate ACK] ");
+		if( ta->dupack_num){
+			if( ta->flags&TCP_A_DUPLICATE_ACK ){
+				proto_tree_add_none_format(flags_tree, hf_tcp_analysis_duplicate_ack, tvb, 0, 0, "This is a TCP duplicate ack");
+				if(check_col(pinfo->cinfo, COL_INFO)){
+					col_prepend_fstr(pinfo->cinfo, COL_INFO, "[TCP Dup ACK %d#%d] ", ta->dupack_frame, ta->dupack_num);
+				}
 			}
+			proto_tree_add_uint(tree, hf_tcp_analysis_duplicate_ack_num,
+				tvb, 0, 0, ta->dupack_num);
+			proto_tree_add_uint(tree, hf_tcp_analysis_duplicate_ack_frame,
+				tvb, 0, 0, ta->dupack_frame);
 		}
 		if( ta->flags&TCP_A_ZERO_WINDOW_PROBE ){
 			proto_tree_add_none_format(flags_tree, hf_tcp_analysis_zero_window_probe, tvb, 0, 0, "This is a TCP zero-window-probe");
@@ -777,11 +995,21 @@ tcp_analyze_seq_init(void)
 		g_hash_table_destroy(tcp_rel_seq_table);
 		tcp_rel_seq_table = NULL;
 	}
+	if( tcp_pdu_tracking_table ){
+		g_hash_table_foreach_remove(tcp_rel_seq_table,
+			free_all_acked, NULL);
+		g_hash_table_destroy(tcp_pdu_tracking_table);
+		tcp_pdu_tracking_table = NULL;
+	}
 
 	/*
 	 * Now destroy the chunk from which the conversation table
 	 * structures were allocated.
 	 */
+	if (tcp_next_pdu_chunk) {
+		g_mem_chunk_destroy(tcp_next_pdu_chunk);
+		tcp_next_pdu_chunk = NULL;
+	}
 	if (tcp_analysis_chunk) {
 		g_mem_chunk_destroy(tcp_analysis_chunk);
 		tcp_analysis_chunk = NULL;
@@ -804,6 +1032,12 @@ tcp_analyze_seq_init(void)
 			tcp_acked_equal);
 		tcp_rel_seq_table = g_hash_table_new(tcp_acked_hash,
 			tcp_acked_equal);
+		tcp_pdu_tracking_table = g_hash_table_new(tcp_acked_hash,
+			tcp_acked_equal);
+		tcp_next_pdu_chunk = g_mem_chunk_new("tcp_next_pdu_chunk",
+			sizeof(struct tcp_next_pdu),
+			tcp_next_pdu_count * sizeof(struct tcp_next_pdu),
+			G_ALLOC_ONLY);
 		tcp_analysis_chunk = g_mem_chunk_new("tcp_analysis_chunk",
 			sizeof(struct tcp_analysis),
 			tcp_analysis_count * sizeof(struct tcp_analysis),
@@ -1040,7 +1274,7 @@ desegment_tcp(tvbuff_t *tvb, packet_info *pinfo, int offset,
 			tcp_fragment_table,
 			seq - tsk->start_seq,
 			nxtseq - seq,
-			(nxtseq < (tsk->start_seq + tsk->tot_len)) );
+			(LT_SEQ (nxtseq,tsk->start_seq + tsk->tot_len)) );
 
 		if(!ipfd_head){
 			/* fragment_add() returned NULL, This means that
@@ -1066,7 +1300,7 @@ desegment_tcp(tvbuff_t *tvb, packet_info *pinfo, int offset,
 		   Call the normal subdissector.
 		*/
 		decode_tcp_ports(tvb, offset, pinfo, tree,
-				sport, dport);
+				sport, dport, 0);
 		called_dissector = TRUE;
 
 		/* Did the subdissector ask us to desegment some more data
@@ -1105,7 +1339,7 @@ desegment_tcp(tvbuff_t *tvb, packet_info *pinfo, int offset,
 		 * Note that the last segment may include more than what
 		 * we needed.
 		 */
-		if(nxtseq >= (tsk->start_seq + tsk->tot_len)){
+		if(GE_SEQ(nxtseq, tsk->start_seq + tsk->tot_len)){
 			/*
 			 * OK, this is the last segment.
 			 * Let's call the subdissector with the desegmented
@@ -1135,7 +1369,7 @@ desegment_tcp(tvbuff_t *tvb, packet_info *pinfo, int offset,
 
 			/* call subdissector */
 			decode_tcp_ports(next_tvb, 0, pinfo, tree,
-				sport, dport);
+				sport, dport, 0);
 			called_dissector = TRUE;
 
 			/*
@@ -1305,7 +1539,7 @@ desegment_tcp(tvbuff_t *tvb, packet_info *pinfo, int offset,
 		    tcp_fragment_table,
 		    tsk->seq - tsk->start_seq,
 		    nxtseq - tsk->start_seq,
-		    (nxtseq < tsk->start_seq + tsk->tot_len));
+		    LT_SEQ (nxtseq, tsk->start_seq + tsk->tot_len));
 
 		/* this is the next segment in the sequence we want */
 		new_tsk = g_mem_chunk_alloc(tcp_segment_key_chunk);
@@ -1382,6 +1616,7 @@ tcp_dissect_pdus(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 		 void (*dissect_pdu)(tvbuff_t *, packet_info *, proto_tree *))
 {
   volatile int offset = 0;
+  int offset_before;
   guint length_remaining;
   guint plen;
   guint length;
@@ -1497,8 +1732,12 @@ tcp_dissect_pdus(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 
     /*
      * Step to the next PDU.
+     * Make sure we don't overflow.
      */
+    offset_before = offset;
     offset += plen;
+    if (offset <= offset_before)
+      break;
   }
 }
 
@@ -1516,8 +1755,10 @@ dissect_tcpopt_maxseg(const ip_tcp_opt *optp, tvbuff_t *tvb,
   guint16 mss;
 
   mss = tvb_get_ntohs(tvb, offset + 2);
-  proto_tree_add_text(opt_tree, tvb, offset,      optlen,
-			"%s: %u bytes", optp->name, mss);
+  proto_tree_add_boolean_hidden(opt_tree, hf_tcp_option_mss, tvb, offset,
+				optlen, TRUE);
+  proto_tree_add_uint_format(opt_tree, hf_tcp_option_mss_val, tvb, offset,
+			     optlen, mss, "%s: %u bytes", optp->name, mss);
   tcp_info_append_uint(pinfo, "MSS", mss);
 }
 
@@ -1528,8 +1769,11 @@ dissect_tcpopt_wscale(const ip_tcp_opt *optp, tvbuff_t *tvb,
   guint8 ws;
 
   ws = tvb_get_guint8(tvb, offset + 2);
-  proto_tree_add_text(opt_tree, tvb, offset,      optlen,
-			"%s: %u (multiply by %u)", optp->name, ws, 1 << ws);
+  proto_tree_add_boolean_hidden(opt_tree, hf_tcp_option_wscale, tvb, 
+				offset, optlen, TRUE);
+  proto_tree_add_uint_format(opt_tree, hf_tcp_option_wscale_val, tvb,
+			     offset, optlen, ws, "%s: %u (multiply by %u)", 
+			     optp->name, ws, 1 << ws);
   tcp_info_append_uint(pinfo, "WS", ws);
 }
 
@@ -1548,6 +1792,8 @@ dissect_tcpopt_sack(const ip_tcp_opt *optp, tvbuff_t *tvb,
     if (field_tree == NULL) {
       /* Haven't yet made a subtree out of this option.  Do so. */
       field_tree = proto_item_add_subtree(tf, *optp->subtree_index);
+      proto_tree_add_boolean_hidden(field_tree, hf_tcp_option_sack, tvb, 
+				    offset, optlen, TRUE);
     }
     if (optlen < 4) {
       proto_tree_add_text(field_tree, tvb, offset,      optlen,
@@ -1555,6 +1801,9 @@ dissect_tcpopt_sack(const ip_tcp_opt *optp, tvbuff_t *tvb,
       break;
     }
     leftedge = tvb_get_ntohl(tvb, offset);
+    proto_tree_add_uint_format(field_tree, hf_tcp_option_sack_sle, tvb, 
+			       offset, 4, leftedge, 
+			       "left edge = %u", leftedge);
     optlen -= 4;
     if (optlen < 4) {
       proto_tree_add_text(field_tree, tvb, offset,      optlen,
@@ -1564,8 +1813,9 @@ dissect_tcpopt_sack(const ip_tcp_opt *optp, tvbuff_t *tvb,
     /* XXX - check whether it goes past end of packet */
     rightedge = tvb_get_ntohl(tvb, offset + 4);
     optlen -= 4;
-    proto_tree_add_text(field_tree, tvb, offset,      8,
-        "left edge = %u, right edge = %u", leftedge, rightedge);
+    proto_tree_add_uint_format(field_tree, hf_tcp_option_sack_sre, tvb, 
+			       offset+4, 4, rightedge, 
+			       "right edge = %u", rightedge);
     tcp_info_append_uint(pinfo, "SLE", leftedge);
     tcp_info_append_uint(pinfo, "SRE", rightedge);
     offset += 8;
@@ -1579,6 +1829,8 @@ dissect_tcpopt_echo(const ip_tcp_opt *optp, tvbuff_t *tvb,
   guint32 echo;
 
   echo = tvb_get_ntohl(tvb, offset + 2);
+  proto_tree_add_boolean_hidden(opt_tree, hf_tcp_option_echo, tvb, offset,
+				optlen, TRUE);
   proto_tree_add_text(opt_tree, tvb, offset,      optlen,
 			"%s: %u", optp->name, echo);
   tcp_info_append_uint(pinfo, "ECHO", echo);
@@ -1592,6 +1844,8 @@ dissect_tcpopt_timestamp(const ip_tcp_opt *optp, tvbuff_t *tvb,
 
   tsv = tvb_get_ntohl(tvb, offset + 2);
   tser = tvb_get_ntohl(tvb, offset + 6);
+  proto_tree_add_boolean_hidden(opt_tree, hf_tcp_option_time_stamp, tvb, 
+				offset, optlen, TRUE);
   proto_tree_add_text(opt_tree, tvb, offset,      optlen,
     "%s: tsval %u, tsecr %u", optp->name, tsv, tser);
   tcp_info_append_uint(pinfo, "TSV", tsv);
@@ -1605,6 +1859,8 @@ dissect_tcpopt_cc(const ip_tcp_opt *optp, tvbuff_t *tvb,
   guint32 cc;
 
   cc = tvb_get_ntohl(tvb, offset + 2);
+  proto_tree_add_boolean_hidden(opt_tree, hf_tcp_option_cc, tvb, offset,
+				optlen, TRUE);
   proto_tree_add_text(opt_tree, tvb, offset,      optlen,
 			"%s: %u", optp->name, cc);
   tcp_info_append_uint(pinfo, "CC", cc);
@@ -1725,11 +1981,19 @@ static const ip_tcp_opt tcpopts[] = {
 
 void
 decode_tcp_ports(tvbuff_t *tvb, int offset, packet_info *pinfo,
-	proto_tree *tree, int src_port, int dst_port)
+	proto_tree *tree, int src_port, int dst_port, guint32 nxtseq)
 {
   tvbuff_t *next_tvb;
   int low_port, high_port;
 
+/*qqq   see if it is an unaligned PDU */
+  if(nxtseq && tcp_analyze_seq && (!tcp_desegment)){
+    guint32 seq;
+    seq=nxtseq-tvb_reported_length_remaining(tvb, offset);
+    offset=scan_for_next_pdu(pinfo, offset, seq, nxtseq);
+  }
+
+
   next_tvb = tvb_new_subset(tvb, offset, -1, -1);
 
 /* determine if this packet is part of a conversation and call dissector */
@@ -1737,7 +2001,7 @@ decode_tcp_ports(tvbuff_t *tvb, int offset, packet_info *pinfo,
 
   if (try_conversation_dissector(&pinfo->src, &pinfo->dst, PT_TCP,
 		src_port, dst_port, next_tvb, pinfo, tree))
-    return;
+    goto end_decode_tcp_ports;
 
   /* Do lookups with the subdissector table.
      We try the port number with the lower value first, followed by the
@@ -1763,17 +2027,28 @@ decode_tcp_ports(tvbuff_t *tvb, int offset, packet_info *pinfo,
   }
   if (low_port != 0 &&
       dissector_try_port(subdissector_table, low_port, next_tvb, pinfo, tree))
-    return;
+    goto end_decode_tcp_ports;
+
   if (high_port != 0 &&
       dissector_try_port(subdissector_table, high_port, next_tvb, pinfo, tree))
-    return;
+    goto end_decode_tcp_ports;
 
   /* do lookup with the heuristic subdissector table */
   if (dissector_try_heuristic(heur_subdissector_list, next_tvb, pinfo, tree))
-    return;
+    goto end_decode_tcp_ports;
+
 
   /* Oh, well, we don't know this; dissect it as data. */
   call_dissector(data_handle,next_tvb, pinfo, tree);
+  return;
+
+end_decode_tcp_ports:
+  /* if !visited, check want_pdu_tracking and store it in table */
+  /* XXX fix nxtseq so that it always has valid content and skip the ==0 check */
+  if((!pinfo->fd->flags.visited) && nxtseq && tcp_analyze_seq && pinfo->want_pdu_tracking){
+    pdu_store_sequencenumber_of_next_pdu(pinfo, nxtseq+pinfo->bytes_until_next_pdu);
+  }
+
 }
 
 
@@ -1792,7 +2067,6 @@ dissect_tcp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   guint      bpos;
   guint      optlen;
   guint32    nxtseq;
-  guint      len;
   guint      reported_len;
   vec_t      cksum_vec[4];
   guint32    phdr[2];
@@ -1809,7 +2083,8 @@ dissect_tcp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
      tcph_count=0;
   }
   tcph=&tcphstruct[tcph_count];
-
+  /* XXX add to ipv6 so this works for that protocol as well */
+  tcph->ip_header=pinfo->private_data;
 
   if (check_col(pinfo->cinfo, COL_PROTOCOL))
     col_set_str(pinfo->cinfo, COL_PROTOCOL, "TCP");
@@ -1858,7 +2133,6 @@ dissect_tcp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   tcph->th_hlen = hi_nibble(th_off_x2) * 4;  /* TCP header length, in bytes */
 
   reported_len = tvb_reported_length(tvb);
-  len = tvb_length(tvb);
 
   /* Compute the length of data in this segment. */
   tcph->th_seglen = reported_len - tcph->th_hlen;
@@ -1961,9 +2235,8 @@ dissect_tcp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
    * Assume, initially, that we can't desegment.
    */
   pinfo->can_desegment = 0;
-
   th_sum = tvb_get_ntohs(tvb, offset + 16);
-  if (!pinfo->fragmented && len >= reported_len) {
+  if (!pinfo->fragmented && tvb_bytes_exist(tvb, 0, reported_len)) {
     /* The packet isn't part of an un-reassembled fragmented datagram
        and isn't truncated.  This means we have all the data, and thus
        can checksum it and, unless it's being returned in an error
@@ -1997,7 +2270,7 @@ dissect_tcp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
         g_assert_not_reached();
         break;
       }
-      cksum_vec[3].ptr = tvb_get_ptr(tvb, offset, len);
+      cksum_vec[3].ptr = tvb_get_ptr(tvb, offset, reported_len);
       cksum_vec[3].len = reported_len;
       computed_cksum = in_cksum(&cksum_vec[0], 4);
       if (computed_cksum == 0) {
@@ -2016,6 +2289,7 @@ dissect_tcp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
         /* Checksum is invalid, so we're not willing to desegment it. */
         desegment_ok = FALSE;
+        pinfo->noreassembly_reason = " (incorrect TCP checksum)";
       }
     } else {
       proto_tree_add_uint_format(tcp_tree, hf_tcp_checksum, tvb,
@@ -2125,7 +2399,7 @@ dissect_tcp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
            we don't report it as a malformed frame. */
         save_fragmented = pinfo->fragmented;
         pinfo->fragmented = TRUE;
-        decode_tcp_ports(tvb, offset, pinfo, tree, tcph->th_sport, tcph->th_dport);
+        decode_tcp_ports(tvb, offset, pinfo, tree, tcph->th_sport, tcph->th_dport, nxtseq);
         pinfo->fragmented = save_fragmented;
       }
     }
@@ -2243,6 +2517,14 @@ proto_register_tcp(void)
 		{ "Duplicate ACK",		"tcp.analysis.duplicate_ack", FT_NONE, BASE_NONE, NULL, 0x0,
 			"This is a duplicate ACK", HFILL }},
 
+		{ &hf_tcp_analysis_duplicate_ack_num,
+		{ "Duplicate ACK #",		"tcp.analysis.duplicate_ack_num", FT_UINT32, BASE_DEC, NULL, 0x0,
+			"This is duplicate ACK number #", HFILL }},
+
+		{ &hf_tcp_analysis_duplicate_ack_frame,
+		{ "Duplicate to the ACK in frame",		"tcp.analysis.duplicate_ack_frame", FT_FRAMENUM, BASE_NONE, NULL, 0x0,
+			"This is a duplicate to the ACK in frame #", HFILL }},
+
 		{ &hf_tcp_analysis_zero_window_violation,
 		{ "Zero Window Violation",		"tcp.analysis.zero_window_violation", FT_NONE, BASE_NONE, NULL, 0x0,
 			"This is a zero-window violation, an attempt to write >1 byte to a zero-window", HFILL }},
@@ -2260,7 +2542,7 @@ proto_register_tcp(void)
 		    "", HFILL}},
 
 		{ &hf_tcp_analysis_acks_frame,
-		  { "This is an ACK to the segment in frame",            "tcp.analysis.acks_frame", FT_UINT32, BASE_DEC, NULL, 0x0,
+		  { "This is an ACK to the segment in frame",            "tcp.analysis.acks_frame", FT_FRAMENUM, BASE_NONE, NULL, 0x0,
 		    "Which previous segment is this an ACK for", HFILL}},
 
 		{ &hf_tcp_analysis_ack_rtt,
@@ -2298,6 +2580,56 @@ proto_register_tcp(void)
 		{ &hf_tcp_segments,
 		{ "TCP Segments", "tcp.segments", FT_NONE, BASE_NONE, NULL, 0x0,
 			"TCP Segments", HFILL }},
+		{ &hf_tcp_option_mss,
+		  { "TCP MSS Option", "tcp.options.mss", FT_BOOLEAN, 
+		    BASE_NONE, NULL, 0x0, "TCP MSS Option", HFILL }},
+		{ &hf_tcp_option_mss_val,
+		  { "TCP MSS Option Value", "tcp.options.mss_val", FT_UINT16,
+		    BASE_DEC, NULL, 0x0, "TCP MSS Option Value", HFILL}},
+		{ &hf_tcp_option_wscale,
+		  { "TCP Window Scale Option", "tcp.options.wscale", 
+		    FT_BOOLEAN, 
+		    BASE_NONE, NULL, 0x0, "TCP Window Option", HFILL}},
+		{ &hf_tcp_option_wscale_val,
+		  { "TCP Windows Scale Option Value", "tcp.options.wscale_val",
+		    FT_UINT8, BASE_DEC, NULL, 0x0, "TCP Window Scale Value",
+		    HFILL}},
+		{ &hf_tcp_option_sack_perm, 
+		  { "TCP Sack Perm Option", "tcp.options.sack_perm", 
+		    FT_BOOLEAN,
+		    BASE_NONE, NULL, 0x0, "TCP Sack Perm Option", HFILL}},
+		{ &hf_tcp_option_sack,
+		  { "TCP Sack Option", "tcp.options.sack", FT_BOOLEAN, 
+		    BASE_NONE, NULL, 0x0, "TCP Sack Option", HFILL}},
+		{ &hf_tcp_option_sack_sle,
+		  {"TCP Sack Left Edge", "tcp.options.sack_le", FT_UINT32,
+		   BASE_DEC, NULL, 0x0, "TCP Sack Left Edge", HFILL}},
+		{ &hf_tcp_option_sack_sre,
+		  {"TCP Sack Right Edge", "tcp.options.sack_re", FT_UINT32,
+		   BASE_DEC, NULL, 0x0, "TCP Sack Right Edge", HFILL}},
+		{ &hf_tcp_option_echo,
+		  { "TCP Echo Option", "tcp.options.echo", FT_BOOLEAN, 
+		    BASE_NONE, NULL, 0x0, "TCP Sack Echo", HFILL}},
+		{ &hf_tcp_option_echo_reply,
+		  { "TCP Echo Reply Option", "tcp.options.echo_reply", 
+		    FT_BOOLEAN,
+		    BASE_NONE, NULL, 0x0, "TCP Echo Reply Option", HFILL}},
+		{ &hf_tcp_option_time_stamp,
+		  { "TCP Time Stamp Option", "tcp.options.time_stamp", 
+		    FT_BOOLEAN,
+		    BASE_NONE, NULL, 0x0, "TCP Time Stamp Option", HFILL}},
+		{ &hf_tcp_option_cc,
+		  { "TCP CC Option", "tcp.options.cc", FT_BOOLEAN, BASE_NONE,
+		    NULL, 0x0, "TCP CC Option", HFILL}},
+		{ &hf_tcp_option_ccnew,
+		  { "TCP CC New Option", "tcp.options.ccnew", FT_BOOLEAN, 
+		    BASE_NONE, NULL, 0x0, "TCP CC New Option", HFILL}},
+		{ &hf_tcp_option_ccecho,
+		  { "TCP CC Echo Option", "tcp.options.ccecho", FT_BOOLEAN,
+		    BASE_NONE, NULL, 0x0, "TCP CC Echo Option", HFILL}},
+		{ &hf_tcp_option_md5,
+		  { "TCP MD5 Option", "tcp.options.md5", FT_BOOLEAN, BASE_NONE,
+		    NULL, 0x0, "TCP MD5 Option", HFILL}},
 	};
 	static gint *ett[] = {
 		&ett_tcp,
@@ -2323,7 +2655,7 @@ proto_register_tcp(void)
 
 	/* Register configuration preferences */
 	tcp_module = prefs_register_protocol(proto_tcp, NULL);
-	prefs_register_bool_preference(tcp_module, "tcp_summary_in_tree",
+	prefs_register_bool_preference(tcp_module, "summary_in_tree",
 	    "Show TCP summary in protocol tree",
 "Whether the TCP summary line should be shown in the protocol tree",
 	    &tcp_summary_in_tree);
@@ -2335,11 +2667,11 @@ proto_register_tcp(void)
 	    "Allow subdissector to desegment TCP streams",
 "Whether subdissector can request TCP streams to be desegmented",
 	    &tcp_desegment);
-	prefs_register_bool_preference(tcp_module, "tcp_analyze_sequence_numbers",
+	prefs_register_bool_preference(tcp_module, "analyze_sequence_numbers",
 	    "Analyze TCP sequence numbers",
 	    "Make the TCP dissector analyze TCP sequence numbers to find and flag segment retransmissions, missing segments and RTT",
 	    &tcp_analyze_seq);
-	prefs_register_bool_preference(tcp_module, "tcp_relative_sequence_numbers",
+	prefs_register_bool_preference(tcp_module, "relative_sequence_numbers",
 	    "Use relative sequence numbers",
 	    "Make the TCP dissector use relative sequence numbers instead of absolute ones. To use this option you must also enable \"Analyze TCP sequence numbers\".",
 	    &tcp_relative_seq);
