@@ -1,6 +1,6 @@
 /* packet-tcp.h
  *
- * $Id: packet-tcp.h,v 1.13 2002/12/17 11:49:32 sahlberg Exp $
+ * $Id: packet-tcp.h,v 1.15 2003/04/23 10:20:29 sahlberg Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -45,6 +45,9 @@ struct tcpheader {
 	guint16 th_dport;
 	guint8  th_hlen;
 	guint8  th_flags;
+	/* this can either be ipv4 or ipv6, make sure you know which
+	   one you get. */
+	void *ip_header;
 };
 
 /*
@@ -82,6 +85,6 @@ tcp_dissect_pdus(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 		 void (*dissect_pdu)(tvbuff_t *, packet_info *, proto_tree *));
 
 extern void decode_tcp_ports(tvbuff_t *, int, packet_info *,
-	proto_tree *, int, int);
+	proto_tree *, int, int, guint32);
 
 #endif
