@@ -2,7 +2,7 @@
  *
  * Routines to dissect WTP component of WAP traffic.
  *
- * $Id: packet-wtp.c,v 1.42 2002/12/19 11:22:38 sahlberg Exp $
+ * $Id: packet-wtp.c,v 1.45 2003/04/20 11:36:16 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -213,6 +213,7 @@ static const fragment_items wtp_frag_items = {
     &hf_wtp_fragment_multiple_tails,
     &hf_wtp_fragment_too_long_fragment,
     &hf_wtp_fragment_error,
+    NULL,
     "fragments"
 };
 
@@ -555,7 +556,7 @@ dissect_wtp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		/* Iterate through missing packets */
 		for (i = 0; i < numMissing; i++)
 		{
-		    proto_tree_add_item(wtp_tree, hf_wtp_header_sequence_number, tvb, offCur + i, 1, bo_little_endian);
+		    proto_tree_add_item(wtp_tree, hf_wtp_header_sequence_number, tvb, offCur + 4 + i, 1, bo_little_endian);
 		}
 		break;
 
@@ -595,15 +596,16 @@ dissect_wtp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
      * Any remaining data ought to be WSP data (if not WTP ACK, NACK
      * or ABORT pdu), so hand off (defragmented) to the WSP dissector
      */
-    if ((tvb_length_remaining(tvb, offCur + cbHeader + vHeader) > 0) &&
+    if ((tvb_reported_length_remaining(tvb, offCur + cbHeader + vHeader) > 0) &&
 	! ((pdut==ACK) || (pdut==NEGATIVE_ACK) || (pdut==ABORT)))
     {
 	int	dataOffset = offCur + cbHeader + vHeader;
-	guint32 dataLen = tvb_length_remaining(tvb, offCur + cbHeader + vHeader);
+	gint    dataLen = tvb_reported_length_remaining(tvb, dataOffset);
 	gboolean save_fragmented;
 
-	if ((pdut == SEGMENTED_INVOKE) || (pdut == SEGMENTED_RESULT) ||
-	    (((pdut == INVOKE) || (pdut == RESULT)) && (!fTTR)))
+	if (((pdut == SEGMENTED_INVOKE) || (pdut == SEGMENTED_RESULT) ||
+	    (((pdut == INVOKE) || (pdut == RESULT)) && (!fTTR))) &&
+	    tvb_bytes_exist(tvb, dataOffset, dataLen))
 	{					/* 1st part of segment	*/
 	    save_fragmented = pinfo->fragmented;
 	    pinfo->fragmented = TRUE;
@@ -632,9 +634,13 @@ dissect_wtp_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	    }
 	    pinfo->fragmented = save_fragmented;
 	}
-	else			/* Normal packet, call next dissector	*/
+	else
 	{
-	    wsp_tvb = tvb_new_subset(tvb, dataOffset, -1, dataLen);
+	    /*
+	     * Normal packet, or not all the fragment data is available;
+	     * call next dissector.
+	     */
+	    wsp_tvb = tvb_new_subset(tvb, dataOffset, -1, -1);
 	    call_dissector(wsp_handle, wsp_tvb, pinfo, tree);
 	}
     }
