@@ -1,7 +1,7 @@
 /* plugin_api.c
  * Routines for Ethereal plugins.
  *
- * $Id: plugin_api.c,v 1.43 2002/11/14 18:54:53 guy Exp $
+ * $Id: plugin_api.c,v 1.50 2003/05/01 21:10:43 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * Copyright 2000 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -210,4 +210,48 @@ plugin_address_table_init(plugin_address_table_t *pat)
 	p_fragment_delete			= pat->p_fragment_delete;
 	p_show_fragment_tree			= pat->p_show_fragment_tree;
 	p_show_fragment_seq_tree		= pat->p_show_fragment_seq_tree;
+
+	p_register_tap				= pat->p_register_tap;
+	p_tap_queue_packet			= pat->p_tap_queue_packet;
+
+	p_asn1_open				= pat->p_asn1_open;
+	p_asn1_close				= pat->p_asn1_close;
+	p_asn1_octet_decode			= pat->p_asn1_octet_decode;
+	p_asn1_tag_decode			= pat->p_asn1_tag_decode;
+	p_asn1_id_decode			= pat->p_asn1_id_decode;
+	p_asn1_length_decode			= pat->p_asn1_length_decode;
+	p_asn1_header_decode			= pat->p_asn1_header_decode;
+	p_asn1_eoc				= pat->p_asn1_eoc;
+	p_asn1_eoc_decode			= pat->p_asn1_eoc_decode;
+	p_asn1_null_decode			= pat->p_asn1_null_decode;
+	p_asn1_bool_decode			= pat->p_asn1_bool_decode;
+	p_asn1_int32_value_decode		= pat->p_asn1_int32_value_decode;
+	p_asn1_int32_decode			= pat->p_asn1_int32_decode;
+	p_asn1_uint32_value_decode		= pat->p_asn1_uint32_value_decode;
+	p_asn1_uint32_decode			= pat->p_asn1_uint32_decode;
+	p_asn1_bits_decode			= pat->p_asn1_bits_decode;
+	p_asn1_string_value_decode		= pat->p_asn1_string_value_decode;
+	p_asn1_string_decode			= pat->p_asn1_string_decode;
+	p_asn1_octet_string_decode		= pat->p_asn1_octet_string_decode;
+	p_asn1_subid_decode			= pat->p_asn1_subid_decode;
+	p_asn1_oid_value_decode			= pat->p_asn1_oid_value_decode;
+	p_asn1_oid_decode			= pat->p_asn1_oid_decode;
+	p_asn1_sequence_decode			= pat->p_asn1_sequence_decode;
+	p_asn1_err_to_str			= pat->p_asn1_err_to_str;
+
+	p_proto_item_set_end			= pat->p_proto_item_set_end;
+	p_proto_tree_add_none_format		= pat->p_proto_tree_add_none_format;
+
+	p_except_init				= pat->p_except_init;
+	p_except_deinit				= pat->p_except_deinit;
+	p_except_rethrow			= pat->p_except_rethrow;
+	p_except_throwd				= pat->p_except_throwd;
+	p_except_throwf				= pat->p_except_throwf;
+	p_except_unhandled_catcher     		= pat->p_except_unhandled_catcher;
+	p_except_take_data			= pat->p_except_take_data;
+	p_except_set_allocator			= pat->p_except_set_allocator;
+	p_except_alloc				= pat->p_except_alloc;
+	p_except_free				= pat->p_except_free;
+	p_except_pop				= pat->p_except_pop;
+	p_except_setup_try			= pat->p_except_setup_try;
 }
