@@ -1,7 +1,7 @@
 /* plugin_api.h
  * Routines for Ethereal plugins.
  *
- * $Id: plugin_api.h,v 1.44 2002/11/14 18:54:53 guy Exp $
+ * $Id: plugin_api.h,v 1.51 2003/05/01 21:10:43 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * Copyright 2000 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -76,7 +76,7 @@
 #define proto_is_protocol_enabled	(*p_proto_is_protocol_enabled)
 
 #define proto_item_get_len		(*p_proto_item_get_len)
-#define proto_item_set_len		(*p_proto_item_set_len)
+#define proto_item_set_end		(*p_proto_item_set_end)
 #define proto_item_set_text		(*p_proto_item_set_text)
 #define proto_item_append_text		(*p_proto_item_append_text)
 #define	proto_item_add_subtree		(*p_proto_item_add_subtree)
@@ -240,7 +240,51 @@
 #define fragment_delete			(*p_fragment_delete)
 #define show_fragment_tree		(*p_show_fragment_tree)
 #define show_fragment_seq_tree		(*p_show_fragment_seq_tree)
-                                                
+
+#define register_tap			(*p_register_tap)
+#define tap_queue_packet		(*p_tap_queue_packet)
+
+#define	asn1_open			(*p_asn1_open)
+#define	asn1_close			(*p_asn1_close)
+#define	asn1_octet_decode		(*p_asn1_octet_decode)
+#define	asn1_tag_decode			(*p_asn1_tag_decode)
+#define	asn1_id_decode			(*p_asn1_id_decode)
+#define	asn1_length_decode		(*p_asn1_length_decode)
+#define	asn1_header_decode		(*p_asn1_header_decode)
+#define	asn1_eoc			(*p_asn1_eoc)
+#define	asn1_eoc_decode			(*p_asn1_eoc_decode)
+#define	asn1_null_decode		(*p_asn1_null_decode)
+#define	asn1_bool_decode		(*p_asn1_bool_decode)
+#define	asn1_int32_value_decode		(*p_asn1_int32_value_decode)
+#define	asn1_int32_decode		(*p_asn1_int32_decode)
+#define	asn1_uint32_value_decode	(*p_asn1_uint32_value_decode)
+#define	asn1_uint32_decode		(*p_asn1_uint32_decode)
+#define	asn1_bits_decode		(*p_asn1_bits_decode)
+#define	asn1_string_value_decode	(*p_asn1_string_value_decode)
+#define	asn1_string_decode		(*p_asn1_string_decode)
+#define	asn1_octet_string_decode	(*p_asn1_octet_string_decode)
+#define	asn1_subid_decode		(*p_asn1_subid_decode)
+#define	asn1_oid_value_decode		(*p_asn1_oid_value_decode)
+#define	asn1_oid_decode			(*p_asn1_oid_decode)
+#define	asn1_sequence_decode		(*p_asn1_sequence_decode)
+#define	asn1_err_to_str			(*p_asn1_err_to_str)
+
+#define proto_item_set_len		(*p_proto_item_set_len)
+#define	proto_tree_add_none_format	(*p_proto_tree_add_none_format)
+
+#define except_init			(*p_except_init)
+#define except_deinit			(*p_except_deinit)
+#define except_rethrow			(*p_except_rethrow)
+#define except_throwd			(*p_except_throwd)
+#define except_throwf			(*p_except_throwf)
+#define except_unhandled_catcher       	(*p_except_unhandled_catcher)
+#define except_take_data	       	(*p_except_take_data)
+#define except_set_allocator	       	(*p_except_set_allocator)
+#define except_alloc			(*p_except_alloc)
+#define except_free			(*p_except_free)
+#define except_pop			(*p_except_pop)
+#define except_setup_try		(*p_except_setup_try)
+
 #endif
 
 #include <epan/packet.h>
@@ -250,6 +294,9 @@
 #include "packet-giop.h"
 #include "packet-tpkt.h"
 #include "packet-tcp.h"
+#include "tap.h"
+#include "asn1.h"
+#include "epan/except.h"
 
 #include "plugin_table.h"
 
