@@ -2,7 +2,7 @@
  * Declarations of a list of "p_" names; included in various places
  * to declare them as variables or as function members.
  *
- * $Id: plugin_api_decls.h,v 1.6 2002/11/14 18:54:53 guy Exp $
+ * $Id: plugin_api_decls.h,v 1.13 2003/05/01 21:10:43 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * Copyright 2000 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -250,3 +250,46 @@ addr_fragment_delete			p_fragment_delete;
 addr_show_fragment_tree			p_show_fragment_tree;
 addr_show_fragment_seq_tree		p_show_fragment_seq_tree;
 
+addr_register_tap			p_register_tap;
+addr_tap_queue_packet			p_tap_queue_packet;
+
+addr_asn1_open				p_asn1_open;
+addr_asn1_close				p_asn1_close;
+addr_asn1_octet_decode			p_asn1_octet_decode;
+addr_asn1_tag_decode			p_asn1_tag_decode;
+addr_asn1_id_decode			p_asn1_id_decode;
+addr_asn1_length_decode			p_asn1_length_decode;
+addr_asn1_header_decode			p_asn1_header_decode;
+addr_asn1_eoc				p_asn1_eoc;
+addr_asn1_eoc_decode			p_asn1_eoc_decode;
+addr_asn1_null_decode			p_asn1_null_decode;
+addr_asn1_bool_decode			p_asn1_bool_decode;
+addr_asn1_int32_value_decode		p_asn1_int32_value_decode;
+addr_asn1_int32_decode			p_asn1_int32_decode;
+addr_asn1_uint32_value_decode		p_asn1_uint32_value_decode;
+addr_asn1_uint32_decode			p_asn1_uint32_decode;
+addr_asn1_bits_decode			p_asn1_bits_decode;
+addr_asn1_string_value_decode		p_asn1_string_value_decode;
+addr_asn1_string_decode			p_asn1_string_decode;
+addr_asn1_octet_string_decode		p_asn1_octet_string_decode;
+addr_asn1_subid_decode			p_asn1_subid_decode;
+addr_asn1_oid_value_decode		p_asn1_oid_value_decode;
+addr_asn1_oid_decode			p_asn1_oid_decode;
+addr_asn1_sequence_decode		p_asn1_sequence_decode;
+addr_asn1_err_to_str			p_asn1_err_to_str;
+
+addr_proto_item_set_end			p_proto_item_set_end;
+addr_proto_tree_add_none_format		p_proto_tree_add_none_format;
+
+addr_except_init			p_except_init;
+addr_except_deinit			p_except_deinit;
+addr_except_rethrow			p_except_rethrow;
+addr_except_throwd			p_except_throwd;
+addr_except_throwf			p_except_throwf;
+addr_except_unhandled_catcher		p_except_unhandled_catcher;
+addr_except_take_data			p_except_take_data;
+addr_except_set_allocator		p_except_set_allocator;
+addr_except_alloc			p_except_alloc;
+addr_except_free			p_except_free;
+addr_except_pop				p_except_pop;
+addr_except_setup_try			p_except_setup_try;
