@@ -1,7 +1,7 @@
 /* plugin_table.h
  * Table of exported addresses for Ethereal plugins.
  *
- * $Id: plugin_table.h,v 1.56 2002/12/02 23:34:40 guy Exp $
+ * $Id: plugin_table.h,v 1.63 2003/05/01 21:10:43 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * Copyright 2000 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -279,6 +279,58 @@ typedef unsigned char *(*addr_fragment_delete)(packet_info *, guint32, GHashTabl
 typedef gboolean (*addr_show_fragment_tree)(fragment_data *, const fragment_items *, proto_tree *, packet_info *, tvbuff_t *);
 typedef gboolean (*addr_show_fragment_seq_tree)(fragment_data *, const fragment_items *, proto_tree *, packet_info *, tvbuff_t *);
 
+typedef int (*addr_register_tap)(char *);
+typedef void (*addr_tap_queue_packet)(int, packet_info *, void *);
+
+typedef void (*addr_asn1_open)(ASN1_SCK *, tvbuff_t *, int );
+typedef void (*addr_asn1_close)(ASN1_SCK *, int *);
+typedef int (*addr_asn1_octet_decode)(ASN1_SCK *, guchar *);
+typedef int (*addr_asn1_tag_decode)(ASN1_SCK *, guint *);
+typedef int (*addr_asn1_id_decode)(ASN1_SCK *, guint *, guint *, guint *);
+typedef int (*addr_asn1_length_decode)(ASN1_SCK *, gboolean *, guint *);
+typedef int (*addr_asn1_header_decode)(ASN1_SCK *, guint *, guint *, guint *,
+			gboolean *, guint *);
+typedef int (*addr_asn1_eoc)(ASN1_SCK *, int );
+typedef int (*addr_asn1_eoc_decode)(ASN1_SCK *, int );
+typedef int (*addr_asn1_null_decode)(ASN1_SCK *, int );
+typedef int (*addr_asn1_bool_decode)(ASN1_SCK *, int , gboolean *);
+typedef int (*addr_asn1_int32_value_decode)(ASN1_SCK *, int , gint32 *);
+typedef int (*addr_asn1_int32_decode)(ASN1_SCK *, gint32 *, guint *);
+typedef int (*addr_asn1_uint32_value_decode)(ASN1_SCK *, int , guint *);
+typedef int (*addr_asn1_uint32_decode)(ASN1_SCK *, guint32 *, guint *);
+typedef int (*addr_asn1_bits_decode)(ASN1_SCK *, int , guchar **,
+                             guint *, guchar *);
+typedef int (*addr_asn1_string_value_decode)(ASN1_SCK *, int ,
+			guchar **);
+typedef int (*addr_asn1_string_decode)(ASN1_SCK *, guchar **, guint *,
+			guint *, guint );
+typedef int (*addr_asn1_octet_string_decode)(ASN1_SCK *, guchar **, guint *,
+			guint *);
+typedef int (*addr_asn1_subid_decode)(ASN1_SCK *, subid_t *);
+typedef int (*addr_asn1_oid_value_decode)(ASN1_SCK *, int , subid_t **,
+			guint *);
+typedef int (*addr_asn1_oid_decode)( ASN1_SCK *, subid_t **, guint *, guint *);
+typedef int (*addr_asn1_sequence_decode)( ASN1_SCK *, guint *, guint *);
+
+typedef char *(*addr_asn1_err_to_str)(int );
+
+typedef void (*addr_proto_item_set_end)(proto_item*, tvbuff_t *, gint);
+
+typedef proto_item* (*addr_proto_tree_add_none_format)(proto_tree*, int, tvbuff_t*, gint, gint, const char*, ...);
+
+typedef int (*addr_except_init)(void);
+typedef void (*addr_except_deinit)(void);
+typedef void (*addr_except_rethrow)(except_t *);
+typedef void (*addr_except_throwd)(long, long, const char *, void *);
+typedef void (*addr_except_throwf)(long, long, const char *, ...);
+typedef void (*(*addr_except_unhandled_catcher)(void (*)(except_t *)))(except_t *);
+typedef void *(*addr_except_take_data)(except_t *);
+typedef void (*addr_except_set_allocator)(void *(*)(size_t), void (*)(void *));
+typedef void *(*addr_except_alloc)(size_t);
+typedef void (*addr_except_free)(void *);
+typedef struct except_stacknode *(*addr_except_pop)(void);
+typedef void (*addr_except_setup_try)(struct except_stacknode *, struct except_catch *, const except_id_t [], size_t);
+
 typedef struct  {
 
 #include "plugin_api_decls.h"
