@@ -1,7 +1,7 @@
 /* prefs.c
  * Routines for handling preferences
  *
- * $Id: prefs.c,v 1.92 2002/12/20 01:48:54 guy Exp $
+ * $Id: prefs.c,v 1.100 2003/04/21 21:28:31 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -387,7 +387,7 @@ prefs_apply_all(void)
  */
 static pref_t *
 register_preference(module_t *module, const char *name, const char *title,
-    const char *description)
+    const char *description, pref_type_t type)
 {
 	pref_t *preference;
 	const guchar *p;
@@ -396,6 +396,7 @@ register_preference(module_t *module, const char *name, const char *title,
 	preference->name = name;
 	preference->title = title;
 	preference->description = description;
+	preference->type = type;
 	if (title != NULL)
 		preference->ordinal = module->numprefs;
 	else
@@ -423,6 +424,15 @@ register_preference(module_t *module, const char *name, const char *title,
 	 */
 	g_assert(find_preference(module, name) == NULL);
 
+	if (type != PREF_OBSOLETE) {
+		/*
+		 * Make sure the preference name doesn't begin with the
+		 * module name, as that's redundant and Just Silly.
+		 */
+		g_assert((strncmp(name, module->name, strlen(module->name)) != 0) ||
+			(((name[strlen(module->name)]) != '.') && ((name[strlen(module->name)]) != '_')));
+	}
+
 	/*
 	 * There isn't already one with that name, so add the
 	 * preference.
@@ -490,8 +500,8 @@ prefs_register_uint_preference(module_t *module, const char *name,
 {
 	pref_t *preference;
 
-	preference = register_preference(module, name, title, description);
-	preference->type = PREF_UINT;
+	preference = register_preference(module, name, title, description,
+	    PREF_UINT);
 	preference->varp.uint = var;
 	preference->info.base = base;
 }
@@ -505,8 +515,8 @@ prefs_register_bool_preference(module_t *module, const char *name,
 {
 	pref_t *preference;
 
-	preference = register_preference(module, name, title, description);
-	preference->type = PREF_BOOL;
+	preference = register_preference(module, name, title, description,
+	    PREF_BOOL);
 	preference->varp.boolp = var;
 }
 
@@ -520,8 +530,8 @@ prefs_register_enum_preference(module_t *module, const char *name,
 {
 	pref_t *preference;
 
-	preference = register_preference(module, name, title, description);
-	preference->type = PREF_ENUM;
+	preference = register_preference(module, name, title, description,
+	    PREF_ENUM);
 	preference->varp.enump = var;
 	preference->info.enum_info.enumvals = enumvals;
 	preference->info.enum_info.radio_buttons = radio_buttons;
@@ -536,8 +546,22 @@ prefs_register_string_preference(module_t *module, const char *name,
 {
 	pref_t *preference;
 
-	preference = register_preference(module, name, title, description);
-	preference->type = PREF_STRING;
+	preference = register_preference(module, name, title, description,
+	    PREF_STRING);
+
+	/*
+	 * String preference values should be non-null (as you can't
+	 * keep them null after using the preferences GUI, you can at best
+	 * have them be null strings) and freeable (as we free them
+	 * if we change them.
+	 *
+	 * If the value is a null pointer, make it a copy of a null
+	 * string, otherwise make it a copy of the value.
+	 */
+	if (*var == NULL)
+		*var = g_strdup("");
+	else
+		*var = g_strdup(*var);
 	preference->varp.string = var;
 	preference->saved_val.string = NULL;
 }
@@ -548,10 +572,7 @@ prefs_register_string_preference(module_t *module, const char *name,
 void
 prefs_register_obsolete_preference(module_t *module, const char *name)
 {
-	pref_t *preference;
-
-	preference = register_preference(module, name, NULL, NULL);
-	preference->type = PREF_OBSOLETE;
+	register_preference(module, name, NULL, NULL, PREF_OBSOLETE);
 }
 
 typedef struct {
@@ -1571,7 +1592,7 @@ set_pref(gchar *pref_name, gchar *value)
     pref = find_preference(module, dotp);
 
     if (pref == NULL) {
-      if (strncmp(pref_name, "mgcp.", 5) == 0) {
+      if (strcmp(module->name, "mgcp") == 0) {
         /*
          * XXX - "mgcp.display raw text toggle" and "mgcp.display dissect tree"
          * rather than "mgcp.display_raw_text" and "mgcp.display_dissect_tree"
@@ -1618,20 +1639,106 @@ set_pref(gchar *pref_name, gchar *value)
           /* Otherwise it's from the command line, and we don't bother
              mapping it. */
 	}
-      } else if (strncmp(pref_name, "smb.", 4) == 0) {
+      } else if (strcmp(module->name, "smb") == 0) {
         /* Handle old names for SMB preferences. */
         if (strcmp(dotp, "smb.trans.reassembly") == 0)
           pref = find_preference(module, "trans_reassembly");
         else if (strcmp(dotp, "smb.dcerpc.reassembly") == 0)
           pref = find_preference(module, "dcerpc_reassembly");
-      } else if (strncmp(pref_name, "ndmp.", 5) == 0) {
+      } else if (strcmp(module->name, "ndmp") == 0) {
         /* Handle old names for NDMP preferences. */
         if (strcmp(dotp, "ndmp.desegment") == 0)
           pref = find_preference(module, "desegment");
-      } else if (strncmp(pref_name, "diameter.", 9) == 0) {
+      } else if (strcmp(module->name, "diameter") == 0) {
         /* Handle old names for Diameter preferences. */
         if (strcmp(dotp, "diameter.desegment") == 0)
           pref = find_preference(module, "desegment");
+      } else if (strcmp(module->name, "pcli") == 0) {
+        /* Handle old names for PCLI preferences. */
+        if (strcmp(dotp, "pcli.udp_port") == 0)
+          pref = find_preference(module, "udp_port");
+      } else if (strcmp(module->name, "artnet") == 0) {
+        /* Handle old names for ARTNET preferences. */
+        if (strcmp(dotp, "artnet.udp_port") == 0)
+          pref = find_preference(module, "udp_port");
+      } else if (strcmp(module->name, "mapi") == 0) {
+        /* Handle old names for MAPI preferences. */
+        if (strcmp(dotp, "mapi_decrypt") == 0)
+          pref = find_preference(module, "decrypt");
+      } else if (strcmp(module->name, "fc") == 0) {
+        /* Handle old names for Fibre Channel preferences. */
+        if (strcmp(dotp, "reassemble_fc") == 0)
+          pref = find_preference(module, "reassemble");
+        else if (strcmp(dotp, "fc_max_frame_size") == 0)
+          pref = find_preference(module, "max_frame_size");
+      } else if (strcmp(module->name, "fcip") == 0) {
+        /* Handle old names for Fibre Channel-over-IP preferences. */
+        if (strcmp(dotp, "desegment_fcip_messages") == 0)
+          pref = find_preference(module, "desegment");
+        else if (strcmp(dotp, "fcip_port") == 0)
+          pref = find_preference(module, "target_port");
+      } else if (strcmp(module->name, "gtp") == 0) {
+        /* Handle old names for GTP preferences. */
+        if (strcmp(dotp, "gtpv0_port") == 0)
+          pref = find_preference(module, "v0_port");
+        else if (strcmp(dotp, "gtpv1c_port") == 0)
+          pref = find_preference(module, "v1c_port");
+        else if (strcmp(dotp, "gtpv1u_port") == 0)
+          pref = find_preference(module, "v1u_port");
+        else if (strcmp(dotp, "gtp_dissect_tpdu") == 0)
+          pref = find_preference(module, "dissect_tpdu");
+        else if (strcmp(dotp, "gtpv0_dissect_cdr_as") == 0)
+          pref = find_preference(module, "v0_dissect_cdr_as");
+        else if (strcmp(dotp, "gtpv0_check_etsi") == 0)
+          pref = find_preference(module, "v0_check_etsi");
+        else if (strcmp(dotp, "gtpv1_check_etsi") == 0)
+          pref = find_preference(module, "v1_check_etsi");
+      } else if (strcmp(module->name, "ip") == 0) {
+        /* Handle old names for IP preferences. */
+        if (strcmp(dotp, "ip_summary_in_tree") == 0)
+          pref = find_preference(module, "summary_in_tree");
+      } else if (strcmp(module->name, "iscsi") == 0) {
+        /* Handle old names for iSCSI preferences. */
+        if (strcmp(dotp, "iscsi_port") == 0)
+          pref = find_preference(module, "target_port");
+      } else if (strcmp(module->name, "lmp") == 0) {
+        /* Handle old names for LMP preferences. */
+        if (strcmp(dotp, "lmp_version") == 0)
+          pref = find_preference(module, "version");
+      } else if (strcmp(module->name, "mtp3") == 0) {
+        /* Handle old names for MTP3 preferences. */
+        if (strcmp(dotp, "mtp3_standard") == 0)
+          pref = find_preference(module, "standard");
+      } else if (strcmp(module->name, "nlm") == 0) {
+        /* Handle old names for NLM preferences. */
+        if (strcmp(dotp, "nlm_msg_res_matching") == 0)
+          pref = find_preference(module, "msg_res_matching");
+      } else if (strcmp(module->name, "ppp") == 0) {
+        /* Handle old names for PP preferences. */
+        if (strcmp(dotp, "ppp_fcs") == 0)
+          pref = find_preference(module, "fcs_type");
+        else if (strcmp(dotp, "ppp_vj") == 0)
+          pref = find_preference(module, "decompress_vj");
+      } else if (strcmp(module->name, "rsvp") == 0) {
+        /* Handle old names for RSVP preferences. */
+        if (strcmp(dotp, "rsvp_process_bundle") == 0)
+          pref = find_preference(module, "process_bundle");
+      } else if (strcmp(module->name, "tcp") == 0) {
+        /* Handle old names for TCP preferences. */
+        if (strcmp(dotp, "tcp_summary_in_tree") == 0)
+          pref = find_preference(module, "summary_in_tree");
+        else if (strcmp(dotp, "tcp_analyze_sequence_numbers") == 0)
+          pref = find_preference(module, "analyze_sequence_numbers");
+        else if (strcmp(dotp, "tcp_relative_sequence_numbers") == 0)
+          pref = find_preference(module, "relative_sequence_numbers");
+      } else if (strcmp(module->name, "udp") == 0) {
+        /* Handle old names for UDP preferences. */
+        if (strcmp(dotp, "udp_summary_in_tree") == 0)
+          pref = find_preference(module, "summary_in_tree");
+      } else if (strcmp(module->name, "ndps") == 0) {
+        /* Handle old names for NDPS preferences. */
+        if (strcmp(dotp, "desegment_ndps") == 0)
+          pref = find_preference(module, "desegment_tcp");
       }
     }
     if (pref == NULL)
@@ -1672,10 +1779,9 @@ set_pref(gchar *pref_name, gchar *value)
       break;
 
     case PREF_STRING:
-      if (*pref->varp.string == NULL || strcmp(*pref->varp.string, value) != 0) {
+      if (strcmp(*pref->varp.string, value) != 0) {
         module->prefs_changed = TRUE;
-        if (*pref->varp.string != NULL)
-          g_free(*pref->varp.string);
+        g_free(*pref->varp.string);
         *pref->varp.string = g_strdup(value);
       }
       break;
