@@ -1,7 +1,7 @@
 /* reassemble.c
  * Routines for {fragment,segment} reassembly
  *
- * $Id: reassemble.c,v 1.28 2002/12/19 11:22:38 sahlberg Exp $
+ * $Id: reassemble.c,v 1.38 2003/04/30 22:13:05 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -98,8 +98,8 @@ fragment_hash(gconstpointer k)
 }
 
 /*
- * XXX - we use the frame_data structure for a frame as the key
- * structure, with the frame number as the item compared.
+ * XXX - we use the frame number as the key (we can't use the frame_data
+ * structure, as in Tethereal there's only one such structure).
  *
  * This won't work if there's more than one form of reassembly using
  * the reassembled-packet hash tables going on in the frame, and two
@@ -115,18 +115,13 @@ fragment_hash(gconstpointer k)
 static gint
 reassembled_equal(gconstpointer k1, gconstpointer k2)
 {
-	const frame_data* key1 = (const frame_data*) k1;
-	const frame_data* key2 = (const frame_data*) k2;
-
-	return (key1->num == key2->num);
+	return ((guint32)k1 == (guint32)k2);
 }
 
 static guint
 reassembled_hash(gconstpointer k)
 {
-	const frame_data* key = (const frame_data*) k;
-
-	return key->num;
+	return (guint32)k;
 }
 
 /*
@@ -178,8 +173,18 @@ free_all_reassembled_fragments(gpointer key_arg _U_, gpointer value,
 	fragment_data *fd_head;
 
 	for (fd_head = value; fd_head != NULL; fd_head = fd_head->next) {
-		if(fd_head->data && !(fd_head->flags&FD_NOT_MALLOCED))
+		if(fd_head->data && !(fd_head->flags&FD_NOT_MALLOCED)) {
 			g_free(fd_head->data);
+
+			/*
+			 * A reassembled packet is inserted into the
+			 * hash table once for every frame that made
+			 * up the reassembled packet; clear the data
+			 * pointer so that we only free the data the
+			 * first time we see it.
+			 */
+			fd_head->data = NULL;
+		}
 	}
 
 	return TRUE;
@@ -401,6 +406,62 @@ fragment_set_partial_reassembly(packet_info *pinfo, guint32 id, GHashTable *frag
 	}
 }
 
+/*
+ * This function gets rid of an entry from a fragment table, given
+ * a pointer to the key for that entry; it also frees up the key
+ * and the addresses in it.
+ */
+static void
+fragment_unhash(GHashTable *fragment_table, fragment_key *key)
+{
+	/*
+	 * Free up the copies of the addresses from the old key.
+	 */
+	g_free((gpointer)key->src.data);
+	g_free((gpointer)key->dst.data);
+
+	/*
+	 * Remove the entry from the fragment table.
+	 */
+	g_hash_table_remove(fragment_table, key);
+
+	/*
+	 * Free the key itself.
+	 */
+	g_mem_chunk_free(fragment_key_chunk, key);
+}
+
+/*
+ * This function adds fragment_data structure to a reassembled-packet
+ * hash table, using the frame numbers of each of the frames from
+ * which it was reassembled as keys, and sets the "reassembled_in"
+ * frame number.
+ */
+void
+fragment_reassembled(fragment_data *fd_head, packet_info *pinfo,
+	     GHashTable *reassembled_table)
+{
+	fragment_data *fd;
+
+	if (fd_head->next == NULL) {
+		/*
+		 * This was not fragmented, so there's no fragment
+		 * table; just hash it using the current frame number.
+		 */
+		g_hash_table_insert(reassembled_table, (gpointer)pinfo->fd->num,
+		    fd_head);
+	} else {
+		/*
+		 * Hash it with the frame numbers for all the frames.
+		 */
+		for (fd = fd_head->next; fd != NULL; fd = fd->next){
+			g_hash_table_insert(reassembled_table,
+			    (gpointer)fd->frame, fd_head);
+		}
+	}
+	fd_head->reassembled_in = pinfo->fd->num;
+}
+
 /*
  * This function adds a new fragment to the fragment hash table.
  * If this is the first fragment seen for this datagram, a new entry
@@ -422,63 +483,16 @@ fragment_set_partial_reassembly(packet_info *pinfo, guint32 id, GHashTable *frag
  * with the new fragment. FD_TOOLONGFRAGMENT and FD_MULTIPLETAILS flags
  * are lowered when a new extension process is started.
  */
-fragment_data *
-fragment_add(tvbuff_t *tvb, int offset, packet_info *pinfo, guint32 id,
-	     GHashTable *fragment_table, guint32 frag_offset,
+static gboolean
+fragment_add_work(fragment_data *fd_head, tvbuff_t *tvb, int offset,
+	     packet_info *pinfo, guint32 frag_offset,
 	     guint32 frag_data_len, gboolean more_frags)
 {
-	fragment_key key, *new_key;
-	fragment_data *fd_head;
 	fragment_data *fd;
 	fragment_data *fd_i;
 	guint32 max, dfpos;
 	unsigned char *old_data;
 
-	/* create key to search hash with */
-	key.src = pinfo->src;
-	key.dst = pinfo->dst;
-	key.id  = id;
-
-	fd_head = g_hash_table_lookup(fragment_table, &key);
-
-	/* have we already seen this frame ?*/
-	if (pinfo->fd->flags.visited) {
-		if (fd_head != NULL && fd_head->flags & FD_DEFRAGMENTED) {
-			return fd_head;
-		} else {
-			return NULL;
-		}
-	}
-
-	if (fd_head==NULL){
-		/* not found, this must be the first snooped fragment for this
-                 * packet. Create list-head.
-		 */
-		fd_head=g_mem_chunk_alloc(fragment_data_chunk);
-		/* head/first structure in list only holds no other data than
-                 * 'datalen' then we don't have to change the head of the list
-                 * even if we want to keep it sorted
-                 */
-		fd_head->next=NULL;
-		fd_head->datalen=0;
-		fd_head->offset=0;
-		fd_head->len=0;
-		fd_head->flags=0;
-		fd_head->data=NULL;
-
-		/*
-		 * We're going to use the key to insert the fragment,
-		 * so allocate a structure for it, and copy the
-		 * addresses, allocating new buffers for the address
-		 * data.
-		 */
-		new_key = g_mem_chunk_alloc(fragment_key_chunk);
-		COPY_ADDRESS(&new_key->src, &key.src);
-		COPY_ADDRESS(&new_key->dst, &key.dst);
-		new_key->id = key.id;
-		g_hash_table_insert(fragment_table, new_key, fd_head);
-	}
-
 	/* create new fd describing this fragment */
 	fd = g_mem_chunk_alloc(fragment_data_chunk);
 	fd->next = NULL;
@@ -545,7 +559,7 @@ fragment_add(tvbuff_t *tvb, int offset, packet_info *pinfo, guint32 id,
 			fd->flags      |= FD_TOOLONGFRAGMENT;
 			fd_head->flags |= FD_TOOLONGFRAGMENT;
 			LINK_FRAG(fd_head,fd);
-			return (fd_head);
+			return TRUE;
 		}
 		/* make sure it doesnt conflict with previous data */
 		if ( memcmp(fd_head->data+fd->offset,
@@ -553,11 +567,11 @@ fragment_add(tvbuff_t *tvb, int offset, packet_info *pinfo, guint32 id,
 			fd->flags      |= FD_OVERLAPCONFLICT;
 			fd_head->flags |= FD_OVERLAPCONFLICT;
 			LINK_FRAG(fd_head,fd);
-			return (fd_head);
+			return TRUE;
 		}
 		/* it was just an overlap, link it and return */
 		LINK_FRAG(fd_head,fd);
-		return (fd_head);
+		return TRUE;
 	}
 
 
@@ -576,7 +590,7 @@ fragment_add(tvbuff_t *tvb, int offset, packet_info *pinfo, guint32 id,
 		/* if we dont know the datalen, there are still missing
 		 * packets. Cheaper than the check below.
 		 */
-		return NULL;
+		return FALSE;
 	}
 
 
@@ -593,7 +607,7 @@ fragment_add(tvbuff_t *tvb, int offset, packet_info *pinfo, guint32 id,
 
 	if (max < (fd_head->datalen)) {
 		/* we have not received all packets yet */
-		return NULL;
+		return FALSE;
 	}
 
 
@@ -647,8 +661,227 @@ fragment_add(tvbuff_t *tvb, int offset, packet_info *pinfo, guint32 id,
 	/* mark this packet as defragmented.
            allows us to skip any trailing fragments */
 	fd_head->flags |= FD_DEFRAGMENTED;
+	fd_head->reassembled_in=pinfo->fd->num;
 
-	return fd_head;
+	return TRUE;
+}
+
+fragment_data *
+fragment_add_common(tvbuff_t *tvb, int offset, packet_info *pinfo, guint32 id,
+	     GHashTable *fragment_table, guint32 frag_offset,
+	     guint32 frag_data_len, gboolean more_frags,
+	     gboolean check_already_added)
+{
+	fragment_key key, *new_key;
+	fragment_data *fd_head;
+	fragment_data *fd_item;
+	gboolean already_added=pinfo->fd->flags.visited;
+
+	/* create key to search hash with */
+	key.src = pinfo->src;
+	key.dst = pinfo->dst;
+	key.id  = id;
+
+	fd_head = g_hash_table_lookup(fragment_table, &key);
+
+	/*
+	 * "already_added" is true if "pinfo->fd->flags.visited" is true;
+	 * if "pinfo->fd->flags.visited", this isn't the first pass, so
+	 * we've already done all the reassembly and added all the
+	 * fragments.
+	 *
+	 * If it's not true, but "check_already_added" is true, just check
+	 * if we have seen this fragment before, i.e., if we have already
+	 * added it to reassembly.
+	 * That can be true even if "pinfo->fd->flags.visited" is false
+	 * since we sometimes might call a subdissector multiple times.
+	 * As an additional check, just make sure we have not already added 
+	 * this frame to the reassembly list, if there is a reassembly list;
+	 * note that the first item in the reassembly list is not a
+	 * fragment, it's a data structure for the reassembled packet.
+	 * We don't check it because its "frame" member isn't initialized
+	 * to anything, and because it doesn't count in any case.
+	 */
+	if (!already_added && check_already_added && fd_head != NULL) {
+		for(fd_item=fd_head->next;fd_item;fd_item=fd_item->next){
+			if(pinfo->fd->num==fd_item->frame){
+				already_added=TRUE;
+			}
+		}
+	}
+	/* have we already added this frame ?*/
+	if (already_added) {
+		if (fd_head != NULL && fd_head->flags & FD_DEFRAGMENTED) {
+			return fd_head;
+		} else {
+			return NULL;
+		}
+	}
+
+	if (fd_head==NULL){
+		/* not found, this must be the first snooped fragment for this
+                 * packet. Create list-head.
+		 */
+		fd_head=g_mem_chunk_alloc(fragment_data_chunk);
+
+		/* head/first structure in list only holds no other data than
+                 * 'datalen' then we don't have to change the head of the list
+                 * even if we want to keep it sorted
+                 */
+		fd_head->next=NULL;
+		fd_head->datalen=0;
+		fd_head->offset=0;
+		fd_head->len=0;
+		fd_head->flags=0;
+		fd_head->data=NULL;
+		fd_head->reassembled_in=0;
+
+		/*
+		 * We're going to use the key to insert the fragment,
+		 * so allocate a structure for it, and copy the
+		 * addresses, allocating new buffers for the address
+		 * data.
+		 */
+		new_key = g_mem_chunk_alloc(fragment_key_chunk);
+		COPY_ADDRESS(&new_key->src, &key.src);
+		COPY_ADDRESS(&new_key->dst, &key.dst);
+		new_key->id = key.id;
+		g_hash_table_insert(fragment_table, new_key, fd_head);
+	}
+
+	if (fragment_add_work(fd_head, tvb, offset, pinfo, frag_offset,
+	    frag_data_len, more_frags)) {
+		/*
+		 * Reassembly is complete.
+		 */
+		return fd_head;
+	} else {
+		/*
+		 * Reassembly isn't complete.
+		 */
+		return NULL;
+	}
+}
+
+fragment_data *
+fragment_add(tvbuff_t *tvb, int offset, packet_info *pinfo, guint32 id,
+	     GHashTable *fragment_table, guint32 frag_offset,
+	     guint32 frag_data_len, gboolean more_frags)
+{
+	return fragment_add_common(tvb, offset, pinfo, id, fragment_table,
+	    frag_offset, frag_data_len, more_frags, TRUE);
+}
+
+/*
+ * For use when you can have multiple fragments in the same frame added
+ * to the same reassembled PDU, e.g. with ONC RPC-over-TCP.
+ */
+fragment_data *
+fragment_add_multiple_ok(tvbuff_t *tvb, int offset, packet_info *pinfo,
+			 guint32 id, GHashTable *fragment_table,
+			 guint32 frag_offset, guint32 frag_data_len,
+			 gboolean more_frags)
+{
+	return fragment_add_common(tvb, offset, pinfo, id, fragment_table,
+	    frag_offset, frag_data_len, more_frags, FALSE);
+}
+
+fragment_data *
+fragment_add_check(tvbuff_t *tvb, int offset, packet_info *pinfo,
+	     guint32 id, GHashTable *fragment_table,
+	     GHashTable *reassembled_table, guint32 frag_offset,
+	     guint32 frag_data_len, gboolean more_frags)
+{
+	fragment_key key, *new_key, *old_key;
+	gpointer orig_key, value;
+	fragment_data *fd_head;
+
+	/*
+	 * If this isn't the first pass, look for this frame in the table
+	 * of reassembled packets.
+	 */
+	if (pinfo->fd->flags.visited)
+		return g_hash_table_lookup(reassembled_table,
+		    (gpointer)pinfo->fd->num);
+
+	/* create key to search hash with */
+	key.src = pinfo->src;
+	key.dst = pinfo->dst;
+	key.id  = id;
+
+	if (!g_hash_table_lookup_extended(fragment_table, &key,
+					  &orig_key, &value)) {
+		/* not found, this must be the first snooped fragment for this
+                 * packet. Create list-head.
+		 */
+		fd_head=g_mem_chunk_alloc(fragment_data_chunk);
+
+		/* head/first structure in list only holds no other data than
+                 * 'datalen' then we don't have to change the head of the list
+                 * even if we want to keep it sorted
+                 */
+		fd_head->next=NULL;
+		fd_head->datalen=0;
+		fd_head->offset=0;
+		fd_head->len=0;
+		fd_head->flags=0;
+		fd_head->data=NULL;
+		fd_head->reassembled_in=0;
+
+		/*
+		 * We're going to use the key to insert the fragment,
+		 * so allocate a structure for it, and copy the
+		 * addresses, allocating new buffers for the address
+		 * data.
+		 */
+		new_key = g_mem_chunk_alloc(fragment_key_chunk);
+		COPY_ADDRESS(&new_key->src, &key.src);
+		COPY_ADDRESS(&new_key->dst, &key.dst);
+		new_key->id = key.id;
+		g_hash_table_insert(fragment_table, new_key, fd_head);
+
+		orig_key = new_key;	/* for unhashing it later */
+	} else {
+		/*
+		 * We found it.
+		 */
+		fd_head = value;
+	}
+
+	/*
+	 * If this is a short frame, then we can't, and don't, do
+	 * reassembly on it.  We just give up.
+	 */
+	if (tvb_reported_length(tvb) > tvb_length(tvb))
+		return NULL;
+
+	if (fragment_add_work(fd_head, tvb, offset, pinfo, frag_offset,
+	    frag_data_len, more_frags)) {
+		/*
+		 * Reassembly is complete.
+		 * Remove this from the table of in-progress
+		 * reassemblies, add it to the table of
+		 * reassembled packets, and return it.
+		 */
+
+		/*
+		 * Remove this from the table of in-progress reassemblies,
+		 * and free up any memory used for it in that table.
+		 */
+		old_key = orig_key;
+		fragment_unhash(fragment_table, old_key);
+
+		/*
+		 * Add this item to the table of reassembled packets.
+		 */
+		fragment_reassembled(fd_head, pinfo, reassembled_table);
+		return fd_head;
+	} else {
+		/*
+		 * Reassembly isn't complete.
+		 */
+		return NULL;
+	}
 }
 
 /*
@@ -699,7 +932,8 @@ fragment_add_seq_work(fragment_data *fd_head, tvbuff_t *tvb, int offset,
 			}
 		} else {
 			/* this was the first tail fragment, now we know the
-			 * length of the packet
+			 * sequence number of that fragment (which is NOT
+			 * the length of the packet!)
 			 */
 			fd_head->datalen = fd->offset;
 		}
@@ -714,8 +948,9 @@ fragment_add_seq_work(fragment_data *fd_head, tvbuff_t *tvb, int offset,
 		fd->flags      |= FD_OVERLAP;
 		fd_head->flags |= FD_OVERLAP;
 
-		/* make sure its not too long */
+		/* make sure it's not past the end */
 		if (fd->offset > fd_head->datalen) {
+			/* new fragment comes after the end */
 			fd->flags      |= FD_TOOLONGFRAGMENT;
 			fd_head->flags |= FD_TOOLONGFRAGMENT;
 			LINK_FRAG(fd_head,fd);
@@ -730,23 +965,68 @@ fragment_add_seq_work(fragment_data *fd_head, tvbuff_t *tvb, int offset,
 		  }
 		  last_fd=fd_i;
 		}
-		if(fd_i->datalen!=fd->datalen){
-			fd->flags      |= FD_OVERLAPCONFLICT;
-			fd_head->flags |= FD_OVERLAPCONFLICT;
+		if(fd_i){
+			/* new fragment overlaps existing fragment */
+			if(fd_i->len!=fd->len){
+				/*
+				 * They have different lengths; this
+				 * is definitely a conflict.
+				 */
+				fd->flags      |= FD_OVERLAPCONFLICT;
+				fd_head->flags |= FD_OVERLAPCONFLICT;
+				LINK_FRAG(fd_head,fd);
+				return TRUE;
+			}
+			g_assert(fd_head->len >= dfpos + fd->len);
+			if ( memcmp(fd_head->data+dfpos,
+			    tvb_get_ptr(tvb,offset,fd->len),fd->len) ){
+				/*
+				 * They have the same length, but the
+				 * data isn't the same.
+				 */
+				fd->flags      |= FD_OVERLAPCONFLICT;
+				fd_head->flags |= FD_OVERLAPCONFLICT;
+				LINK_FRAG(fd_head,fd);
+				return TRUE;
+			}
+			/* it was just an overlap, link it and return */
+			LINK_FRAG(fd_head,fd);
+			return TRUE;
+		} else {
+			/*
+			 * New fragment doesn't overlap an existing
+			 * fragment - there was presumably a gap in
+			 * the sequence number space.
+			 *
+			 * XXX - what should we do here?  Is it always
+			 * the case that there are no gaps, or are there
+			 * protcols using sequence numbers where there
+			 * can be gaps?
+			 *
+			 * If the former, the check below for having
+			 * received all the fragments should check for
+			 * holes in the sequence number space and for the
+			 * first sequence number being 0.  If we do that,
+			 * the only way we can get here is if this fragment
+			 * is past the end of the sequence number space -
+			 * but the check for "fd->offset > fd_head->datalen"
+			 * would have caught that above, so it can't happen.
+			 *
+			 * If the latter, we don't have a good way of
+			 * knowing whether reassembly is complete if we
+			 * get packet out of order such that the "last"
+			 * fragment doesn't show up last - but, unless
+			 * in-order reliable delivery of fragments is
+			 * guaranteed, an implementation of the protocol
+			 * has no way of knowing whether reassembly is
+			 * complete, either.
+			 *
+			 * For now, we just link the fragment in and
+			 * return.
+			 */
 			LINK_FRAG(fd_head,fd);
 			return TRUE;
 		}
-		g_assert(fd_head->len >= dfpos + fd->len);
-		if ( memcmp(fd_head->data+dfpos,
-			tvb_get_ptr(tvb,offset,fd->len),fd->len) ){
-			fd->flags      |= FD_OVERLAPCONFLICT;
-			fd_head->flags |= FD_OVERLAPCONFLICT;
-			LINK_FRAG(fd_head,fd);
-			return TRUE;
-		}
-		/* it was just an overlap, link it and return */
-		LINK_FRAG(fd_head,fd);
-		return TRUE;
 	}
 
 	/* If we have reached this point, the packet is not defragmented yet.
@@ -760,8 +1040,9 @@ fragment_add_seq_work(fragment_data *fd_head, tvbuff_t *tvb, int offset,
 
 
 	if( !(fd_head->datalen) ){
-		/* if we dont know the datalen, there are still missing
-		 * packets. Cheaper than the check below.
+		/* if we dont know the sequence number of the last fragment,
+		 * there are definitely still missing packets. Cheaper than
+		 * the check below.
 		 */
 		return FALSE;
 	}
@@ -837,6 +1118,7 @@ fragment_add_seq_work(fragment_data *fd_head, tvbuff_t *tvb, int offset,
 	/* mark this packet as defragmented.
            allows us to skip any trailing fragments */
 	fd_head->flags |= FD_DEFRAGMENTED;
+	fd_head->reassembled_in=pinfo->fd->num;
 
 	return TRUE;
 }
@@ -882,6 +1164,7 @@ fragment_add_seq(tvbuff_t *tvb, int offset, packet_info *pinfo, guint32 id,
                  * packet. Create list-head.
 		 */
 		fd_head=g_mem_chunk_alloc(fragment_data_chunk);
+
 		/* head/first structure in list only holds no other data than
                  * 'datalen' then we don't have to change the head of the list
                  * even if we want to keep it sorted
@@ -892,6 +1175,7 @@ fragment_add_seq(tvbuff_t *tvb, int offset, packet_info *pinfo, guint32 id,
 		fd_head->len=0;
 		fd_head->flags=FD_BLOCKSEQUENCE;
 		fd_head->data=NULL;
+		fd_head->reassembled_in=0;
 
 		/*
 		 * We're going to use the key to insert the fragment,
@@ -920,42 +1204,6 @@ fragment_add_seq(tvbuff_t *tvb, int offset, packet_info *pinfo, guint32 id,
 	}
 }
 
-/*
- * This function gets rid of an entry from a fragment table, given
- * a pointer to the key for that entry; it also frees up the key
- * and the addresses in it.
- */
-static void
-fragment_unhash(GHashTable *fragment_table, fragment_key *key)
-{
-	/*
-	 * Free up the copies of the addresses from the old key.
-	 */
-	g_free((gpointer)key->src.data);
-	g_free((gpointer)key->dst.data);
-
-	/*
-	 * Remove the entry from the fragment table.
-	 */
-	g_hash_table_remove(fragment_table, key);
-
-	/*
-	 * Free the key itself.
-	 */
-	g_mem_chunk_free(fragment_key_chunk, key);
-}
-
-/*
- * This function adds fragment_data structure to a reassembled-packet
- * hash table, using the frame data structure as the key.
- */
-static void
-fragment_reassembled(fragment_data *fd_head, packet_info *pinfo,
-	     GHashTable *reassembled_table)
-{
-	g_hash_table_insert(reassembled_table, pinfo->fd, fd_head);
-}
-
 /*
  * This does the work for "fragment_add_seq_check()" and
  * "fragment_add_seq_next()".
@@ -1007,7 +1255,8 @@ fragment_add_seq_check_work(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	 * If so, look for it in the table of reassembled packets.
 	 */
 	if (pinfo->fd->flags.visited)
-		return g_hash_table_lookup(reassembled_table, pinfo->fd);
+		return g_hash_table_lookup(reassembled_table,
+		    (gpointer)pinfo->fd->num);
 
 	/* create key to search hash with */
 	key.src = pinfo->src;
@@ -1031,6 +1280,7 @@ fragment_add_seq_check_work(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		fd_head->len=0;
 		fd_head->flags=FD_BLOCKSEQUENCE;
 		fd_head->data=NULL;
+		fd_head->reassembled_in=0;
 
 		if (!more_frags) {
 			/*
@@ -1084,8 +1334,8 @@ fragment_add_seq_check_work(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	}
 
 	/*
-	 * If this is a short frame, then we can't, and don't, do
-	 * reassembly on it.
+	 * If we don't have all the data that is in this fragment,
+	 * then we can't, and don't, do reassembly on it.
 	 *
 	 * If it's the first frame, handle it as an unfragmented packet.
 	 * Otherwise, just handle it as a fragment.
@@ -1093,7 +1343,7 @@ fragment_add_seq_check_work(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	 * If "more_frags" isn't set, we get rid of the entry in the
 	 * hash table for this reassembly, as we don't need it any more.
 	 */
-	if (tvb_reported_length(tvb) > tvb_length(tvb)) {
+	if (!tvb_bytes_exist(tvb, offset, frag_data_len)) {
 		if (!more_frags) {
 			/*
 			 * Remove this from the table of in-progress
@@ -1157,6 +1407,72 @@ fragment_add_seq_next(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	    more_frags, TRUE);
 }
 
+/*
+ * Process reassembled data; if we're on the frame in which the data
+ * was reassembled, put the fragment information into the protocol
+ * tree, and construct a tvbuff with the reassembled data, otherwise
+ * just put a "reassembled in" item into the protocol tree.
+ */
+tvbuff_t *
+process_reassembled_data(tvbuff_t *tvb, packet_info *pinfo, char *name,
+    fragment_data *fd_head, const fragment_items *fit,
+    gboolean *update_col_infop, proto_tree *tree)
+{
+	tvbuff_t *next_tvb;
+	gboolean update_col_info;
+
+	if (pinfo->fd->num == fd_head->reassembled_in) {
+		/*
+		 * OK, we have the complete reassembled payload.
+		 * Allocate a new tvbuff, referring to the reassembled
+		 * payload.
+		 */
+		if (fd_head->flags & FD_BLOCKSEQUENCE) {
+			next_tvb = tvb_new_real_data(fd_head->data,
+			      fd_head->len, fd_head->len);
+		} else {
+			next_tvb = tvb_new_real_data(fd_head->data,
+			      fd_head->datalen, fd_head->datalen);
+		}
+
+		/*
+		 * Add the tvbuff to the list of tvbuffs to which
+		 * the tvbuff we were handed refers, so it'll get
+		 * cleaned up when that tvbuff is cleaned up.
+		 */
+		tvb_set_child_real_data_tvbuff(tvb, next_tvb);
+
+		/* Add the defragmented data to the data source list. */
+		add_new_data_source(pinfo, next_tvb, name);
+
+		/* show all fragments */
+		if (fd_head->flags & FD_BLOCKSEQUENCE) {
+			update_col_info = !show_fragment_seq_tree(fd_head,
+			    fit,  tree, pinfo, next_tvb);
+		} else {
+			update_col_info = !show_fragment_tree(fd_head,
+			    fit, tree, pinfo, next_tvb);
+		}
+		if (update_col_infop != NULL)
+			*update_col_infop = update_col_info;
+	} else {
+		/* We don't have the complete reassembled payload. */
+		next_tvb = NULL;
+
+		/*
+		 * If there's a field to use for the number of
+		 * the frame in which the packet was reassembled,
+		 * add it to the protocol tree.
+		 */
+		if (fit->hf_reassembled_in != NULL) {
+			proto_tree_add_uint(tree,
+			    *(fit->hf_reassembled_in), tvb,
+			    0, 0, fd_head->reassembled_in);
+		}
+	}
+	return next_tvb;
+}
+
 /*
  * Show a single fragment in a fragment subtree.
  */
