@@ -2,7 +2,7 @@
  * Defines for smb packet dissection
  * Copyright 1999, Richard Sharpe <rsharpe@ns.aus.com>
  *
- * $Id: smb.h,v 1.48 2003/01/22 00:40:31 sahlberg Exp $
+ * $Id: smb.h,v 1.52 2003/04/13 23:58:37 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -26,6 +26,8 @@
 #ifndef _SMB_H
 #define _SMB_H
 
+extern gboolean sid_name_snooping;
+
 /* SMB command codes, from the SNIA CIFS spec. */
 extern const value_string smb_cmd_vals[];
 extern const value_string trans2_cmd_vals[];
@@ -153,6 +155,7 @@ extern const value_string nt_cmd_vals[];
 #define SMBE_eas_didnt_fit 275     /* Extended attributes didn't fit */
 #define SMBE_eas_nsup 282          /* Extended attributes not supported */
 #define SMBE_notify_buf_small 1022 /* Buffer too small to return change notify. */
+#define SMBE_serverunavailable 1722/* Server unavailable */
 #define SMBE_unknownipc 2142
 #define SMBE_noipc 66              /* don't support ipc */
 
@@ -288,6 +291,10 @@ typedef struct conv_tables {
 	GHashTable *matched;
 	/* this tables is used by DCERPC over SMB reassembly*/
 	GHashTable *dcerpc_fid_to_frame;
+	/* this table is to keep track of which frames we saw which dcerpc
+	   pdu in */
+	GHashTable *dcerpc_frame_to_dcerpc_pdu;	  
+
 	/* This table is used to track TID->services for a conversation */
 	GHashTable *tid_service;
         gboolean raw_ntlmssp;   /* Do extended security exc use raw ntlmssp */
@@ -324,7 +331,6 @@ extern int dissect_ipc_state(tvbuff_t *tvb, proto_tree *parent_tree,
     int offset, gboolean setstate);
 
 extern gboolean smb_dcerpc_reassembly;
-extern GHashTable *dcerpc_fragment_table;
 
 /*
  * NT and DOS error codes used by other dissectors.
