@@ -1,6 +1,6 @@
 /* libpcap.c
  *
- * $Id: libpcap.c,v 1.91 2003/01/23 04:04:00 guy Exp $
+ * $Id: libpcap.c,v 1.95 2003/03/25 06:04:54 guy Exp $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -213,8 +213,8 @@ static const struct {
 	{ 106,		WTAP_ENCAP_LINUX_ATM_CLIP },
 	{ 107,		WTAP_ENCAP_FRELAY },	/* Frame Relay */
 	{ 108,		WTAP_ENCAP_NULL },	/* OpenBSD loopback */
-#if 0
 	{ 109,		WTAP_ENCAP_ENC },	/* OpenBSD IPSEC enc */
+#if 0
 	{ 110,		WTAP_ENCAP_LANE_802_3 },/* ATM LANE 802.3 */
 	{ 111,		WTAP_ENCAP_HIPPI },	/* NetBSD HIPPI */
 #endif
@@ -324,7 +324,7 @@ static const struct {
 #if defined(DLT_ATM_RFC1483) && (DLT_ATM_RFC1483 == 13)
 	{ 13,		WTAP_ENCAP_ATM_RFC1483 },
 #elif defined(DLT_ENC) && (DLT_ENC == 13)
-	/* Put entry for DLT_ENC here */
+	{ 13,		WTAP_ENCAP_ENC },
 #endif
 
 	/*
@@ -517,9 +517,17 @@ int libpcap_open(wtap *wth, int *err)
 	 *	FDDI		15
 	 *
 	 * which correspond to DLT_IEEE802 (used for Token Ring),
-	 * DLT_PPP, and DLT_SLIP_BSDOS, respectively.  We shall
-	 * assume that if the minor version number is 2, and
-	 * the network type is 6, 9, or 15, that it's AIX libpcap.
+	 * DLT_PPP, and DLT_SLIP_BSDOS, respectively.  The ifType value
+	 * for a loopback interface is 24, which currently isn't
+	 * used by any version of libpcap I know about (and, as
+	 * tcpdump.org are assigning DLT_ values above 100, and
+	 * NetBSD started assigning values starting at 50, and
+	 * the values chosen by other libpcaps appear to stop at
+	 * 19, it's probably not going to be used by any libpcap
+	 * in the future).
+	 *
+	 * We shall assume that if the minor version number is 2, and
+	 * the network type is 6, 9, 15, or 24, that it's AIX libpcap.
 	 *
 	 * I'm assuming those older versions of libpcap didn't
 	 * use DLT_IEEE802 for Token Ring, and didn't use DLT_SLIP_BSDOS
@@ -548,6 +556,11 @@ int libpcap_open(wtap *wth, int *err)
 			hdr.network = 10;	/* DLT_FDDI, FDDI */
 			aix = TRUE;
 			break;
+
+		case 24:
+			hdr.network = 0;	/* DLT_NULL, loopback */
+			aix = TRUE;
+			break;
 		}
 	}
 	file_encap = wtap_pcap_encap_to_wtap_encap(hdr.network);
@@ -1217,15 +1230,32 @@ static int wtap_wtap_encap_to_pcap_encap(int encap)
 {
 	unsigned int i;
 
-	/*
-	 * Special-case WTAP_ENCAP_FDDI and WTAP_ENCAP_FDDI_BITSWAPPED;
-	 * both of them get mapped to DLT_FDDI (even though that may
-	 * mean that the bit order in the FDDI MAC addresses is wrong;
-	 * so it goes - libpcap format doesn't record the byte order,
-	 * so that's not fixable).
-	 */
-	if (encap == WTAP_ENCAP_FDDI || encap == WTAP_ENCAP_FDDI_BITSWAPPED)
+	switch (encap) {
+
+	case WTAP_ENCAP_FDDI:
+	case WTAP_ENCAP_FDDI_BITSWAPPED:
+		/*
+		 * Special-case WTAP_ENCAP_FDDI and
+		 * WTAP_ENCAP_FDDI_BITSWAPPED; both of them get mapped
+		 * to DLT_FDDI (even though that may mean that the bit
+		 * order in the FDDI MAC addresses is wrong; so it goes
+		 * - libpcap format doesn't record the byte order,
+		 * so that's not fixable).
+		 */
 		return 10;	/* that's DLT_FDDI */
+
+	case WTAP_ENCAP_PPP_WITH_PHDR:
+		/*
+		 * Also special-case PPP and Frame Relay with direction
+		 * bits; map them to PPP and Frame Relay, even though
+		 * that means that the direction of the packet is lost.
+		 */
+		return 9;
+
+	case WTAP_ENCAP_FRELAY_WITH_PHDR:
+		return 107;
+	}
+
 	for (i = 0; i < NUM_PCAP_ENCAPS; i++) {
 		if (pcap_to_wtap_map[i].wtap_encap_value == encap)
 			return pcap_to_wtap_map[i].dlt_value;
