@@ -1,6 +1,6 @@
 /* netxray.c
  *
- * $Id: netxray.c,v 1.74 2003/01/10 04:04:41 guy Exp $
+ * $Id: netxray.c,v 1.81 2003/03/31 21:11:49 guy Exp $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -66,9 +66,33 @@ struct netxray_hdr {
 };
 
 /*
- * # of ticks that equal 1 second
+ * Capture type, in xxb[20].
  */
-static double TpS[] = { 1e6, 1193000.0, 1193180.0 };
+#define CAPTYPE_NDIS	0	/* Capture on network interface using NDIS */
+#define CAPTYPE_BROUTER	1	/* Bridge/router captured with pod */
+#define CAPTYPE_GIGPOD	2	/* gigabit Ethernet captured with pod */
+#define CAPTYPE_PPP	3	/* PPP captured with pod */
+#define CAPTYPE_FRELAY	4	/* Frame Relay captured with pod */
+#define CAPTYPE_BROUTER2 5	/* Bridge/router captured with pod */
+#define CAPTYPE_HDLC	6	/* HDLC (X.25, ISDN) captured with pod */
+#define CAPTYPE_SDLC	7	/* SDLC captured with pod */
+#define CAPTYPE_HDLC2	8	/* HDLC captured with pod */
+#define CAPTYPE_BROUTER3 9	/* Bridge/router captured with pod */
+#define CAPTYPE_SMDS	10	/* SMDS DXI */
+#define CAPTYPE_BROUTER4 11	/* Bridge/router captured with pod */
+#define CAPTYPE_BROUTER5 12	/* Bridge/router captured with pod */
+
+/*
+ * # of ticks that equal 1 second
+ *
+ * XXX - the third item was 1193180.0, presumably because somebody found
+ * it gave the right answer for some captures, but 3 times that, i.e.
+ * 3579540.0, appears to give the right answer for some other captures.
+ *
+ * Is there something else in the file header to indicate which of those
+ * is correct?
+ */
+static double TpS[] = { 1e6, 1193000.0, 3579540.0 };
 #define NUM_NETXRAY_TIMEUNITS (sizeof TpS / sizeof TpS[0])
 
 /* Version number strings. */
@@ -236,13 +260,36 @@ int netxray_open(wtap *wth, int *err)
 		} else if (memcmp(hdr.version, vers_2_000, sizeof vers_2_000) == 0
 		    || memcmp(hdr.version, vers_2_001, sizeof vers_2_001) == 0
 		    || memcmp(hdr.version, vers_2_002, sizeof vers_2_002) == 0) {
-			if (hdr.timeunit > NUM_NETXRAY_TIMEUNITS) {
-				g_message("netxray: Unknown timeunit %u",
-					  hdr.timeunit);
-				*err = WTAP_ERR_UNSUPPORTED;
-				return -1;
+			/*
+			 * It appears that, at least for Ethernet
+			 * captures, if hdr.xxb[20] is 2, that indicates
+			 * that it's a gigabit Ethernet capture, possibly
+			 * from a special whizzo gigabit pod, and also
+			 * indicates that the time stamps have some
+			 * higher resolution than in other captures,
+			 * possibly thanks to a high-resolution
+			 * timer on the pod.
+			 */
+			if (hdr.network == 0 && hdr.xxb[20] == CAPTYPE_GIGPOD) {
+				/*
+				 * It appears that the time units for these
+				 * captures are nanoseconds, unless
+				 * hdr.timeunit is 2, in which case it's
+				 * 1/31250000.0 of a second.
+				 */
+				if (hdr.timeunit == 2)
+					timeunit = 31250000.0;
+				else
+					timeunit = 1e9;
+			} else {
+				if (hdr.timeunit > NUM_NETXRAY_TIMEUNITS) {
+					g_message("netxray: Unknown timeunit %u",
+					    hdr.timeunit);
+					*err = WTAP_ERR_UNSUPPORTED;
+					return -1;
+				}
+				timeunit = TpS[hdr.timeunit];
 			}
-			timeunit = TpS[hdr.timeunit];
 			version_major = 2;
 			file_type = WTAP_FILE_NETXRAY_2_00x;
 		} else {
@@ -270,20 +317,27 @@ int netxray_open(wtap *wth, int *err)
 		 *
 		 * In version 2, it looks as if there's stuff in the "xxb"
 		 * words of the file header to specify what particular
-		 * type of WAN capture we have; we handle the ones we've
-		 * seen, and punt on the others.
+		 * type of WAN capture we have.
 		 */
 		if (version_major == 2) {
 			switch (hdr.xxb[20]) {
 
-			case 4:
+			case CAPTYPE_PPP:
+				/*
+				 * PPP.
+				 */
+				file_encap = WTAP_ENCAP_PPP_WITH_PHDR;
+				break;
+
+			case CAPTYPE_FRELAY:
 				/*
 				 * Frame Relay.
 				 */
-				file_encap = WTAP_ENCAP_FRELAY;
+				file_encap = WTAP_ENCAP_FRELAY_WITH_PHDR;
 				break;
 
-			case 6:
+			case CAPTYPE_HDLC:
+			case CAPTYPE_HDLC2:
 				/*
 				 * Various HDLC flavors?
 				 */
@@ -308,6 +362,13 @@ int netxray_open(wtap *wth, int *err)
 				}
 				break;
 
+			case CAPTYPE_SDLC:
+				/*
+				 * SDLC.
+				 */
+				file_encap = WTAP_ENCAP_SDLC;
+				break;
+
 			default:
 				g_message("netxray: WAN capture subtype 0x%02x unknown or unsupported",
 				   hdr.xxb[20]);
@@ -633,8 +694,9 @@ netxray_set_pseudo_header(wtap *wth, const guint8 *pd, int len,
 			break;
 
 		case WTAP_ENCAP_LAPB:
+		case WTAP_ENCAP_FRELAY_WITH_PHDR:
 			/*
-			 * LAPB/X.25.
+			 * LAPB/X.25 and Frame Relay.
 			 *
 			 * The bottommost bit of byte 12 of "hdr.hdr_2_x.xxx"
 			 * is the direction flag.  (Probably true for other
@@ -644,6 +706,12 @@ netxray_set_pseudo_header(wtap *wth, const guint8 *pd, int len,
 			    (hdr->hdr_2_x.xxx[12] & 0x01) ? 0x00 : FROM_DCE;
 			break;
 
+		case WTAP_ENCAP_PPP_WITH_PHDR:
+		case WTAP_ENCAP_SDLC:
+			pseudo_header->p2p.sent =
+			    (hdr->hdr_2_x.xxx[12] & 0x01) ? TRUE : FALSE;
+			break;
+
 		case WTAP_ENCAP_ATM_PDUS_UNTRUNCATED:
 			pseudo_header->atm.flags = 0;
 			/*
@@ -755,33 +823,39 @@ netxray_close(wtap *wth)
 	g_free(wth->capture.netxray);
 }
 
-static const int wtap_encap[] = {
-    -1,		/* WTAP_ENCAP_UNKNOWN -> unsupported */
-    0,		/* WTAP_ENCAP_ETHERNET -> NDIS Ethernet */
-    1,		/* WTAP_ENCAP_TOKEN_RING -> NDIS Token Ring */
-    -1,		/* WTAP_ENCAP_SLIP -> unsupported */
-    -1,		/* WTAP_ENCAP_PPP -> unsupported */
-    2,		/* WTAP_ENCAP_FDDI -> NDIS FDDI */
-    2,		/* WTAP_ENCAP_FDDI_BITSWAPPED -> NDIS FDDI */
-    -1,		/* WTAP_ENCAP_RAW_IP -> unsupported */
-    -1,		/* WTAP_ENCAP_ARCNET -> unsupported */
-    -1,		/* WTAP_ENCAP_ATM_RFC1483 -> unsupported */
-    -1,		/* WTAP_ENCAP_LINUX_ATM_CLIP -> unsupported */
-    -1,		/* WTAP_ENCAP_LAPB -> unsupported */
-    -1,		/* WTAP_ENCAP_ATM_PDUS_UNTRUNCATED -> unsupported */
-    -1		/* WTAP_ENCAP_NULL -> unsupported */
+static const struct {
+	int	wtap_encap_value;
+	int	ndis_value;
+} wtap_encap_1_1[] = {
+    { WTAP_ENCAP_ETHERNET, 0 },		/* -> NDIS Ethernet */
+    { WTAP_ENCAP_TOKEN_RING, 1 },	/* -> NDIS Token Ring */
+    { WTAP_ENCAP_FDDI, 2 },		/* -> NDIS FDDI */
+    { WTAP_ENCAP_FDDI_BITSWAPPED, 2 },	/* -> NDIS FDDI */
 };
-#define NUM_WTAP_ENCAPS (sizeof wtap_encap / sizeof wtap_encap[0])
+#define NUM_WTAP_ENCAPS_1_1 (sizeof wtap_encap_1_1 / sizeof wtap_encap_1_1[0])
+
+static int
+wtap_encap_to_netxray_1_1_encap(int encap)
+{
+    unsigned int i;
+
+    for (i = 0; i < NUM_WTAP_ENCAPS_1_1; i++) {
+    	if (encap == wtap_encap_1_1[i].wtap_encap_value)
+	    return wtap_encap_1_1[i].ndis_value;
+    }
+
+    return -1;
+}
 
 /* Returns 0 if we could write the specified encapsulation type,
    an error indication otherwise. */
-int netxray_dump_can_write_encap(int encap)
+int netxray_dump_can_write_encap_1_1(int encap)
 {
     /* Per-packet encapsulations aren't supported. */
     if (encap == WTAP_ENCAP_PER_PACKET)
 	return WTAP_ERR_ENCAP_PER_PACKET_UNSUPPORTED;
 
-    if (encap < 0 || (unsigned)encap >= NUM_WTAP_ENCAPS || wtap_encap[encap] == -1)
+    if (wtap_encap_to_netxray_1_1_encap(encap) == -1)
 	return WTAP_ERR_UNSUPPORTED_ENCAP;
 
     return 0;
@@ -914,7 +988,7 @@ static gboolean netxray_dump_close_1_1(wtap_dumper *wdh, int *err)
     file_hdr.nframes = htolel(netxray->nframes);
     file_hdr.start_offset = htolel(CAPTUREFILE_HEADER_SIZE);
     file_hdr.end_offset = htolel(filelen);
-    file_hdr.network = htoles(wtap_encap[wdh->encap]);
+    file_hdr.network = htoles(wtap_encap_to_netxray_1_1_encap(wdh->encap));
     file_hdr.timelo = htolel(0);
     file_hdr.timehi = htolel(0);
 
@@ -934,6 +1008,48 @@ static gboolean netxray_dump_close_1_1(wtap_dumper *wdh, int *err)
     return TRUE;
 }
 
+static const struct {
+	int	wtap_encap_value;
+	int	ndis_value;
+} wtap_encap_2_0[] = {
+    { WTAP_ENCAP_ETHERNET, 0 },			/* -> NDIS Ethernet */
+    { WTAP_ENCAP_TOKEN_RING, 1 },		/* -> NDIS Token Ring */
+    { WTAP_ENCAP_FDDI, 2 },			/* -> NDIS FDDI */
+    { WTAP_ENCAP_FDDI_BITSWAPPED, 2 },		/* -> NDIS FDDI */
+    { WTAP_ENCAP_PPP_WITH_PHDR, 3 },		/* -> NDIS WAN */
+    { WTAP_ENCAP_FRELAY_WITH_PHDR, 3 },		/* -> NDIS WAN */
+    { WTAP_ENCAP_LAPB, 3 },			/* -> NDIS WAN */
+    { WTAP_ENCAP_SDLC, 3 },			/* -> NDIS WAN */
+};
+#define NUM_WTAP_ENCAPS_2_0 (sizeof wtap_encap_2_0 / sizeof wtap_encap_2_0[0])
+
+static int
+wtap_encap_to_netxray_2_0_encap(int encap)
+{
+    unsigned int i;
+
+    for (i = 0; i < NUM_WTAP_ENCAPS_2_0; i++) {
+    	if (encap == wtap_encap_2_0[i].wtap_encap_value)
+	    return wtap_encap_2_0[i].ndis_value;
+    }
+
+    return -1;
+}
+
+/* Returns 0 if we could write the specified encapsulation type,
+   an error indication otherwise. */
+int netxray_dump_can_write_encap_2_0(int encap)
+{
+    /* Per-packet encapsulations aren't supported. */
+    if (encap == WTAP_ENCAP_PER_PACKET)
+	return WTAP_ERR_ENCAP_PER_PACKET_UNSUPPORTED;
+
+    if (wtap_encap_to_netxray_2_0_encap(encap) == -1)
+	return WTAP_ERR_UNSUPPORTED_ENCAP;
+
+    return 0;
+}
+
 /* Returns TRUE on success, FALSE on failure; sets "*err" to an error code on
    failure */
 gboolean netxray_dump_open_2_0(wtap_dumper *wdh, gboolean cant_seek, int *err)
@@ -1003,11 +1119,22 @@ static gboolean netxray_dump_2_0(wtap_dumper *wdh,
     rec_hdr.orig_len = htoles(phdr->len);
     rec_hdr.incl_len = htoles(phdr->caplen);
 
-    if (phdr->pkt_encap == WTAP_ENCAP_IEEE_802_11_WITH_RADIO)
-    {
-        rec_hdr.xxx[12] = pseudo_header->ieee_802_11.channel;
-        rec_hdr.xxx[13] = pseudo_header->ieee_802_11.data_rate;
-        rec_hdr.xxx[14] = pseudo_header->ieee_802_11.signal_level;
+    switch (phdr->pkt_encap) {
+
+    case WTAP_ENCAP_IEEE_802_11_WITH_RADIO:
+	rec_hdr.xxx[12] = pseudo_header->ieee_802_11.channel;
+	rec_hdr.xxx[13] = pseudo_header->ieee_802_11.data_rate;
+	rec_hdr.xxx[14] = pseudo_header->ieee_802_11.signal_level;
+	break;
+
+    case WTAP_ENCAP_PPP_WITH_PHDR:
+    case WTAP_ENCAP_SDLC:
+	rec_hdr.xxx[12] |= pseudo_header->p2p.sent ? 0x01 : 0x00;
+	break;
+
+    case WTAP_ENCAP_FRELAY_WITH_PHDR:
+	rec_hdr.xxx[12] |= (pseudo_header->x25.flags & FROM_DCE) ? 0x00 : 0x01;
+	break;
     }
 
     nwritten = fwrite(&rec_hdr, 1, sizeof(rec_hdr), wdh->fh);
@@ -1068,9 +1195,28 @@ static gboolean netxray_dump_close_2_0(wtap_dumper *wdh, int *err)
     file_hdr.nframes = htolel(netxray->nframes);
     file_hdr.start_offset = htolel(CAPTUREFILE_HEADER_SIZE);
     file_hdr.end_offset = htolel(filelen);
-    file_hdr.network = htoles(wtap_encap[wdh->encap]);
+    file_hdr.network = htoles(wtap_encap_to_netxray_2_0_encap(wdh->encap));
     file_hdr.timelo = htolel(0);
     file_hdr.timehi = htolel(0);
+    switch (wdh->encap) {
+
+    case WTAP_ENCAP_PPP_WITH_PHDR:
+	file_hdr.xxb[20] = CAPTYPE_PPP;
+	break;
+
+    case WTAP_ENCAP_FRELAY_WITH_PHDR:
+	file_hdr.xxb[20] = CAPTYPE_FRELAY;
+	break;
+
+    case WTAP_ENCAP_LAPB:
+	file_hdr.xxb[20] = CAPTYPE_HDLC;
+	file_hdr.xxb[28] = 0;
+	break;
+
+    case WTAP_ENCAP_SDLC:
+	file_hdr.xxb[20] = CAPTYPE_SDLC;
+	break;
+    }
 
     memset(hdr_buf, '\0', sizeof hdr_buf);
     memcpy(hdr_buf, &file_hdr, sizeof(file_hdr));
